/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef DIRVIEW_H
#define DIRVIEW_H

#include <qlistview.h>
#include <qpopupmenu.h>
#include <qfile.h>

enum FilterTypes {HTML, TEXT, IMAGE, BACKUP};

class Directory: public QListViewItem
{
 public:
		Directory(QListView *parent, bool showAllFiles=TRUE);
		Directory(Directory *parent, const char *filename, bool showAllFiles=TRUE);
		virtual const char* text( int column ) const;
		virtual QString fullName();
		virtual void setOpen(bool);
		virtual void setup();
		virtual void setFilter(FilterTypes filterType, const QString &filterString);
 protected:
		bool showAll;
 	QString htmlFilter;
		QString textFilter;
		QString imageFilter;
		QString backupFilter;
		virtual void initView();
 private:
		QFile f;
		Directory *p;
		bool readable;
};

class DirView: public QListView
{
		Q_OBJECT
 public:
  DirView(bool showAllFiles=TRUE, QWidget *parent=0, const char *name=0);
		virtual QString getFileName(const QListViewItem *item);
		virtual void setStyle(GUIStyle s);
		virtual void setPalette(const QPalette &p);
		virtual void setFilter(FilterTypes filterType, const QString &filterString, bool updateView=FALSE);
		virtual void showAllFiles(bool enable=TRUE);
 protected:
		bool showAll;
		QPopupMenu *RBMenu;
		int openID;
		int renameID;
		int removeID;
		int mkdirID;
 	QString htmlFilter;
		QString textFilter;
		QString imageFilter;
		QString backupFilter;
	protected slots:
		virtual void fileMenu(QListViewItem *item, const QPoint &point, int column);
	 virtual void openFile();
		virtual void renameFile();
		virtual void removeFile();
		virtual void makeDir();
};

#endif
