/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

#include <qlayout.h>
#include "editorview.moc.h"

KTabBar::KTabBar(QWidget *parent, const char *name):QTabBar(parent, name)
{
		tabsList=new QList<QTab>;
}

KTabBar::~KTabBar()
{
		delete tabsList;
}

QSize KTabBar::sizeHint() const
{
		if (currentTab()!=-1) return QTabBar::sizeHint();
		return QSize(0, fontMetrics().height()+10);																						
}

int KTabBar::addTab(QTab *newTab)
{
		int id=QTabBar::addTab(newTab);
		tabsList->append(newTab);
		return id;
}

void KTabBar::removeTab(int id)
{
		QTab *tab;
		uint width=0;
		uint i;
		for (i=0; i<tabsList->count(); i++)
				{
						tab=tabsList->at(i);
						if (tab && tab->id == id)
								{
										width=tab->r.width()-4;
								 	tabList()->removeRef(tab);
										tabsList->removeRef(tab);
										break;
								};
				};
		if (width)
				for (i=0; i<tabsList->count(); i++)
						{
								tab=tabsList->at(i);
								if (tab && tab->id > id)
										{
												tab->r.moveBy(-width, 0);
												width=tab->r.width()-4;
										};
						};
		repaint();
};

void KTabBar::setTabLabel(int id, const char *label)
{
		QTab *t=tab(id);
		if (t)
				t->label=label;
		else
				return;
  QFontMetrics fm=fontMetrics();
		uint textWidth;
		QRect rect;
		for (uint i=0; i<tabsList->count(); i++)
				{
						t=tabsList->at(i);
						textWidth=fm.width(t->label);
						t->r.setRect(rect.isEmpty() ? 0 : rect.right()-3, 0, textWidth+24, fm.height()+10);
						rect=rect.unite(t->r);
				};
		repaint();
}

EditorView::EditorView(QWidget *parent, const char *name):QWidget(parent, name)

{
		QVBoxLayout *vbox=new QVBoxLayout(this);

		tabs=new KTabBar(this);
		tabs->setMinimumSize(tabs->sizeHint());
		tabs->setFocusPolicy(ClickFocus);
		vbox->addWidget(tabs);

		widgets=new QWidgetStack(this);
		//widgets->setMinimumSize(widgets->sizeHint());
		vbox->addWidget(widgets, 1);
		
		vbox->activate();

		connect(tabs, SIGNAL(selected(int)), widgets, SLOT(raiseWidget(int)));
		connect(widgets, SIGNAL(aboutToShow(int)), this, SIGNAL(aboutToShow(int)));
		connect(widgets, SIGNAL(aboutToShow(QWidget*)), this, SIGNAL(aboutToShow(QWidget*)));
}

int EditorView::addTab(QWidget *child, const char *label)
{
		QTab *tab = new QTab();
		tab->label = label;
		tab->enabled = TRUE;
		int id = tabs->addTab(tab);
		widgets->addWidget(child, id);
		widgets->raiseWidget(id);
		
		tabs->repaint();
		widgets->show();
		
		return id;
}

void EditorView::removeTab(QWidget *child)
{
		int id=widgets->id(child);
		widgets->removeWidget(child);
		tabs->removeTab(id);
		tabs->setCurrentTab(tabs->currentTab());
		widgets->raiseWidget(tabs->currentTab());
}

void EditorView::showPage(QWidget *child)
{
		int id=widgets->id(child);
		tabs->setCurrentTab(id);
}

void EditorView::setTabLabel(QWidget *child, const char *label)
{
		int id=widgets->id(child);
		tabs->setTabLabel(id, label);
}
