/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

#ifndef _HTML_H_

#define _HTML_H_

#include "htmltypes.h"

/*----------------------------------------------------HTML Tags-----------------------------------------------------*/

const char* GLOBAL_STRUCTURE[]={"Global document structure", "!DOCTYPE", "HTML", "HEAD", "TITLE", "META", "BODY"};
const char* BODY_STRUCTURE[]={"Body structure", "DIV", "SPAN", "H1", "H2", "H3", "H4", "H5", "H6", "ADDRESS", "CENTER"};
const char* TEXT_DIRECTION[]={"Text direction", "BDO"};
const char* TEXT_STRUCTURE[]={"Text structure", "EM", "STRONG", "DFN", "CODE", "SAMP", "KBD", "VAR", "CITE", "ABBR", "ACRONYM", "BLOCKQUOTE", "Q", "SUB", "SUP"};
const char* TEXT_PARAGRAPHS[]={"Lines and Paragraphs", "P", "BR", "HR", "PRE"};
const char* TEXT_CHANGES[]={"Text changes markup", "INS", "DEL"};
const char* LISTS[]={"Lists", "UL", "OL", "LI", "DL", "DT", "DD", "DIR", "MENU"};
const char* TABLES[]={"Tables", "TABLE", "CAPTION", "THEAD", "TFOOT", "TBODY", "COLGROUP", "COL", "TR", "TH", "TD"};
const char* LINKS[]={"Links", "A", "LINK", "BASE"};
const char* OBJECTS[]={"Images and other objects", "IMG", "OBJECT", "PARAM", "APPLET", "MAP", "AREA"};
const char* STYLE_SHEETS[]={"Style sheets", "STYLE"};
const char* FONTS[]={"Font style", "TT", "I", "B", "BIG", "SMALL", "STRIKE", "S", "U", "FONT", "BASEFONT"};
const char* FRAMES[]={"Frames", "FRAMESET", "FRAME", "NOFRAMES", "IFRAME"};
const char* FORMS[]={"Forms", "FORM", "INPUT", "BUTTON", "SELECT", "OPTGROUP", "OPTION", "TEXTAREA", "ISINDEX", "LABEL", "FIELDSET", "LEGEND"};
const char* SCRIPTS[]={"Scripts", "SCRIPT", "NOSCRIPT"};

const int TAG_GROUPS_COUNT=15;

const char** TAG_GROUPS[TAG_GROUPS_COUNT]=
{
	GLOBAL_STRUCTURE,
	BODY_STRUCTURE,
	TEXT_DIRECTION,
	TEXT_STRUCTURE,
	TEXT_PARAGRAPHS,
	TEXT_CHANGES,
	LISTS,
	TABLES,
	LINKS,
	OBJECTS,
	STYLE_SHEETS,
	FONTS,
	FRAMES,
	FORMS,
	SCRIPTS
};

const int TAG_GROUPS_SIZE[TAG_GROUPS_COUNT]=
{
	sizeof(GLOBAL_STRUCTURE)/sizeof(char*),
	sizeof(BODY_STRUCTURE)/sizeof(char*),
	sizeof(TEXT_DIRECTION)/sizeof(char*),
	sizeof(TEXT_STRUCTURE)/sizeof(char*),
	sizeof(TEXT_PARAGRAPHS)/sizeof(char*),
	sizeof(TEXT_CHANGES)/sizeof(char*),
	sizeof(LISTS)/sizeof(char*),
	sizeof(TABLES)/sizeof(char*),
	sizeof(LINKS)/sizeof(char*),
	sizeof(OBJECTS)/sizeof(char*),
	sizeof(STYLE_SHEETS)/sizeof(char*),
	sizeof(FONTS)/sizeof(char*),
	sizeof(FRAMES)/sizeof(char*),
	sizeof(FORMS)/sizeof(char*),
	sizeof(SCRIPTS)/sizeof(char*)
};

/*----------------------------------------------HTML Entities-------------------------------------------------------*/

const char* EVENTS_ENTITY[]={"onclick", "ondblclick", "onmousedown", "onmouseup", "onmouseover", "onmousemove", "onmouseout", "onkeypress", "onkeydown", "onkeyup"};
const char* EVENTS_ENTITY_NAME[]={"On click", "On double click", "On mouse down", "On mouse up", "On mouse over", "On mouse move", "On mouse out", "On key press", "On key down", "On key up"};
const int EVENTS_ENTITY_SIZE=sizeof(EVENTS_ENTITY)/sizeof(char*);

/*---------------------------------------HTML Tags Descriptions-------------------------------------------------*/

const HTMLTag TAGS[]=
{
	{"!DOCTYPE", FALSE, FALSE, FALSE, FORBIDDEN, DOCTYPE},
	{"HTML", FALSE, TRUE, FALSE, OPTIONAL, ABSENT},
	{"HEAD", FALSE, TRUE, FALSE, OPTIONAL, HEAD},
	{"TITLE", FALSE, TRUE, FALSE, REQUIRED, ABSENT},
	{"META", FALSE, TRUE, FALSE, FORBIDDEN, META},
	{"BODY", TRUE, TRUE, TRUE, OPTIONAL, BODY},
	{"DIV", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"SPAN", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"H1", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"H2", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"H3", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"H4", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"H5", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"H6", TRUE, TRUE, TRUE, REQUIRED, ALIGN},
	{"ADDRESS", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"CENTER", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"BDO", TRUE, TRUE, FALSE, REQUIRED, ABSENT},
	{"EM", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"STRONG", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"DFN", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"CODE", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"SAMP", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"KBD", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"VAR", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"CITE", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"ABBR", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"ACRONYM", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"BLOCKQUOTE", TRUE, TRUE, TRUE, REQUIRED, QUOTE},
	{"Q", TRUE, TRUE, TRUE, REQUIRED, QUOTE},
	{"SUB", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"SUP", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"P", TRUE, TRUE, TRUE, OPTIONAL, ALIGN},
	{"BR", TRUE, FALSE, FALSE, FORBIDDEN, BR},
	{"PRE", TRUE, TRUE, TRUE, REQUIRED, PRE},
	{"INS", TRUE, TRUE, TRUE, REQUIRED, EDIT},
	{"DEL", TRUE, TRUE, TRUE, REQUIRED, EDIT},
	{"UL", TRUE, TRUE, TRUE, REQUIRED, UL},
	{"OL", TRUE, TRUE, TRUE, REQUIRED, OL},
	{"LI", TRUE, TRUE, TRUE, OPTIONAL, LI},
	{"DL", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"DT", TRUE, TRUE, TRUE, OPTIONAL, ABSENT},
	{"DD", TRUE, TRUE, TRUE, OPTIONAL, ABSENT},
	{"DIR", TRUE, TRUE, TRUE, REQUIRED, COMPACT},
	{"MENU", TRUE, TRUE, TRUE, REQUIRED, COMPACT},
	{"TABLE", TRUE, TRUE, TRUE, REQUIRED, TABLE},
	{"CAPTION", TRUE, TRUE, TRUE, REQUIRED, CAPTION},
	{"THEAD", TRUE, TRUE, TRUE, OPTIONAL, THEAD},
	{"TFOOT", TRUE, TRUE, TRUE, OPTIONAL, THEAD},
	{"TBODY", TRUE, TRUE, TRUE, OPTIONAL, THEAD},
	{"COLGROUP", TRUE, TRUE, TRUE, OPTIONAL, COLGROUP},
	{"COL", TRUE, TRUE, TRUE, FORBIDDEN, COLGROUP},
	{"TR", TRUE, TRUE, TRUE, OPTIONAL, TR},
	{"TH", TRUE, TRUE, TRUE, OPTIONAL, TD},
	{"TD", TRUE, TRUE, TRUE, OPTIONAL, TD},
	{"A", TRUE, TRUE, TRUE, REQUIRED, A},
	{"LINK", TRUE, TRUE, TRUE, FORBIDDEN, LINK},
	{"BASE", FALSE, FALSE, FALSE, FORBIDDEN, BASE},
	{"IMG", TRUE, TRUE, TRUE, FORBIDDEN, IMG},
	{"OBJECT", TRUE, TRUE, TRUE, REQUIRED, OBJECT},
	{"PARAM", FALSE, FALSE, FALSE, FORBIDDEN, PARAM},
	{"APPLET", TRUE, FALSE, FALSE, REQUIRED, APPLET},
	{"MAP", TRUE, TRUE, TRUE, REQUIRED, MAP},
	{"AREA", TRUE, TRUE, TRUE, FORBIDDEN, AREA},
	{"STYLE", FALSE, TRUE, FALSE, REQUIRED, STYLE},
	{"TT", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"I", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"B", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"BIG", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"SMALL", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"STRIKE", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"S", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"U", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"BASEFONT", FALSE, FALSE, FALSE, FORBIDDEN, BASEFONT},
	{"FONT", TRUE, TRUE, FALSE, REQUIRED, FONT},
	{"HR", TRUE, FALSE, TRUE, FORBIDDEN, HR},
	{"FRAMESET", TRUE, FALSE, FALSE, REQUIRED, FRAMESET},
	{"FRAME", TRUE, FALSE, FALSE, FORBIDDEN, FRAME},
	{"NOFRAMES", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"IFRAME", TRUE, FALSE, FALSE, REQUIRED, IFRAME},
	{"FORM", TRUE, TRUE, TRUE, REQUIRED, FORM},
	{"INPUT", TRUE, TRUE, TRUE, FORBIDDEN, INPUT},
	{"BUTTON", TRUE, TRUE, TRUE, REQUIRED, BUTTON},
	{"SELECT", TRUE, TRUE, TRUE, REQUIRED, SELECT},
	{"OPTGROUP", TRUE, TRUE, TRUE, REQUIRED, OPTGROUP},
	{"OPTION", TRUE, TRUE, TRUE, OPTIONAL, OPTION},
	{"TEXTAREA", TRUE, TRUE, TRUE, REQUIRED, TEXTAREA},
	{"ISINDEX", TRUE, TRUE, FALSE, FORBIDDEN, ISINDEX},
	{"LABEL", TRUE, TRUE, TRUE, REQUIRED, LABEL},
	{"FIELDSET", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"LEGEND", TRUE, TRUE, TRUE, REQUIRED, LEGEND},
	{"SCRIPT", FALSE, FALSE, FALSE, REQUIRED, SCRIPT},
	{"NOSCRIPT", TRUE, TRUE, TRUE, REQUIRED, ABSENT},
	{"\0"}
};

#endif
