/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

#ifndef _HTML_EDIT_H_

#define _HTML_EDIT_H_

#include <keditcl.h>
#include <kcolorbtn.h>
#include <kprocess.h>
#include <qtabdialog.h>
#include "htmltypes.h"

// temporarily
#include "mykedit.h"

typedef QDict<QString> StrDict;
typedef QDictIterator<QString> StrDictIterator;

typedef QDict<HTMLTag> TagDict;

class HTMLEdit: public MyKEdit
{
	Q_OBJECT
public:
	enum {ANCHOR=0, COMMENT, STYLE, IMAGE, SCRIPT, TABLE, QUOTED, DEFAULT};
	HTMLEdit (KApplication *a=NULL, QWidget *parent=NULL, const char *name=NULL, const char *filename=NULL);
	virtual ~HTMLEdit();
	virtual int getFile(QFile &file, QString mask);
	virtual void createTableTemplate();
	virtual void insertTag(const QString &tag, const QString &attributes=0); 
	virtual void insertTagWithEdit(const QString &tag, const QString &attributes=0);
	virtual QString getTag();
	virtual QString getTagElement();
	virtual QString getTagAttributes();
	virtual int getTagGroup(QString *tag);
	virtual int browseExternal(bool tempFile=FALSE);
	virtual void setExternalBrowser(const char *browser);
	virtual QString projectRoot();
	virtual int startImport(const char *command);
	virtual int startExport(const char *command);
	virtual int startFilter(const char *command);
	virtual void print(const bool pixmapPrinting=FALSE);
protected:
	int cursorOn, cursorTimer;
	char* tempFile;
	int tempFileTimer;
	QColor palette[8];
	TagDict *tagDescriptions;
	QString externalBrowser;
	QString basePath;
	KShellProcess filter;
	QString filterBuffer;
	bool chooserConnected;
	inline int tabLength() {return fontMetrics().width("x")*8;};
	int textWidthWithTabs(const char *s, int nChars);
	virtual void paintCell(QPainter *painter, int row, int col);
	virtual void setupPainter(QPainter *painter);
 virtual void closeEvent(QCloseEvent *e);
	virtual void focusInEvent(QFocusEvent *e);
	virtual void focusOutEvent(QFocusEvent *e);
	virtual void resizeEvent(QResizeEvent *e);
	virtual void timerEvent (QTimerEvent *t);
	virtual void initPalette();
	virtual void initTagDescriptions();
	virtual void updateTag(const QString &tag);
	static QString stripQuotes(const QString *s);
	virtual StrDict* parseAttributes(const QString* attributes);
public slots: 
 virtual void openTagChooser();
	virtual void editTag();
	virtual void setProjectRoot();
protected slots:
	virtual void makeCursorVisible();
	virtual void insertTagFromChooser();
	virtual void filterOutput(KProcess *proc, char *buffer, int buflen);
	virtual void filterError(KProcess *proc, char *buffer, int buflen);
	virtual void filterTerminated(KProcess *proc);
};

class TableTemplateDialog: public QDialog
{
public:
	TableTemplateDialog(QWidget *parent=0, const char *name=0);
	QString getCaption();
	int getColumns();
	int getRows();
	int getHeaderColumns();
	int getHeaderRows();
protected:
	QLineEdit *tableCaption, *headWidth, *headHeight, *width, *height;
};

class TagChooserDialog: public QDialog
{
	Q_OBJECT
public:
	TagChooserDialog(QWidget *parent=0, const char *name=0);
	virtual const char* getTag();
protected:
	QListBox *tagList;
public slots:
	virtual void groupHighlighted(int index);
	virtual void tagSelected(int);
signals:
 void tagSelected();	
};

class AttributesTab: public QWidget
{
	Q_OBJECT
public:
	AttributesTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
};

class CoreTab: public AttributesTab
{
public:
	CoreTab(QWidget *parent=0, const char *name=0);
};

class EventsTab: public AttributesTab
{
public:
	EventsTab(QWidget *parent=0, const char *name=0);
};

class I18nTab: public AttributesTab
{
public:
	I18nTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QLineEdit *lang;
	QComboBox *dir;
};

class ImageTab: public AttributesTab
{
	Q_OBJECT
public:
	ImageTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLabel *preview;
	QLineEdit *imgSrc, *imgWidth, *imgHeight, *longDesc;
	QComboBox *align;
	QCheckBox *ismap;
	virtual bool eventFilter(QObject *o, QEvent *ev);
protected slots:
	virtual void imgSrcClicked();
	virtual void recalcSizeClicked();
	virtual void longDescClicked();
};

class DocTypeTab: public AttributesTab
{
public:
	DocTypeTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *dtd;
};

class MetaTab: public AttributesTab
{
public:
	MetaTab(QWidget *parent=0, const char *name=0);
};

class AlignTab: public AttributesTab
{
public:
	AlignTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align;
};

class FrameSetTab: public AttributesTab
{
public:
	FrameSetTab(QWidget *parent=0, const char *name=0);
};

class LabelTab: public AttributesTab
{
public:
	LabelTab(QWidget *parent=0, const char *name=0);
};

class BodyTab: public AttributesTab
{
 Q_OBJECT
public:
	BodyTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	enum {BGCOLOR, TEXT, LINK, VLINK, ALINK};
	QColor palette[5];
	QLineEdit *bgEdit;
	QLineEdit *bgcolorEdit, *textEdit, *linkEdit, *vlinkEdit, *alinkEdit;
	KColorButton	*bgcolorButton, *textButton, *linkButton, *vlinkButton, *alinkButton;
 virtual bool eventFilter(QObject *obj, QEvent *ev);
	virtual void updateEdit(QLineEdit *edit, KColorButton *button);
protected slots:
	virtual void bgClicked();
	virtual void colorChanged(const QColor&);
};

class QuoteTab: public AttributesTab
{
 Q_OBJECT
public:
	QuoteTab(const char *baseDir, QWidget *parent=0, const char *name=0);
protected:
	QString basePath;
	QLineEdit *cite;
protected slots:
	virtual void citeClicked();
};

class BRTab: public AttributesTab
{
public:
	BRTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *clear;
};

class PreTab: public AttributesTab
{
public:
	PreTab(QWidget *parent=0, const char *name=0);
};

class EditTab: public AttributesTab
{
 Q_OBJECT
public:
	EditTab(const char *baseDir, QWidget *parent=0, const char *name=0);
protected:
	QString basePath;
	QLineEdit *cite;
protected slots:
	virtual void citeClicked();
};

class ULTab: public AttributesTab
{
public:
	ULTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *type;
	QCheckBox *compact;
};

class OLTab: public AttributesTab
{
public:
	OLTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *type;
	QLineEdit *start;
	QCheckBox *compact;
};

class LITab: public AttributesTab
{
public:
	LITab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *type;
	QLineEdit *value;
};

class CompactTab: public AttributesTab
{
public:
	CompactTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QCheckBox *compact;
};

class CaptionTab: public AttributesTab
{
public:
	CaptionTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align;
};

class BaseTab: public AttributesTab
{
 Q_OBJECT
public:
	BaseTab(const char *baseDir, QWidget *parent=0, const char *name=0);
protected:
	QString basePath;
	QLineEdit *href;
protected slots:
	virtual void hrefClicked();
};

class MapTab: public AttributesTab
{
public:
	MapTab(QWidget *parent=0, const char *name=0);
};

class StyleTab: public AttributesTab
{
public:
	StyleTab(QWidget *parent=0, const char *name=0);
};

class HRTab: public AttributesTab
{
public:
	HRTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align;
	QCheckBox *noshade;
};

class OptGroupTab: public AttributesTab
{
public:
	OptGroupTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QCheckBox *disabled;
};

class OptionTab: public AttributesTab
{
public:
	OptionTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QCheckBox *selected;
	QCheckBox *disabled;
};

class IsIndexTab: public AttributesTab
{
public:
	IsIndexTab(QWidget *parent=0, const char *name=0);
};

class LegendTab: public AttributesTab
{
public:
	LegendTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align;
};

class ScriptTab: public AttributesTab
{
	Q_OBJECT
public:
	ScriptTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *src;
	QCheckBox *defer;
protected slots:
	virtual void srcClicked();
};

class ATab: public AttributesTab
{
 Q_OBJECT
public:
	ATab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *href;
	QComboBox *shape;
protected slots:
	virtual void hrefClicked();
};

class TableTab: public AttributesTab
{
	Q_OBJECT
public:
	TableTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
 QComboBox *frame, *rules, *align;
	QLineEdit *bgcolorEdit;
	KColorButton	*bgcolorButton;
 virtual bool eventFilter(QObject *obj, QEvent *ev);
protected slots:
	virtual void colorChanged(const QColor &color);
};

class THeadTab: public AttributesTab
{
public:
	THeadTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align, *valign;
};

class TRTab: public AttributesTab
{
	Q_OBJECT
public:
	TRTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align, *valign;
	QLineEdit *bgcolorEdit;
	KColorButton	*bgcolorButton;
 virtual bool eventFilter(QObject *obj, QEvent *ev);
protected slots:
	virtual void colorChanged(const QColor &color);
};

class TDTab: public AttributesTab
{
	Q_OBJECT
public:
	TDTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align, *valign, *scope;
	QCheckBox *nowrap;
	QLineEdit *bgcolorEdit;
	KColorButton	*bgcolorButton;
 virtual bool eventFilter(QObject *obj, QEvent *ev);
protected slots:
	virtual void colorChanged(const QColor &color);
};

class ColGroupTab: public AttributesTab
{
public:
	ColGroupTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *align, *valign;
};

class AreaTab: public AttributesTab
{
 Q_OBJECT
public:
	AreaTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *href;
	QCheckBox *nohref;
	QComboBox *shape;
protected slots:
	virtual void hrefClicked();
};

class BaseFontTab: public AttributesTab
{
	Q_OBJECT
public:
	BaseFontTab(QWidget *parent=0, const char *name=0);
	virtual void setParameters(StrDict *parameters);
protected:
	QLineEdit *colorEdit;
	KColorButton	*colorButton;
 virtual bool eventFilter(QObject *obj, QEvent *ev);
protected slots:
	virtual void colorChanged(const QColor &color);
};

class FontTab: public AttributesTab
{
	Q_OBJECT
public:
	FontTab(QWidget *parent=0, const char *name=0);
	virtual void setParameters(StrDict *parameters);
protected:
	QLineEdit *colorEdit;
	KColorButton	*colorButton;
 virtual bool eventFilter(QObject *obj, QEvent *ev);
protected slots:
	virtual void colorChanged(const QColor &color);
};

class FrameTab: public AttributesTab
{
	Q_OBJECT
public:
	FrameTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *src, *longDesc;
	QComboBox *frameBorder, *scrolling;
	QCheckBox *noresize;
protected slots:
	virtual void srcClicked();
	virtual void longDescClicked();
};

class IFrameTab: public AttributesTab
{
	Q_OBJECT
public:
	IFrameTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *src, *longDesc;
	QComboBox *frameBorder, *scrolling, *align;
protected slots:
	virtual void srcClicked();
	virtual void longDescClicked();
};

class FormTab: public AttributesTab
{
 Q_OBJECT
public:
	FormTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *action;
	QComboBox *method, *enctype;
protected slots:
	virtual void actionClicked();
};

class SelectTab: public AttributesTab
{
public:
	SelectTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QCheckBox *multiple;
	QCheckBox *disabled;
};

class TextAreaTab: public AttributesTab
{
public:
	TextAreaTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QCheckBox *readOnly;
	QCheckBox *disabled;
};

class ButtonTab: public AttributesTab
{
public:
	ButtonTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *type;
	QCheckBox *disabled;
};

class InputTab: public AttributesTab
{
	Q_OBJECT
public:
	InputTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *src, *usemap;
	QComboBox *type, *align;
	QCheckBox *readOnly, *checked, *disabled;
protected slots:
	virtual void srcClicked();
	virtual void usemapClicked();
};

class ParamTab: public AttributesTab
{
public:
	ParamTab(QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QComboBox *valueType;
};

class ObjectTab: public AttributesTab
{
	Q_OBJECT
public:
	ObjectTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *codeBase, *classID, *data, *archive, *usemap;
	QComboBox *align;
	QCheckBox *declare;
protected slots:
	virtual void codeBaseClicked();
	virtual void classIDClicked();
	virtual void dataClicked();
	virtual void archiveClicked();
	virtual void usemapClicked();
};

class AppletTab: public AttributesTab
{
	Q_OBJECT
public:
	AppletTab(const char *baseDir, QWidget *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
protected:
	QString basePath;
	QLineEdit *codeBase, *archive;
	QComboBox *align;
protected slots:
	virtual void codeBaseClicked();
	virtual void archiveClicked();
};

class HeadTab: public AttributesTab
{
 Q_OBJECT
public:
	HeadTab(const char *baseDir, QWidget *parent=0, const char *name=0);
protected:
	QString basePath;
	QLineEdit *profile;
protected slots:
	virtual void profileClicked();
};

class LinkTab: public AttributesTab
{
 Q_OBJECT
public:
	LinkTab(const char *baseDir, QWidget *parent=0, const char *name=0);
protected:
	QString basePath;
	QLineEdit *href;
protected slots:
	virtual void hrefClicked();
};

class TagEditDialog: public QTabDialog
{
public:
	TagEditDialog(const HTMLTag *element, HTMLEdit *parent=0, const char *name=0);
	virtual QString getParameters();
	virtual void setParameters(StrDict *parameters);
};

#endif
