/*
  $Id: mykedit.cpp,v 1.2 1999/01/09 17:41:30 pfeiffer Exp $

    Better Keybindings for the KEdit class in libkdeui,
    will hopefully be merged into kdelibs after the KDE 1.1 Release

    Copyright (C) 1999 Carsten Pfeiffer <carpdjih@sp.zrz.tu-berlin.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
  */


#include <qregexp.h>

#include "mykedit.h"


MyKEdit::MyKEdit( KApplication *a, QWidget *parent, const char *name, const char *filename ) : KEdit( a, parent, name, filename )
{
  myLeftWords.append( "\\s[^\\s\\t\\n\\r]" );
  //  myLeftWords.append( "[^\\s]\\s" );
  myRightWords.append( "\\s[^\\s\\t\\n\\r]" );
  myRightWords.append( "[^\\s\\t]\\n\\r?" );
}


MyKEdit::~MyKEdit()
{
}


void MyKEdit::keyPressEvent( QKeyEvent *e )
{
  if ((e->state() & ControlButton ) && (e->key() == Key_Left) ){
    jumpOneWordLeft();
    emit CursorPositionChanged();
    return;
  }

  if ((e->state() & ControlButton ) && (e->key() == Key_Right) ){
    jumpOneWordRight();
    emit CursorPositionChanged();
    return;
  }

  if ((e->state() & ControlButton ) && (e->key() == Key_Home) ){
    setCursorPosition( 0, 0 );
    emit CursorPositionChanged();
    return;
  }

  if ((e->state() & ControlButton ) && (e->key() == Key_End) ){
    int lines = numLines() - 1;
    setCursorPosition( lines, lineLength( lines ) );
    emit CursorPositionChanged();
    return;
  }


  KEdit::keyPressEvent( e );
}


void MyKEdit::jumpOneWordRight()
{
  int col, line;
  cursorPosition( &line, &col );

  int i;
  int tmppos  = -1;
  int newpos  = -1;
  int newline = -1;

  QRegExp s_pattern;
  QString string;

  for ( s_pattern = myRightWords.first();
	( !s_pattern.isEmpty() && s_pattern.isValid() );
	s_pattern = myRightWords.next() )
  {

    for(i = line; i < numLines(); i++) {
      string = textLine(i);
      tmppos = string.find(s_pattern, i == line ? col : 0 );

      // word found
      if( tmppos != -1 ) {

	if ( newline == -1 && newpos == -1 ) {
	  newline = i;
	  newpos = tmppos;
	}

	else { // remember the nearest word
	  if ( i < newline ) {
	    newline = i;
	    newpos = tmppos;
	  }
	  else if ( i == newline ) {
	    if ( tmppos < newpos ) {
	      newpos = tmppos;
	    }
	  }
	}
	
	break;
      }
    }
  }

  if ( newline != -1 && newpos != -1 )
    setCursorPosition( newline, ++newpos, false );
}


void MyKEdit::jumpOneWordLeft()
{
  int col, line;
  cursorPosition( &line, &col );

  int i;
  int tmppos  = -1;
  int newpos  = -1;
  int newline = -1;

  QRegExp s_pattern;
  QString string;

  if ( col >= 2 )
    col -= 2;
  else if ( line > 0 ) {
    line--;
    col = lineLength( line );
  }

  for ( s_pattern = myLeftWords.first();
	( !s_pattern.isEmpty() && s_pattern.isValid() );
	s_pattern = myLeftWords.next() )
  {

    for(i = line; i >= 0; i--) {
      string = textLine(i);
      tmppos = string.findRev(s_pattern, line == i ? col : string.length() );

      // word found
      if( tmppos != -1 ) {

	if ( newline == -1 && newpos == -1 ) {
	  newline = i;
	  newpos = tmppos;
	}

	else { // remember the nearest word
	  if ( i > newline ) {
	    newline = i;
	    newpos = tmppos;
	  }
	  else if ( i == newline ) {
	    if ( tmppos > newpos ) {
	      newpos = tmppos;
	    }
	  }
	}
	
	break;
      }
    }
  }

  if ( newline != -1 && newpos != -1 )
    setCursorPosition( newline, ++newpos, false );
}
