/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

//#include <iostream.h>
#include <config.h>
#include <qclipboard.h>

//Next line must be defined _ONLY_ in CVS repository
//#define HAVE_KIMGIO 1

#ifdef HAVE_KIMGIO
#include <kimgio.h>
#else
#ifdef HAVE_QIMGIO
#include <qimageio.h>
#else
#error WebMaker requires kimgio or qimgio for compilation!
#endif
#endif

#include <kiconloader.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kurl.h>
#include "webmaker.moc.h"

// Constructor of Application

HTMLEditor::HTMLEditor(const char *name):KTopLevelWidget(name)
{
#ifdef HAVE_KIMGIO
		kimgioRegister();
#else
		qInitImageIO();
#endif				
		setCaption("WebMaker");
		ReadSettings();
		InitMenuBar();
		InitStatusBar();
		InitMainWidget();
		InitToolBar();
		InitTimer();
		ProcessParameters();
}

HTMLEditor::~HTMLEditor()
{
		delete StatusBarTimer;
		delete toolBar(mainToolBar);
		delete toolBar(tagToolBar);
		delete MainWidget;
		delete statusBar();
		delete menuBar();
		delete EditorsList;
		delete EditorMenu;
		delete pluginList;
		delete recentFilesList;
}

bool HTMLEditor::eventFilter(QObject*, QEvent *ev)
{
		if (ev->type() == Event_MouseButtonPress) 
				{
						QMouseEvent *e = (QMouseEvent *)ev;
						if (e->button() == RightButton) 
								{
										//     QMouseEvent trickEvent(Event_MouseButtonPress, e->pos(), LeftButton, LeftButton);
										//     kapp->sendEvent(o, &trickEvent);
								};
				};
		return FALSE; 
}

void HTMLEditor::InitStatusBar()
{
		KStatusBar *StatusBar = new KStatusBar(this);
		StatusBar->setInsertOrder(KStatusBar::RightToLeft);
		StatusBar->insertItem("Line: 000000 Col: 000", ID_STATUS_LINE_COL);
		StatusBar->insertItem("XXX", ID_STATUS_INS);
		StatusBar->setAlignment(ID_STATUS_INS, AlignCenter);
		StatusBar->insertItem("", ID_STATUS_MAIN);
	
		StatusBar->changeItem("Line: 1 Col: 1", ID_STATUS_LINE_COL);
		StatusBar->changeItem("INS", ID_STATUS_INS);
	
		setStatusBar(StatusBar);
}

void HTMLEditor::InitMenuBar()
{
		unsigned int i;
		
		KMenuBar *MenuBar = new KMenuBar(this);
	
		file = new QPopupMenu();
		file->insertItem(i18n("Ne&w"), this, SLOT(NewFile()), CTRL + Key_N);
		file->insertItem(i18n("New From &Template..."), this, SLOT(NewFileFromTemplate()));
		file->insertItem(i18n("&Open..."), this, SLOT(OpenFile()), CTRL + Key_O);

		recent = new QPopupMenu();
	 for (i=0; i<recentFilesList->count(); i++)
    recent->insertItem(recentFilesList->at(i));
  file->insertItem (i18n("Open &Recent"), recent);
  connect(recent, SIGNAL(activated(int)), SLOT(OpenRecentFile(int)));
		
 	ID_Save = file->insertItem(i18n("&Save"), this, SLOT(SaveFile()), CTRL + Key_S);
		ID_SaveAs = file->insertItem(i18n("Save &as..."), this, SLOT(SaveFileAs()), SHIFT + CTRL + Key_S);
		ID_SaveAll = file->insertItem(i18n("Sa&ve All"), this, SLOT(SaveAllFiles()));
		file->insertItem(i18n("&Close"), this, SLOT(Close()), CTRL + Key_W);
		file->insertItem(i18n("C&lose All"), this, SLOT(CloseAll()));
		file->insertSeparator();
		file->insertItem(i18n("&Insert..."), this, SLOT(InsertFile()));
		file->insertSeparator();
	
		PluginInfo *plugin;
	
		for (i=0;i<pluginList->count(); i++)
				{
						plugin=pluginList->at(i);
						if (plugin->type==PluginInfo::IMPORT)
								plugin->ID=file->insertItem(i18n(plugin->name));
				};
	
		file->insertSeparator();
	
		for (i=0;i<pluginList->count(); i++)
				{
						plugin=pluginList->at(i);
						if (plugin->type==PluginInfo::EXPORT)
								plugin->ID=file->insertItem(i18n(plugin->name));
				};
	
		file->insertSeparator();
		ID_Print = file->insertItem(i18n("&Print..."), this, SLOT(Print()), CTRL + Key_P);
		file->insertSeparator();
		file->insertItem(i18n("E&xit"), this, SLOT(Exit()));
		MenuBar->insertItem(i18n("&File"), file);
	
		edit = new QPopupMenu();
		ID_Cut = edit->insertItem(i18n("Cu&t"), this, SLOT(Cut()), CTRL + Key_X);
		ID_Copy = edit->insertItem(i18n("&Copy"), this, SLOT(Copy()), CTRL + Key_C);
		ID_Paste = edit->insertItem(i18n("&Paste"), this, SLOT(Paste()), CTRL + Key_V);
		edit->insertItem(i18n("C&lear"), this, SLOT(Clear()));
		edit->insertItem(i18n("&Select All"), this, SLOT(SelectAll()));
		edit->insertSeparator();
		edit->insertItem(i18n("&Find..."), this, SLOT(Search()), CTRL + Key_F);
		edit->insertItem(i18n("&Replace..."), this, SLOT(Replace()), CTRL + Key_R);
		edit->insertItem(i18n("Find &Again"), this, SLOT(SearchAgain()));
		edit->insertSeparator();
		edit->insertItem(i18n("&Go to Line..."), this, SLOT(GotoLine()), CTRL + Key_G);
		MenuBar->insertItem(i18n("&Edit"), edit);
	
		tools = new QPopupMenu();
		ID_InsertTag = tools->insertItem(i18n("Tag &Chooser"), this, SLOT(TagChooser()), CTRL + Key_E);
		ID_EditTag = tools->insertItem(i18n("&Edit tag..."), this, SLOT(EditTag()), Key_F4);
		ID_TableTemplate = tools->insertItem(i18n("&Table Template..."), this, SLOT(TableTemplate()));
		tools->insertSeparator();
	
		for (i=0;i<pluginList->count(); i++)
				{
						plugin=pluginList->at(i);
						if (plugin->type==PluginInfo::FILTER)
								plugin->ID=tools->insertItem(i18n(plugin->name));
				};
	
		MenuBar->insertItem(i18n("&Tools"), tools);
	
		tags = new QPopupMenu();
		tags->insertItem(i18n("Bold"), this, SLOT(Bold()), SHIFT + CTRL + Key_B);
		tags->insertItem(i18n("Italic"), this, SLOT(Italic()), SHIFT + CTRL + Key_I);
		tags->insertItem(i18n("Underline"), this, SLOT(Underline()), SHIFT + CTRL + Key_U);
		tags->insertSeparator();
		tags->insertItem(i18n("Font Size +1"), this, SLOT(IncreaseFont()), CTRL + Key_Equal);
		tags->insertItem(i18n("Font Size -1"), this, SLOT(DecreaseFont()), CTRL + Key_Minus);
		tags->insertSeparator();
		tags->insertItem(i18n("Quote"), this, SLOT(Quote()), SHIFT + CTRL + Key_Q);
		tags->insertItem(i18n("Center"), this, SLOT(Center()), SHIFT + CTRL + Key_C);
		tags->insertItem(i18n("Horizontal Rule"), this, SLOT(HR()), SHIFT + CTRL + Key_H);
		tags->insertItem(i18n("Non-breaking Space"), this, SLOT(nbsp()), SHIFT + Key_Space);
		tags->insertItem(i18n("Line Break"), this, SLOT(LineBreak()), SHIFT + Key_Enter);
		tags->insertItem(i18n("Paragraph"), this, SLOT(Paragraph()), SHIFT + CTRL + Key_P);
		tags->insertSeparator();
		tags->insertItem(i18n("H1"), this, SLOT(H1()), CTRL + Key_1);
		tags->insertItem(i18n("H2"), this, SLOT(H2()), CTRL + Key_2);
		tags->insertItem(i18n("H3"), this, SLOT(H3()), CTRL + Key_3);
		MenuBar->insertItem(i18n("T&ags"), tags);
	
		QPopupMenu* view = new QPopupMenu();
		view->insertItem(i18n("&View in External Browser"), this, SLOT(ExternalBrowser()), Key_F11);
		MenuBar->insertItem(i18n("&View"), view);
	
		options = new QPopupMenu();
		options->setCheckable(TRUE);
	
		toolbars=new QPopupMenu();
		toolbars->setCheckable(TRUE);
		ID_MainToolBar = toolbars->insertItem(i18n("Main Toolbar"), this, SLOT(ToggleMainToolBar()));
		ID_TagToolBar = toolbars->insertItem(i18n("Tag Toolbar"), this, SLOT(ToggleTagToolBar()));
		toolbars->setItemChecked(ID_MainToolBar, showMainToolBar);
		toolbars->setItemChecked(ID_TagToolBar, showTagToolBar);
		options->insertItem(i18n("&Toolbars"), toolbars);
	
		options->insertItem(i18n("&Font..."), this, SLOT(ChangeFont()));
		ID_AutoIndent = options->insertItem(i18n("Auto &Indent"), this, SLOT(ToggleAutoIndent()));
		options->setItemChecked(ID_AutoIndent, indentMode);
		ID_ShowAllFiles = options->insertItem(i18n("Show All Files"), this, SLOT(ToggleFileView()));
		options->setItemChecked(ID_ShowAllFiles, showAllFiles);
		options->insertSeparator();
		ID_BrowseTempFile = options->insertItem(i18n("Browse Temporary File"), this, SLOT(ToggleBrowseTemp()));
		options->setItemChecked(ID_BrowseTempFile, browseTempCopy);
		int item=options->insertItem(i18n("&External Browsers..."), this, SLOT(DefaultHandler()));
		options->setItemEnabled(item, FALSE);
		options->insertSeparator();
		ID_AutoSaveOptions = options->insertItem(i18n("Auto Save Options"), this, SLOT(ToggleAutoSave()));
		options->setItemChecked(ID_AutoSaveOptions, autoSaveOptions);
		options->insertItem(i18n("&Save Options"), this, SLOT(SaveSettings()));
		MenuBar->insertItem(i18n("&Options"), options);

		windows=new QPopupMenu();
		MenuBar->insertItem(i18n("&Windows"), windows);
		
		MenuBar->insertSeparator();
	
		QPopupMenu* help;
		QString helpText;
		helpText.sprintf("%s - KDE HTML Editor\nVersion %s\n\nCopyright 1998, 1999\nAlexei Dets <dets@services.ru>",
																			APP_NAME, APP_VERSION);
		help = kapp->getHelpMenu(TRUE, i18n(helpText));
		MenuBar->insertItem( i18n("&Help"), help);

		connect(file, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(edit, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(tools, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(tags, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(view, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(options, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(windows, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
		connect(file, SIGNAL(activated(int)), this, SLOT(StartPlugin(int)));
  connect(windows, SIGNAL(activated(int)), SLOT(ChangeWindow(int)));
		setMenu(MenuBar);

		/*================Init editor RMB menu=====================*/
		
		EditorMenu = new QPopupMenu();
		ID_EditorInsertTag = EditorMenu->insertItem(i18n("&Tag Chooser"), this, SLOT(TagChooser()));
		ID_EditorEditTag = EditorMenu->insertItem(i18n("&Edit tag..."), this, SLOT(EditTag()));
		EditorMenu->insertSeparator();
		efile = new QPopupMenu();
		efile->insertItem(i18n("Ne&w..."), this, SLOT(NewFile()));
		efile->insertItem(i18n("&Open..."), this, SLOT(OpenFile()));
		ID_EditorSave=efile->insertItem(i18n("&Save"), this, SLOT(SaveFile()));
		ID_EditorSaveAs=efile->insertItem(i18n("S&ave as..."), this, SLOT(SaveFileAs()));
		efile->insertItem(i18n("&Close"), this, SLOT(Close()));
		EditorMenu->insertItem(i18n("&File"), efile);
		EditorMenu->insertSeparator();
		ID_EditorCut = EditorMenu->insertItem(i18n("Cu&t"), this, SLOT(Cut()));
		ID_EditorCopy = EditorMenu->insertItem(i18n("&Copy"), this, SLOT(Copy()));
		ID_EditorPaste = EditorMenu->insertItem(i18n("&Paste"), this, SLOT(Paste()));
		connect(EditorMenu, SIGNAL(aboutToShow()), this, SLOT(UpdateMenuBar()));
}

void HTMLEditor::InitToolBar()
{
		KIconLoader *loader = kapp->getIconLoader();
		QPixmap pixmap;
	
		KToolBar *ToolBar = new KToolBar(this);
		pixmap = loader->loadIcon("filenew.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(NewFile()), TRUE, i18n("New Document"));
		pixmap = loader->loadIcon("fileopen.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(OpenFile()), TRUE, i18n("Open Document"));
		pixmap = loader->loadIcon("filefloppy.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(SaveFile()), TRUE, i18n("Save Document"));
		pixmap = loader->loadIcon("fileprint.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Print()), TRUE, i18n("Print Document"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("browse.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(ExternalBrowser()), TRUE, i18n("View in External Browser"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("editcut.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Cut()), TRUE, i18n("Cut"));
		pixmap = loader->loadIcon("editcopy.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Copy()), TRUE, i18n("Copy"));
		pixmap = loader->loadIcon("editpaste.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Paste()), TRUE, i18n("Paste"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("search.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Search()), TRUE, i18n("Search"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("help.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Help()), TRUE, i18n("Help"));
	
		ToolBar->setBarPos( KToolBar::Top );
		mainToolBar=addToolBar(ToolBar);
		if (showMainToolBar)
				enableToolBar(KToolBar::Show, mainToolBar);
		else
				enableToolBar(KToolBar::Hide, mainToolBar);
	
	
		ToolBar = new KToolBar(this);
		pixmap = loader->loadIcon("tag_p.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Paragraph()), TRUE, i18n("<P>"));
		pixmap = loader->loadIcon("tag_nbsp.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(nbsp()), TRUE, i18n("&nbsp;"));
		pixmap = loader->loadIcon("tag_br.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(LineBreak()), TRUE, i18n("<BR>"));
		pixmap = loader->loadIcon("tag_hr.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(HR()), TRUE, i18n("<HR>"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("anchor.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Anchor()), TRUE, i18n("<A>"));
		pixmap = loader->loadIcon("tag_img.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Image()), TRUE, i18n("<IMG>"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("div_left.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Left()), TRUE, i18n("<DIV align=\"left\">"));
		pixmap = loader->loadIcon("div_center.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Center()), TRUE, i18n("<DIV align=\"center\">"));
		pixmap = loader->loadIcon("div_right.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Right()), TRUE, i18n("<DIV align=\"right\">"));
		pixmap = loader->loadIcon("div_justify.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Justify()), TRUE, i18n("<DIV align=\"justify\">"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("tag_b.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Bold()), TRUE, i18n("<B>"));
		pixmap = loader->loadIcon("tag_i.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Italic()), TRUE, i18n("<I>"));
		pixmap = loader->loadIcon("tag_u.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Underline()), TRUE, i18n("<U>"));
	
		ToolBar->insertSeparator();
		ToolBar->insertSeparator();
	
		pixmap = loader->loadIcon("tag_q.xpm");
		ToolBar->insertButton(pixmap, 0,
																								SIGNAL(clicked()), this,
																								SLOT(Quote()), TRUE, i18n("<Q>"));

		ToolBar->setBarPos( KToolBar::Top );
		tagToolBar=addToolBar(ToolBar);
		if (showTagToolBar)
				enableToolBar(KToolBar::Show, tagToolBar);
		else
				enableToolBar(KToolBar::Hide, tagToolBar); 
}

void HTMLEditor::InitMainWidget()
{
		MainWidget=new KNewPanner(this);
		MainWidget->setUnits(KNewPanner::Absolute);
		
		Dirs=new DirView(showAllFiles, MainWidget);
		Dirs->setFilter(HTML, htmlFilter);
		Dirs->setFilter(TEXT, textFilter);
		Dirs->setFilter(IMAGE, imageFilter);
		Dirs->setFilter(BACKUP, backupFilter);
		if (!htmlFilter.isEmpty() || !textFilter.isEmpty() ||
						!imageFilter.isEmpty() || !backupFilter.isEmpty())
				Dirs->setFilter(HTML, 0, TRUE);
		Dirs->setFocusPolicy(ClickFocus);
		
		connect(Dirs, SIGNAL(doubleClicked(QListViewItem*)),
										this, SLOT(OpenFileFromList(QListViewItem*)));
		connect(Dirs, SIGNAL(returnPressed(QListViewItem*)),
										this, SLOT(OpenFileFromList(QListViewItem*)));
		
  Editors=new EditorView(MainWidget);
		EditorsList=new HTMLEditList();
		EditorsList->setAutoDelete(TRUE);
		connect(Editors, SIGNAL(aboutToShow(QWidget*)), this, SLOT(SetCurrentEditor(QWidget*)));
		NewFile();
		
		MainWidget->setSeparatorPos(Dirs->sizeHint().width()*2);
		MainWidget->activate(Dirs, Editors);
		
		setView(MainWidget);
}

void HTMLEditor::InitTimer()
{
		StatusBarTimer = new QTimer(this);
		connect(StatusBarTimer, SIGNAL(timeout()),this,SLOT(RefreshMainStatus()));
}

void HTMLEditor::ProcessParameters()
{
		if (kapp->argc()==1) return;
		
		QTextStream cout(stdout, IO_WriteOnly);
		
		for (int i=1; i<kapp->argc(); i++)
				{
						QString fileName(kapp->argv()[i]);
						if (fileName=="--help" || fileName=="-h")
								{
										cout << APP_NAME << " - KDE HTML Editor\n";
										cout << "Version " << APP_VERSION << "\n";
										cout << "Copyright 1998, 1999, Alexei Dets <dets@services.ru>\n\n";
										cout << "Usage: webmaker [FILE1] [FILE2] ...\n";
										cout << "                --version, -v      - version information\n";
										cout << "                --help, -h         - this help message\n";
										exit(0);
										return;
								};
						if (fileName=="--version" || fileName=="-v")
								{
										cout << APP_NAME << " - KDE HTML Editor\n";
										cout << "Version " << APP_VERSION << "\n";
										cout << "Copyright 1998, 1999, Alexei Dets <dets@services.ru>\n";
										exit(0);
										return;
								};
						QFileInfo file(fileName);
						if (!file.exists() || file.isDir() || !file.isReadable()) continue;
		
						if (Editor->isModified() || Editor->getName()!="Untitled") NewFile();
		
						int result=Editor->loadFile(file.absFilePath(), KEdit::OPEN_READWRITE);
						if (result!=KEdit::KEDIT_OK) return; //    !!!
		
						QString s;
						s.sprintf(i18n("File opened: %s"), fileName.data());
						SetMainStatus(s);
						UpdateLineColStatus();
				};
}

void HTMLEditor::NewFile()
{
		Editor = new HTMLEdit( kapp, Editors, "Editor", "Untitled");
			
		connect(Editor, SIGNAL(CursorPositionChanged()), this, SLOT(UpdateLineColStatus()));
		connect(Editor, SIGNAL(toggle_overwrite_signal()), this, SLOT(ToggleInsStatus()));
		connect(Editor, SIGNAL(fileChanged()), this, SLOT(UpdateCaption()));
		connect(Editor, SIGNAL(loading()), this, SLOT(Loading()));
		connect(Editor, SIGNAL(saving()), this, SLOT(Saving()));

		Editor->installRBPopup(EditorMenu);
		Editor->installEventFilter(this);
	
		Editor->setFont(editorFont);
		Editor->setAutoIndentMode(indentMode);
		Editor->setExternalBrowser(browser.data());
	
		Editor->setFillColumnMode(0,FALSE);
		Editor->setWordWrap(FALSE);

		EditorsList->append(Editor);
		Editors->addTab(Editor, "Untitled");
		Editor->setFocusPolicy(StrongFocus);

		KDNDDropZone *dropZone=new KDNDDropZone(Editor, DndURL);
		connect(dropZone, SIGNAL(dropAction(KDNDDropZone*)),
										this, SLOT(OpenFileFromDrop(KDNDDropZone*)));

		UpdateCaption();
		UpdateLineColStatus();

		Editor->setEnabled(TRUE);
}

void HTMLEditor::NewFileFromTemplate()
{
		if (Editor->isModified() || Editor->getName()!="Untitled") NewFile();
		
		int result;
	
		if	(Editor->newFile()==KEdit::KEDIT_USER_CANCEL) return;
		UpdateLineColStatus();
	
 retry:
	
		QString fileName=KFileDialog::getOpenFileName(templatePath, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		result=Editor->loadFile(fileName, KEdit::OPEN_INSERT);
		switch (result)
				{
				case KEdit::KEDIT_OK:
						{
								QString s;
								s.sprintf(i18n("Template opened: %s"), fileName.data());
								SetMainStatus(s); 
								UpdateLineColStatus();
								break;
						};
				case KEdit::KEDIT_RETRY: goto retry; break;
				default: break;
				}
}

void HTMLEditor::OpenFile()
{
		if (Editor->isModified() || Editor->getName()!="Untitled") NewFile();
		
		int result;
	
 retry:
	
		result=Editor->openFile(KEdit::OPEN_READWRITE);
		switch (result)
				{
				case KEdit::KEDIT_OK:
						{
								QString s;
								s.sprintf(i18n("File opened: %s"), Editor->getName().data());
								SetMainStatus(s);
								UpdateLineColStatus();
								AddRecentFile(Editor->getName());
								break;
						};
				case KEdit::KEDIT_RETRY: goto retry; break;
				default: break;
				}
}

void HTMLEditor::OpenRecentFile(int id)
{
		QString fileName(recent->text(id));
		QFileInfo file(fileName);
		if (!file.exists() || file.isDir() || !file.isReadable()) return;
		
		if (Editor->isModified() || Editor->getName()!="Untitled") NewFile();
		
		int result=Editor->loadFile(fileName, KEdit::OPEN_READWRITE);
		if (result!=KEdit::KEDIT_OK) return; //    !!!
		
		QString s;
		s.sprintf(i18n("File opened: %s"), fileName.data());
		SetMainStatus(s);
		UpdateLineColStatus();
		AddRecentFile(Editor->getName());
}

void HTMLEditor::OpenFileFromList(QListViewItem *item)
{
		QString fileName(Dirs->getFileName(item));
		QFileInfo file(fileName);
		if (!file.exists() || file.isDir() || !file.isReadable()) return;
		
		if (Editor->isModified() || Editor->getName()!="Untitled") NewFile();
		
		int result=Editor->loadFile(fileName, KEdit::OPEN_READWRITE);
		if (result!=KEdit::KEDIT_OK) return; //    !!!
		
		QString s;
		s.sprintf(i18n("File opened: %s"), fileName.data());
		SetMainStatus(s);
		UpdateLineColStatus();
		AddRecentFile(Editor->getName());
}

void HTMLEditor::OpenFileFromDrop(KDNDDropZone *dropZone)
{
		QStrList URLList(dropZone->getURLList());
		KURL url;
		for (uint i=0; i<URLList.count(); i++)
				{
						url=URLList.at(i);
						if (url.isLocalFile())
								{
										QString fileName(url.path());
										QFileInfo file(fileName);
										if (!file.exists() || file.isDir() || !file.isReadable()) continue;
		
										if (Editor->isModified() || Editor->getName()!="Untitled") NewFile();
		
										int result=Editor->loadFile(fileName, KEdit::OPEN_READWRITE);
										if (result!=KEdit::KEDIT_OK) return; //    !!!
		
										QString s;
										s.sprintf(i18n("File opened: %s"), fileName.data());
										SetMainStatus(s);
										UpdateLineColStatus();
										AddRecentFile(Editor->getName());
								}
						else
								{
										KMsgBox::message(this, "Warning", "Only local files allowed!");
								}
				};
}

void HTMLEditor::SaveFile()
{
		if (Editor->isModified())
				{
						if (Editor->doSave()==KEdit::KEDIT_OK)
								{
										UpdateCaption();
										QString s;
										s.sprintf(i18n("Saved: %s"), Editor->getName().data());
										SetMainStatus(s);
										AddRecentFile(Editor->getName());
								}
				}
		else
				{
						SetMainStatus(i18n("No changes need to be saved"));
				}
}

void HTMLEditor::SaveFileAs()
{
		if (Editor->saveAs()== KEdit::KEDIT_OK)
				{
						UpdateCaption();
						QString s;
						s.sprintf(i18n("Saved as: %s"), Editor->getName().data());
						SetMainStatus(s);
						AddRecentFile(Editor->getName());
				}
}

void HTMLEditor::SaveAllFiles()
{
		HTMLEdit *currentEditor=Editor;
		Editor=EditorsList->first();
		while (Editor)
				{
						if (Editor->isModified())
								{
										if (Editor->getName()=="Untitled") Editors->showPage(Editor);
										if (Editor->doSave()==KEdit::KEDIT_OK)
												{
														UpdateCaption();
														QString s;
														s.sprintf(i18n("Saved: %s"), Editor->getName().data());
														SetMainStatus(s);
														AddRecentFile(Editor->getName());
												}
								}
						Editor=EditorsList->next();
				}
		Editors->showPage(currentEditor);
}

void HTMLEditor::Close()
{
		if (Editor->close())
				{
						HTMLEdit *temp=Editor;
						Editors->removeTab(Editor);
						EditorsList->removeRef(temp);
				};
		if (EditorsList->count()==0) NewFile();
}

void HTMLEditor::CloseAll()
{
		bool accept=TRUE;
		HTMLEdit *editor=EditorsList->first();
		while (editor)
				{
						accept=editor->close();
						if (accept)
								{
										HTMLEdit *temp=editor;
						    editor=EditorsList->next();
									 Editors->removeTab(temp);
										EditorsList->removeRef(temp);
								}
						else
								break;
				}
		if (accept) NewFile();
}

void HTMLEditor::Print()
{
		Editor->print(pixmapPrinting);
}

void HTMLEditor::InsertFile()
{
		int result;
	
 retry:
	
		result=Editor->insertFile();
		switch (result)
				{
				case KEdit::KEDIT_OK:	
						{
								SetMainStatus(i18n("File inserted"));
								UpdateLineColStatus();
								break;
						};
				case KEdit::KEDIT_RETRY: goto retry; break;
				default: break;
				};
}

void HTMLEditor::StartPlugin(int ID)
{
		PluginInfo *plugin;
		unsigned int num=pluginList->count();
		for (unsigned int i=0; i<num; i++)
				{
						plugin=pluginList->at(i);
						if (plugin->ID==ID)
								{
										switch(plugin->type)
												{	
												case PluginInfo::IMPORT:
														{
																if (Editor->isModified() || Editor->getName()!="Untitled") NewFile();
																Editor->startImport(plugin->commandLine);
																break;
														};
												case PluginInfo::EXPORT: Editor->startExport(plugin->commandLine);break;
												case PluginInfo::FILTER: Editor->startFilter(plugin->commandLine);break;
												};
										break;
								}
				}
}

void HTMLEditor::Exit()
{
		close();
}

void HTMLEditor::closeEvent(QCloseEvent* e)
{
		bool accept=TRUE;
		HTMLEdit *editor=EditorsList->first();
		while (editor)
				{
						accept=editor->close();
						if (accept)
								{
										HTMLEdit *temp=editor;
						    editor=EditorsList->next();
									 Editors->removeTab(temp);
										EditorsList->removeRef(temp);
								}
						else
								break;
				}
		if (accept)
				{
						SaveSettings(TRUE);
						e->accept();
				};
}

void HTMLEditor::Cut()
{
		Editor->cut();
		Editor->toggleModified(TRUE);
		UpdateLineColStatus();
}

void HTMLEditor::Copy()
{
		Editor->copyText();
}

void HTMLEditor::Paste()
{
		Editor->paste();
		Editor->toggleModified(TRUE);
		UpdateLineColStatus();
}

void HTMLEditor::Clear()
{
		Editor->clear();
		Editor->toggleModified(TRUE);
		UpdateLineColStatus();
}

void HTMLEditor::SelectAll()
{
		Editor->selectAll();
}

void HTMLEditor::Search()
{
		Editor->Search();
		UpdateLineColStatus();
}

void HTMLEditor::Replace()
{
		Editor->Replace();
		UpdateLineColStatus();
}

void HTMLEditor::SearchAgain()
{
		Editor->repeatSearch();
		UpdateLineColStatus();
}

void HTMLEditor::GotoLine()
{
		Editor->doGotoLine();
}

void HTMLEditor::TagChooser()
{
		Editor->openTagChooser();
}

void HTMLEditor::EditTag()
{
		Editor->editTag();
}

void HTMLEditor::TableTemplate()
{
		Editor->createTableTemplate();
}

void HTMLEditor::Bold()
{
		Editor->insertTag("B");
}

void HTMLEditor::Italic()
{
		Editor->insertTag("I");
}

void HTMLEditor::Underline()
{
		Editor->insertTag("U");
}

void HTMLEditor::IncreaseFont()
{
		Editor->insertTag("FONT", " size=\"+1\"");
}

void HTMLEditor::DecreaseFont()
{
		Editor->insertTag("FONT", " size=\"-1\"");
}

void HTMLEditor::Quote()
{
		Editor->insertTag("Q");
}

void HTMLEditor::Anchor()
{
		Editor->insertTagWithEdit("A");
}

void HTMLEditor::Image()
{
		Editor->insertTagWithEdit("IMG");
}

void HTMLEditor::Center()
{
		Editor->insertTag("DIV", " align=\"center\"");
}

void HTMLEditor::Left()
{
		Editor->insertTag("DIV", " align=\"left\"");
}

void HTMLEditor::Right()
{
		Editor->insertTag("DIV", " align=\"right\"");
}

void HTMLEditor::Justify()
{
		Editor->insertTag("DIV", " align=\"justify\"");
}

void HTMLEditor::HR()
{
		Editor->insertTag("HR");
}

void HTMLEditor::nbsp()
{
		int line, col;
		Editor->cursorPosition(&line, &col);
		Editor->insertAt("&nbsp;", line, col);
}

void HTMLEditor::LineBreak()
{
		Editor->insertTag("BR");
}

void HTMLEditor::Paragraph()
{
		Editor->insertTag("P");
}

void HTMLEditor::H1()
{
		Editor->insertTag("H1");
}

void HTMLEditor::H2()
{
		Editor->insertTag("H2");
}

void HTMLEditor::H3()
{
		Editor->insertTag("H3");
}

void HTMLEditor::ExternalBrowser()
{
		Editor->browseExternal(browseTempCopy);
}

void HTMLEditor::ToggleMainToolBar()
{
		showMainToolBar=!showMainToolBar;
		enableToolBar(KToolBar::Toggle, mainToolBar);
		toolbars->setItemChecked(ID_MainToolBar, showMainToolBar);
}

void HTMLEditor::ToggleTagToolBar()
{
		showTagToolBar=!showTagToolBar;
		enableToolBar(KToolBar::Toggle, tagToolBar);
		toolbars->setItemChecked(ID_TagToolBar, showTagToolBar);
}

void HTMLEditor::ChangeFont()
{
		Editor->selectFont();
		editorFont=Editor->font();
}

void HTMLEditor::ToggleAutoIndent()
{
		indentMode = !indentMode;
		Editor->setAutoIndentMode(indentMode);
		options->setItemChecked(ID_AutoIndent, indentMode);
		if (indentMode)
				SetMainStatus(i18n("Auto Indent Mode: On"));
		else
				SetMainStatus(i18n("Auto Indent Mode: Off"));
}

void HTMLEditor::ToggleFileView()
{
		showAllFiles = !showAllFiles;
		options->setItemChecked(ID_ShowAllFiles, showAllFiles);
		if (showAllFiles)
				SetMainStatus(i18n("Show All Files: On. Updating disk info..."));
		else
				SetMainStatus(i18n("Show All Files: Off. Updating disk info..."));
		Dirs->showAllFiles(showAllFiles);
}

void HTMLEditor::ToggleBrowseTemp()
{
		browseTempCopy=!browseTempCopy;
		options->setItemChecked(ID_BrowseTempFile, browseTempCopy);
		if (browseTempCopy)
				SetMainStatus(i18n("Browse Temporary File: On"));
		else
				SetMainStatus(i18n("Browse Temporary File: Off"));
}

void HTMLEditor::ToggleAutoSave()
{
		autoSaveOptions = !autoSaveOptions;
		options->setItemChecked(ID_AutoSaveOptions, autoSaveOptions);
		if (autoSaveOptions)
				SetMainStatus(i18n("Auto Save Options: On"));
		else
				SetMainStatus(i18n("Auto Save Options: Off"));
}

void HTMLEditor::ReadSettings()
{
		KConfig* config=kapp->getConfig();
		config->setGroup("Editor");
		editorFont=config->readFontEntry("Font", kapp->font());
		indentMode=config->readBoolEntry( "Auto Indent", FALSE);
		browseTempCopy=config->readBoolEntry("Browse Temporary File", TRUE);
		browser=config->readEntry("External Browser", "/usr/local/bin/netscape");
		templatePath=config->readEntry("Template Path", kapp->kde_datadir() + "/" + kapp->appName() + "/templates");
		pixmapPrinting=config->readBoolEntry( "WYSIWYG Printing", FALSE);
		config->setGroup("Toolbars");
		showMainToolBar=config->readBoolEntry( "Main Toolbar", TRUE);
		showTagToolBar=config->readBoolEntry( "Tag Toolbar", TRUE);
	
		pluginList=new QPluginList();
		QString nameMask("Name%d");
		QString commandMask("Command%d");
		QString name;
		QString command;
		uint i=1;
	
		config->setGroup("Import Plugins");
		name.sprintf(nameMask, i);
		command.sprintf(commandMask, i);
		while (config->hasKey(name))
				{
						PluginInfo *plugin=new PluginInfo;
						plugin->name=config->readEntry(name, 0);
						plugin->commandLine=config->readEntry(command, 0);
						plugin->type=PluginInfo::IMPORT;
						pluginList->append(plugin);
						i++;
						name.sprintf(nameMask, i);
						command.sprintf(commandMask, i);
				};
	
		config->setGroup("Export Plugins");
		i=1;
		name.sprintf(nameMask, i);
		command.sprintf(commandMask, i);
		while (config->hasKey(name))
				{
						PluginInfo *plugin=new PluginInfo;
						plugin->name=config->readEntry(name, 0);
						plugin->commandLine=config->readEntry(command, 0);
						plugin->type=PluginInfo::EXPORT;
						pluginList->append(plugin);
						i++;
						name.sprintf(nameMask, i);
						command.sprintf(commandMask, i);
				};
	
		config->setGroup("Filter Plugins");
		i=1;
		name.sprintf(nameMask, i);
		command.sprintf(commandMask, i);
		while (config->hasKey(name))
				{
						PluginInfo *plugin=new PluginInfo;
						plugin->name=config->readEntry(name, 0);
						plugin->commandLine=config->readEntry(command, 0);
						plugin->type=PluginInfo::FILTER;
						pluginList->append(plugin);
						i++;
						name.sprintf(nameMask, i);
						command.sprintf(commandMask, i);
				};
	
		config->setGroup("General");
		QSize size(500, 300);
		resize(config->readSizeEntry("Geometry", &size));
		autoSaveOptions=config->readBoolEntry("Auto Save Options", FALSE);
		maxRecentCount=config->readUnsignedNumEntry("Recent Files Count", 5);
		recentFilesList=new QStrList();
		config->readListEntry("Recent Files", *recentFilesList, ':');
		
		config->setGroup("Directory Tree");
		htmlFilter=config->readEntry("HTML Files", "");
		textFilter=config->readEntry("Text Files", "");
		imageFilter=config->readEntry("Image Files", "");
		backupFilter=config->readEntry("Backup Files", "");
		showAllFiles=config->readBoolEntry("Show All Files", FALSE);
}

void HTMLEditor::SaveSettings()
{
		SaveSettings(FALSE);
}

void HTMLEditor::SaveSettings(bool isExit)
{
		KConfig* config = kapp->getConfig();

		if (autoSaveOptions || !isExit)
				{
						config->setGroup("Editor");
						config->writeEntry("Font", editorFont);
						config->writeEntry("Auto Indent", indentMode);
						config->writeEntry("Browse Temporary File", browseTempCopy);
						config->writeEntry("External Browser", browser);
						config->writeEntry("Template Path", templatePath);
						config->writeEntry("WYSIWYG Printing", pixmapPrinting);
	
						config->setGroup("Toolbars");
						config->writeEntry("Main Toolbar", showMainToolBar);
						config->writeEntry("Tag Toolbar", showTagToolBar);
	
						config->setGroup("General");
						config->writeEntry("Auto Save Options", autoSaveOptions);
						config->writeEntry("Recent Files Count",	maxRecentCount);
		
						config->setGroup("Directory Tree");
						config->writeEntry("HTML Files", htmlFilter);
						config->writeEntry("Text Files", textFilter);
						config->writeEntry("Image Files", imageFilter);
						config->writeEntry("Backup Files", backupFilter);
						config->writeEntry("Show All Files", showAllFiles);
				}
		
		config->setGroup("General");
		config->writeEntry("Geometry", size());
		config->writeEntry("Recent Files", *recentFilesList, ':');
		
		config->sync();
}

void HTMLEditor::Help()
{
		KMsgBox::message(this, i18n("Sorry!"), i18n("Sorry, help is not available."));
}

/*====================Support routines===============================*/

void HTMLEditor::SetMainStatus(QString s)
{
		StatusBarTimer->stop();
		statusBar()->changeItem(s.data(),ID_STATUS_MAIN);
		StatusBarTimer->start(10000,TRUE);
}

void HTMLEditor::RefreshMainStatus()
{
		statusBar()->changeItem("",ID_STATUS_MAIN);
}

void HTMLEditor::UpdateLineColStatus()
{
		QString linenumber;
		linenumber.sprintf(i18n("Line: %d Col: %d"), Editor->currentLine() + 1, Editor->currentColumn() + 1);
		statusBar()->changeItem(linenumber.data(), ID_STATUS_LINE_COL);
}

void HTMLEditor::ToggleInsStatus()
{
		if(Editor->isOverwriteMode())
				{
						statusBar()->changeItem("OVR",ID_STATUS_INS);
				}
		else
				{
						statusBar()->changeItem("INS",ID_STATUS_INS);
				}
}

void HTMLEditor::ChangeWindow(int id)
{
		Editors->showPage(EditorsList->at(id));
}

void HTMLEditor::SetCurrentEditor(QWidget *editor)
{
	 Editor=(HTMLEdit*)editor;
		//Editor->raise();
		Editor->setFocus();
		
		UpdateCaption();
		UpdateLineColStatus();
}

void HTMLEditor::UpdateCaption()
{
		QString title;
		title.sprintf("%s: %s", APP_NAME, Editor->getName().data());
		setCaption(title);
		Editors->setTabLabel(Editor, QFileInfo(Editor->getName()).fileName());
}

void HTMLEditor::UpdateMenuBar()
{
		//Update "Windows" menu
		windows->clear();
		QString itemText(1024);
		HTMLEdit *editor=EditorsList->first();
		while (editor)
				{
						itemText.sprintf("%d: %s", EditorsList->at()+1, editor->getName().data());
						windows->insertItem(itemText);
						editor=EditorsList->next();
				};
		
		//All stuff is completely wrong :-((( Commenting out...
		//Must handle enabling/disabling of items in menus
		
		/*
		uint i, count;
		
		if (Editor->getTag().isEmpty())
				{
						tools->setItemEnabled(ID_InsertTag, TRUE);
						tools->setItemEnabled(ID_EditTag, FALSE);
						tools->setItemEnabled(ID_TableTemplate, TRUE);
						EditorMenu->setItemEnabled(ID_EditorInsertTag, TRUE);
						EditorMenu->setItemEnabled(ID_EditorEditTag, FALSE);
						count=tags->count();
						for (i=0; i<count; i++)
								{
										tags->setItemEnabled(tags->idAt(i), TRUE);
								};
				}
		else
				{
						tools->setItemEnabled(ID_InsertTag, FALSE);
						tools->setItemEnabled(ID_EditTag, TRUE);
						tools->setItemEnabled(ID_TableTemplate, FALSE);
						EditorMenu->setItemEnabled(ID_EditorInsertTag, FALSE);
						EditorMenu->setItemEnabled(ID_EditorEditTag, TRUE);
						count=tags->count();
						for (i=0; i<count; i++)
								{
										tags->setItemEnabled(tags->idAt(i), FALSE);
								};
				};
		if (QApplication::clipboard()->text())
				{
						edit->setItemEnabled(ID_Paste, TRUE);
						EditorMenu->setItemEnabled(ID_EditorPaste, TRUE);
				}
		else
				{
						edit->setItemEnabled(ID_Paste, FALSE);
						EditorMenu->setItemEnabled(ID_EditorPaste, FALSE);
				};
		if (Editor->markedText().isEmpty())
				{
						edit->setItemEnabled(ID_Cut, FALSE);
						edit->setItemEnabled(ID_Copy, FALSE);
						EditorMenu->setItemEnabled(ID_EditorCut, FALSE);
						EditorMenu->setItemEnabled(ID_EditorCopy, FALSE);
				}
		else
				{
						edit->setItemEnabled(ID_Cut, TRUE);
						edit->setItemEnabled(ID_Copy, TRUE);
						EditorMenu->setItemEnabled(ID_EditorCut, TRUE);
						EditorMenu->setItemEnabled(ID_EditorCopy, TRUE);
				};
		if (Editor->isModified() || Editor->getName()!="Untitled")
				{
						file->setItemEnabled(ID_Save, TRUE);
						file->setItemEnabled(ID_SaveAs, TRUE);
						file->setItemEnabled(ID_Print, TRUE);
						efile->setItemEnabled(ID_EditorSave, TRUE);
						efile->setItemEnabled(ID_EditorSaveAs, TRUE);
				}
		else
				{
						file->setItemEnabled(ID_Save, FALSE);
						file->setItemEnabled(ID_SaveAs, FALSE);
						file->setItemEnabled(ID_Print, FALSE);
						efile->setItemEnabled(ID_EditorSave, FALSE);
						efile->setItemEnabled(ID_EditorSaveAs, FALSE);
				};
		file->setItemEnabled(ID_SaveAll, FALSE);
		HTMLEdit *editor=EditorsList->first();
		while (editor)
				{
						if (editor->isModified() || editor->getName()!="Untitled")
								{
											file->setItemEnabled(ID_SaveAll, TRUE);
											break;
								};
						editor=EditorsList->next();
				}
		*/
}

void HTMLEditor::AddRecentFile(const char* fileName)
{
  if (recentFilesList->find(fileName)!=-1) return; // it's already there

  if (recentFilesList->count() < maxRecentCount)
    recentFilesList->append(fileName);
  else
				{
						recentFilesList->removeFirst();
						recentFilesList->append(fileName);
				};

  recent->clear();
  for (uint i=0; i < recentFilesList->count(); i++)
    recent->insertItem(recentFilesList->at(i));
}

void HTMLEditor::Loading()
{
		SetMainStatus("Loading...");
}

void HTMLEditor::Saving()
{
		SetMainStatus("Saving...");
}

void HTMLEditor::DefaultHandler()
{
		KMsgBox::message(this, i18n("Sorry!"), i18n("Sorry, feature is not implemented yet."));
}
