/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

#ifndef _WEBMAKER_H_

#define _WEBMAKER_H_

#include <ktopwidget.h>
#include <knewpanner.h>
#include "dirview.moc.h"
#include "editorview.moc.h"
#include "htmledit.moc.h"

#define APP_NAME "WebMaker"
#define APP_VERSION "0.8.0"

const int ID_STATUS_MAIN=0;
const int ID_STATUS_LINE_COL=1;
const int ID_STATUS_INS=2;

struct PluginInfo
{
	enum {IMPORT, EXPORT, FILTER};
	QString name;
	QString commandLine;
	int type;
	int ID;
};

typedef QList<PluginInfo> QPluginList;
typedef QList<HTMLEdit> HTMLEditList;

class HTMLEditor: public KTopLevelWidget
{
 Q_OBJECT
public:
	 KNewPanner *MainWidget;
	 DirView *Dirs;
	 EditorView *Editors;
		HTMLEditList *EditorsList;
  HTMLEdit *Editor;
  QPopupMenu *EditorMenu;
		QPopupMenu *efile;
		
  HTMLEditor (const char *name=0);
  virtual ~HTMLEditor();
public slots:
  void NewFile();
  void NewFileFromTemplate();
  void OpenFile();
  void OpenRecentFile(int id);
		void OpenFileFromList(QListViewItem *item);
		void OpenFileFromDrop(KDNDDropZone *dropZone);
  void SaveFile();
  void SaveFileAs();
		void SaveAllFiles();
		void Close();
		void CloseAll();
  void InsertFile();
		void StartPlugin(int ID);
		void Print();
  void Exit();
  
  void Cut();
  void Copy();
  void Paste();
		void Clear();
  void SelectAll();
  void Search();
  void Replace();
  void SearchAgain();
  void GotoLine();
  
  void TagChooser();
  void EditTag();
  void TableTemplate();
  
  void Bold();
  void Italic();
  void Underline();
  void IncreaseFont();
  void DecreaseFont();
  void Quote();
		void Anchor();
  void Image();
  void Center();
  void Left();
  void Right();
  void Justify();
  void HR();
  void nbsp();
  void LineBreak();
  void Paragraph();
  void H1();
  void H2();
  void H3();

  void ExternalBrowser();
  
  void ToggleMainToolBar();
  void ToggleTagToolBar();
  void ChangeFont();
  void ToggleAutoIndent();
		void ToggleFileView();
  void ToggleBrowseTemp();
  void ToggleAutoSave();
  void ReadSettings();
  void SaveSettings();
  void SaveSettings(bool isExit);

  void Help();
protected:
		//Menu Items IDs
		int ID_Save,
		    ID_SaveAs,
				  ID_SaveAll,
						ID_Print,
				  ID_EditorSave,
				  ID_EditorSaveAs,

						ID_Cut,
						ID_Copy,
						ID_Paste,
						ID_EditorCut,
						ID_EditorCopy,
						ID_EditorPaste,

						ID_InsertTag,
						ID_EditTag,
				  ID_TableTemplate,
						ID_EditorInsertTag,
						ID_EditorEditTag,

						ID_MainToolBar,
						ID_TagToolBar,
						ID_AutoIndent,
						ID_ShowAllFiles,
						ID_BrowseTempFile,
				  ID_AutoSaveOptions;
	
  int mainToolBar, tagToolBar;
		QPopupMenu *file;
		QPopupMenu *recent;
		QPopupMenu *edit;
		QPopupMenu *tools;
		QPopupMenu *tags;
  QPopupMenu *options;
  QPopupMenu *toolbars;
  QPopupMenu *windows;
  QTimer* StatusBarTimer;
  //configurable section
  bool showMainToolBar;
  bool showTagToolBar;
  QFont editorFont;
  bool indentMode;
  bool browseTempCopy;
  QString browser;
  QString templatePath;
		bool pixmapPrinting;
		QPluginList *pluginList;
		bool showAllFiles;          //Dirs must contain all files or web-related only
 	QString htmlFilter;         //File masks for directory tree
		QString textFilter;
		QString imageFilter;
		QString backupFilter;
		bool autoSaveOptions;
		uint maxRecentCount;        //Maximum number of items in "Open Recent" menu
		QStrList *recentFilesList;
  //end of configurable cection
  void InitStatusBar();
  void InitMenuBar();
  void InitToolBar();
		void InitMainWidget();
  void InitTimer();
		void ProcessParameters();
  void SetMainStatus(QString s);
  
  virtual bool eventFilter(QObject *o, QEvent *ev);
  virtual void closeEvent(QCloseEvent*e);
protected slots:
		void ChangeWindow(int id);
		void SetCurrentEditor(QWidget *editor);
  void RefreshMainStatus();
  void UpdateLineColStatus();
  void ToggleInsStatus();
  void UpdateCaption();
		void UpdateMenuBar();
		void AddRecentFile(const char* fileName);
  void Loading();
  void Saving();
  void DefaultHandler();
};

#endif
