#ifndef HEXEDIT_H
#define HEXEDIT_H

/* hexedit.h by Adam Rogoyski (apoc@laker.net) Temperanc on EFNet irc
 * Copyright (C) 1998, 1999 Adam Rogoyski
 * --- GNU General Public License Disclamer ---
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef HAVE_CONFIG_H
#include "../config.h"
#endif

#include <ctype.h>

#ifdef HAVE_NCURSES_H
#include <ncurses.h>
#else
#ifdef HAVE_CURSES_H
#include <curses.h>
#endif
#endif

#include <errno.h>

#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif

#ifdef STDC_HEADERS
#include <stdlib.h>
#include <string.h>
#endif

#include <stdio.h>
#include <signal.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SYS_IOCTL_H
#include <sys/ioctl.h>
#endif

#include <sys/stat.h>
#include <sys/types.h>

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#define  INSERT            1
#define  DELETE            2
#define  CHANGE            3

#define  MODIFIED          1
#define  READ_ONLY         2

#ifndef  __PDCURSES__
#define  ALT_V             118
#endif
#define  BACKSPACE         127
#define  CONTROL_A         1
#define  CONTROL_B         2
#define  CONTROL_C         3
#define  CONTROL_D         4
#define  CONTROL_E         5
#define  CONTROL_F         6
#define  CONTROL_G         7
#define  CONTROL_H         8
#define  CONTROL_I         9
#define  CONTROL_L         12
#define  CONTROL_N         14
#define  CONTROL_O         15
#define  CONTROL_P         16
#define  CONTROL_R         18
#define  CONTROL_T         20
#define  CONTROL_U         21
#define  CONTROL_V         22
#define  CONTROL_W         23
#define  CONTROL_X         24
#define  CONTROL_Y         25
#define  CONTROL_SLASH     31
#define  ESCAPE_CHARACTER  27
#define  SPACEBAR          32
#define  TAB               9

#define  M_0xFFFFFFF0      0xfffffff0
#define  M_0xF0            0xf0
#define  M_0x10            0x10
#define  M_0x0F            0x0f

#define  MAIN_HEIGHT       (LINES - 2)
#define  MAIN_TOP_LINE     1
#define  MAIN_BOTTOM_LINE  (LINES - 3)
#define  BOTTOM_LINE       (LINES - 2)

#define  EXTRA_BUF         1024
#define  NAME_POS          57

#define  FILE_MODE         0
#define  HEX_MODE          1
#define  ASCII_MODE        2
#define  CALCULATOR_MODE   3

#define  ASCII_CHAR_SET    0
#define  EBCDIC_CHAR_SET   1

void         asciiMode (wchar_t);
void         binary_calculator (void);
struct
foundit *    boyer_moore_search (unsigned char *, unsigned char *,
                                 unsigned long, unsigned long);
unsigned
char         bytecmp (const unsigned char *, const unsigned char *,
                      long);
char *       chompWhiteSpace (char *);
int          cursor_ascii_to_ebcdic (int);
void         deleteByte (int);
void         drawdump (long);
void         endKey (void);
void         exitProgram (void);
void         exitSave (int);
char *       fileSelect (void);
wchar_t      findEBCDIC (wchar_t);
int          getHexValue (wchar_t);
char         getAsciiValue (wchar_t);
void         handleArrowKeys (wchar_t);
void         handleInterrupt (int);
#ifndef __PDCURSES__
   void      handleSigwinch (int);
#endif
void         hexMode (wchar_t);
struct
ret_string * hex_string_box (WINDOW *, int, int);
void         homeKey (void);
void         init (int, char **);
void         insertByte (unsigned char, int);
int          isHexChar (wchar_t);
int          mappos (int);
int          mapcur (int);
void         over_write (wchar_t);
void         pagedown (void);
void         pageup (void);
void         printHelpWindow (void);
void         printStatusWindow (void);
int          popupFileSave (void);
void         popupGotoOffset (long);
void         popupHelp (void);
WINDOW *     popupWindow (int, int);
void         pushUndo (int, unsigned long, unsigned char);
void         redraw (void);
void         search (int);
char *       stringBox (WINDOW *, int, int, int, int, char *);
void         switchModes (void);
void         tabcursor (int *);
void         table_clear (void);
void *       table_lookup (long);
void         table_remove (long);
void         undoLastChange (void);
void         update_table (long);
void         usage (char *);

   /* functions specific to file.c and filekeys.c */
void statWindow    (const char * const);
void helpWindow    (const char * const);
struct FileNames * getDirectory (const char * const);
struct FileNames * merge (struct FileNames *, struct FileNames *);
struct FileNames * msort (struct FileNames *);
void printPage     (const struct FileNames *);
void file_key_up   (int *);
void file_key_down (int *, int);
void file_redraw   (int *);
void file_pagedown (int *, int);
void file_pageup   (int *);
void file_home     (int *);
void file_end      (int *, int);


   /* functions specific to calc.c */

#define BIN_BOX  1
#define OCT_BOX  2
#define DEC_BOX  3
#define HEX_BOX  4
#define BOX_LEN  64

struct calcEntryBox;
void calcStatWindow (void);
void drawEntryBox   (WINDOW *, struct calcEntryBox *, int, int, int);
void calcDrawBoxes (struct calcEntryBox *, struct calcEntryBox *,
                    struct calcEntryBox *, struct calcEntryBox *, int);
void calcNumberKey (struct calcEntryBox *, struct calcEntryBox *,
                    struct calcEntryBox *, struct calcEntryBox *,
                    wchar_t, int);
void calcArrowKey  (struct calcEntryBox *, struct calcEntryBox *,
                    struct calcEntryBox *, struct calcEntryBox *,
                    wchar_t, int *);

void calcBinaryString  (struct calcEntryBox *);
void calcOctalString   (struct calcEntryBox *);
void calcDecimalString (struct calcEntryBox *);
void calcHexString     (struct calcEntryBox *);



#ifdef MAIN_C
long offset = 0x00;              /* where in the file you are */
int position = 0;                /* where on the line you are */
int cursor_x = 10;               /* cursor tracking in hex mode */
int cursor_y = MAIN_TOP_LINE;
int acursor_x = 0;               /* cursor tracking in ascii mode */
int acursor_y = 0;
unsigned char *filebuf = NULL;   /* the file in memory */
int extrabuf = EXTRA_BUF;        /* always malloc extra bytes for
                                    inserting text */
int *newlines = NULL;            /* used in ascii mode, where to start lines */
#else
extern long offset;
extern int position;
extern int cursor_x;
extern int cursor_y;
extern int acursor_x;
extern int acursor_y;
extern unsigned char *filebuf;
extern int extrabuf;
extern int *newlines;
extern const unsigned char EBCDIC[];
extern const unsigned char ASCII_to_EBCDIC[];
#endif

struct
{
   WINDOW *wmain, *wstatus, *whelp;
   long filesize;
   char *progname;
   int mode;
   int modified;
   int tabb;
   int charset;
   int spacing;
} Globals;

struct foundit
   /* used for returning from searching */
{
   int flag;
   unsigned long offset;
};

struct ret_string
   /* used for returning from hexstring box */
{
   int len;
   unsigned char *str;
};

struct Bold_Table
   /* table of modified bytes to be marked as changed (bolded) */
{
   long len;
   long num;
   long *table;
} bold_table;


struct Change
   /* A node in the ChangeLog stack */
{
   int type;
   unsigned long offset;
   unsigned char v;
   struct Change *p;
};

struct ChangeLog
   /* Stack of changes to use with undo */
{
   int s;
   struct Change *base;
   struct Change *top;
} UndoStack;

struct FileNames
   /* node for linked list of filenames */
{
   char *filename;
   struct FileNames *p;
};


struct calcEntryBox
{
   char *tokens;
   char s[BOX_LEN + 1];
   int x;
   int pos_y;
   int pos_x;
   union
   {
      int i;
      unsigned int ui;
      long l;
      unsigned long ul;
   } value;
};


#endif
