/* init.c by Adam Rogoyski (apoc@laker.net) Temperanc on EFNet irc
 * Copyright (C) 1998, 1999 Adam Rogoyski
 * --- GNU General Public License Disclamer ---
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */


#include "hexedit.h"

#if defined(HAVE_GETOPT_H) && defined(HAVE_GETOPT_LONG)
#include <getopt.h>
#else
#include "../gnu/getopt.h"
#include "../gnu/getopt.c"
#include "../gnu/getopt1.c"
#endif

void
init (int argc, char **argv)
{
   unsigned char strbuf[2048];
   int i = 0;
   FILE *fp = NULL;
   int len = 0;
   int result = 0;
   struct stat filestat;
   void *p;

   int option_index = 0;
   struct option long_options[] = 
   {
      { "help", 0, 0, 0 },
      { "readonly", 0, 0, 0 },
      { "version", 0, 0, 0 },
      { NULL, 0, 0, 0 }
   };

   Globals.modified = 0;

   while ((i = getopt_long (argc, argv, "hrv", long_options,
              &option_index)) != EOF)
   {
      switch (i)
      {
         case 0:
            switch (option_index)
            {
               case 0:
                  usage (*argv);
                  exit (EXIT_SUCCESS);

               case 1:
                  Globals.modified = READ_ONLY;
                  break;

               case 2:
                  printf ("%s\n", VERSION);
                  exit (EXIT_SUCCESS);
            }
            break;
         case 'h':
            usage (*argv);
            exit (EXIT_SUCCESS);

         case 'r':
            Globals.modified = READ_ONLY;
            break;

         case 'v':
            printf ("%s\n", VERSION);
            exit (EXIT_SUCCESS);
      }
   }

   i = 0;
   p = initscr ();
   if (p == NULL)
   {
      fprintf (stderr, "Cannot initialize screen - curses!\n");
      exit (EXIT_FAILURE);
   }
   keypad (stdscr, TRUE);
   scrollok (stdscr, FALSE);
   cbreak ();
   noecho ();
   refresh ();

   Globals.wstatus = newwin (1, COLS, 0, 0);
   if (Globals.wstatus == NULL)
   {
      fprintf (stderr, "Cannot open status window - curses!\n");
      exit (EXIT_FAILURE);
   }
   scrollok (Globals.wstatus, FALSE);
   wattrset (Globals.wstatus, A_REVERSE);

   Globals.wmain = newwin (MAIN_HEIGHT, COLS, MAIN_TOP_LINE, 0);
   if (Globals.wmain == NULL)
   {
      fprintf (stderr, "Cannot open main window - curses!\n");
      exit (EXIT_FAILURE);
   }
   scrollok (Globals.wmain, FALSE);

   Globals.whelp = newwin (1, COLS, LINES - 1, 0);
   if (Globals.whelp == NULL)
   {
      fprintf (stderr, "Cannot open help window - curses!\n");
      exit (EXIT_FAILURE);
   }
   scrollok (Globals.whelp, FALSE);

#ifdef __PDCURSES__
   leaveok(Globals.wmain, FALSE);  /* Stupid macro */
#else
   leaveok (Globals.wmain, FALSE);
#endif


#ifndef __PDCURSES__
   signal (SIGWINCH, handleSigwinch);
#endif

   if (optind < argc)
   {
      fp = fopen (*(argv + optind), "rb");
      if (!fp)
      {
         erase (); refresh (); endwin ();
         perror (*(argv + optind));
         exit (EXIT_FAILURE);
      }
      *(argv + 1) = *(argv + optind);
      optind = argc;
   }


   while (!fp && optind <= argc)
   {
      *(argv + 1) = fileSelect ();

      fp = fopen (*(argv + 1), "rb");
      if (!fp)
      {
         #define FILE_ERR_WIDTH  50
         #define FILE_ERR_HEIGHT 6
         WINDOW *wpopup = popupWindow (FILE_ERR_WIDTH, FILE_ERR_HEIGHT);

         werase (wpopup);
         box (wpopup, 0, 0);
         wmove (wpopup, 1, (FILE_ERR_WIDTH / 2)
                 - (strlen ("Cannot Open File") / 2));
         wprintw (wpopup, "Cannot Open FILE");
         wmove (wpopup, FILE_ERR_HEIGHT - 3, (FILE_ERR_WIDTH / 2) -
                (strlen (strerror (errno)) + strlen ("Reason: ")) / 2);
         wprintw (wpopup, "Reason: %s", strerror (errno));
         wrefresh (wpopup);
         getch ();
      }
      else
         optind++;
   }

   if (!fp)
   {
      erase (); refresh (); endwin ();
      perror (*(argv + 1));
      exit (EXIT_FAILURE);
   }


   result = stat (*(argv + 1), &filestat);
   if (result == -1)
   {
      perror (*(argv + 1));
      exit (EXIT_FAILURE);
   }
   if (filestat.st_size == 0)
   {
      fprintf (stderr, "%s: File is Zero Bytes - Nothing to Edit!\n",
               *(argv + 1));
      exit (EXIT_FAILURE);
   }
   filebuf = malloc (filestat.st_size + 1 + extrabuf);
   if (!filebuf)
   {
      fprintf (stderr, "Not enough Memory!\n");
      exit (EXIT_FAILURE);
   }
   memset (filebuf, 0, filestat.st_size + 1 + extrabuf);



   i = 0;
   memset (strbuf, 0, 2048);
   while ((result = fread (strbuf, 1, 2048, fp)) > 0)
   {
      if (ferror (fp))
      {
         perror (*(argv + 1));
         exit (EXIT_FAILURE);
      }
      memcpy (filebuf + i, strbuf, result);
      i += result;
      if (i > filestat.st_size)
      {
         fprintf (stderr, "file has changed, restart\n");
         exit (EXIT_FAILURE);
      }
      memset (strbuf, 0, 2048);
   }
   *(filebuf + i) = '\0';
   result = fclose (fp);
   if (!fp)
   {
      perror ("fclose");
   }

   Globals.filesize = filestat.st_size - 1;
   Globals.mode = HEX_MODE;
   if (Globals.modified != READ_ONLY)
      Globals.modified = 0;
   Globals.tabb = 0;
   Globals.charset = ASCII_CHAR_SET;
   Globals.spacing = 1;
   len = strlen (*(argv + 1)) + 1;
   Globals.progname = malloc (len);
   strcpy (Globals.progname, *(argv + 1));
   signal (SIGINT, handleInterrupt);

   newlines = malloc (BOTTOM_LINE * sizeof (int));
   if (!newlines)
   {
      erase ();
      refresh ();
      endwin ();
      fprintf (stderr, "Not enough memory.\n");
      exit (EXIT_FAILURE);
   }

   bold_table.len = EXTRA_BUF;
   bold_table.num = 0;
   bold_table.table = malloc (sizeof (long) * bold_table.len);
   if (!bold_table.table)
   {
      erase ();
      refresh ();
      endwin ();
      fprintf (stderr, "Not enough memory.\n");
      exit (EXIT_FAILURE);   
   }
   memset (bold_table.table, 0, EXTRA_BUF * sizeof (long));

   UndoStack.s = 0;
   UndoStack.base = NULL;
   UndoStack.top = NULL;

   for (i = 0; i < COLS; i++)
      waddch (Globals.wstatus, ' ');
   printStatusWindow ();
   printHelpWindow ();

   drawdump (offset);
   wrefresh (Globals.wmain);
   cursor_y = MAIN_TOP_LINE;
   cursor_x = 10;
   move (cursor_y, cursor_x);
   refresh ();
}
