/* misc.c by Adam Rogoyski (apoc@laker.net) Temperanc on EFNet irc
 * Copyright (C) 1998, 1999 Adam Rogoyski
 * --- GNU General Public License Disclamer ---
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */


#include "hexedit.h"

#ifdef HAVE_TERMIO_H
#include <termio.h>
#else
#ifdef HAVE_TERMIOS_H
#include <termios.h>
#else
#ifdef HAVE_SYS_TERMIOS_H
#include <sys/termios.h>
#endif
#endif
#endif

int    table_compare (const void *, const void *);
void * table_lookup (long);


int
mappos (int pos)
/* This is used for cursor_x.  It has a value of 0-F, depending on what
 * byte you are on in the line, and this returns the corresponding screen
 * cursor position */
{
   if (Globals.spacing)
   {
      switch (pos)
      {
         case 0:
            return 10;
         case 1:
            return 13;
         case 2:
            return 16;
         case 3:
            return 19;
         case 4:
            return 23;
         case 5:
            return 26;
         case 6:
            return 29;
         case 7:
            return 32;
         case 8:
            return 37;
         case 9:
            return 40;
         case 10:
            return 43;
         case 11:
            return 46;
         case 12:
            return 50;
         case 13:
            return 53;
         case 14:
            return 56;
         case 15:
            return 59;
      }
   }
   else 
   {
      switch (pos)
      {
         case 0:
            return 10;
         case 1:
            return 12;
         case 2:
            return 14;
         case 3:
            return 16;
         case 4:
            return 19;
         case 5:
            return 21;
         case 6:
            return 23;
         case 7:
            return 25;
         case 8:
            return 28;
         case 9:
            return 30;
         case 10:
            return 32;
         case 11:
            return 34;
         case 12:
            return 37;
         case 13:
            return 39;
         case 14:
            return 41;
         case 15:
            return 43;
      }
   }
   return 0;
}



int
mapcur (int cur)
/* This takes the cursor position and returns what byte it is on that line
 */
{
   if (Globals.spacing)
   {
      switch (cur)
      {
         case 10:
         case 11:
            return 0;
         case 13:
         case 14:
            return 1;
         case 16:
         case 17:
            return 2;
         case 19:
         case 20:
            return 3;
         case 23:
         case 24:
            return 4;
         case 26:
         case 27:
            return 5;
         case 29:
         case 30:
            return 6;
         case 32:
         case 33:
            return 7;
         case 37:
         case 38:
            return 8;
         case 40:
         case 41:
            return 9;
         case 43:
         case 44:
            return 10;
         case 46:
         case 47:
            return 11;
         case 50:
         case 51:
            return 12;
         case 53:
         case 54:
            return 13;
         case 56:
         case 57:
            return 14;
         case 59:
         case 60:
            return 15;
      }
   }
   else
   {
      switch (cur)
      {
         case 10:
         case 11:
            return 0;
         case 12:
         case 13:
            return 1;
         case 14:
         case 15:
            return 2;
         case 16:
         case 17:
            return 3;
         case 19:
         case 20:
            return 4;
         case 21:
         case 22:
            return 5;
         case 23:
         case 24:
            return 6;
         case 25:
         case 26:
            return 7;
         case 28:
         case 29:
            return 8;
         case 30:
         case 31:
            return 9;
         case 32:
         case 33:
            return 10;
         case 34:
         case 35:
            return 11;
         case 37:
         case 38:
            return 12;
         case 39:
         case 40:
            return 13;
         case 41:
         case 42:
            return 14;
         case 43:
         case 44:
            return 15;
      }
   }
   return 0;
}


int
cursor_ascii_to_ebcdic (int cur)
{
   if (!Globals.spacing)
   {
      switch (cur)
      {
         case 10:
            return 10;
         case 11:
            return 11;
         case 13:
            return 12;
         case 14:
            return 13;
         case 16:
            return 14;
         case 17:
            return 15;
         case 19:
            return 16;
         case 20:
            return 17;
         case 23:
            return 19;
         case 24:
            return 20;
         case 26:
            return 21;
         case 27:
            return 22;
         case 29:
            return 23;
         case 30:
            return 24;
         case 32:
            return 25;
         case 33:
            return 26;
         case 37:
            return 28;
         case 38:
            return 29;
         case 40:
            return 30;
         case 41:
            return 31;
         case 43:
            return 32;
         case 44:
            return 33;
         case 46:
            return 34;
         case 47:
            return 35;
         case 50:
            return 37;
         case 51:
            return 38;
         case 53:
            return 39;
         case 54:
            return 40;
         case 56:
            return 41;
         case 57:
            return 42;
         case 59:
            return 43;
         case 60:
            return 44;
      }
   }
   else
   {
      switch (cur)
      {
         case 10:
            return 10;
         case 11:
            return 11;
         case 12:
            return 13;
         case 13:
            return 14;
         case 14:
            return 16;
         case 15:
            return 17;
         case 16:
            return 19;
         case 17:
            return 20;
         case 19:
            return 23;
         case 20:
            return 24;
         case 21:
            return 26;
         case 22:
            return 27;
         case 23:
            return 29;
         case 24:
            return 30;
         case 25:
            return 32;
         case 26:
            return 33;
         case 28:
            return 37;
         case 29:
            return 38;
         case 30:
            return 40;
         case 31:
            return 41;
         case 32:
            return 43;
         case 33:
            return 44;
         case 34:
            return 46;
         case 35:
            return 47;
         case 37:
            return 50;
         case 38:
            return 51;
         case 39:
            return 53;
         case 40:
            return 54;
         case 41:
            return 56;
         case 42:
            return 57;
         case 43:
            return 59;
         case 44:
            return 60;
      }
   }
   return 0;
}



void
usage (char *prog)
{
   printf ("[N]Curses Hexedit by Adam Rogoyski (apoc@laker.net)\n" \
     "Copyright (C) 1998, 1999 Adam Rogoyski\n" \
     "This is free software; see the source for copying conditions.\n" \
     "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n" \
     "PARTICULAR PURPOSE.\n\n" \
     "Usage: %s [options] [file]\n" \
     "Options:\n" \
     "  -h, --help              Print this message and exit\n"
     "  -r, --readonly          Do not modifying of the file\n"
     "  -v, --version           Print the version number and exit\n",
     prog);
}



void
handleInterrupt (int i)
#define QUIT_BOX_WIDTH  50
#define QUIT_BOX_HEIGHT 6
{
   int done = 0;
   int j = 0;
   int value = 1;
   wchar_t in = 0;
   WINDOW *win = popupWindow (QUIT_BOX_WIDTH, QUIT_BOX_HEIGHT);

   signal (SIGINT, SIG_IGN);


   box (win, 0, 0);
   wmove (win, 2, QUIT_BOX_WIDTH / 2 - strlen ("Quit without Saving?") / 2);
   wprintw (win, "Quit without Saving?");
   wmove (win, 4, QUIT_BOX_WIDTH / 2 - 5 - strlen (" Yes "));
   wattrset (win, A_REVERSE);
   wprintw (win, " Yes ");
   wattroff (win, A_REVERSE);
   wmove (win, 4, QUIT_BOX_WIDTH / 2 + 5 - strlen (" No "));
   wprintw (win, " No ");
   wmove (win, 4, QUIT_BOX_WIDTH / 2 - 5 - strlen (" Yes "));
   wrefresh (win);

   wmove (Globals.whelp, 0, 0);
   wprintw (Globals.whelp, "^G/^X/Escape Cancel");
   for (j = strlen ("^G/^X/Escape Cancel"); j < COLS; j++)
      wprintw (Globals.whelp, " ");
   wrefresh (Globals.whelp);

   while (!done && (in = getch ()) != '\n')
   {
      switch (in)
      {
         case TAB:
         case KEY_RIGHT:
         case KEY_LEFT:
            value ^= 1;
            if (value)
            {
               wmove (win, 4, QUIT_BOX_WIDTH / 2 - 5 - strlen (" Yes "));
               wattrset (win, A_REVERSE);
               wprintw (win, " Yes ");
               wattroff (win, A_REVERSE);
               wmove (win, 4, QUIT_BOX_WIDTH / 2 + 5 - strlen (" No "));
               wprintw (win, " No ");
               wmove (win, 4, QUIT_BOX_WIDTH / 2 - 5 - strlen (" Yes "));
            }
            else
            {
               wmove (win, 4, QUIT_BOX_WIDTH / 2 - 5 - strlen (" Yes "));
               wattroff (win, A_REVERSE);
               wprintw (win, " Yes ");
               wmove (win, 4, QUIT_BOX_WIDTH / 2 + 5 - strlen (" No "));
               wattrset (win, A_REVERSE);
               wprintw (win, " No ");
               wmove (win, 4, QUIT_BOX_WIDTH / 2 + 5 - strlen (" No "));
            }
            break;
         case 'n':
         case 'N':
         case CONTROL_G:
         case CONTROL_X:
         case ESCAPE_CHARACTER:
            value = 0;
            done = 1;
            break;
         case 'y':
         case 'Y':
            value = 1;
            done = 1;
            break;
      }
      wrefresh (win);
   }   

   if (value)
      exitProgram ();
   redraw ();
   signal (SIGINT, handleInterrupt);
}


#ifndef __PDCURSES__
void
handleSigwinch (int i)
{
   char *tty = NULL;
   int fd = 0;
   int result = 0;
   struct winsize win;

   tty = ttyname (0);
   if (!tty)
      return;
   fd = open (tty, O_RDWR);
   if (fd == -1)
   {
      perror (tty);
      return;
   }
   result = ioctl (fd, TIOCGWINSZ, &win);
   if (result == -1)
   {
      perror ("ioctl TIOCGWINSZ");
      return;
   }
   delwin (Globals.wmain);
   delwin (Globals.wstatus);
   delwin (Globals.whelp);
   endwin ();
   COLS = win.ws_col;
   LINES = win.ws_row;
   initscr ();
   if (stdscr == NULL)
   {
      fprintf (stderr, "Cannot initialize screen - curses!\n");
      exit (EXIT_FAILURE);
   }
   keypad (stdscr, TRUE);
   scrollok (stdscr, FALSE);
   cbreak ();
   noecho ();
   refresh ();
   Globals.wmain = newwin (MAIN_HEIGHT, COLS, MAIN_TOP_LINE, 0);
   if (Globals.wmain == NULL)
   {
      fprintf (stderr, "Cannot open larger main window - curses!\n");
      exit (EXIT_FAILURE);
   }
   scrollok (Globals.wmain, FALSE);
   #ifdef __PDCURSES__
   leaveok(Globals.wmain, FALSE);  /* Stupid macro */
   #else
      leaveok (Globals.wmain, FALSE);
   #endif
   Globals.wstatus = newwin (1, COLS, 0, 0);
   if (Globals.wstatus == NULL)
   {
      fprintf (stderr, "Cannot open status window - curses!\n");
      exit (EXIT_FAILURE);
   }
   scrollok (Globals.wstatus, FALSE);
   wattrset (Globals.wstatus, A_REVERSE);
   Globals.whelp = newwin (1, COLS, LINES - 1, 0);
   if (Globals.whelp == NULL)
   {
      fprintf (stderr, "Cannot open help window - curses!\n");
      exit (EXIT_FAILURE);
   }
   scrollok (Globals.whelp, FALSE);
   free (newlines);
   newlines = malloc (BOTTOM_LINE * sizeof (int));
   if (!newlines)
   {
      erase ();
      refresh ();
      endwin ();
      fprintf (stderr, "Not enough Memory\n");
      exit (EXIT_FAILURE);
   }
   Globals.wmain->_cury = cursor_y - 1;
   Globals.wmain->_curx = cursor_x;
   stdscr->_cury = cursor_y;
   stdscr->_curx = cursor_x;
   if (cursor_y >= BOTTOM_LINE)
      cursor_y = BOTTOM_LINE;
   if (Globals.mode == FILE_MODE)
   {
      extern struct FileNames *fp;
      extern struct FileNames **pages;
      extern int current_page;
      extern int num_pages;
      char trunc_file[PATH_MAX + 1];
      struct FileNames *p = *pages;
      struct FileNames *pfront = *pages;
      int i = 0;
      int j = 0;
      int n = 0;

      while (p->p)
      {
         p = p->p;
         n++;
      }
      free (pages);
      num_pages = ((n - 1) / (LINES - 2)) + 1;
      pages = malloc (sizeof (struct FileNames *) * num_pages);
      if (!pages)
      {
         erase (); refresh (); endwin ();
         fprintf (stderr, "Not enough memory!\n");
         exit (EXIT_FAILURE);
      }
      memset (pages, 0x00, num_pages * sizeof (struct FileNames *));
      *pages = pfront;
      p = pfront;
      for (i = 1; i < num_pages; i++)
      {
         for (j = 0; (j < (LINES - 2)) && p; j++)
            p = p->p;
         *(pages + i) = p;
      }

      i = 0;
      p = pfront;
      while (p != fp)
      {
         if (p == *(pages + i + 1))
            i++;
         p = p->p;
         
      }
      if (p == *(pages + i + 1))
         i++;
      current_page = i;
      cursor_y = MAIN_TOP_LINE;
      p = *(pages + current_page);
      while (p != fp)
      {
         p = p->p;
         cursor_y++;
      }
      refresh ();
      wrefresh (Globals.wmain);
      wrefresh (Globals.wstatus);
      wrefresh (Globals.whelp);
      printPage (*(pages + current_page));
      memset (trunc_file, 0x00, PATH_MAX + 1);
      strncpy (trunc_file, fp->filename, COLS - NAME_POS);
      wmove    (Globals.wmain, cursor_y - MAIN_TOP_LINE, NAME_POS);
      wattrset (Globals.wmain, A_BOLD);
      wprintw  (Globals.wmain, "%s", trunc_file);
      wattroff (Globals.wmain, A_BOLD);

      statWindow (fp->filename);
      helpWindow ("^C ^X  Exit Program   ^M  Select File");
      move (cursor_y, NAME_POS);
      wrefresh (Globals.wmain);
      wrefresh (Globals.wstatus);
      wrefresh (Globals.whelp);
   }
   else
   {
      redraw ();
      redraw ();
   }
   refresh ();
   signal (SIGWINCH, handleSigwinch);
}
#endif


void
exitProgram (void)
{
   int i = 0;
   struct Change *cp = NULL;

   move (LINES - 1, 0);
   wattroff (Globals.wmain, A_REVERSE);
   for (i = 0; i < COLS; i++)
      wprintw (stdscr, " ");
   wrefresh (stdscr);
   werase (Globals.whelp);
   wrefresh (Globals.whelp);
   delwin (Globals.wstatus);
   delwin (Globals.wmain);
   delwin (Globals.whelp);
   delwin (stdscr);
   free (filebuf);
   free (Globals.progname);
   free (newlines);
   while (UndoStack.base)
   {
      cp = UndoStack.base->p;
      free (UndoStack.base);
      UndoStack.base = cp;
   }
   endwin ();
   exit (EXIT_SUCCESS);
}


void
exitSave (int saved)
{
   if (saved)
      exitProgram ();
   redraw ();
}


char *
chompWhiteSpace (char *str)
{
   int i = 0;
   i = 0;
   while (*(str + i))
   {
      if ((*(str + i) == ' ') || (*(str + i) == '\r')
         || (*(str + i) == '\t') || (*(str + i) == '\n'))
      {
         *(str + i) = 0;
         break;
      }
      else
         i++;
   }
   return str;
}


int
isHexChar (wchar_t c)
{
   switch (c)
   {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
      case 'a':
      case 'b':
      case 'c':
      case 'd':
      case 'e':
      case 'f':
      case 'A':
      case 'B':
      case 'C':
      case 'D':
      case 'E':
      case 'F':
         return 1;
      default:
        return 0;
   }
   return 0;
}


int
getHexValue (wchar_t c)
{
   switch (c)
   {
      case '0':
         return 0x00;
      case '1':
         return 0x01;
      case '2':
         return 0x02;
      case '3':
         return 0x03;
      case '4':
         return 0x04;
      case '5':
         return 0x05;
      case '6':
         return 0x06;
      case '7':
         return 0x07;
      case '8':
         return 0x08;
      case '9':
         return 0x09;
      case 'a':
      case 'A':
         return 0x0A;
      case 'b':
      case 'B':
         return 0x0B;
      case 'c':
      case 'C':
         return 0x0C;
      case 'd':
      case 'D':
         return 0x0D;
      case 'e':
      case 'E':
         return 0x0E;
      case 'f':
      case 'F':
         return 0x0F;
   }
   return 0;
}

char
getAsciiValue (wchar_t c)
{
   switch (c)
   {
      case 0:
         return '0';
      case 1:
         return '1';
      case 2:
         return '2';
      case 3:
         return '3';
      case 4:
         return '4';
      case 5:
         return '5';
      case 6:
         return '6';
      case 7:
         return '7';
      case 8:
         return '8';
      case 9:
         return '9';
      case 10:
         return 'A';
      case 11:
         return 'B';
      case 12:
         return 'C';
      case 13:
         return 'D';
      case 14:
         return 'E';
      case 15:
         return 'F';
   }
   return '\0';
}


void
switchModes (void)
{
   if (Globals.mode == HEX_MODE)
      Globals.mode = ASCII_MODE;
   else
      Globals.mode = HEX_MODE;
   redraw ();
}


void
update_table (long off)
/* Add an entry to the bold table where changes in the file were made so
 * we know these show up as bold
 */
{
   if (bold_table.num >= bold_table.len)
   {
      bold_table.len += EXTRA_BUF;
      bold_table.table = realloc (bold_table.table,
                                  bold_table.len * sizeof (long));
      if (!bold_table.table)
      {
         erase ();
         refresh ();
         endwin ();
         fprintf (stderr, "Not enough Memory!\n");
         exit (EXIT_FAILURE);
      }
      memset (bold_table.table + bold_table.len - EXTRA_BUF,
              0, EXTRA_BUF * sizeof (long));
   }
   if (!table_lookup (off))
   {
      *(bold_table.table + bold_table.num) = off;
      bold_table.num++;
      qsort (bold_table.table, bold_table.num, sizeof (long),
             table_compare);
   }
}


void *
table_lookup (long off)
{
   if (bold_table.num > 0)
      return bsearch (&off, bold_table.table, bold_table.num, sizeof (long),
                      table_compare);
   return NULL;
}


void
table_remove (long off)
{
   int i = 0;
   for (i = 0; i < bold_table.num; i++)
   {
      if (*(bold_table.table + i) == off)
      {
         if (i < (bold_table.num - 1))
            memmove (bold_table.table + i, bold_table.table + i + 1,
                     bold_table.num - i - 1);
         bold_table.num--;
      }
   }
}


int
table_compare (const void *n1, const void *n2)
{
   long *l1 = (void *) n1;
   long *l2 = (void *) n2;
   if (*l1 > *l2)
      return 1;
   if (*l1 < *l2)
      return -1;
   return 0;
}


void
table_clear ()
{
   int i = 0;
   for (i = 0; i < bold_table.num; i++)
      *(bold_table.table + i) = -1;
}
