/* print.c by Adam Rogoyski (apoc@laker.net) Temperanc on EFNet irc
 * Copyright (C) 1998, 1999 Adam Rogoyski
 * --- GNU General Public License Disclamer ---
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */


#include "hexedit.h"

long findNewline (long);
void drawAsciiDump (long, long *);

const unsigned char EBCDIC[] = {
'.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.',
'.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.',
'.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.',
'.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.',
' ', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '<', '(', '+', '|',
'&', '.', '.', '.', '.', '.', '.', '.', '.', '.', '!', '$', '*', ')', ';', '^',
'-', '/', '.', '.', '.', '.', '.', '.', '.', '.', '|', ',', '%', '_', '>', '?',
'.', '.', '.', '.', '.', '.', '.', '.', '.', '`', ':', '#', '@', '\'', '=','"',
'.', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', '.', '{', '.', '.', '.', '.',
'.', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', '.', '}', '.', '.', '.', '.',
'.', '~', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '.', '.', '.', '[', '.', '.',
'.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', '.', ']', '.', '.',
'{', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', '.', '.', '.', '.', '.', '.',
'}', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', '.', '.', '.', '.', '.', '.',
'\\', '.', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '.', '.', '.', '.', '.','.',
'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.', '.', '.', '.', '.', '.',
};



void
drawdump (long off)
{
   int pos = 0;
   int space = 0;
   int cur_y = 0;
   long *bold = NULL;
   long upto = (off & M_0xFFFFFFF0) + ((MAIN_HEIGHT) * M_0x10);

   werase (Globals.wmain);
   off &= M_0xFFFFFFF0;
   while ((off < upto) && (off < Globals.filesize + 1))
   {
      if (pos == 0)
         wprintw (Globals.wmain, "%08X ", off);
      if (pos == 4 || pos == 12)
         space++;
      else if (pos == 8)
      {
         if (Globals.spacing)
            space = 3;
         else
            space = 2;
      }
      bold = table_lookup (off);
      if (bold)
         wattrset (Globals.wmain, A_BOLD);
      if (Globals.spacing)
      {
         wmove (Globals.wmain, cur_y, (pos * 3) + 10 + space);
         wprintw (Globals.wmain, "%02lX ", *(filebuf + off));
      }
      else
      {
         wmove (Globals.wmain, cur_y, (pos * 2) + 10 + space);
         wprintw (Globals.wmain, "%02lX", *(filebuf + off));
      }      
      wmove (Globals.wmain, cur_y, COLS - M_0x10 + pos);
      if (Globals.charset == ASCII_CHAR_SET)
      {
         if ((*(filebuf + off) > 32) && (*(filebuf + off) < 127))
            wprintw (Globals.wmain, "%c", *(filebuf + off));
         else
            wprintw (Globals.wmain, ".");
      }
      else /* EBCDIC */
      {
         wprintw (Globals.wmain, "%c", EBCDIC[*(filebuf + off)]);
      }
      if (bold)
         wattroff (Globals.wmain, A_BOLD);
      if (pos == 15)
      {
         if (cur_y == BOTTOM_LINE)
            break;
         cur_y++;
         pos = 0;
         space = 0;
      }
      else
         pos++;
      off++;
   }
   position = pos;
}


long
findNewline (long off)
/* finds the first newline before the current offset.  This is used for
 * drawing the control-e ascii/text screen
 */
{
   while (off > 0x00)
   {
      if (*(filebuf + off) == '\n')
      {
         off++;
         break;
      }
      off--;
   }
   return off;
}


void
drawAsciiDump (long off, long *offs)
{
   int move = 0;
   int i = 0;
   long *bold = NULL;

   while (1)
   {
      if (*offs > Globals.filesize)
         break;
      #ifdef __NCURSES_H     /* i don't know why this works */
      if (Globals.wmain->_cury == MAIN_BOTTOM_LINE)
      #else
      if (Globals.wmain->_cury == MAIN_HEIGHT)
      #endif
         break;
      if (Globals.wmain->_curx == COLS - 1)
      {
         move = 1;
      }
      if (Globals.wmain->_curx == 0)
         *(newlines + i++) = *offs;
      bold = table_lookup (*offs);
      if (bold)
         wattrset (Globals.wmain, A_BOLD);
      if (Globals.charset == ASCII_CHAR_SET)
      {
         if ((*(filebuf + *offs)) == '\n')
         {
            int cury = Globals.wmain->_cury;
            wprintw (Globals.wmain, ".");
            wmove (Globals.wmain, cury + 1, 0);
         }
         else if ((*(filebuf + *offs)) == ' ')
         {
            wprintw (Globals.wmain, " ");
         } 
         else if ((*(filebuf + *offs) > 32) && (*(filebuf + *offs) < 127))
            wprintw (Globals.wmain, "%c", *(filebuf + *offs));
         else
            wprintw (Globals.wmain, ".");
      }
      else /* EBCDIC */
      {
         if ((*(filebuf + *offs)) == EBCDIC['\n'])
         {
            int cury = Globals.wmain->_cury;
            wprintw (Globals.wmain, ".");
            wmove (Globals.wmain, cury + 1, 0);
         }
         else if ((*(filebuf + *offs)) == EBCDIC[' '])
         {
            wprintw (Globals.wmain, " ");
         } 
         else
            wprintw (Globals.wmain, "%c", EBCDIC[*(filebuf + *offs)]);
      }
     
      if (bold)
         wattroff (Globals.wmain, A_BOLD);
      (*offs)++;
      if (move)
      {
         wmove (Globals.wmain, Globals.wmain->_cury + 1, 0);
         move = 0;
         continue;
      }
      if (*offs > Globals.filesize)
         break;
   }
   *(newlines + i) = *offs + 1;
}

void
drawAscii (long off)
{
   int i = 0;
   long offs = 0;
   long os = 0;
   wmove (Globals.wmain, 0, 0);
   os = offs = findNewline (off);
   memset (newlines, 0, BOTTOM_LINE * sizeof (int));
   drawAsciiDump (off, &offs);
   while (offset >= offs)
   {
      memset (newlines, 0, BOTTOM_LINE * sizeof (int));
      werase (Globals.wmain);
      wmove (Globals.wmain, 0, 0);
      drawAsciiDump (offs, &offs);
      if (offset >= Globals.filesize)
         break;
   }
      acursor_y = 1;
      acursor_x = 0;
      for (i = 0; i < BOTTOM_LINE; i++)
      {
         if (offset < *(newlines + i))
         {
            acursor_y = i;
            if (i > 0)
               acursor_x = offset - *(newlines + i - 1);
            else
               acursor_x = offset - os;
            if (acursor_x >= COLS)
            {
               acursor_y += (int) (acursor_x / COLS);
               acursor_x %= COLS;
            }
            break;
         }
      }
}


void
printStatusWindow (void)
{
   static char *buf = NULL;
   static int first = 0;
   static int length = 0;
   double percent = 0.0;
   if (!first)
   {
      length = COLS - 54; /* 54 is rest of status bar needed for other info */
      buf = malloc (PATH_MAX + 1);
      if (!buf)
      {
         fprintf (stderr, "Not enough Memory\n");
         exit (EXIT_FAILURE);
      }
      memset (buf, 0, PATH_MAX + 1);
      first = 1;
   }
   memset (buf, ' ', PATH_MAX);
   length = COLS - 54;
   if (strlen (Globals.progname) < length)
      strncpy (buf, Globals.progname, strlen (Globals.progname));
   else
      strncpy (buf, Globals.progname, length - 1);
   *(buf + length - 1) = '\0';
   if (Globals.filesize <= 0)
      percent = 0.0;
   else
      percent = ((float) offset / (float) Globals.filesize) * 100;
   wmove (Globals.wstatus, 0, 0);
   wprintw (Globals.wstatus,
           "File: %-s %7s Offset: 0x%08lX / 0x%08lX (%%%02.0f) ",
                  buf, 
                  Globals.charset == ASCII_CHAR_SET ? "ASCII" : "EBCDIC",
                  offset, Globals.filesize, percent);
      wmove (Globals.wstatus, 0, COLS - 1);
   if (Globals.modified == MODIFIED)
      wprintw (Globals.wstatus, "M");
   else if (Globals.modified == READ_ONLY)
      wprintw (Globals.wstatus, "R");
   else
      wprintw (Globals.wstatus, " ");
   wrefresh (Globals.wstatus);
}


void
printHelpWindow (void)
{
   int len = COLS;

      /* Only what can fit on the screen will be displayed */   
   wmove (Globals.whelp, 0, 0);
   if (len > strlen ("^G Help"))
   {
      wprintw (Globals.whelp, "^G");
      wprintw (Globals.whelp, " Help");
      len -= strlen ("^G Help");
   }
   if (len > strlen ("   ^C Exit (No Save)"))
   {
      wprintw (Globals.whelp, "   ^C");
      wprintw (Globals.whelp, " Exit (No Save)");
      len -= strlen ("   ^C Exit (No Save)");
   }
   if (len > strlen ("   ^T goTo Offset"))
   {
      wprintw (Globals.whelp, "   ^T");
      wprintw (Globals.whelp, " goTo Offset");
      len -= strlen ("   ^T goTo Offset");
   }
   if (len > strlen ("   ^X Exit and Save"))
   {
      wprintw (Globals.whelp, "   ^X");
      wprintw (Globals.whelp, " Exit and Save");
      len -= strlen ("   ^X Exit and Save");
   }
   if (len > strlen ("   ^W Search"))
   {
      wprintw (Globals.whelp, "   ^W");
      wprintw (Globals.whelp, " Search");
      len -= strlen ("   ^W Search");
   }
   if (len > strlen ("   ^U Undo"))
   {
      wprintw (Globals.whelp, "   ^U");
      wprintw (Globals.whelp, " Undo");
      len -= strlen ("   ^U Undo");
   }
   if (len > strlen ("   ^L Redraw"))
   {
      wprintw (Globals.whelp, "   ^L");
      wprintw (Globals.whelp, " Redraw");
      len -= strlen ("   ^L Redraw");
   }
   if (len > strlen ("   ^E Ascii Mode"))
   {
      wprintw (Globals.whelp, "   ^E");
      wprintw (Globals.whelp, " Ascii Mode");
      len -= strlen ("   ^E Ascii Mode");
   }
   if (len > strlen ("   ^R CharSet"))
   {
      wprintw (Globals.whelp, "   ^R");
      wprintw (Globals.whelp, " CharSet");
      len -= strlen ("   ^R CharSet");
   }
   if (len > strlen ("   ^P Spacing"))
   {
      wprintw (Globals.whelp, "   ^P");
      wprintw (Globals.whelp, " Spacing");
      len -= strlen ("   ^P Spacing");
   }
   wrefresh (Globals.whelp);
}


void
redraw ()
{
   int i = 0;
   long off = offset;

   off -= M_0x10 * (cursor_y - MAIN_TOP_LINE);
   if (off < 0x00)
      off = 0;
   werase (Globals.wstatus);
   werase (Globals.wmain);
   werase (Globals.whelp);
   wrefresh (Globals.wstatus);
   wrefresh (Globals.wmain);
   wrefresh (Globals.whelp);
   move (0, 0);
   wmove (Globals.wstatus, 0, 0);
   for (i = 0; i < COLS; i++)
      wprintw (Globals.wstatus, " ");

   if (Globals.mode == CALCULATOR_MODE)
   {
      extern struct calcEntryBox *bbox;
      extern struct calcEntryBox *obox;
      extern struct calcEntryBox *dbox;
      extern struct calcEntryBox *hbox;
      extern int calc_mode;

      wmove (Globals.wmain, 1, (COLS / 2) 
             - (strlen ("Binary Calculator") / 2));
      wprintw (Globals.wmain, "Binary Calculator");

      calcStatWindow ();
      wmove (Globals.whelp, 0, 0);
      wprintw (Globals.whelp, "^X/^G  Return   ^C  Exit");
      calcDrawBoxes (bbox, obox, dbox, hbox, calc_mode);
      wrefresh (Globals.wstatus);
      wrefresh (Globals.whelp);
      wrefresh (Globals.wmain);
      refresh ();
   }
   else
   {
      printStatusWindow ();
      printHelpWindow ();
      wmove (Globals.wmain, 0, 0);
      if (Globals.mode == HEX_MODE)
      {
         drawdump (off);
         wrefresh (Globals.wmain);
        if (Globals.tabb)
            move (cursor_y, COLS - M_0x10 + mapcur (cursor_x));
         else
            move (cursor_y, cursor_x);
      }
      else
      {
         drawAscii (off);
         wrefresh (Globals.wmain);
         move (acursor_y, acursor_x);
      }
   }
   refresh ();
}
