/* $Id: kget.cpp,v 1.4 1998/10/10 12:30:38 matt Exp $
   $Log: kget.cpp,v $
   Revision 1.4  1998/10/10 12:30:38  matt
   changed default "killall pppd" to "kppp -k"
   deleted testDir()
   status messages - number of transfers

   Revision 1.15  1998/08/23 13:25:16  matt
   Added prompt toggle button - as a solution for a request from Graham TerMarsch

   Revision 1.14  1998/05/06 21:04:36  matt
   Now you cannot stop http transfers - only delete

   Revision 1.13  1998/05/06 19:27:55  matt
   Updated to work with Beta4

   Revision 1.12  1998/04/22 20:26:44  matt
   Added paste button - suggestion from Joe Orton

   Revision 1.11  1998/03/19 16:09:33  matt
   Added transfer speed in status bar

   Revision 1.10  1998/03/14 08:50:48  matt
   Added RMB menu

   Revision 1.9  1998/03/13 11:51:22  matt
   Added pasteTransfer() : now you can use Ctrl-v to paste in transfer

   Revision 1.8  1998/03/13 11:31:38  matt
   Removed bug : deleting stopped transfer caused segfault

   Revision 1.7  1998/03/02 14:30:37  matt
   Removed bugs with deleteCurrent and slotFinished

   Revision 1.6  1998/02/18 16:45:14  matt
   Added settings processing

   Revision 1.5  1998/02/16 11:37:09  matt
   Now computing size in kB and MB

   Revision 1.4  1998/02/15 22:05:27  matt
   Added queueing

   Revision 1.3  1998/02/15 21:09:33  matt
   Showing progress in TabListBox. Deleting transfers after finishing KIOJob.

   Revision 1.2  1998/02/14 15:38:05  matt
   Added KIO - KIOJob, KIOServer, KIOSlave
   copying now works, at least for local files

   Revision 1.1  1998/01/29 18:21:05  matt
   Renamed kdownload.cpp to kget.cpp

   Revision 1.1  1998/01/28 16:51:53  matt
   Renamed kftp.cpp to kdownload.cpp

   Revision 1.1  1998/01/28 14:02:38  matt
   Renamed kftp.cc to kftp.cpp

   Revision 1.4  1998/01/24 13:14:00  matt
   Updated code - removed useless parts

   Revision 1.3  1998/01/24 12:43:10  matt
   Updated status icons - added trying icon

   Revision 1.2  1998/01/24 12:36:01  matt
   Added key accelerators

   Revision 1.1.1.1  1998/01/24 09:11:31  matt
   Imported sources


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <dirent.h>
#include <sys/stat.h>

#include <iostream.h>

extern "C" {
#include <mediatool.h>
}

#include <qdir.h>
#include <qpainter.h>
#include <qclipbrd.h>

#include <kapp.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kurl.h>
#include <kstdaccel.h>
#include <kmsgbox.h>
#include <kwm.h>
#include <kstring.h>
#include <kaudio.h>

#include "kget.h"
#include "version.h"
#include "about.h"
#include "utils.h"
#include "ksavedialog.h"
#include "kquestiondialog.h"
#include "kgetoptdlg.h"
#include "docking.h"
#include "kioserver.h"

KMainWidget*	p_kmain;
DockWidget*     dock_widget;
KStdAccel*      keys;

QString currentFilename;
QString currentDirectory;


int main( int argc, char **argv ) {
  KConfig* cfg;

  KApplication a( argc, argv, "kget" );

  cfg = KApplication::getKApplication()->getConfig();
  keys = new KStdAccel(cfg);

  KIOJob::InitStatic();
  KMyTabListBox::InitStatic();
  DockWidget::InitStatic();

  warning("1. Init KIOManager\n");
  
  new KIOServer();

  KMainWidget kmain;
  p_kmain = &kmain;
  
  warning("2. Init window\n");

  a.setMainWidget(&kmain);

  return a.exec();
}


KMainWidget::KMainWidget( const char *name )
  : KTopLevelWidget( name )
{
  pppstats = NULL;
  audio = NULL;

  promptMode = true;
  disconnectMode = false;

  readSettings();

  if ( properties != "" )
    setGeometry(KWM::setProperties(winId(), properties));
  else
    resize( 420, 200 );

  setCaption("KGet "KGETVERSION);
  setMinimumSize( 350, 150 );

  setupMenuBar();
  setupToolBar();
  setupStatusBar();
  setupTabListBox();
  setupPPPStats();

  GlobalItemList.setAutoDelete( TRUE );
  deleteMode = FALSE;

  // Timer used for animation
  animTimer = new QTimer( this );
  animCounter = 0;
  connect( animTimer, SIGNAL( timeout() ),
	   this, SLOT( slotAnimTimeout() ) );

  if ( animateIcons )
    animTimer->start( 200 );

  currentDirectory = QDir::currentDirPath();

  dock_widget = new DockWidget("dockw");

  readTransfers();

  if ( dockWindow )
    dock_widget->dock();
  else
    this->show();
}


KMainWidget::~KMainWidget () {
  if ( animTimer )
    {
      animTimer->stop();
      delete animTimer;
    }
  
  delete myMenuBar;
  delete myStatusBar;
  delete myToolBar;
  delete myTabListBox;
  delete pppstats;

  if ( audio )
    delete audio;
}


void KMainWidget::setupMenuBar(){
  file = new QPopupMenu ();
  options = new QPopupMenu ();
  help = new QPopupMenu ();
  
  help->insertItem (i18n("&About..."),
		    this, 	SLOT(about()));
  help->insertItem (i18n("&Help"),
		    this, 	SLOT(helpSelected()));
  
  options->setCheckable(true);
  options->insertItem( i18n("Show &Statusbar"),
		       this, SLOT(slotShowStatusbar()) );
  options->insertItem( i18n("Show &Toolbar"),
		       this, SLOT(slotShowToolbar()) );
  options->insertSeparator();
  options->insertItem( i18n("&Animate Status"),
		       this, SLOT(slotAnimateStatus()) );
  options->insertSeparator();
  options->insertItem( i18n("&Dock Window"),
		       this, SLOT(slotDockWindow()) );
  options->insertSeparator();
  options->insertItem( i18n("S&ettings..."),
		       this, SLOT(slotSettings()));
  options->setItemChecked( options->idAt( ID_STATUSBAR ), showStatusbar );
  options->setItemChecked( options->idAt( ID_TOOLBAR ), showToolbar );
  options->setItemChecked( options->idAt( ID_ANIMATEICONS ), animateIcons );
  options->setItemChecked( options->idAt( ID_DOCKWINDOW ), dockWindow );


  file->insertItem(i18n("&Open Transfer"),
		   this, 	SLOT(openTransfer()), keys->open());
  file->insertItem(i18n("&Paste Transfer"),
		   this, 	SLOT(pasteTransfer()), keys->paste());
  file->insertSeparator(-1);
  file->insertItem (i18n("&Quit"),
		    this,	SLOT(quit()), keys->quit());


  myMenuBar = new KMenuBar (this, "myMenuBar");
  myMenuBar->insertItem (i18n("&File"), file);
  myMenuBar->insertItem (i18n("&Options"), options);
  myMenuBar->insertSeparator(-1);
  myMenuBar->insertItem (i18n("&Help"), help);

  setMenu(myMenuBar);
  
}



void KMainWidget::setupToolBar(){
  myToolBar = new KToolBar( this );

  myToolBar->insertButton(Icon("filenew.xpm"), ITEM_OPEN,
			  SIGNAL(clicked()), this,
			  SLOT(openTransfer()), TRUE, i18n("Open transfer"));

  myToolBar->insertButton(Icon("reload.xpm"), ITEM_RESUME,
			  SIGNAL(clicked()), this,
			  SLOT(resumeCurrent()), FALSE, i18n("Resume transfer"));

  myToolBar->insertButton(Icon("stop.xpm"), ITEM_STOP,
			  SIGNAL(clicked()), this,
			  SLOT(stopCurrent()), FALSE, i18n("Stop transfer"));

  myToolBar->insertButton(Icon("exit.xpm"), ITEM_DELETE,
			  SIGNAL(clicked()), this,
			  SLOT(deleteCurrent()), FALSE, i18n("Delete transfer"));

  myToolBar->insertButton(Icon("bottom.xpm"), ITEM_DOCK,
			  SIGNAL(clicked()), this,
			  SLOT(dockHide()), FALSE, i18n("Dock window"));

  myToolBar->setItemEnabled( ITEM_DOCK, dockWindow );

  myToolBar->insertButton(Icon("editpaste.xpm"), ITEM_PASTE,
			  SIGNAL(clicked()), this,
			  SLOT(pasteTransfer()), TRUE, i18n("Paste transfer"));

  myToolBar->insertSeparator();

  myToolBar->insertButton(Icon("help.xpm"), ITEM_PROMPT,
			  SIGNAL(clicked()), this,
			  SLOT(togglePrompt()), TRUE, i18n("Prompt mode"));
  myToolBar->setToggle(ITEM_PROMPT, true);
  ((KToolBarButton*)myToolBar->getButton(ITEM_PROMPT))->on(true);

  myToolBar->insertButton(Icon("ppp-off.xpm"), ITEM_DISCONNECT,
			  SIGNAL(clicked()), this,
			  SLOT(toggleDisconnect()), TRUE, i18n("Disconnect mode"));
  myToolBar->setToggle(ITEM_DISCONNECT, true);
  ((KToolBarButton*)myToolBar->getButton(ITEM_DISCONNECT))->on(false);

  myToolBar->setBarPos( toolbarPos );
  addToolBar(myToolBar);

  if ( !showToolbar )
    myToolBar->enable( KToolBar::Hide );
  else
    myToolBar->enable( KToolBar::Show );
}



void KMainWidget::setupPPPStats(){
  if ( pppstats ){
    delete pppstats;
    pppstats = 0;
    myStatusBar->changeItem("     ", ID_TRANSFER_SPEED);
  }

  if ( showSpeed ){
    pppstats = new PPPStats();
    
    connect( pppstats, SIGNAL( changeStats( uint ) ), this, SLOT( updateStats( uint ) ) );
  }
}



void KMainWidget::updateStats( uint rxDelta )
{
  QString s;
  float fsize;

  if ( statsSpeed )
    rxDelta /= statsSpeed;

  if ( rxDelta > 1048576 ){
    fsize = (float) rxDelta / (float) 1048576;
    s.sprintf ( "%.2f MB/sec", fsize);
  } else if ( rxDelta > 1024 ){
    fsize = (float) rxDelta / (float) 1024;
    s.sprintf ( "%.2f kB/sec", fsize);
  } else {
    s.sprintf ( "%d B/sec", rxDelta);
  }

  myStatusBar->changeItem(s.data(), ID_TRANSFER_SPEED);
  
}


void KMainWidget::setupStatusBar(){
  myStatusBar = new KStatusBar( this );
  myStatusBar->insertItem("00000 B/sec", ID_TRANSFER_SPEED);
  myStatusBar->changeItem("     ", ID_TRANSFER_SPEED);
  myStatusBar->insertItem(i18n("Welcome to KGet"), ID_URL_TEXT);
 
  myStatusBar->setInsertOrder(KStatusBar::RightToLeft);

  setStatusBar( myStatusBar );

  if ( !showStatusbar )
    myStatusBar->enable( KStatusBar::Hide );
  else
    myStatusBar->enable( KStatusBar::Show );
}


void KMainWidget::setupTabListBox() {
  myTabListBox = new KMyTabListBox(this, "transferList", 4);

  myTabListBox->setFont( tablistboxfont );

  setView( myTabListBox, FALSE);

  KDNDDropZone *myDropZone = new KDNDDropZone(this, DndURL);
  connect( myDropZone, SIGNAL( dropAction( KDNDDropZone *) ), 
	   this, SLOT( slotDropEvent( KDNDDropZone *) ) );

  connect( myTabListBox, SIGNAL( highlighted( int, int )), 
	   this, SLOT( slotHighlightedEvent( int )));
  connect( myTabListBox, SIGNAL( midClick( int, int )), 
	   this, SLOT( slotMidClickEvent( int )));
  connect( myTabListBox, SIGNAL( selected( int, int )), 
	   this, SLOT( slotSelectedEvent( int )));
  connect( myTabListBox, SIGNAL( popupMenu( int, int )), 
	   this, SLOT( slotPopupMenu( int )));
}



void KMainWidget::slotSettings() {
  KGETOptDlg dlg;
  QStrList strlist( true );
  QStrList prxStrList ( true );
  QStrList sndStrList ( true );
  QStrList dirStrList ( true );
  
  // read entries from config file
  KConfig* config = KApplication::getKApplication()->getConfig();
  QString oldgroup = config->group();

  // transmit data to FontDlg
  dlg.setFontData( tablistboxfont );

  // now read data for TransferDlg
  config->setGroup("Transfer");
  strlist.append(config->readEntry("UseQueueing", "No"));
  strlist.append(config->readEntry("MaxSimConnections", "3"));
  strlist.append(config->readEntry("UseReconnecting", "No"));
  strlist.append(config->readEntry("ReconnectTime", "15"));
  strlist.append(config->readEntry("ShowSpeed", "Yes"));
  strlist.append(config->readEntry("StatsSpeed", "5"));
  strlist.append(config->readEntry("PPPLink", "0"));
  strlist.append(config->readEntry("DisconnectCommand", "kppp -k"));
  
  // transmit data to TransferDlg
  dlg.setTransferData(&strlist);

  // now read data for KProxyDlg
  config->setGroup("Proxy");
  prxStrList.append(config->readEntry("UseProxy", "No"));
  prxStrList.append(config->readEntry("HTTP-Proxy", ""));
  prxStrList.append(config->readEntry("FTP-Proxy", ""));
  prxStrList.append(config->readEntry("NoProxyFor", ""));

  // transmit data to KProxyDlg
  dlg.setProxyData(&prxStrList);
  
  // now read data for SoundDlg
  config->setGroup("Sound");
  sndStrList.append(config->readEntry("UseSound", "Yes"));
  sndStrList.append(config->readEntry("Finished", audioFinished.data()));
  sndStrList.append(config->readEntry("FinishedAll", audioFinishedAll.data()));

  // transmit data to SoundDlg
  dlg.setSoundData(&sndStrList);

  // now read data for DirDlg
  config->setGroup("Directories");
  dirStrList.append(config->readEntry("Dir1", bDir1.data()));
  dirStrList.append(config->readEntry("Dir2", bDir2.data()));
  dirStrList.append(config->readEntry("Dir3", bDir3.data()));
  dirStrList.append(config->readEntry("Dir4", bDir4.data()));

  // transmit data to DirDlg
  dlg.setDirData(&dirStrList);

  // show the dialog
  int ret = dlg.exec();
  if( ret == QDialog::Accepted )
    {
      // write back the entries from FontDlg
      config->setGroup("Font");
      config->writeEntry( "Font", dlg.dataFont() );
      tablistboxfont = dlg.dataFont();
      myTabListBox->setFont( tablistboxfont );

      // write back the entries from KProxyDlg
      config->setGroup("Proxy");
      strlist = dlg.dataProxy();
      config->writeEntry( "UseProxy", strlist.first() );
      config->writeEntry( "HTTP-Proxy", strlist.next() );
      config->writeEntry( "FTP-Proxy", strlist.next() );
      config->writeEntry( "NoProxyFor", strlist.next() );

      // write back the entries from TransferDlg
      config->setGroup("Transfer");
      strlist = dlg.dataTransfer();
      QString s;
      s = strlist.first();
      config->writeEntry( "UseQueueing", s.data() );
      if ( s == "Yes" )
	queueingEnabled = TRUE;
      else
	queueingEnabled = FALSE;

      s = strlist.next();
      config->writeEntry( "MaxSimConnections", s.data() );
      maxSimultaneousConnections = s.toUInt();
      checkQueue();

      s = strlist.next();
      config->writeEntry( "UseReconnecting", s.data() );
      if ( s == "Yes" )
	reconnectingEnabled = TRUE;
      else
	reconnectingEnabled = FALSE;

      s = strlist.next();
      config->writeEntry( "ReconnectTime", s.data() );
      reconnectingTime = s.toUInt();

      s = strlist.next();
      config->writeEntry( "ShowSpeed", s.data() );
      if ( s == "Yes" )
	showSpeed = TRUE;
      else
	showSpeed = FALSE;

      setupPPPStats();

      s = strlist.next();
      config->writeEntry( "StatsSpeed", s.data() );
      statsSpeed = s.toUInt();

      s = strlist.next();
      config->writeEntry( "PPPLink", s.data() );
      pppLink = s.toUInt();

      disconnectCommand = strlist.next();
      config->writeEntry( "DisconnectCommand", disconnectCommand.data() );

      // write back the entries from DirDlg
      config->setGroup("Directories");
      strlist = dlg.dataDir();

      s = strlist.first();
      config->writeEntry( "Dir1", s.data() );
      bDir1 = s.data();

      s = strlist.next();
      config->writeEntry( "Dir2", s.data() );
      bDir2 = s.data();

      s = strlist.next();
      config->writeEntry( "Dir3", s.data() );
      bDir3 = s.data();

      s = strlist.next();
      config->writeEntry( "Dir4", s.data() );
      bDir4 = s.data();

      // write back the entries from SoundDlg
      config->setGroup("Sound");
      strlist = dlg.dataSound();

      s = strlist.first();
      config->writeEntry( "UseSound", s.data() );
      if ( s == "Yes" ){
	useSound = TRUE;
	if ( ! audio )
	  audio = new KAudio;
	if ( audio->serverStatus() ){
	  warning("Failed contacting audio server !");
	  useSound = FALSE;
	}
      }
      else {
	useSound = FALSE;
	if ( audio )
	  delete audio;
      }
      s = strlist.next();
      config->writeEntry( "Finished", s.data() );
      audioFinished = s;

      s = strlist.next();
      config->writeEntry( "FinishedAll", s.data() );
      audioFinishedAll = s;


    }

  // restore the group
  config->setGroup( oldgroup );
}


void KMainWidget::readSettings(){
  /* read the settings. We have some defaults, if the 
     settings are not defined or if there are unknown
     words in it. */
  
  KConfig *config = KApplication::getKApplication()->getConfig();
  QString entry;

  // read settings for directory buttons
  config->setGroup("Directories");
  bDir1 = config->readEntry("Dir1", "");
  bDir2 = config->readEntry("Dir2", "");
  bDir3 = config->readEntry("Dir3", "");
  bDir4 = config->readEntry("Dir4", "");

  // read sound settings
  config->setGroup("Sound");
  entry = config->readEntry("UseSound", "Yes");
  if ( entry == "Yes" ){
    useSound = TRUE;
    if ( ! audio )
      audio = new KAudio;
    if ( audio->serverStatus() ){
      warning("Failed contacting audio server !");
      useSound = FALSE;
    }
  }
  else {
    useSound = FALSE;
    if ( audio )
      delete audio;
  }

  QString path = KApplication::getKApplication()->kde_datadir() + "/kget/sounds/finished.wav";
  audioFinished = config->readEntry("Finished", path.data());

  path = KApplication::getKApplication()->kde_datadir() + "/kget/sounds/finishedall.wav";
  audioFinishedAll = config->readEntry("FinishedAll", path.data());

  // read transfer settings
  config->setGroup("Transfer");
  entry = config->readEntry("UseQueueing", "No");
  if ( entry == "Yes" )
    queueingEnabled = TRUE;
  else
    queueingEnabled = FALSE;

  maxSimultaneousConnections = config->readNumEntry("MaxSimConnections", 3);

  entry = config->readEntry("UseReconnecting", "No");
  if ( entry == "Yes" )
    reconnectingEnabled = TRUE;
  else
    reconnectingEnabled = FALSE;

  reconnectingTime = config->readNumEntry("ReconnectTime", 15);

  entry = config->readEntry("ShowSpeed", "Yes");
  if ( entry == "Yes" )
    showSpeed = TRUE;
  else
    showSpeed = FALSE;

  statsSpeed = config->readNumEntry("StatsSpeed", 5);
  pppLink = config->readNumEntry("PPPLink", 0);
  disconnectCommand = config->readEntry("DisconnectCommand", "kppp -k");

  // read misc settings
  config->setGroup( "Misc" );
  entry = config->readEntry("Toolbar", "top");
  showToolbar = true;
  if ( entry == "top" )
    toolbarPos = KToolBar::Top;
  else if ( entry == "left" )
    toolbarPos = KToolBar::Left;
  else if ( entry == "right" )
    toolbarPos = KToolBar::Right;    
  else if ( entry == "bottom" )
    toolbarPos = KToolBar::Bottom;    
  else if ( entry == "floating" )
    toolbarPos = KToolBar::Floating;    
  else
    showToolbar = false;

  showStatusbar = config->readBoolEntry("Statusbar", TRUE);

  animateIcons = config->readBoolEntry("AnimateIcons", TRUE);

  dockWindow = config->readBoolEntry("DockWindow", FALSE);

  useSound = config->readBoolEntry("UseSound", TRUE);

  // read font settings
  config->setGroup( "Font" );
  QFont font("helvetica",12);
  tablistboxfont = config->readFontEntry("Font", &font );

  // read geometry settings
  config->setGroup("Geometry");
  properties = config->readEntry("Properties", "" );

}



void KMainWidget::writeSettings(){
  KConfig *config = KApplication::getKApplication()->getConfig();
  QString entry;

  config->setGroup( "Misc" );
  if ( !showToolbar )
    config->writeEntry( "Toolbar", "hide" );
  else if ( myToolBar->barPos() == KToolBar::Top )
    config->writeEntry( "Toolbar", "top" );
  else if ( myToolBar->barPos() == KToolBar::Bottom )
    config->writeEntry( "Toolbar", "bottom" );
  else if ( myToolBar->barPos() == KToolBar::Left )
    config->writeEntry( "Toolbar", "left" );
  else if ( myToolBar->barPos() == KToolBar::Right )
    config->writeEntry( "Toolbar", "right" );
  else if ( myToolBar->barPos() == KToolBar::Floating )
    config->writeEntry( "Toolbar", "floating" );

  config->writeEntry( "Statusbar", showStatusbar );

  config->writeEntry( "AnimateIcons", animateIcons );

  config->writeEntry( "DockWindow", dockWindow );

  config->setGroup( "Font" );
  config->writeEntry("Font",      tablistboxfont );

  config->setGroup( "Geometry" );
  properties = KWM::getProperties(winId());
  config->writeEntry("Properties", properties );

  config->sync();
}



void KMainWidget::about(){
  QDialog *dlg = new About(0);

  QPoint point = this->mapToGlobal (QPoint (0,0));

  QRect pos = this->geometry();
  dlg->setGeometry(point.x() + pos.width()/2  - dlg->width()/2,
		   point.y() + pos.height()/2 - dlg->height()/2, 
		   dlg->width(),dlg->height());
  dlg->exec();
  delete dlg;
}


void KMainWidget::helpSelected(){
  KApplication::getKApplication()->invokeHTMLHelp( "" , "" );
}


int KMainWidget::saveAs( QString filename, bool resume ){
    
  KSaveDialog *box;

  QFileInfo info;

 try_again:

  if ( promptMode ) {
    box = new KSaveDialog(this, currentDirectory.data(), filename.data(),
		  bDir1.data(), bDir2.data(), bDir3.data(), bDir4.data());
    box->show();
    
    if (!box->result())
      return KSTATUS_USER_CANCEL;
    
    currentFilename = box->fileSelected();
    currentDirectory = box->dirEntered();
    
    delete box;
  }
  else {
    currentFilename = filename;
    //    currentDirectory = QDir::currentDirPath();
  }

  if ( currentFilename.isEmpty() )
    return KSTATUS_USER_CANCEL;
  
  QString dbg = currentDirectory + QDir::separator() + currentFilename;
  warning( "Saving file : %s\n", dbg.data() );
  info.setFile(currentDirectory + QDir::separator() + currentFilename);
  
  if(info.exists()){

    // Ftp supports resuming
    if ( resume ) {
      KMsgBox *conf = new KMsgBox( this,
				   i18n("Warning:"), 	
				   i18n("A File with this Name exists already !"),
				   KMsgBox::EXCLAMATION,
				   i18n("Overwrite"),
				   i18n("Resume"),
				   i18n("Cancel"));

      conf->show();
    
      switch (conf->result()) {
      case 1 : // Overwrite
	if (! info.isWritable() ) {
	  KMsgBox::message( this,
			    i18n("Sorry"),
			    i18n("Cannot write to this file."));
	  return KSTATUS_NOPERMISSIONS;
	}
	return KSTATUS_OVERWRITE;
	break;
      case 2 : // Resume
	if (! info.isWritable() ) {
	  KMsgBox::message( this,
			    i18n("Sorry"),
			    i18n("Cannot write to this file."));
	  return KSTATUS_NOPERMISSIONS;
	}
	return KSTATUS_RESUME;
	break;
      default : // Cancel
	goto try_again;
	break;
      }
    }
    else {             // If resuming is not supported
      KMsgBox *conf = new KMsgBox( this,
				   i18n("Warning:"), 	
				   i18n("A File with this Name exists already !"),
				   KMsgBox::EXCLAMATION,
				   i18n("Overwrite"),
				   i18n("Cancel"));

      conf->show();
    
      switch (conf->result()) {
      case 1 : // Overwrite
	if (! info.isWritable() ) {
	  KMsgBox::message( this,
			    i18n("Sorry"),
			    i18n("Cannot write to this file."));
	  return KSTATUS_NOPERMISSIONS;
	}
	return KSTATUS_OVERWRITE;
	break;
      default : // Cancel
	goto try_again;
	break;
      }
    }
  }
  
  return KSTATUS_OK;
}



void KMainWidget::readTransfers() {
  warning("readTransfers()");
  QStrList trlist;

  KConfig *config = KApplication::getKApplication()->getConfig();
  config->setGroup( "Running Transfers" );

  if (config->readListEntry( "Items", trlist )){
    QString f = trlist.first();
    QString g = trlist.next();
  
    while ( f && g ) {
      addTransfer( f, g );
      f = trlist.next();
      g = trlist.next();
    }
  }
}



void KMainWidget::saveTransfers() {
  warning("saveTransfers()");

  QStrList trlist;
  KItem *item;
  KConfig *config;

  config = KApplication::getKApplication()->getConfig();
  config->setGroup( "Running Transfers" );

  for ( item =GlobalItemList.first(); item != 0; item=GlobalItemList.next()){
    trlist.append(item->getSrc());
    trlist.append(item->getDest());
  }
  config->writeEntry( "Items", trlist );

}


void KMainWidget::quit(){
  if ( myTabListBox->count() > 0 )
    saveTransfers();
  else {
    KConfig *config = KApplication::getKApplication()->getConfig();
    config->setGroup( "Running Transfers" );
    config->writeEntry( "Items", "" );
  }

  deleteMode = TRUE;
  KIOJob::deleteAllJobs();
  writeSettings();
  KApplication::getKApplication()->quit();
}


void KMainWidget::resumeCurrent() {
  int id = myTabListBox->currentItem();

  if ( id < 0 )
    return;

  if ( ! myTabListBox->isMarked( id ) )
    return;

  KItem *it = GlobalItemList.at(id);

  KIOJob *job = new KIOJob( it->getID() );
  
  Connect( job );

  it->setJob( job );

  KURL u = it->getSrc();

  QString f = u.protocol();

  if ( f == "ftp" )
    job->resume( it->getSrc(), it->getDest() ); // resume selected KIOJob
  else if ( f == "http" && it->getStatus() == KItem::TR_LATER ){
    job->setOverWriteExistingFiles( true );
    job->copy( it->getSrc(), it->getDest() ); // copy selected KIOJob
  }

  warning( "Resuming job %d from GlobalItemList with id %d\n",
	   id, it->getID());

  it->setStatus( KItem::TR_TRYING );
  
  updateToolbar();

  if ( queueingEnabled )
    checkQueue();
}


void KMainWidget::stopCurrent() {
  int id = myTabListBox->currentItem();

  if ( id < 0 )
    return;

  if ( ! myTabListBox->isMarked( id ) )
    return;

  KItem *it = GlobalItemList.at(id);

  warning( "Stopping job %d from GlobalItemList with id %d\n",
	   id,
	   it->getID());

  it->setStatus( KItem::TR_STOPPED );
  it->getJob()->stop(); // stop selected KIOJob

  updateToolbar();

  if ( queueingEnabled )
    checkQueue();
}


void KMainWidget::deleteCurrent() {

  int id = myTabListBox->currentItem();

  if ( id < 0 )
    return;

  if ( ! myTabListBox->isMarked( id ) )
    return;

  KItem *it = GlobalItemList.at(id);

  warning( "Canceling job %d from GlobalItemList with id %d\n",
	   id,
	   it->getID());

  if ( it->getStatus() == KItem::TR_STOPPED || it->getStatus() == KItem::TR_LATER ){
    slotFinished( it->getID());
    return;
  }
  
  it->getJob()->cancel(); // stop selected KIOJob
}


void KMainWidget::openTransfer() {
  QString newtransfer;

  KQuestionDialog *box = new KQuestionDialog(i18n("Open transfer :"), this);
  box->show();

  if (!box->result())   /* cancelled */
    return;

  newtransfer = box->getAnswer();
  delete box;

  if ( newtransfer.isEmpty() ) /* answer is "" */
    return;

  addTransfer( newtransfer );
}



void KMainWidget::pasteTransfer() {
  QString newtransfer;
  QClipboard *cb = QApplication::clipboard();

  newtransfer = cb->text();

  if ( promptMode ){
    KQuestionDialog *box = new KQuestionDialog(i18n("Open transfer :"), this);
    box->setText( newtransfer.data() );
    box->show();
    
    if (!box->result())   // cancelled
      return;
    
    newtransfer = box->getAnswer();
    delete box;
  }

  if ( newtransfer.isEmpty() ) // answer is
    return;

  addTransfer( newtransfer );
}



void KMainWidget::addTransfer( QString s, QString d ) {
  cout << "addTransfer   " << s << endl << d << endl;
  QString f;
  KURL url;
  int result;

  url = s;
  KItem *item;

  for ( item =GlobalItemList.first(); item != 0; item=GlobalItemList.next())
    if ( item->getSrc() == s ){ // if we find this URL in the list
      QString e;
      e << i18n( "Already saving URL \n" ) << s.data();
      KMsgBox::message( this, i18n( "KGet Error" ), e.data() );
      return;
    }

  f = url.protocol();

  QString dest;

  if ( !d ) {    // Ask for the name where to save it
    if ( f == "ftp" )
      result =  saveAs( url.filename(), true );   // ftp can resume
    else if ( f == "http" )
      result =  saveAs( url.filename(), false );  // http can't
    else
      return;

    dest = "file:" + currentDirectory + "/" + currentFilename;

  } else {
    if ( f == "ftp" )
      result =  KSTATUS_RESUME;    // ftp can resume
    else if ( f == "http" )
      result = KSTATUS_OVERWRITE;  // http can't
    else
      return;

    KURL url2(d);
    dest = d;
    currentFilename = url2.filename();
  }


  if ( result == KSTATUS_OK || result == KSTATUS_RESUME ||
       result == KSTATUS_OVERWRITE ) {
    myTabListBox->insertItem( "disconnect\n" + currentFilename + "\n   \n     \n");
    // name of the file insert into TabListBox
	  
    int id = newKIOJobID();
    if ( id < 0 ) {
      fprintf( stderr, "cannot find new id for KIOJob\n");
      exit(0);
    }
    KIOJob *job = new KIOJob( id );
    job->setOverWriteExistingFiles( true );

    KItem *item = new KItem( s, dest, id, job );

    // append new KItem to global list
    GlobalItemList.append ( item );

    Connect( job );
      
    if ( !queueingEnabled ||
	 GlobalItemList.count() <= maxSimultaneousConnections )
      // if queueing is not enabled
      // or if there is less items in the list then maxSimultaneousConnections
      {
	if ( result == KSTATUS_OK ) {
	  job->copy( s.data(), dest.data() );
	} else if ( result == KSTATUS_OVERWRITE ){
	  job->copy( s.data(), dest.data() );
	} else if ( result == KSTATUS_RESUME )
	  job->resume( s.data(), dest.data() );

	item->setStatus( KItem::TR_TRYING );
      } else{
	item->setStatus( KItem::TR_QUEUED );
	checkQueue();
      }
    // otherwise check queue for number of running transfers
  }
  QString tmps;
  int a = myTabListBox->count();

  if ( a == 1)
    ksprintf( &tmps, "1 %s", i18n("transfer") );
  else
    ksprintf( &tmps, "%d %s", a, i18n("transfers") );

  myStatusBar->changeItem( tmps, ID_URL_TEXT);
}



int KMainWidget::newKIOJobID() {
  KItem *item;
  int i;

  for ( i = 0; i < 10000; i++ ){
    for (item=GlobalItemList.first(); item != 0; item=GlobalItemList.next()){
      if ( item->getID() == i )
	break;
    }
    if ( item == 0 ) // if we do not find this id in the list
      return i;
  }
  return -1;
}



void KMainWidget::checkQueue() {
  uint numRun = 0;
  uint numQue = 0;
  int status;
  KItem *item;

  warning("checking queue\n");
  if ( GlobalItemList.count() == 0 )
    return;

  for ( item=GlobalItemList.first(); item != 0; item=GlobalItemList.next()){
    status = item->getStatus();
    if ( status == KItem::TR_RUNNING || status == KItem::TR_TRYING )
      numRun++;
    else if ( status == KItem::TR_QUEUED )
      numQue++;
  }

  item=GlobalItemList.first();
  while ( numRun < maxSimultaneousConnections && item != 0 ){
    status = item->getStatus();
    if (  status == KItem::TR_QUEUED ){
      warning("Queueing:  Starting another queued job\n");
      myStatusBar->changeItem( i18n("Starting another queued job."), ID_URL_TEXT);
      item->getJob()->copy( item->getSrc(), item->getDest());
      item->setStatus( KItem::TR_TRYING );
      numRun++;
    } else if ( status == KItem::TR_STOPPED && numQue == 0 ) {
      warning("Queueing:  Starting another stopped job\n");
      myStatusBar->changeItem( i18n("Starting another stopped job."), ID_URL_TEXT);

      KIOJob *job = new KIOJob( item->getID() );

      Connect( job );

      item->setJob( job );
      item->setStatus( KItem::TR_TRYING );

      job->resume( item->getSrc(), item->getDest() );
      numRun++;
    }
    item=GlobalItemList.next();
  }
}



void KMainWidget::slotAnimTimeout(){
  KItem *item;
  QString s; int index;

  if( ! animateIcons) return;
  
  if ( GlobalItemList.isEmpty() ) return;

  animCounter++;
  if ( animCounter == myTabListBox->getPhasesNum() )
    animCounter = 0;

  myTabListBox->setAutoUpdate(FALSE);
  for ( item=GlobalItemList.first(); item != 0; item=GlobalItemList.next())
    if ( item->getStatus() == KItem::TR_RUNNING ){
      index = GlobalItemList.find( item );
      s.sprintf("connect%i", animCounter);
      myTabListBox->changeItemPart( s, index, 0 );
    } else if ( item->getStatus() == KItem::TR_TRYING ){
      index = GlobalItemList.find( item );
      s.sprintf("try%i", animCounter);
      myTabListBox->changeItemPart( s, index, 0 );
    } else if ( item->getStatus() == KItem::TR_LATER ){
      index = GlobalItemList.find( item );
      myTabListBox->changeItemPart( "later", index, 0 );
    }
  myTabListBox->setAutoUpdate(TRUE);
  myTabListBox->repaint();
}



void KMainWidget::slotFinished( int id ) {
  KItem *item;

  if ( deleteMode )     // so that we don't start new jobs
    return;

  for ( item=GlobalItemList.first(); item != 0; item=GlobalItemList.next())
    if ( item->getID() == id ) {
      int index = GlobalItemList.find( item );
      GlobalItemList.remove( item );

      myTabListBox->removeItem( index );

      if ( myTabListBox->count() == 0){
	myStatusBar->changeItem( i18n("No files"), ID_URL_TEXT);
      } else
	myStatusBar->changeItem( "", ID_URL_TEXT);

      updateToolbar();

      if ( useSound && audio )
	if ( GlobalItemList.isEmpty() )
	  audio->play( audioFinishedAll );
	else
	  audio->play( audioFinished );

      if ( queueingEnabled )
	checkQueue();

      if ( !isTransfer() && disconnectMode )
	disconnect();

      QString tmps;
      int a = myTabListBox->count();

      if ( a == 1)
	ksprintf( &tmps, "1 %s", i18n("transfer") );
      else
	ksprintf( &tmps, "%d %s", a, i18n("transfers") );
      
      return;
    }
  
  warning( "Wrong finished for KIOJob : %d\n", id);
  return;
}



void KMainWidget::slotTryLater( int id ) {
  KItem *item;

  for ( item=GlobalItemList.first(); item != 0; item=GlobalItemList.next())
    if ( item->getID() == id ) {
      item->setStatus( KItem::TR_LATER );

      int index = GlobalItemList.find( item );
      item = GlobalItemList.take( index );
      GlobalItemList.append( item );

      QString str = myTabListBox->text( index );
      myTabListBox->removeItem( index );

      myTabListBox->insertItem( str );
      myTabListBox->unmarkAll();
      updateToolbar();

      if ( queueingEnabled )
	checkQueue();

      if ( !isTransfer() && disconnectMode )
	disconnect();

      return;
    }
  
  warning( "Wrong trylater for KIOJob : %d\n", id);
  return;
}


void KMainWidget::slotStarted( int id ) {
  KItem *item;

  for ( item =GlobalItemList.first(); item != 0; item=GlobalItemList.next())
    if ( item->getID() == id ) {
      if ( item->getStatus() == KItem::TR_TRYING ){
	item->setStatus( KItem::TR_RUNNING );
	myStatusBar->changeItem( i18n("Starting transfer."), ID_URL_TEXT);
      }
    }
}



void KMainWidget::slotProgress( int percent, int size, int id ) {
  KItem *item;
  int index;
  float fsize;

  for ( item =GlobalItemList.first(); item != 0; item=GlobalItemList.next())
    if ( item->getID() == id ) {
      item->setSize( size );
      index = GlobalItemList.find ( item );

      myStatusBar->changeItem( "", ID_URL_TEXT);

      QString s;
      s.sprintf("%d", percent);
      myTabListBox->setAutoUpdate(FALSE);
      myTabListBox->changeItemPart( s, index, 2);

      if ( size > 1048576 ){
	fsize = (float) size / (float) 1048576;
	s.sprintf ( "%.2f MB", fsize);
      } else if ( size > 1024 ){
	fsize = (float) size / (float) 1024;
	s.sprintf ( "%.2f kB", fsize);
      } else {
	s.sprintf ( "%d B", size);
      }

      myTabListBox->changeItemPart( s, index, 3);
      myTabListBox->setAutoUpdate(TRUE);
      myTabListBox->repaint();
      return;
    } 

  warning ( "Wrong progress for KIOJob : %d\n", id);
  return;
}


void KMainWidget::slotError( bool confirm, const char* s, int id ) {
  KItem *item;

  warning("slotError()");
  for ( item =GlobalItemList.first(); item != 0; item=GlobalItemList.next())
      if ( item->getID() == id ) {
	  
	  if ( confirm ) { // Errors that can be caused by accident
	      if ( promptMode ){
		  KMsgBox *m = new KMsgBox( 0L, i18n("Error"), 
					    s, KMsgBox::EXCLAMATION, 
					    i18n("Try later"), 
					    i18n("Cancel transfer") );
		  
		  m->show();
		  
		  switch (m->result()) {
		  case 1:
		      item->getJob()->cancel(1);
		      break;
		  case 2:
		      item->getJob()->cancel();
		      break;
		  }
	      } else // When prompt mode is off => set it implicitly to TRY LATER
		  item->getJob()->cancel(1);
	  }
	  else {  // Errors, that cannot be caused by accident ( Malformed URL..)
	      if ( promptMode ) {  // show a message
		  KMsgBox::message( 0L, i18n("Error"), 
				    s, KMsgBox::EXCLAMATION, 
				    i18n("Ok"));
	      }
	      item->getJob()->cancel();
	  }
	
	  return;
      }

  warning ( "Wrong error for KIOJob : %d\n", id);
  return;
}


void KMainWidget::slotDropEvent( KDNDDropZone * _dropZone ) {
  QString s;

  QStrList & list = _dropZone->getURLList();

  for ( s = list.first(); s != 0L; s = list.next() ) {
    addTransfer( s );
  }

  myTabListBox->unmarkAll();
}


void KMainWidget::slotHighlightedEvent( int Index ) {
  if ( Index >= 0 ) {
    myStatusBar->changeItem(GlobalItemList.at(Index)->getSrc(), ID_URL_TEXT);
    updateToolbar();
  }
}


void KMainWidget::slotSelectedEvent( int Index ) {
  if ( Index >= 0 ) {
    QClipboard *cb = QApplication::clipboard();
    cb->setText( GlobalItemList.at(Index)->getSrc() );
    updateToolbar();
  }
}


void KMainWidget::slotMidClickEvent( int Index ) {
  myTabListBox->unmarkItem( Index );
  myStatusBar->changeItem( "", ID_URL_TEXT);
  updateToolbar();
}


void KMainWidget::slotPopupMenu( int Index ) {
  QPopupMenu *menu = new QPopupMenu;

  myTabListBox->setCurrentItem(Index);
  myTabListBox->unmarkAll();
  myTabListBox->markItem( Index );

  menu->insertItem(i18n("Move to beginning"), this,
		   SLOT(slotMoveToBegin()));
  menu->insertItem(i18n("Move to end"), this,
		   SLOT(slotMoveToEnd()));
  menu->insertItem(i18n("Try Later"), this,
		   SLOT(slotMenuTryLater()));

  menu->popup(QCursor::pos());
}


void KMainWidget::slotMoveToBegin() {
  int index = myTabListBox->currentItem();

  if ( index <= 0 )
    return;

  KItem *item = GlobalItemList.take( index );
  GlobalItemList.insert( 0, item );

  QString str = myTabListBox->text( index );
  myTabListBox->removeItem( index );

  myTabListBox->insertItem( str, 0 );
  myTabListBox->unmarkAll();
  updateToolbar();
}


void KMainWidget::slotMoveToEnd() {
  int index = myTabListBox->currentItem();

  if ( index < 0 )
    return;

  KItem *item = GlobalItemList.take( index );
  GlobalItemList.append( item );

  QString str = myTabListBox->text( index );
  myTabListBox->removeItem( index );

  myTabListBox->insertItem( str );
  myTabListBox->unmarkAll();
  updateToolbar();
}


void KMainWidget::slotMenuTryLater() {
  int index = myTabListBox->currentItem();

  if ( index < 0 )
    return;

  KItem *item = GlobalItemList.at( index );

  if ( item->getStatus() == KItem::TR_LATER ){
    warning( "job %d from GlobalItemList with id %d is already set for TRY_LATER\n",
	     index,
	     item->getID());
    return;
  }

  warning( "Setting job %d from GlobalItemList with id %d as TRY_LATER\n",
	   index,
	   item->getID());
  slotTryLater( item->getID() );

  item->getJob()->stop(); // stop selected KIOJob

}



void KMainWidget::closeEvent( QCloseEvent * ){
  quit();
}


void KMainWidget::slotShowStatusbar() {

  showStatusbar = !showStatusbar;
  options->setItemChecked( options->idAt( ID_STATUSBAR ), showStatusbar );
  if ( !showStatusbar )
    myStatusBar->enable( KStatusBar::Hide );
  else
    myStatusBar->enable( KStatusBar::Show );
  resizeEvent( 0L );
}


void KMainWidget::slotShowToolbar() {

  showToolbar = !showToolbar;
  options->setItemChecked( options->idAt( ID_TOOLBAR ), showToolbar );
  if ( !showToolbar )
    myToolBar->enable( KToolBar::Hide );
  else
    myToolBar->enable( KToolBar::Show );
  resizeEvent( 0L );
}


void KMainWidget::slotAnimateStatus() {

  animateIcons = !animateIcons;
  options->setItemChecked( options->idAt( ID_ANIMATEICONS ), animateIcons );

  if ( !animateIcons && animTimer->isActive() )
    animTimer->stop();
  else
    animTimer->start( 200 );
}


void KMainWidget::slotDockWindow() {

  bool lastdock = dockWindow;

  dockWindow = !dockWindow;
  options->setItemChecked( options->idAt( ID_DOCKWINDOW ), dockWindow );
  myToolBar->setItemEnabled( ITEM_DOCK, dockWindow );

  if ( !dockWindow && lastdock )
    dock_widget->undock();
  else if ( dockWindow && !lastdock ){
    dock_widget->dock();
    this->hide();
  }
}


void KMainWidget::togglePrompt() {

  promptMode = !promptMode;

  if ( promptMode ) {
    ((KToolBarButton*)myToolBar->getButton(ITEM_PROMPT))->on(true);
    myStatusBar->changeItem( i18n("Prompt mode on."), ID_URL_TEXT);
  }
  else {
    ((KToolBarButton*)myToolBar->getButton(ITEM_PROMPT))->on(false);
    myStatusBar->changeItem( i18n("Prompt mode off."), ID_URL_TEXT);
  }
}


void KMainWidget::toggleDisconnect() {

  disconnectMode = !disconnectMode;

  if ( disconnectMode ) {
    ((KToolBarButton*)myToolBar->getButton(ITEM_DISCONNECT))->on(true);
    myStatusBar->changeItem( i18n("Disconnect mode on."), ID_URL_TEXT);
  }
  else {
    ((KToolBarButton*)myToolBar->getButton(ITEM_DISCONNECT))->on(false);
    myStatusBar->changeItem( i18n("Disconnect mode off."), ID_URL_TEXT);
  }
}



void KMainWidget::dockHide(){
  this->hide();
}


void KMainWidget::updateToolbar() {
  int id;
  int count = myTabListBox->count();

  if ( count == 0)
    goto updcln;

  id = myTabListBox->currentItem();

  if ( id >= count || id < 0)
    goto updcln;

  if ( myTabListBox->isMarked(id) ){
    //    warning("Enabling delete for item no. %d\n", id);
    myToolBar->setItemEnabled( ITEM_DELETE, TRUE);

    KItem *item = GlobalItemList.at(id);
    KURL url = item->getSrc();
    QString f = url.protocol();

    switch ( item->getStatus() ){
    case KItem::TR_RUNNING :
    case KItem::TR_TRYING :
      if ( f != "http" ){
	myToolBar->setItemEnabled( ITEM_STOP, TRUE);
      }
      myToolBar->setItemEnabled( ITEM_RESUME, FALSE);
      break;
    case KItem::TR_STOPPED :
      myToolBar->setItemEnabled( ITEM_STOP, FALSE);
      if ( f != "http" )
	myToolBar->setItemEnabled( ITEM_RESUME, TRUE);
      break;
    case KItem::TR_LATER :
      myToolBar->setItemEnabled( ITEM_STOP, FALSE);
      myToolBar->setItemEnabled( ITEM_RESUME, TRUE);
      break;
    }
    return;
  }

 updcln:
  myToolBar->setItemEnabled( ITEM_RESUME, FALSE);
  myToolBar->setItemEnabled( ITEM_STOP, FALSE);
  myToolBar->setItemEnabled( ITEM_DELETE, FALSE); 
  return;
}



bool KMainWidget::isTransfer() {
  KItem *item;

  if ( GlobalItemList.count() <= 0 )
    return false;
  else {
    for ( item=GlobalItemList.first(); item != 0; item=GlobalItemList.next())
      {
	int status = item->getStatus();
	if ( status == KItem::TR_RUNNING || status == KItem::TR_TRYING )
	  return true;
      }
  }
  return false;
}


void KMainWidget::Connect( KIOJob *job ) {
  connect( job, SIGNAL( progress( int, int, int ) ),
	   this, SLOT( slotProgress( int, int, int ) ) );
  connect( job, SIGNAL( jobstarted( int ) ),
	   this, SLOT( slotStarted( int ) ) );
  connect( job, SIGNAL( finished( int ) ),
	   this, SLOT( slotFinished( int ) ) );
  connect( job, SIGNAL( trylater( int ) ),
	   this, SLOT( slotTryLater( int ) ) );
  connect( job, SIGNAL( error( bool, const char*, int ) ),
	   this, SLOT( slotError( bool, const char*, int ) ) );
}


void KMainWidget::disconnect() {

  system( disconnectCommand.data() );

}
