/* $Id: kget.h,v 1.1.1.1 1998/05/06 19:13:31 matt Exp $
   $Log: kget.h,v $
   Revision 1.1.1.1  1998/05/06 19:13:31  matt
   Imported sources

   Revision 1.1  1998/01/29 18:21:19  matt
   Renamed kdownload.h to kget.h

   Revision 1.1  1998/01/28 16:51:30  matt
   Renamed kftp.h to kdownload.h

   Revision 1.1  1998/01/28 14:02:58  matt
   Renamed kftp.hh to kftp.h

   Revision 1.1.1.1  1998/01/24 09:11:31  matt
   Imported sources


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   */


#ifndef _KGET_H_
#define _KGET_H_

#include <drag.h>
#include <qtimer.h>

#include <kmenubar.h>
#include <ktopwidget.h>

#include "kmytablistbox.h"
#include "PPPStats.h"

// StatusBar field IDs
#define ID_URL_TEXT 1
#define ID_TRANSFER_SPEED 2

// Options menu IDs
#define ID_STATUSBAR     0
#define ID_TOOLBAR       1
#define ID_ANIMATEICONS  3
#define ID_DOCKWINDOW    5

class KIOJob;
class KAudio;

class KItem : public QObject {

 public:

    enum { TR_STOPPED = 0,
	   TR_RUNNING = 1,
	   TR_TRYING  = 2,
	   TR_QUEUED  = 3,
	   TR_LATER   = 4
    };

    KItem( const char *_src, const char *_dest, int _id, KIOJob *_job )
    { src=_src; dest=_dest; id=_id; job=_job; status = TR_STOPPED; }

    const char *getSrc() const { return src; }
    const char *getDest() const { return dest; }
    int getID() const { return id; }
    KIOJob* getJob() const { return job; }
    int getStatus() const { return status; }
    int getSize() const { return size; }

    void setStatus( int _status ) { status = _status; }
    void setJob( KIOJob *_job ) { job = _job; }
    void setSize( int _size) { size = _size; }

 private:  

    QString src;
    QString dest;
    int id;
    KIOJob *job;
    int size;

    int status;
};


class KMainWidget : public KTopLevelWidget {

    Q_OBJECT

 public:
  
    KMainWidget( const char *name=0 );
    ~KMainWidget();
  
    QPopupMenu *file, *options, *help;

    enum { KSTATUS_OK 		= 0, 
	   KSTATUS_OS_ERROR  	= 1, 
	   KSTATUS_USER_CANCEL 	= 2,
	   KSTATUS_RETRY 		= 3,
	   KSTATUS_NOPERMISSIONS 	= 4,
	   KSTATUS_OVERWRITE      = 5,
	   KSTATUS_RESUME         = 6
    };

    enum { ITEM_OPEN   = 0,
	   ITEM_RESUME = 1,
	   ITEM_STOP   = 2,
	   ITEM_DELETE = 3,
	   ITEM_DOCK   = 4,
	   ITEM_PASTE  = 5,
	   ITEM_PROMPT = 6,
	   ITEM_DISCONNECT = 7
    };
  
    bool isTransfer();

 protected:

    void  closeEvent( QCloseEvent * );

    void readSettings();
    void writeSettings();

    void readTransfers();
    void saveTransfers();

    void setupMenuBar();
    void setupToolBar();
    void setupStatusBar();
    void setupTabListBox();
    void setupPPPStats();

    void updateToolbar();
    int saveAs(QString, bool);
    void addTransfer(QString s, QString d = NULL );

    int newKIOJobID();
    void checkQueue();

    void Connect( KIOJob *job );

    void disconnect();

    bool showToolbar;
    KToolBar::BarPosition toolbarPos;
    bool showStatusbar;

    bool queueingEnabled;
    uint maxSimultaneousConnections;
    bool reconnectingEnabled;
    uint reconnectingTime;
    bool showSpeed;
    uint statsSpeed;
    uint pppLink;

    bool deleteMode;

    bool dockWindow;

    bool useSound;

    bool animateIcons;

    /**
     * The image from 0...animatedLogo.count()-1 we are currently
     * displaying.
     * 
     * @see #animatedLogo
     */
    uint animCounter;
    /**
     * Times used to display the animated logo.
     *
     * @see #animatedLogo
     */
    QTimer *animTimer;

  
    private:    

    KMenuBar *myMenuBar;
    KToolBar *myToolBar;
    KStatusBar *myStatusBar;
    KMyTabListBox *myTabListBox;

    KAudio *audio;
    PPPStats *pppstats;

    QString audioFinished;
    QString audioFinishedAll;

    QFont  tablistboxfont;

    QString properties;

    QList<KItem> GlobalItemList;

    // directories for directory buttons in save dialog
    QString bDir1;
    QString bDir2;
    QString bDir3;
    QString bDir4;

    bool promptMode;
    bool disconnectMode;

    QString disconnectCommand;

public slots:  

    void quit();
    void about();
    void helpSelected();

    void openTransfer();
    void pasteTransfer();
    void resumeCurrent();
    void stopCurrent();
    void deleteCurrent();

    void togglePrompt();
    void toggleDisconnect();

    void slotAnimTimeout();

    void slotShowStatusbar();
    void slotShowToolbar();
    void slotAnimateStatus();
    void slotDockWindow();
    void slotSettings();

    void dockHide();

    void slotProgress( int, int, int );
    void slotFinished( int );
    void slotStarted( int );
    void slotTryLater( int );
    void slotError( bool, const char*, int );
    
    /// Drag and Drop
    void slotDropEvent( KDNDDropZone * _dropZone );

    /// Click on TabListBox
    void slotHighlightedEvent( int Index );
    void slotMidClickEvent( int Index );
    void slotSelectedEvent( int Index );

    void slotPopupMenu( int Index );
    void slotMoveToBegin();
    void slotMoveToEnd();
    void slotMenuTryLater();

    void updateStats( uint );

};

extern KMainWidget *p_kmain;

#endif
