#include <kapp.h>
#include <klocale.h>
#include "kgetoptdlg.h"


KGETOptDlg::KGETOptDlg(QWidget *parent, const char *name, WFlags f)
  : QDialog(parent, name, true, f)
{
  resize(410, 390);
  // i do not want to be resized!
  setMaximumSize(410, 390);
  setMinimumSize(410, 390);
  
  // KTabCtl
  tabDlg = new KTabCtl(this);
  tabDlg->setGeometry(10, 10, 390, 330);
  
  prxDlg = new KProxyDlg(tabDlg);
  tabDlg->addTab(prxDlg, "Proxy");
  tnfDlg = new TransferDlg(tabDlg);
  tabDlg->addTab(tnfDlg, "Transfer");
  sndDlg = new SoundDlg(tabDlg);
  tabDlg->addTab(sndDlg, "Sound");
  dirDlg = new DirDlg(tabDlg);
  tabDlg->addTab(dirDlg, "Directories");
  fntDlg = new FontDlg(tabDlg);
  tabDlg->addTab(fntDlg, "Font");

  // help button
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(11, 350, 72, 27);
  connect(help, SIGNAL(clicked()), SLOT(helpShow()));

  // ok button
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(230, 350, 72, 27);
  ok->setDefault(TRUE);
  connect(ok, SIGNAL(clicked()), SLOT(accept()));
  
  // cancel button
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(318, 350, 72, 27);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));

  // my name is
  setCaption(i18n("KGet Settings"));
}

KGETOptDlg::~KGETOptDlg()
{
  // now delete everything we allocated before
  delete help;
  delete ok;
  delete cancel;
  delete fntDlg;
  delete prxDlg;
  delete tnfDlg;
  delete sndDlg;
  delete dirDlg;
  delete tabDlg;
}

void KGETOptDlg::helpShow()
{
  kapp->invokeHTMLHelp(0, 0);
}


void KGETOptDlg::setFontData(QFont font)
// forward data to FontDlg
{
  fntDlg->setData(font);
}

QFont KGETOptDlg::dataFont() const
{
  return(fntDlg->data()); 
}


void KGETOptDlg::setProxyData(QStrList *strList)
// forward data to KProxyDlg
{
  prxDlg->setData(strList);
}

QStrList KGETOptDlg::dataProxy() const
{
  return(prxDlg->data());
}


void KGETOptDlg::setTransferData(QStrList *strList)
// forward data to TransferDlg
{
  tnfDlg->setData(strList);
}

QStrList KGETOptDlg::dataTransfer() const
{
  return(tnfDlg->data());
}


void KGETOptDlg::setSoundData(QStrList *strList)
// forward data to SoundDlg
{
  sndDlg->setData(strList);
}

QStrList KGETOptDlg::dataSound() const
{
  return(sndDlg->data());
}

void KGETOptDlg::setDirData(QStrList *strList)
// forward data to DirDlg
{
  dirDlg->setData(strList);
}

QStrList KGETOptDlg::dataDir() const
{
  return(dirDlg->data());
}
