#include "file.h"

#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>

#include <qregexp.h>

#include <qdir.h>

KProtocolFILE::KProtocolFILE()
{
	file = NULL;
}

KProtocolFILE::~KProtocolFILE()
{
	Close();
}

int KProtocolFILE::Open(KURL *url, int mode, unsigned long int offset)
{
  struct stat st;
  Close();

  size=0;
  
  if( mode & READ )
  {
    file = fopen( url->path(), "rb");
    if( !file )
    {
      Error(KIO_ERROR_CouldNotRead,"Can't open File for reading",errno);
      return(FAIL);
    }
    fstat(fileno(file), &st);
    size = st.st_size;
  }
  else if( mode & RESUME )
  {
    file = fopen( url->path(), "rb");
    if( !file )
    {
      Error(KIO_ERROR_CouldNotRead,"Can't open File for reading",errno);
      return(FAIL);
    }
    fseek(file, offset, SEEK_SET);
    fstat(fileno(file), &st);
    size = st.st_size;
  }
  else if( mode & WRITE )
  {
    if( !( mode & OVERWRITE ) )
    {
      if(stat( url->path(), &st) != -1)
      {
	Error( KIO_ERROR_FileExists, "File already exists", errno );
	return(FAIL);
      }
    }

    file = fopen( url->path(), "wb");
    if( !file )
    {
      Error(KIO_ERROR_CouldNotWrite,"Can't open file for writing",errno);
      return(FAIL);
    }
  }
  else if( mode & APPEND )
  {
    file = fopen( url->path(), "ab");
    if( !file )
    {
      Error(KIO_ERROR_CouldNotWrite,"Can't open file for writing",errno);
      return(FAIL);
    }
    printf("file: appending to file %s\n", url->path());
  }

  return SUCCESS;
}

int KProtocolFILE::Close()
{
  if( file )
    fclose(file);
  file = NULL;
  return SUCCESS;
}


long KProtocolFILE::Write(void *buffer, long len)
{
	long n = fwrite(buffer,1,len,file);
	if(n < 0)
	{
		Error(KIO_ERROR_CouldNotRead,"Writing to file failed",errno);
		return(FAIL);
	}
	return(n);
}

long KProtocolFILE::Size()
{
	if(!file) return(FAIL);
	return(size);
}

int KProtocolFILE::atEOF()
{
	if(!file) return(FAIL);
	return(feof(file));
}
