/* $Id: main.cpp,v 1.1.1.1 1998/05/06 19:13:37 matt Exp $
   $Log: main.cpp,v $
   Revision 1.1.1.1  1998/05/06 19:13:37  matt
   Imported sources

   Revision 1.2  1998/03/02 14:36:27  matt
   kgetslave based on kioslave from kfm


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   */

#include <kapp.h>
#include <qfileinf.h>

#include "kioslave_ipc.h"
#include "main.h"
#include "kio_errors.h"
#include "manage.h"
#include "protocol.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <errno.h>

#define FTP_LOGIN "anonymous"
#define FTP_PASSWD "joedoe@nowhere.crg"

void sig_handler(int);
void sig_handler2(int);

KIOSlave *slave = 0L;

int main(int argc, char **argv)
{
  printf("A new kgetslave has been started\n");
    
  if ( argc != 2 )
    {
      fprintf( stderr, "Usage: kgetslave path\n");
      exit(1);
    }

  signal(SIGTERM,sig_handler);
  signal(SIGCHLD,sig_handler2);
    
  KApplication a( argc, argv );

  KIOSlave slave( argv[1] );
    
  a.exec();
  return(0);
}

void KIOSlave::ProcessError(KProtocol *prot, const char *srcurl)
{
  int KError, SysError;
  QString message;

  prot->GetLastError(KError, message, SysError);

  fprintf( stderr, "KIOSlave-ERROR (%s): %s; %s\n",srcurl,message.data(),
	   strerror(SysError));
  ipc->fatalError(KError, srcurl, SysError);
}

KIOSlave::KIOSlave( char * _path )
{
  ipc = new KIOSlaveIPC( _path );
    
  if ( !ipc->isConnected() )
    {
      fprintf( stderr, "Could not connect to KIO om path %s\n", _path );
      exit(1);
    }

  copyDestFile = 0L;

  connect( ipc, SIGNAL( copy( const char*, const char*, bool ) ),
	   this, SLOT( copy( const char*, const char*, bool ) ) );
  connect( ipc, SIGNAL( resume( const char*, const char* ) ),
	   this, SLOT( resume( const char*, const char* ) ) );
  connect( ipc, SIGNAL( cleanUp() ), this, SLOT( cleanUp() ) );
  connect( ipc, SIGNAL( getPID() ), this, SLOT( getPID() ) );

  ipc->hello();
}

KIOSlave::~KIOSlave()
{
  warning("delete ~KIOSlave");
  delete ipc;
}

void KIOSlave::getPID()
{
  ipc->setPID( (int)getpid() );
}

void KIOSlave::slotRedirection( const char *_url )
{
  sRedirection = _url;
}


void KIOSlave::copy( const char *_src_url, const char *_dest_url, bool _overwriteExistingFiles )
{
  KURL u( _src_url );
  if ( u.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_src_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _src_url, 0 );
      return;
    }

  KURL su( u.nestedURL() );
  if ( su.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_src_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _src_url, 0 );
      return;
    }

  KURL u2( _dest_url );
  if ( u2.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_dest_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _dest_url, 0 );
      return;
    }
  KURL du( u2.nestedURL() );
  if ( du.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_dest_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _dest_url, 0 );
      return;
    }

  if( ProtocolSupported( _src_url ) && ProtocolSupported( _dest_url ) )
    {
      KProtocol *src_prot = CreateProtocol(_src_url);
      KProtocol *dest_prot = CreateProtocol(_dest_url);

      int destmode = KProtocol::WRITE;
      if( _overwriteExistingFiles ) destmode |= KProtocol::OVERWRITE;
	
      if( dest_prot->Open(&du, destmode) != KProtocol::SUCCESS )
	{
	  ProcessError(dest_prot, _dest_url);
	  return;
	}

      if( src_prot->Open(&su, KProtocol::READ) != KProtocol::SUCCESS )
	{
	  ProcessError(src_prot, _src_url);
	  return;
	}

      ipc->startedTransfer();

      long c = 0, l = 1;
      long size = src_prot->Size();
      warning( "Size :  %ld", size );
      float last = 0.0;
      char buffer[4096];

      while ( !src_prot->atEOF() )
	{
	  if ( ( l = src_prot->Read( buffer, 4096 ) ) < 0 )
	    {
	      fprintf( stderr, "read error (%ld)\n", l );
	      ProcessError(src_prot, _src_url);
	      return;
	    }
	  if (dest_prot->Write(buffer, l) < l )
	    {
	      ProcessError(dest_prot, _dest_url);
	      return;
	    }
	  c += l;
	  if ( size != 0 && ( ((float)c / (float)size * 100.0 ) - last) >= 1.0 )
	    {
	      last = (float)c / (float)size * 100.0;
	      ipc->progress( (int)last, size );
	    }
	}
      src_prot->Close();
      dest_prot->Close();

      delete src_prot;
      delete dest_prot;
	
      ipc->done();
      return;
    }
  else
    {
      fprintf( stderr, "ERROR: Not implemented\n");
      QString err = "Copy ";
      err += _src_url;
      err += " to ";
      err += _dest_url;
      ipc->fatalError( KIO_ERROR_NotImplemented, err.data(), 0 );
      return;
    }

  ipc->done();
}


void KIOSlave::resume( const char *_src_url, const char *_dest_url )
{
  KURL u( _src_url );
  if ( u.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_src_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _src_url, 0 );
      return;
    }

  KURL su( u.nestedURL() );
  if ( su.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_src_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _src_url, 0 );
      return;
    }

  KURL u2( _dest_url );
  if ( u2.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_dest_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _dest_url, 0 );
      return;
    }
  KURL du( u2.nestedURL() );
  if ( du.isMalformed() )
    {
      fprintf( stderr, "ERROR: Malformed URL '%s'\n",_dest_url );
      ipc->fatalError( KIO_ERROR_MalformedURL, _dest_url, 0 );
      return;
    }

  if( ProtocolSupported( _src_url ) && ProtocolSupported( _dest_url ) )
    {
      KProtocol *src_prot = CreateProtocol(_src_url);
      KProtocol *dest_prot = CreateProtocol(_dest_url);

      int destmode = KProtocol::APPEND;
	
      if( dest_prot->Open(&du, destmode) != KProtocol::SUCCESS )
	{
	  ProcessError(dest_prot, _dest_url);
	  return;
	}

      QFileInfo qinfo( du.path() );
      unsigned long int offset = qinfo.size();
      // what is the offset ?

      if( src_prot->Open(&su, KProtocol::RESUME, offset) != KProtocol::SUCCESS )
	{
	  ProcessError(src_prot, _src_url);
	  return;
	}

      ipc->startedTransfer();

      printf("KGETSLAVE : Resuming file %s at offset %ld\n", _dest_url, offset);

      long c = offset, l = 1;
      long size = src_prot->Size();
      warning( "Size :  %ld", size );
      float last = (float)c / (float)size * 100.0;
      char buffer[4096];

      while ( !src_prot->atEOF() )
	{
	  if ( ( l = src_prot->Read( buffer, 4096 ) ) < 0 )
	    {
	      fprintf( stderr, "read error (%ld)\n", l );
	      ProcessError(src_prot, _src_url);
	      return;
	    }
	  if (dest_prot->Write(buffer, l) < l )
	    {
	      ProcessError(dest_prot, _dest_url);
	      return;
	    }
	  c += l;
	  if ( size != 0 && (( (float)c / (float)size * 100.0 ) - last) >= 1.0 )
	    {
	      last = (float)c / (float)size * 100.0;
	      ipc->progress( (int)last, size );
	    }
	}
      src_prot->Close();
      dest_prot->Close();

      delete src_prot;
      delete dest_prot;
	
      ipc->done();
      return;
    }
  else
    {
      fprintf( stderr, "ERROR: Not implemented\n");
      QString err = "Copy ";
      err += _src_url;
      err += " to ";
      err += _dest_url;
      ipc->fatalError( KIO_ERROR_NotImplemented, err.data(), 0 );
      return;
    }

  ipc->done();
}


void KIOSlave::cleanUp()
{
  ipc->done();
}


void KIOSlave::terminate()
{
  if ( copyDestFile != 0L )
    {
      fclose( copyDestFile);
      unlink( copyDestFileName.data() );
    }
}


// Prevent us from zombies
void sig_handler2( int )
{
  int pid;
  int status;
    
  while( 1 )
    {
      pid = waitpid( -1, &status, WNOHANG );
      if ( pid <= 0 )
	{
	  // Reinstall signal handler, since Linux resets to default after
	  // the signal occured ( BSD handles it different, but it should do
	  // no harm ).
	  signal(SIGCHLD,sig_handler2);
	  return;
	}
    }
}

void sig_handler(int _sig)
{
  fprintf( stderr, "KIOSLAVE GOT TERM\n");
  if ( _sig == SIGTERM )
    if ( slave != 0L )
      slave->terminate();
  exit(0);
}
