/* $Id: manage.cpp,v 1.1.1.1 1998/05/06 19:13:37 matt Exp $
   $Log: manage.cpp,v $
   Revision 1.1.1.1  1998/05/06 19:13:37  matt
   Imported sources

   Revision 1.2  1998/03/02 14:36:28  matt
   kgetslave based on kioslave from kfm


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   */

#include "manage.h"
#include "subprotocol.h"
#include "http.h"
#include "file.h"
#include "ftp.h"
#include "ftp_proxy.h"
#include <ksimpleconfig.h>
#include <kapp.h>

extern int revmatch(const char *host, const char *nplist);

int ProtocolSupported(const char *url)
{
    // Iterate until we saw every subprotocol
    KURL u( url );
    do
    {
	// Get the right most subprotocol
	QString sub( u.nestedURL() );
	if ( !HaveProto( sub ) )
	    return FALSE;
	// Is there something left ?
	if ( u.hasSubProtocol() )
	{
	    QString parent( u.parentURL() );
	    u = parent.data();
	}
    } while ( u.hasSubProtocol() );
    
    return TRUE;
}

int HaveProto(const char *url)
{
	// Something on the local file system ?
	if( url[0] == '/' ) return 1;
	if( strncmp( url, "ftp:", 4 ) == 0) return 1;
	if( strncmp( url, "file:", 5 ) == 0) return 1;
	if( strncmp( url, "http:", 5 ) == 0) return 1;
	return(0);
}

const char *TopLevelURL(const char *url)
{
	const char *lasturl;
	if( ( lasturl = strrchr(url,'#') ) ) return( lasturl+1 );
	return( url );
}

KProtocol *CreateProtocol(const char *url)
{
    QString noPrxFor;
    QString prxStr;
    QString usePrxStr;
    bool do_proxy;
    
    KURL u( url );
    if ( u.isMalformed() )
	return NULL;

    do_proxy = false;
    KSimpleConfig prxcnf(KApplication::localconfigdir() + "/kgetrc", true);
    prxcnf.setGroup("Settings/Proxy");
    noPrxFor = prxcnf.readEntry("NoProxyFor");
    usePrxStr = prxcnf.readEntry("UseProxy");
    prxStr = prxcnf.readEntry("FTP-Proxy");
    if( usePrxStr=="Yes" && ! prxStr.isEmpty() ) { // Do we need proxy?
        do_proxy = true;
    }

    QString lasturl( u.nestedURL() );

    if( do_proxy && ! noPrxFor.isEmpty() ) {
        do_proxy = !revmatch( u.host(), noPrxFor.data() );
    }

    // A Hack. Does allow only one subprotocol.
    if( lasturl[0] == '/' ) return(new KProtocolFILE);
    if( strncmp( lasturl, "file:", 5 ) == 0 ) return( new KProtocolFILE );
    if( strncmp( lasturl, "http:", 5 ) == 0 ) return( new KProtocolHTTP );
    // if( strncmp( lasturl, "ftp:", 4 ) == 0 ) return( new KProtocolFTP );
    if( strncmp( lasturl, "ftp:", 4 ) == 0 )
    {
	if ( do_proxy )
	    return( new KProtocolProxyFTP );
	else
	    return( new KProtocolFTP );
    }     
    return NULL;
}

