/* $Id: kiojob.cpp,v 1.1.1.1 1998/05/06 19:13:31 matt Exp $
   $Log: kiojob.cpp,v $
   Revision 1.1.1.1  1998/05/06 19:13:31  matt
   Imported sources

   Revision 1.1  1998/03/02 21:59:50  matt
   Added resuming


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   */

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <qstrlist.h>
#include <qdict.h>

#include <kurl.h>
#include <kmsgbox.h>
#include <klocale.h>
#include <kstring.h>
#include <kapp.h>

#include "kioserver_ipc.h"
#include "kiojob.h"
#include "kgetslave/kio_errors.h"
#include "passworddialog.h"


QList<KIOJob> *KIOJob::jobList;
QDict<QString> *KIOJob::passwordDict;

KIOJob::KIOJob( int _id )
{
    bAutoDelete = TRUE;
    server = KIOServer::getKIOServer();
    slave = 0L;
    id = _id;
    overwriteExistingFiles = false;
    jobList->setAutoDelete( false );
    jobList->append( this );
}

KIOJob::~KIOJob()
{
  if ( slave != 0L )
  {
    printf("################# WARNING: KIOJOB destructor has still kioslave ################\n");
    disconnect( server, 0, this, 0 );
    disconnect( slave, 0, this, 0 );
    server->freeSlave( slave );
    slave = 0L;
  }

  server->removeJob( this );
  jobList->removeRef( this );
}

void KIOJob::setOverWriteExistingFiles( bool _o )
{
    overwriteExistingFiles = _o;
}


void KIOJob::resume( QStrList & _src_url_list, const char *_dest_dir_url )
{
  action = KIOJob::JOB_RESUME;
    
    // cmSrcURLList.copy( _src_url_list );
    tmpSrcURLList.clear();
    tmpDestURLList.clear();

    // Fill cmDestURLList
    char *p;
    for ( p = _src_url_list.first(); p != 0L; p = _src_url_list.next() )
    {
	QString tmp = p;
	// Is there a trailing '/' ? Delete is, so that KURL::filename works
	if ( tmp.right(1) == "/" && tmp.right(2) != ":/" )
	    tmp.truncate( tmp.length() - 1 );
	KURL su( tmp.data() );
	
	QString d = _dest_dir_url;
	d.detach();
	if ( d.right(1) != "/" )
	    d += "/";
	d += su.filename();

	printf("############# RESUME '%s' to '%s'\n", tmp.data(), d.data());
    
	tmpDestURLList.append( d.data() );
	tmpSrcURLList.append( tmp.data() );
    }

    resume();
}


void KIOJob::resume( const char *_src_url, const char *_dest_url )
{
  action = KIOJob::JOB_RESUME;
    
    tmpSrcURLList.clear();
    tmpDestURLList.clear();    

    QString tmp = _src_url;
    // Is there a trailing '/' ? Delete is, so that KURL::filename works
    if ( tmp.right(1) == "/" && tmp.right(2) != ":/" )
      tmp.truncate( tmp.length() - 1 );
    tmpSrcURLList.append( tmp );

    tmp = _dest_url;
    // Is there a trailing '/' ? Delete is, so that KURL::filename works
    if ( tmp.right(1) == "/" && tmp.right(2) != ":/" )
      tmp.truncate( tmp.length() - 1 );
    tmpDestURLList.append( tmp );

    printf("############# RESUME '%s' to '%s'\n", _src_url, _dest_url);
    
    resume();
}



void KIOJob::resume()
{
    cmSrcURLList.clear();
    cmDestURLList.clear();    

    // Does anybody try to copy a file on itself ?
    QListIterator<char> it3( tmpSrcURLList );
    QListIterator<char> it4( tmpDestURLList );
    for ( ; it3.current(); ++it3 )
    {
        char *p;
	char *p2;
	p = it3.current();
	p2 = it4.current();

	cmSrcURLList.append( p );
	cmDestURLList.append( p2 );
	//    ende:
	++it4;
    }

    QStrList tmpList1;
    QStrList tmpList2;
    
    // Recursive directory    
    QListIterator<char> it( cmSrcURLList );
    QListIterator<char> it2( cmDestURLList );
    for ( ; it.current(); ++it )
    {
        char *p;
	char *p2;
	p = it.current();
	p2 = it2.current();
	KURL su( p );
	KURL du( p2 );
	
	if ( su.isLocalFile() && du.isLocalFile() )
	{
	    struct stat buff;

	    QString supath( su.path() );  // source path
	    supath.detach();
	    KURL::decodeURL( supath );

	    QString dupath( du.path() );  // destination path
	    dupath.detach();
	    KURL::decodeURL( dupath );

	    stat( supath.data(), &buff );
	}
	++it2;
    }

    char *s;
    
    for ( s = tmpList1.first(); s != 0L; s = tmpList1.next() )
	cmSrcURLList.remove( s );
    for ( s = tmpList2.first(); s != 0L; s = tmpList2.next() )
	cmDestURLList.remove( s );
    
    cmCount = cmSrcURLList.count();

    server->getSlave( this );

}



void KIOJob::copy( QStrList & _src_url_list, const char *_dest_dir_url )
{

    action = KIOJob::JOB_COPY;
    
    // cmSrcURLList.copy( _src_url_list );
    tmpSrcURLList.clear();
    tmpDestURLList.clear();

    // Fill cmDestURLList
    char *p;
    for ( p = _src_url_list.first(); p != 0L; p = _src_url_list.next() )
    {
	QString tmp = p;
	// Is there a trailing '/' ? Delete is, so that KURL::filename works
	if ( tmp.right(1) == "/" && tmp.right(2) != ":/" )
	    tmp.truncate( tmp.length() - 1 );
	KURL su( tmp.data() );
	QString enc( su.filename() );
	KURL::encodeURL( enc );
	
	QString d = _dest_dir_url;
	d.detach();
	if ( d.right(1) != "/" )
	    d += "/";
	d += enc;

	printf("############# COPY '%s' to '%s'\n", tmp.data(), d.data());
    
	tmpDestURLList.append( d.data() );
	tmpSrcURLList.append( tmp.data() );
    }

    copy();
}

void KIOJob::copy( const char *_src_url, const char *_dest_url )
{
    action = KIOJob::JOB_COPY;

    tmpSrcURLList.clear();
    tmpDestURLList.clear();    

    QString tmp = _src_url;
    // Is there a trailing '/' ? Delete is, so that KURL::filename works
    if ( tmp.right(1) == "/" && tmp.right(2) != ":/" )
      tmp.truncate( tmp.length() - 1 );
    tmpSrcURLList.append( tmp );

    tmp = _dest_url;
    // Is there a trailing '/' ? Delete is, so that KURL::filename works
    if ( tmp.right(1) == "/" && tmp.right(2) != ":/" )
      tmp.truncate( tmp.length() - 1 );
    tmpDestURLList.append( tmp );

    printf("############# COPY '%s' to '%s'\n", _src_url, _dest_url);
    
    copy();
}

void KIOJob::copy()
{
    cmSrcURLList.clear();
    cmDestURLList.clear();    

    // Does anybody try to copy a file on itself ?
    QListIterator<char> it3( tmpSrcURLList );
    QListIterator<char> it4( tmpDestURLList );
    for ( ; it3.current(); ++it3 )
    {
        char *p;
	char *p2;
	p = it3.current();
	p2 = it4.current();
	
	cmSrcURLList.append( p );
	cmDestURLList.append( p2 );
	++it4;
    }

    QStrList tmpList1;
    QStrList tmpList2;
    
    // Recursive directory    
    QListIterator<char> it( cmSrcURLList );
    QListIterator<char> it2( cmDestURLList );
    for ( ; it.current(); ++it )
    {
        char *p;
	char *p2;
	p = it.current();
	p2 = it2.current();
	KURL su( p );
	KURL du( p2 );
	
	if ( su.isLocalFile() && du.isLocalFile() )
	{
	    struct stat buff;

	    QString supath( su.path() );  // source path
	    supath.detach();

	    QString dupath( du.path() );  // destination path
	    dupath.detach();

	    stat( supath.data(), &buff );
	}
	++it2;
    }

    char *s;
    
    for ( s = tmpList1.first(); s != 0L; s = tmpList1.next() )
	cmSrcURLList.remove( s );
    for ( s = tmpList2.first(); s != 0L; s = tmpList2.next() )
	cmDestURLList.remove( s );
    
    cmCount = cmSrcURLList.count();

    server->getSlave( this );
}


void KIOJob::doIt( KIOSlaveIPC * _slave )
{
    slave = _slave;

    connect( slave, SIGNAL( fatalError( int, const char*, int ) ),
	     this, SLOT( fatalError( int, const char*, int ) ) );
    connect( slave, SIGNAL( setPID( int ) ), this, SLOT( start( int ) ) );
    connect( slave, SIGNAL( closed( KIOSlaveIPC* ) ), this, SLOT( slotSlaveClosed( KIOSlaveIPC* ) ) );

    slave->getPID();
}


void KIOJob::fatalError( int _kioerror, const char* _error, int _errno )
{
  // Assume we want to process this error
  bProcessError = true;
  
  // Allow others to see and process the error
  emit errorFilter( _kioerror, _error, _errno );
  
  // Is the error already process ?
  if ( bProcessError )
    // We process the error.
    processError( _kioerror, _error, _errno );
}


void KIOJob::msgErr( bool confirm, QString s ) {

  warning("msgErr()");
  emit error( confirm, s, id );

}


void KIOJob::processError( int _kioerror, const char* _error, int )
{
    bProcessError = false;
  
    kioError = _kioerror;
    
    // TODO: Remove before final release
    printf("################################# fatalError called '%s'\n",_error);
    
    // We have to delete the password!!! ( if there is one )
    QString url( _error );
    KURL u( _error );
    if ( !u.isMalformed() )
    {
      u.setPassword( "" );
      url = u.url().data();
    }
    
    
    QString msg;

	switch( _kioerror )
	{
	case KIO_ERROR_MalformedURL:
	    ksprintf(&msg, i18n("Malformed URL\n%s"), url.data()); 
	    msgErr( false, msg );
	    break;
	case KIO_ERROR_CouldNotRead:
	    ksprintf(&msg, i18n("Could not read\n%s\nFile does not exist or access denied"), url.data());
	    msgErr( true, msg );
	    break;
	case KIO_ERROR_CouldNotWrite:
	    ksprintf(&msg, i18n("Could not write\n%s\nPerhaps access denied"),
				url.data());
	    msgErr( true, msg );
	    break;
	case KIO_ERROR_CouldNotCreateSocket:
	    ksprintf(&msg, i18n("Could not create Socket for\n%s"), url.data());
	    msgErr( true, msg );
	    break;
	case KIO_ERROR_UnknownHost:
	    ksprintf(&msg, i18n("Unknown host in\n%s"), url.data());
	    msgErr( true, msg );
	    break;
	case KIO_ERROR_CouldNotConnect:
	    ksprintf(&msg, i18n("Could not connect to\n%s"), url.data());
	    msgErr( true, msg );
	    break;
	case KIO_ERROR_CouldNotReadSize:
	    ksprintf(&msg, i18n("Could not read size of \n%s"), url.data());
	    msgErr( true, msg );
	    break;
	case KIO_ERROR_NotImplemented:
	    ksprintf(&msg, i18n("The requested action\n%s\nis not implemented yet."), url.data());
	    msgErr( false, msg );
	    break;
	case KIO_ERROR_CouldNotLogin:
	    {
		KURL u( _error );
		ksprintf(&msg, i18n("Could not login for\n%s\nPerhaps wrong password"), u.host());
		// Remove the password from the dict, since it seems to be wrong.
		if ( u.user() != 0L && u.user()[0] != 0 && u.passwd() != 0L && u.passwd()[0] != 0 )
		{
		    QString tmp;
		    ksprintf(&tmp, "%s@%s", u.user(), u.host());
		    passwordDict->remove( tmp.data() );
		}
	    }
	    msgErr( true, msg );
	    break;
	case KIO_ERROR_FileDoesNotExist:
	    ksprintf(&msg,i18n("File %s\ndoes not exist"), url.data());
	    msgErr( true, msg );
	    break;
	}
    
}


void KIOJob::start( int _pid )
{
    slave->pid = _pid;

    connect( slave, SIGNAL( done() ), this, SLOT( slaveIsReady() ) );
    connect( slave, SIGNAL( data( IPCMemory ) ), this, SLOT( slotData( IPCMemory ) ) );
    connect( slave, SIGNAL( info( const char* ) ), this, SLOT( slotInfo( const char* ) ) );
    connect( slave, SIGNAL( redirection( const char* ) ), this, SLOT( slotRedirection( const char* ) ) );
    connect( slave, SIGNAL( started() ), this, SLOT( slaveStarted() ) );
    
    started = false;
    cleanedUp = false;
    
    slaveIsReady();
}


void KIOJob::slaveStarted()
{
  emit jobstarted( id );
}


void KIOJob::slaveIsReady()
{
    warning("SlaveIsReady\n");

    if ( cleanedUp )
    {
	done();
	return;
    }
    
    switch( action )
    {
    case KIOJob::JOB_COPY:
	{
	    if ( !started )
		connect( slave, SIGNAL( progress( int, int ) ), this, SLOT( slaveProgress( int, int ) ) );
	    
	    if ( cmSrcURLList.count() == 0 )
	    {
		slave->cleanUp();
		cleanedUp = true;
		return;
	    }

	    QString src = completeURL( cmSrcURLList.first() ).data();	    
	    QString dest = completeURL( cmDestURLList.first() ).data();
	    slave->copy( src.data(), dest.data(), overwriteExistingFiles );
	    cmSrcURLList.removeRef( cmSrcURLList.first() );
	    cmDestURLList.removeRef( cmDestURLList.first() );
	}
	break;
    case KIOJob::JOB_RESUME:
	{
	    if ( !started )
		connect( slave, SIGNAL( progress( int, int ) ), this, SLOT( slaveProgress( int, int ) ) );
	    
	    if ( cmSrcURLList.count() == 0 )
	    {
		slave->cleanUp();
		cleanedUp = true;
		return;
	    }

	    QString src = completeURL( cmSrcURLList.first() ).data();	    
	    QString dest = completeURL( cmDestURLList.first() ).data();
	    slave->resume( src.data(), dest.data() );
	    cmSrcURLList.removeRef( cmSrcURLList.first() );
	    cmDestURLList.removeRef( cmDestURLList.first() );
	}
	break;
    default: // Stephan: added default hander
       warning("case not handled here");
    }
    started = true;
}



void KIOJob::slaveProgress( int _percent, int _size )
{
    emit progress( _percent, _size, id );
}

void KIOJob::slotData( IPCMemory _mem )
{
    emit data( _mem.data, _mem.size );
}

void KIOJob::slotRedirection( const char *_url )
{
    printf("KIOJOB::redirection\n");
    
    emit redirection( _url );
}


void KIOJob::slotInfo( const char *_text )
{
    emit info( _text );
}

void KIOJob::cancel( bool try_later )
{
    server->removeJob( this );

    printf("****CANCEL******A\n");
    if ( slave )
    {
	printf("Killing slave\n");
	KIOSlaveIPC *s = slave;
	slave = 0L;
	pid_t p = (pid_t)s->pid;    
	delete s;
	kill( p, SIGTERM );
    }
    
    slave = 0L;
    
    done( try_later );
}

void KIOJob::stop()
{
    printf("****STOP******A\n");
    if ( slave )
    {
	printf("Killing slave\n");
	KIOSlaveIPC *s = slave;
	slave = 0L;
	pid_t p = (pid_t)s->pid;    
	delete s;
	kill( p, SIGTERM );
    }
    
    slave = 0L;
    
    printf("Stopped\n");
    
    if ( bAutoDelete )
	delete this;
}


void KIOJob::done( bool try_later )
{
    printf("Done\n");
    
    server->removeJob( this );
    
    if ( slave != 0L )
    {
	printf("Handing slave back\n");
	disconnect( server, 0, this, 0 );
	disconnect( slave, 0, this, 0 );
	server->freeSlave( slave );
    }
    

    if ( try_later )
      emit trylater( id );
    else
      emit finished( id );
    
    slave = 0L;
    
    if ( bAutoDelete )
	delete this;
}

void KIOJob::deleteAllJobs()
{
    KIOJob *j;
    for ( j = jobList->first(); j != 0L; j = jobList->next() )
	j->cancel();
}

QString KIOJob::completeURL( const char *_url )
{
    KURL u( _url );
    if ( u.isMalformed() )
	return QString( _url );
    
    // We got a user, but no password ?
    if ( u.user() != 0L && u.user()[0] != 0 && ( u.passwd() == 0L || u.passwd()[0] == 0 ) )
    {
	QString head;
	ksprintf(&head, "Password for %s@%s", u.user(), u.host());
	
	QString passwd;
	QString tmp2;
	ksprintf(&tmp2, "%s@%s", u.user(), u.host());
	
	if ( (*passwordDict)[ tmp2.data() ] == 0L )
	{
	    PasswordDialog *dlg = new PasswordDialog( head.data(), 0L, "", true );
	    if ( !dlg->exec() )
	    {
		return QString( _url );
	    }
	    passwd = dlg->password();
	    delete dlg;

	    // If the password is wrong, the error function will remove it from
	    // the dict again.
	    passwordDict->insert( tmp2.data(), new QString( passwd.data() ) );
	}
	else
	    passwd = (*passwordDict)[ tmp2.data() ]->data();
	
	QString tmp;
	tmp << u.user() << ":" << passwd.data();
	QString url = u.url();
	int i = url.find( "@" );
	int j = url.find( "://" );
	url.replace( j + 3, i - ( j + 3 ), tmp.data() );

	return QString( url );
    }
    // We have a user and a password => we want to remember that.
    else if ( u.user() != 0L && u.user()[0] != 0 && u.passwd() != 0L && u.passwd()[0] != 0 )
    {
	QString tmp2;
	ksprintf(&tmp2, "%s@%s", u.user(), u.host());

	// We dont know this passwort ?
	if ( (*passwordDict)[ tmp2.data() ] == 0L )
	    passwordDict->insert( tmp2.data(), new QString( u.passwd() ) );
    }
    
    return QString( _url );
}

void KIOJob::slotSlaveClosed( KIOSlaveIPC* )
{
    // We assumed that the slave is going to die.
    if ( slave == 0L )
	return;
    
    slave = 0L;
    
    emit error( KIO_ERROR_SlaveDied, i18n( "Segmentation fault in io subprocess" ), id );

    cancel();
}

