/* $Id: kioserver.cpp,v 1.1.1.1 1998/05/06 19:13:31 matt Exp $
   $Log: kioserver.cpp,v $
   Revision 1.1.1.1  1998/05/06 19:13:31  matt
   Imported sources

   Revision 1.1  1998/03/22 07:49:19  matt
   *** empty log message ***


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   */

#include <qdir.h>
#include <qstrlist.h>
#include <qdict.h>

#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/wait.h>

#ifdef __FreeBSD__
#include <sys/param.h>
#include <sys/ucred.h>
#endif      

#include <kapp.h>
#include <kconfig.h>

#include "kioserver_ipc.h"
#include "kioserver.h"
#include "utils.h"

KIOServer* KIOServer::pKIOServer;


KIOServer::KIOServer() : KIOSlaveIPCServer()
{
    pKIOServer = this;
    
    freeSlaves.setAutoDelete( false );
    waitingJobs.setAutoDelete( false );

    connect( &m_timer, SIGNAL( timeout() ), this, SLOT( slotTimer() ) );
    m_timer.start( 10000 );
    
    connect( this, SIGNAL( newClient( KIOSlaveIPC* ) ), this, SLOT( newSlave( KIOSlaveIPC* ) ) );
}

KIOServer::~KIOServer()
{
    KIOSlaveIPC *s;
    for ( s = freeSlaves.first(); s != 0L; s = freeSlaves.next() )
    {
	pid_t p = (pid_t)s->pid;
    
	delete s;
        if (p<=0) printf("kgetslave pid is =0, I don't want to hurt myself\n");
        else{
	  kill( p, SIGTERM );
	}  
    }
}


/* -------------------------------------------------------------------
 * Static Functions
 * -----------------------------------------------------------------*/


bool KIOServer::supports( const char *_url, int _mode )
{
    KURL u2( _url );
    if ( u2.isMalformed() )
	return false;
    // We are interested in the right most protocol.
    // HACK
    // We dont check wether the parent protocols support the needed
    // read/write actions.
    KURL u( u2.nestedURL() );
    if ( u.isMalformed() )
	return false;
    
    if ( strcmp( u.protocol(), "http" ) == 0 )
    {
	if ( _mode == KIO_Read )
	    return true;
	return false;
    }
    else if ( strcmp( u.protocol(), "ftp" ) == 0 )
    {
        if ( ( ( KIO_Read | KIO_Write ) & _mode ) == _mode )
	    return true;
	return false;
    }
    else if ( strcmp( u.protocol(), "file" ) == 0 )
    {
        if ( ( ( KIO_Read | KIO_Write ) & _mode ) == _mode )
	    return true;
	return false;
    }

    return false;
}


bool KIOServer::supports( QStrList & _urls, int _mode )
{
    char *s;
    for ( s = _urls.first(); s != 0L; s = _urls.next() )
	if ( !supports( s, _mode ) )
	    return false;
    
    return true;
}



int KIOServer::isDir( QStrList & _urls )
{
    char *s;
    for ( s = _urls.first(); s != 0L; s = _urls.next() )
    {
	int i = isDir( s );
	if ( i <= 0 )
	    return i;
    }
    
    return 1;
}

int KIOServer::isDir( const char *_url )
{
    KURL u( _url );
    if ( u.isMalformed() )
	return false;

    int i = strlen( _url );

    // This is always a directory
    if ( i >= 1 && _url[ i - 1 ] == '/' )
	return 1;
    // With HTTP we can be shure that everything that does not end with '/'
    // is NOT a directory
    else if ( strcmp( u.protocol(), "http" ) == 0 )
	return 0;
    // Local filesystem without subprotocol
    else if ( u.isLocalFile() )
	     // strcmp( u.protocol(), "file" ) == 0 && !u.hasSubProtocol() )
    { 
	struct stat buff;
	if ( stat( u.path(), &buff ) != 0 )
	{
	  warning("FEHLER\n");
	  return 0;
	}
	
	if ( S_ISDIR( buff.st_mode ) )
	    return 1;
	else
	    return 0;
    }
    else
	// We are not shure
	return -1;
}

/* -------------------------------------------------------------------
 * The multi tasking kernel
 * -----------------------------------------------------------------*/

void KIOServer::newSlave( KIOSlaveIPC * _slave )
{
    connect( _slave, SIGNAL( closed( KIOSlaveIPC* ) ), this, SLOT( slotSlaveClosed( KIOSlaveIPC* ) ) );
    
    newSlave2( _slave );
}

void KIOServer::newSlave2( KIOSlaveIPC * _slave )
{
    if ( waitingJobs.count() == 0 )
    {
	freeSlaves.append( _slave );
	_slave->m_time = time( 0L );
	return;
    }
    
    KIOJob* job = waitingJobs.first();
    waitingJobs.removeRef( job );
    job->doIt( _slave );
}

void KIOServer::getSlave( KIOJob *_job )
{
    if ( freeSlaves.count() == 0 )
    {
	waitingJobs.append( _job );
	runNewSlave();
	return;
    }
    
    KIOSlaveIPC *slave = freeSlaves.first();
    freeSlaves.removeRef( slave );
    _job->doIt( slave );
}

void KIOServer::runNewSlave()
{
    QString ipath = kapp->kde_bindir() + "/kgetslave";

    // Keep in sync with the same in kioserver_ipc.cpp!
    QString idir;
    idir.sprintf("/tmp/kio_%i_%i%s",(int)getuid(), (int)getpid(),displayName().data());
    
    if ( fork() == 0 )
    {
        execlp( ipath.data(), "kgetslave", idir.data(), 0 );
	fatal( "ERROR: Could not start kgetslave\n");
        exit( 1 );
    }
}

void KIOServer::freeSlave( KIOSlaveIPC *_slave )
{
    newSlave2( _slave );
}

void KIOServer::slotSlaveClosed( KIOSlaveIPC* _slave )
{
    freeSlaves.removeRef( _slave );
}


QString KIOServer::canonicalURL( const char *_url )
{
    KURL u( _url );
    if ( u.isMalformed() || strcmp( u.protocol(), "file" ) != 0L || u.hasSubProtocol() )
	return QString( _url );
    
    QDir dir( u.path() );
    QString url = dir.canonicalPath();
    if ( url.isEmpty() )
	return QString( _url );
    
    return u.path();
}

void KIOServer::slotTimer()
{
  /*  if ( KIOJob::jobList->count() > 0 )
    printf("====================== ATTENTION: KIOJob memory lack %i ===============\n", KIOJob::jobList->count()); */
    
  QList<KIOSlaveIPC> list = freeSlaves;
 
  time_t t = time( 0L );
  
  KIOSlaveIPC *p;
  for ( p = list.first(); p != 0L; p = list.next() )
  {
    if ( t - p->m_time > 10 )
    {
      pid_t pid = (pid_t)p->pid;    
      freeSlaves.removeRef( p );
      delete p;
      if ( pid <= 0 )
	printf("kioslave pid is =0, I don't want to hurt myself\n");
      else
      {
        kill( pid, SIGTERM );
        int status;
        waitpid( pid, &status, 0 );
      }	
    }
  }
}
