/* $Id: kioserver.h,v 1.1.1.1 1998/05/06 19:13:31 matt Exp $
   $Log: kioserver.h,v $
   Revision 1.1.1.1  1998/05/06 19:13:31  matt
   Imported sources

   Revision 1.1  1998/03/22 07:49:21  matt
   *** empty log message ***


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// KIOSlave
// Server side implementation

#ifndef kioserver_h
#define kioserver_h

#include <qobject.h>
#include <qlist.h>
#include <qdict.h>
#include <qstring.h>
#include <qtimer.h>

#include <kurl.h>

#include "kioserver_ipc.h"

//@Man: KIO Modes
//@{
/** Modes used for the static 'supports' functions of the KIOs */
/// Read from the KIO
#define KIO_Read 1
/// Write to the KIO
#define KIO_Write 2
//@}

#include "kiojob.h"

class KIOServer : public KIOSlaveIPCServer
{
    Q_OBJECT
public:
    KIOServer();
    ~KIOServer();
    
    /**
     * If a job needs a slave, it calls this function. If a 
     * slave is available ( after this call has returned ), the
     * job will be told so.
     */
    void getSlave( KIOJob * _job );
    /// A Job does not need a slave any more.
    void freeSlave( KIOSlaveIPC * _slave );

    void removeJob( KIOJob* _job ) { waitingJobs.removeRef( _job ); }
  
    /// Return a pointer to the running KIOServer
    static KIOServer* getKIOServer() { return pKIOServer; }

    /// Tests wether the list of URLs are all together directories
    /**
      If the protocol of the URL is not "file", then this function
      depends an the convention, that directories have a trailing '/'
      in their URL.
      
      @return 1 if the URL is a directory, 0 if not and <0 if we are unshure.
      */
    static int isDir( QStrList & _urls );
    /// Tests wether _url is a directory or not
    /**
      If the protocol of the URL is not "file", then this function
      depends an the convention, that directories have a trailing '/'
      in their URL.

      @return 1 if the URL is a directory, 0 if not and <0 if we are unshure.
      */
    static int isDir( const char *_url );
    /// Tells wether KIOServer supports an operation on an URL
    /**
      _mode is for example KIO_Read or any of those constants.
      */
    static bool supports( const char *_src_url, int _mode );
    /// Tells wether KIOServer supports an operation on some URLs
    /**
      An operation that is not supported by at least one of the URLs
      will cause this function to return FALSE.
      */
    static bool supports( QStrList & _urls, int _mode );


    /**
     * Does nothing on protocols other then "file" or if there is a sub protocol
     * in the URL. The canonical URL does not contain any symlinks.
     */
    static QString canonicalURL( const char *_url );
    
public slots:
    /**
     * If a new slave is created, this slot is called.
     * This function connects 'slotDirEntry' and 'slotFlushDir' to the new slave
     * and calls 'newSlave2' to do the rest.
     */
    void newSlave( KIOSlaveIPC * );
    void slotSlaveClosed( KIOSlaveIPC* );
    
    /**
     * This function cleansup old kioslaves.
     */
    void slotTimer();
  
signals:
    
protected:
    /// Handles the arrival of a new or freed slave
    void newSlave2( KIOSlaveIPC * );
    
    /// Start a new slave process
    void runNewSlave();
    
    /// List of all slaves without work
    QList<KIOSlaveIPC> freeSlaves;
    /// List of all jobs waiting for a slave
    QList<KIOJob> waitingJobs;

    /**
     * This timer calls @ref #slotTimer.
     */
    QTimer m_timer;
  
    /// A pointer to the running server
    /**
      Sometimes needed by static functions.
      */
    static KIOServer *pKIOServer;
};

#endif
