/* $Id: kioserver_ipc.cpp,v 1.1.1.1 1998/05/06 19:13:31 matt Exp $
   $Log: kioserver_ipc.cpp,v $
   Revision 1.1.1.1  1998/05/06 19:13:31  matt
   Imported sources

   Revision 1.1  1998/03/22 07:49:24  matt
   *** empty log message ***


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// This file has been created by ipcc.pl.
// (c) Torben Weis
//     weis@stud.uni-frankfurt.de

#include <errno.h>
#include <unistd.h>

#include "kioserver_ipc.h"
#include "utils.h"

KIOSlaveIPCServer::KIOSlaveIPCServer()
{
    // Keep in sync with the same in kioserver.cpp!
    QString idir;
    idir.sprintf("/tmp/kio_%i_%i%s",(int)getuid(), (int)getpid(),displayName().data());
    
    serv_sock = new KServerSocket( idir.data() );
    if ( serv_sock->socket() < 0 )
    {
	fprintf( stderr, "ERROR: Could not establish server socket!\n");
	exit(1);
    }
    connect( serv_sock, SIGNAL( accepted(KSocket*) ), 
	    this, SLOT( slotAccept(KSocket*) ) );
}

void KIOSlaveIPCServer::slotAccept( KSocket *_sock )
{
    KIOSlaveIPC * p = new KIOSlaveIPC( _sock );
    emit newClient( p );
}

KIOSlaveIPCServer::~KIOSlaveIPCServer()
{
    delete serv_sock;
}

KIOSlaveIPC::KIOSlaveIPC( KSocket *_sock )
{
    bHeader = TRUE;
    cHeader = 0;
    pBody = 0L;

    connect( _sock, SIGNAL( readEvent(KSocket*) ), this, SLOT( readEvent(KSocket*) ) );
    connect( _sock, SIGNAL( closeEvent(KSocket*) ), this, SLOT( closeEvent(KSocket*) ) );
    _sock->enableRead( TRUE );
    data_sock = _sock;
}

KIOSlaveIPC::~KIOSlaveIPC()
{
    /**
     * MODIFIED
     */
    emit closed( this );
   
    // data_sock->enableRead( FALSE );
    delete data_sock;
    if ( pBody != 0L )
	free( pBody );
}

void KIOSlaveIPC::closeEvent( KSocket * )
{
    printf("CLOSE EVENT\n");
    delete this;
    return;
}

void KIOSlaveIPC::readEvent( KSocket * )
{
    if ( bHeader )
    {
	int n;
    next:
	if ( ( n = read( data_sock->socket(), headerBuffer + cHeader, 1 ) ) < 0 )
        {
	  if ( errno == EINTR )
	    goto next;
	  fprintf( stderr, "ERROR: KIOSlaveIPC::readEvent\n");
	  delete this;
	  return;
        }
	
	if ( headerBuffer[ cHeader ] == ' ' )
	{
	    bHeader = FALSE;
	    cHeader = 0;
	    bodyLen = atoi( headerBuffer );
	    cBody = 0;
	    if ( bodyLen <= 0 )
	    {
		fprintf( stderr, "ERROR: Invalid header\n");
		delete this;
		return;
	    }
	    if ( pBody != 0L )
		free( pBody );
	    pBody = (char*)malloc( bodyLen + 1 );
	}
	else if ( cHeader + n == 10 )
	  {
	    fprintf( stderr, "ERROR: Too long header\n");
	    delete this;
	    return;
	}
	else
	{
	    if ( !isdigit( headerBuffer[ cHeader ] ) )
	    {
		fprintf( stderr, "ERROR: Header must be an int\n");
		delete this;
		return;
	    }

	    cHeader += n;
	    return;
	}
    }
	
    int n;
next2:
    if ( ( n = read( data_sock->socket(), pBody + cBody, bodyLen - cBody ) ) < 0 )
    {
      if ( errno == EINTR )
	goto next2;
      fprintf( stderr, "ERROR: KIOSlaveIPC::readEvent\n");
      delete this;
      return;
    }

    if ( n + cBody == bodyLen )
    {
	pBody[bodyLen] = 0;
	bHeader = TRUE;
	parse( pBody, bodyLen );
	return;
    }
    cBody += n;
}

void KIOSlaveIPC::parse( char *_data, int _len )
{
    int pos = 0;
    char *name = read_string( _data, pos, _len );
    if ( name == 0L )
	return;
    _data += pos;
    _len -= pos;
	if ( strcmp( name, "hello" ) == 0 ) { parse_hello( _data, _len ); } else
	if ( strcmp( name, "progress" ) == 0 ) { parse_progress( _data, _len ); } else
	if ( strcmp( name, "started" ) == 0 ) { parse_started( _data, _len ); } else
	if ( strcmp( name, "info" ) == 0 ) { parse_info( _data, _len ); } else
	if ( strcmp( name, "data" ) == 0 ) { parse_data( _data, _len ); } else
	if ( strcmp( name, "done" ) == 0 ) { parse_done( _data, _len ); } else
	if ( strcmp( name, "fatalError" ) == 0 ) { parse_fatalError( _data, _len ); } else
	if ( strcmp( name, "setPID" ) == 0 ) { parse_setPID( _data, _len ); } else
	if ( strcmp( name, "redirection" ) == 0 ) { parse_redirection( _data, _len ); } else
		return;
	free_string( name );
}

