/* $Id$
   $Log$

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include <ktablistbox.h>
#include <klocale.h>
#include <kapp.h>
#include <kurl.h>
#include <string.h>
#include "sounddlg.h"


SoundDlg::SoundDlg(QWidget *parent, const char *name, WFlags f)
  : QWidget(parent, name, f)
{

  box = new QGroupBox(this, "box");
  box->setGeometry(10,10,365,280);
  box->setTitle(i18n("Sound Options"));

  cb_useSound = new QCheckBox(i18n("Use Sound"), this );
  cb_useSound->setGeometry( 20, 40, 120, 30 );
  connect( cb_useSound, SIGNAL( clicked() ), SLOT( changeSound() ) );
  
  lb_transfer = new QLabel(i18n("Finished transfer:"), this);
  lb_transfer->setAlignment(AlignLeft | AlignVCenter);
  lb_transfer->setGeometry(40, 90, 260, 30);
  le_transfer = new QLineEdit(this);
  le_transfer->setGeometry(150, 90, 200, 30);

  lb_all = new QLabel(i18n("Finished all:"), this);
  lb_all->setAlignment(AlignLeft | AlignVCenter);
  lb_all->setGeometry(40, 150, 260, 30);
  le_all = new QLineEdit(this);
  le_all->setGeometry(150, 150, 200, 30);

}


SoundDlg::~SoundDlg()
{
  // now delete everything we allocated before
  delete box;
  delete lb_transfer;
  delete le_transfer;

  delete lb_all;
  delete le_all;

  delete cb_useSound;
  
  // time to say goodbye ...
}


void SoundDlg::setSound()
{
  // now set all input fields
  le_transfer->setEnabled( useSound );
  le_all->setEnabled( useSound );
  cb_useSound->setChecked( useSound );
}

void SoundDlg::changeSound()
{
  useSound = (! useSound);
  setSound();
}


void SoundDlg::setData(QStrList *strList)
{
  QString tmp;

  tmp = strList->first();
  if( tmp == "Yes" ) {
    useSound = TRUE;
  } else {
    useSound = FALSE;
  }
  setSound();
  
  tmp = strList->next();
  if ( ! tmp.isEmpty() ) {
    printf("Filename for finished transfer found in kgetrc!\n");
    le_transfer->setText( tmp.data() );
  }
  
  tmp = strList->next();
  if ( ! tmp.isEmpty() ) {
    printf("Filename for finished all transfers found in kgetrc!\n");
    le_all->setText( tmp.data() );
  }

}

static QStrList strList(true);

QStrList SoundDlg::data() const
{
  strList.clear();
  QString tmp;
  
  if( useSound == TRUE ) {
    strList.append( "Yes" );
  } else {
    strList.append( "No" );
  }
  
  tmp = le_transfer->text();
  if( tmp.isEmpty() ) {
    printf("No filename for finished transfer selected!\n");
    strList.append("");
  } else {
    strList.append( tmp.data() );
  }

  tmp = le_all->text();
  if( tmp.isEmpty() ) {
    printf("No filename for finished all transfers selected!\n");
    strList.append("");
  } else {
    strList.append( tmp.data() );
  }

  return strList;
}

