/*
  this class manages all instances of the players
  Copyright (C) 1999  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */


#include <kmpgServer.h>


static int instance=0;

KmpgServer::KmpgServer() {
  mixerDevice=new MixerDevice();
  audioDevice=new AudioDevice("/dev/dsp");

  mixerDevice->addListener(audioDevice);
  if (audioDevice->open() == false) {
    KMsgBox::message(NULL,"kmpg AudioDevice","Cannot open /dev/dsp.\n" \
		     "kmpg exists!");
    exit(0);
  }
  GarbageCollector::init();
}


KmpgServer::~KmpgServer() {
  audioDevice->close();
  delete mixerDevice;
  delete audioDevice;
}


void KmpgServer::newPlayer() {
  KmpgMainWindow* player=new KmpgMainWindow();
  player->addListener(mixerDevice);
  player->setGeometry(100+30*instance,100+30*instance,270,125);
  player->show();
  connect(player,SIGNAL(closeEvent(KmpgMainWindow*)),
	  this,SLOT(closePlayer(KmpgMainWindow*)));

  connect(player,SIGNAL(newPlayerEvent()),
	  this,SLOT(newPlayer()));

  connect(player,SIGNAL(quitEvent()),
	  this,SLOT(quit()));

  instance++;
}


void KmpgServer::quit() {
  kapp->quit();
}


void KmpgServer::closePlayer(KmpgMainWindow* player) {
  GenericPlayer* genericPlayer=player->getGenericPlayer();
  
  // this wait until the thread is stopped
  genericPlayer->sendSyncMessage(_NOT_ALLOW_GENERATOR_ENTER);
  player->removeListener(mixerDevice);
 
  player->hide();
  delete player;
  instance--;
  if (instance == 0) {
    quit();
  }

}


void KmpgServer::processCommandLine(CommandLineCreator* cmdLine) {
  KmpgCommandLine::parseIPC(cmdLine);
}

