/*
  gui part for the balance device (very simple)
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */




#include <guiBalanceDevice.h>




GuiBalanceDevice:: GuiBalanceDevice(QWidget *parent=0, const char *name=0 ) :
  GuiDevice(parent,name) {
 balance=0;
 volumeDevice=NULL;
 menuDescription=new MenuDescription("BalanceDevice");
 menuDescription->insertMenu("Center",this,SLOT(centerBalance()));

}


GuiBalanceDevice::~GuiBalanceDevice() {

}



void GuiBalanceDevice::buildGui() {
 slider=new KSlider(-100,100,1,0,KSlider::Horizontal,this);
 slider->setTickmarks( QSlider::NoMarks );
 slider->setGeometry(0,0,90,28);
 slider->setBackgroundColor(black);
 slider->installEventFilter(this);
 connect( slider, SIGNAL(valueChanged(int)),this,SLOT(setBalance(int)) );
}


bool GuiBalanceDevice::eventFilter( QObject *, QEvent *e ) {
  if ( e->type() == Event_MouseButtonPress ) {    // key press
    QMouseEvent* mouseEvent = (QMouseEvent*)e;
    if (mouseEvent->button() ==  RightButton) {
      mousePressEvent(mouseEvent);
      return TRUE;                        // eat event
    }
  }
  return FALSE;    
}


void GuiBalanceDevice::attachTo(VolumeDevice* volumeDevice) {
  this->volumeDevice=volumeDevice;
  GuiDevice::attachTo(volumeDevice);
}


void GuiBalanceDevice::detach() {
  volumeDevice=NULL;
  GuiDevice::detach();
}



int GuiBalanceDevice::getBalance() {
  return balance;
}


void GuiBalanceDevice::setBalance(int value) {
  balance=value;
  if (volumeDevice != NULL) {
    volumeDevice->setBalance(value);
  }
}


void GuiBalanceDevice::centerBalance() {
  setBalance(0);
  slider->setValue(0);
}


void GuiBalanceDevice::mousePressEvent ( QMouseEvent* mouseEvent) {
  if (mouseEvent->button() ==  RightButton) {
    KPopupMenu* popupMenu=menuDescription->createPopupMenu();
    popupMenu->move(mapToGlobal(QPoint(mouseEvent->x(),mouseEvent->y())));
    popupMenu->exec();
    delete popupMenu;
    return;
  }
}




QSize GuiBalanceDevice::sizeHint () {
  return QSize(90,28);
}






