/*
  gui part for the info device (very simple)
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */




#include <guiInfoDevice.h>




GuiInfoDevice::GuiInfoDevice(QWidget *parent=0, 
			       const char *name=0):GuiDevice( parent, name ) {

  infoString=new Buffer(10);
  streamInfoDevice=NULL;

  setMinimumSize( sizeHint() );

  infoDeviceConfig=new InfoDeviceConfig("InfoDeviceConfig",getPreferences());
  getPreferences()->addTab(infoDeviceConfig,infoDeviceConfig->getGroupName());
  insertMenu("Configure",this,SLOT(configure()));
  connect(infoDeviceConfig,SIGNAL(applyEvent()), this, SLOT(apply()));
}


GuiInfoDevice::~GuiInfoDevice() {
  delete infoString;
}


void GuiInfoDevice::attachTo(StreamInfoDevice* streamInfoDevice) {
  GuiDevice::attachTo(streamInfoDevice);
  this->streamInfoDevice=streamInfoDevice;
}


void GuiInfoDevice::detach() {
  GuiDevice::detach();
  streamInfoDevice=NULL;
}



void GuiInfoDevice::processEvent(char eventId) {
  if (eventId!=_STREAMINFO_MUSIC_CHANGE) {
    return;
  }
  update();
}




void GuiInfoDevice::configure() {
  getPreferences()->show();
  apply();
}


void GuiInfoDevice::apply() {
  processEvent(_STREAMINFO_MUSIC_CHANGE);
}


void GuiInfoDevice::mousePressEvent ( QMouseEvent* mouseEvent) {
  if (mouseEvent->button() ==  RightButton) {
    cout << "RightButton"<<endl;
    KPopupMenu* popupMenu=createPopupMenu();
    popupMenu->move(mapToGlobal(QPoint(mouseEvent->x(),mouseEvent->y())));
    popupMenu->exec();
    delete popupMenu;
  }
}



char* GuiInfoDevice::getInfoString() {
  if (streamInfoDevice != NULL) {
    infoDeviceConfig->translate(streamInfoDevice->getMusicInfo(),
				streamInfoDevice->getID3Info(),
				infoString);
  }
  return infoString->getData();
}


char* GuiInfoDevice::getName() {
  if (streamInfoDevice != NULL) {
    MusicInfo* musicInfo=streamInfoDevice->getMusicInfo();
    return musicInfo->getName();
  }
  return "";
}


int GuiInfoDevice::getLen() {
  if (streamInfoDevice != NULL) {
    MusicInfo* musicInfo=streamInfoDevice->getMusicInfo();
    return musicInfo->getLen();
  }
  return 0;
}


QSize GuiInfoDevice::sizeHint () {
  return QSize(140,40);
}


void GuiInfoDevice::paintEvent ( QPaintEvent * paintEvent ) {
  QPainter paint;

  erase(0,0,width(),height());
  paint.begin( this );
  paint.drawText(0,10,getInfoString());
  paint.end();  

}



