#ifndef _APPTDLG_H
#define _APPTDLG_H

// 	$Id: editeventwin.h,v 1.32 1998/08/25 03:00:05 pbrown Exp $	
#include <ktabctl.h>
#include <ktopwidget.h>
#include <klined.h>

#include <qmlined.h>
#include <qlabel.h>
#include <qpushbt.h>
#include <qradiobt.h>
#include <qbttngrp.h>
#include <qchkbox.h>
#include <qdatetm.h>

#include "calobject.h"
#include "eventwingeneral.h"
#include "eventwindetails.h"
#include "eventwinrecur.h"
#include "catdlg.h"
#include "kptbutton.h"

/**
  * This is the class to add/edit a new appointment.
  *
  * @short Creates a dialog box to create/edit an appointment
  * @author Preston Brown
  * @version $Revision: 1.32 $
  */
class EditEventWin : public KTopLevelWidget
{
  Q_OBJECT

public:
/**
  * Constructs a new appointment dialog.
  *
  */  
  EditEventWin( CalObject *calendar, bool todo = FALSE );
  bool isTodo;
  virtual ~EditEventWin(void);

  enum { EVENTADDED, EVENTEDITED, EVENTDELETED };

public slots:

/** Clear eventwin for new event, and preset the dates and times with hint */
  void newEvent( QDateTime from, QDateTime to, bool allDay = FALSE );

/** Edit an existing event. */
  void editEvent( KDPEvent *, QDate qd=QDate::currentDate());

/** cancel is public so TopWidget can call it to close the dialog. */
  void cancel();

signals:
  void eventChanged(KDPEvent *, int);
  void closed(QWidget *);

protected slots:
  void makeEvent();
  void updateCatEdit(QString _str);
  void recurStuffEnable(bool enable);
  void startTimeChanged(QTime newt, int wrapval);
  void endTimeChanged(QTime newt, int wrapval);
  void startDateChanged(QDate);
  void endDateChanged(QDate);
  void prevEvent();
  void nextEvent();
  void saveAndClose();
  void deleteEvent();

protected:
  // methods
  void initConstructor();
  void initMenus();
  void initToolBar();
  void fillInFields(QDate qd);
  void fillInTodoFields();
  void fillInDefaults(QDateTime from, QDateTime to, bool allDay);
  void closeEvent(QCloseEvent *);
  QDate *dateFromText(const char *text);
  void setDuration();

  // variables
  KPTButton *saveExit;
  EventWinGeneral *General;
  EventWinDetails *Details;
  EventWinRecurrence *Recurrence;
  QFrame  *m_frame;
  KTabCtl *tabCtl;
  KToolBar *toolBar;
  CategoryDialog *catDlg;

  CalObject *calendar;
  KDPEvent *currEvent;               // the event being edited, if any.
  bool addFlag;                      // should we add a new event, or modify?

  // temporary variables that hold changes until it is time to commit.
  QDateTime  currStartDateTime;
  QDateTime  currEndDateTime;
  bool prevNextUsed;
};

#endif


