/* 
   KOrganizer (c) 1997,1998 Preston Brown
   All code is covered by the GNU Public License

   $Id: main.cpp,v 1.43 1998/12/23 04:56:42 pbrown Exp $	
*/

#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <qdir.h>

#include <kapp.h>

#include "topwidget.h"
#include "version.h"

KApplication *korganizer;
TopWidget *mainwin;
CalObject *cal;


// display items that are imminent over the next several days as listed
void display_imminent(int numdays, QString fileName)
{
  QDate currDate(QDate::currentDate());
  KDPEvent *currEvent;
  
  if (fileName.isEmpty()) {
    printf(i18n("No default calendar, and none was specified on the command line.\n"));
    exit(0);
  }
  if (!cal->load(fileName)) {
    printf(i18n("Error reading from default calendar.\n"));
    exit(0);
  }

  for (int i = 1; i <= numdays; i++) {
    QList<KDPEvent> tmpList(cal->getEventsForDate(currDate, TRUE));
    printf("%s\n",currDate.toString().data());
    printf("---------------------------------\n");
    if (tmpList.first())
      for (currEvent = tmpList.first(); currEvent; currEvent = tmpList.next())
	currEvent->print(KDPEvent::ASCII);
    else
      printf("(none)\n");
    printf("\n");
    currDate = currDate.addDays(1);
  }
  exit(0);
}

void print_help()
{
  printf("KOrganizer %s\n\n",korgVersion);
  printf("X11/Qt Options:\n");
  printf("-display <display>\n");
  printf("-geometry <geometry>\n"); 
  printf("-name <name>\n");
  printf("-title <title>\n");
  printf("-cmap\n\n");
  printf("KOrganizer Options:\n");
  printf("-c or --calendar <calendar>\n");
  printf("-l or --list\n");
  printf("-s or --show <numdays>\n");
  printf("-h or --help\n");
  exit(0);
}

void signalHandler(int signo)
{
  switch(signo) {
  case SIGSEGV:
    //    int childpid = fork();
    int childpid = 1;
    if (childpid != 0) {
      QMessageBox::critical(0, 
			    "KOrganizer crash analysis",
			    QString("KOrganizer has crashed.  Congratulations,\n"
				    "You have found a bug!  Please send e-mail to\n"
				    "pbrown@redhat.com with the following details:\n\n"
				    "1. What you were doing when the program crashed\n"
				    "2. What version of KOrganizer you are running\n"
				    "3. Any other details you feel are relevant\n\n"
				    "Remember: your bug reports help make the next\n"
				    "release of KOrganizer more bug-free!"));
      exit(-1);
    }
  }
  return;
}

int main (int argc, char **argv)
{

  TopWidget *mainwin = 0L;
  KConfig *config;
  QString fn;
  QString cfgPath;
  QDir qd;
  QFile pidFile;
  QString pidFileName;
  cal = new CalObject; // here so we can display stuff (PGB 07/27/98)

  //  if (signal(SIGSEGV, signalHandler) == SIG_ERR)
  //    debug("warning, can't catch SIGSEGV!");


  // make sure our directory tree exists
  pidFileName = KApplication::localkdedir().data();
  qd.setPath(pidFileName.data());
  if (!qd.exists())
    qd.mkdir(pidFileName.data());
  pidFileName += "/share";
  qd.setPath(pidFileName.data());
  if (!qd.exists())
    qd.mkdir(pidFileName.data());
  pidFileName += "/apps";
  qd.setPath(pidFileName.data());
  if (!qd.exists())
    qd.mkdir(pidFileName.data());
  pidFileName += "/korganizer";
  qd.setPath(pidFileName.data());
  if (!qd.exists())
    qd.mkdir(pidFileName.data(), TRUE);

  cfgPath = KApplication::localconfigdir();
  qd.setPath(cfgPath.data());
  if (!qd.exists())
    qd.mkdir(cfgPath.data());
  cfgPath += "/korganizerrc";

  config = new KConfig(KApplication::kde_configdir() + "/korganizerrc",
		       cfgPath.data());
  config->setGroup("General");

  fn = config->readEntry("Current Calendar");
  config->sync();
  delete config;
 
  // process command line arguments
  if (argc > 1) {
    for (int i = 1; i < argc; i++) {
      if ((strcasecmp(argv[i], "-h") == 0) || 
	  (strcasecmp(argv[i], "--help") == 0))
	print_help();
      else if ((strcasecmp(argv[i], "-c") == 0) || 
	       (strcasecmp(argv[i], "--calendar") == 0)) {
	i++;
	if (argv[i] != 0)
	  fn = argv[i];
      } else if ((strcasecmp(argv[i], "-l") == 0) ||
		 (strcasecmp(argv[i], "--list") == 0)) {

	display_imminent(1, fn);
      } else if ((strcasecmp(argv[i], "-s") == 0) ||
		 (strcasecmp(argv[i], "--show") == 0)) {
	i++;
	if (argv[i] != 0)
	  display_imminent(atoi(argv[i]), fn);
	else
	  display_imminent(1, fn);
      } else {
	fn = argv[i];
      }
    }
  }

  // always try to start the alarm daemon. IT will take care of itself
  // if there is no calendar to work on yet.
  // we may or may not have a filename.
  QString execStr;
  if (!fn.isEmpty())
    execStr.sprintf("%s/alarmd %s",
		    KApplication::kde_bindir().data(),
		    fn.data());
  else
    execStr.sprintf("%s/alarmd",
		    KApplication::kde_bindir().data());
  system(execStr.data());
  
  // those options which should exit are gone by now.
  // restore via session management.
  korganizer = new KApplication(argc, argv, "korganizer");
  if (korganizer->isRestored()) {
    int n = 1;
    while (TopWidget::canBeRestored(n)) {
      (new TopWidget(cal, fn))->restore(n); 
      n++;
    }
  } else {
    mainwin = new TopWidget(cal, fn);
    mainwin->show();
  }

  korganizer->exec();
  
  // cleanup time
  delete mainwin;
  delete cal;
  delete korganizer;

  return 0;
}

