/*
           $Id: topwidget.cpp,v 1.218 1999/01/28 16:57:56 pbrown Exp $

	 Requires the Qt and KDE widget libraries, available at no cost at
	 http://www.troll.no and http://www.kde.org respectively
	 
	 Copyright (C) 1997, 1998 
	 Preston Brown (preston.brown@yale.edu)
	 Fester Zigterman (F.J.F.ZigtermanRustenburg@student.utwente.nl)
	 Ian Dawes (iadawes@globalserve.net)
	 Christopher Beard
	 Laszlo Boloni (boloni@cs.purdue.edu)

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.
	 
	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
	 GNU General Public License for more details.
	 
	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <signal.h>

#include <kfiledialog.h>
#include <qfiledlg.h>
#include <qcursor.h>
#include <qmlined.h>
#include <qmsgbox.h>
#include <qtimer.h>
#include <kiconloader.h>
#include <ksimpleconfig.h>
#include <kstdaccel.h>

#include "misc.h"
#include "version.h"
#include "koarchivedlg.h"
#include "komailclient.h"
#include "calprinter.h"
#include "aboutdlg.h"

#include "topwidget.h"
#include "topwidget.moc"	

#define AGENDABUTTON 0x10
#define NOACCEL 0


QList<TopWidget> TopWidget::windowList;

TopWidget::TopWidget(CalObject *cal, QString fn, QWidget *, 
		     const char *name, bool fnOverride) 
  : KTopLevelWidget( name )
{
  QString titleStr = "KOrganizer ";
  titleStr += korgVersion;
  setCaption(titleStr.data());
  searchDlg = 0L;
  recentPop = 0L;
  viewMode = AGENDAVIEW;
  toolBarEnable = statusBarEnable = TRUE;
  setMinimumSize(600,400);	// make sure we don't get resized too small...

  // get calendar from passed argument to the constructor.
  calendar = cal;

  // create the main layout frames.
  mainFrame    = new QFrame(this);  
  panner = new KNewPanner(mainFrame, "panner",
			  KNewPanner::Vertical,
			  KNewPanner::Absolute, 164);
  leftFrame = new QFrame(panner);
  rightFrame = new QFrame(panner);

  optionsDlg = new OptionsDialog("KOrganizer Configuration Options",
				 this);
  connect(optionsDlg, SIGNAL(configChanged()),
	  this, SLOT(updateConfig()));
  connect(optionsDlg, SIGNAL(closed(QWidget *)), 
	  this, SLOT(cleanWindow(QWidget *)));
  

  QVBoxLayout *layout1 = new QVBoxLayout(leftFrame, 1, -1, "layout1");
  dateNavFrame = new QFrame(leftFrame);
  //  dateNavFrame->setFrameStyle(QFrame::Panel|QFrame::Sunken);
  dateNavigator = new KDateNavigator(dateNavFrame, calendar, TRUE,
				     QDate::currentDate(),"DateNavigator");
  dateNavigator->move(2,2);
  dateNavigator->resize(160, 150);
  dateNavFrame->resize(dateNavigator->width()+4, dateNavigator->height()+4);
  dateNavFrame->setMinimumSize(dateNavFrame->size());
  dateNavFrame->setFixedHeight(dateNavigator->height());
  
  layout1->addWidget(dateNavFrame);
  layout1->addSpacing(5);

  // add this instance of the window to the static list.
  windowList.append(this);
  initCalendar(fn, fnOverride);

  // create the main data display views.
  todoList   = new TodoView(calendar, leftFrame);
  layout1->addWidget(todoList);
  layout1->activate();

  todoView   = new TodoView(calendar, mainFrame);

  QVBoxLayout *layout2 = new QVBoxLayout(rightFrame, 0, -1, "layout2");
  agendaView = new KAgenda(calendar, rightFrame);
  layout2->addWidget(agendaView);
  layout2->activate();

  QVBoxLayout *layout3 = new QVBoxLayout(rightFrame, 0, -1, "layout3");
  listView   = new ListView(calendar, rightFrame);
  layout3->addWidget(listView);
  layout3->activate();

  monthView  = new KDPMonthView(calendar, mainFrame);

  panner->activate(leftFrame, rightFrame);

  // setup toolbar, menubar and status bar, NOTE: this must be done
  // after the widget creation, because setting the menubar, toolbar
  // or statusbar will generate a call to updateRects, which assumes
  // that all of them are around.
  initMenus();
  initToolBar();

  sb = new KStatusBar(this, "sb");
  setStatusBar(sb);

  // set up printing object
  calPrinter = new CalPrinter(this, calendar);

  // hook up the signals/slots of all widgets together so communication
  // can happen when things get clicked.
  hookupSignals();
  
  // set up autoSaving stuff
  autoSaveTimer = new QTimer(this);
  connect(autoSaveTimer, SIGNAL(timeout()),
	  this, SLOT(checkAutoSave()));
  if (calendar->autoSave())
    autoSaveTimer->start(1000*60);
  
  // this is required for KTopLevelWidget for geometry management
  setView(mainFrame, TRUE);
  goToday();

  changeView(viewMode);
  goToday();
  changeAgendaView(agendaViewMode);

  setupRollover();

  // this isn't working right now -- it isn't properly turning
  // off the tool/status bars.  Bug in KTMainWindow?
  /*  if (!toolBarEnable)
    toggleToolBar();

  if (!statusBarEnable)
  toggleStatusBar();*/

}

TopWidget::~TopWidget(void)
{
  // Free memory allocated for widgets (not children)
  // Take this window out of the window list.
  windowList.removeRef( this );

  //KTLW doesn't delete it's main view
  delete tb;
  delete menubar;
  delete mainFrame;
}


void TopWidget::initCalendar(QString fn, bool fnOverride) 
{
  // read the settings from the config file
  readSettings();
  // cmd line argument for filename overrides config files 
  if(!fn.isEmpty() || fnOverride)
    fileName = fn;

  QApplication::setOverrideCursor(waitCursor);
  // note -- we MAY want to remove the second clause of the following
  // boolean expression.  MAYBE.
  if ((!fileName.isEmpty()) && (windowList.count() == 1)) {
    if (!(calendar->load(fileName))) {
      // while failing to load, the calendar object could 
      // have become partially populated.  Clear it out.
      calendar->close();
      fileName = "";
    }
  }
  unsetModified();
  config->setGroup("General");
  config->writeEntry("Current Calendar", fileName);
  QApplication::restoreOverrideCursor();

}

void TopWidget::updateRects()
{
  KTopLevelWidget::updateRects();
  panner->resize(mainFrame->size());
  /*  rightFrame->resize(mainFrame->width()-(panner->separatorPos()+5), 
		     mainFrame->height());

  agendaView->resize(rightFrame->size());
  
  listView->resize(rightFrame->size());*/

  monthView->move(2,2);
  monthView->resize(mainFrame->width()-4, mainFrame->height()-4);
  todoView->move(2,2);
  todoView->resize(mainFrame->width()-4, mainFrame->height()-4);
}

void TopWidget::readSettings()
{
  QString str;

  // read settings from the KConfig, supplying reasonable
  // defaults where none are to be found

  config = korganizer->getConfig();

  int windowWidth = 600;
  int windowHeight = 400;
  viewMode = AGENDAVIEW;

  // add stuff for config file versioning here!
  recentFileList.clear();
  config->setGroup("Recently Opened Files");
  str = config->readEntry("1", "");
  add_recent_file(str);
  str = config->readEntry("2", "");
  add_recent_file(str);
  str = config->readEntry("3", "");
  add_recent_file(str);
  str = config->readEntry("4", "");
  add_recent_file(str);
  str = config->readEntry("5", "");
  add_recent_file(str);
	
  config->setGroup("General");
  str = config->readEntry("Width");
  if (!str.isEmpty())
    windowWidth = str.toInt();
  str = config->readEntry("Height");
  if (!str.isEmpty())
    windowHeight = str.toInt();
  this->resize(windowWidth, windowHeight);

  str = config->readEntry("Separator");
  if (!str.isEmpty())
    panner->setSeparatorPos(str.toInt());

  statusBarEnable = config->readBoolEntry("Status Bar", TRUE);

  toolBarEnable = config->readBoolEntry("Tool Bar", TRUE);

  str = config->readEntry("Current View");
  if (!str.isEmpty())
    viewMode = str.toInt();
  str = config->readEntry("Current Calendar");
  if (!str.isEmpty() && QFile::exists(str.data()))
    fileName = str;

  config->setGroup("Views");
  agendaViewMode = config->readNumEntry("Agenda View", KAgenda::DAY);

  config->setGroup( "Colors" );
  if( config->readBoolEntry( "DefaultColors", TRUE ) == TRUE )
  {
    optionsDlg->setColorDefaults();
    optionsDlg->applyColorDefaults();
  }

  config->sync();
}

void TopWidget::writeSettings()
{
	
  config = korganizer->getConfig();

  config->setGroup("Recently Opened Files");
	
  QString recent_number;
  for(int i = 0; i < (int) recentFileList.count(); i++){
    recent_number.setNum(i+1);
    config->writeEntry(recent_number.data(),recentFileList.at(i));
  }

  QString tmpStr;
  config->setGroup("General");

  tmpStr.sprintf("%d", this->width() );
  config->writeEntry("Width",	tmpStr);
	
  tmpStr.sprintf("%d", this->height() );
  config->writeEntry("Height",	tmpStr);

  tmpStr.sprintf("%d", panner->separatorPos());
  config->writeEntry("Separator", tmpStr);

  tmpStr.sprintf("%s", optionsMenu->isItemChecked(toolBarMenuId) ? 
		 "true" : "false");
  config->writeEntry("Tool Bar", tmpStr);

  tmpStr.sprintf("%s", optionsMenu->isItemChecked(statusBarMenuId) ?
		 "true" : "false");
  config->writeEntry("Status Bar", tmpStr);

  tmpStr.sprintf("%d", viewMode);
  config->writeEntry("Current View", tmpStr);
  config->writeEntry("Current Calendar", fileName);

  config->setGroup("Views");
  config->writeEntry("Agenda View", agendaView->currentView());
  config->sync();

}

void TopWidget::goToday()
{
  dateNavigator->selectDates(QDate::currentDate());
  saveSingleDate = QDate::currentDate();
  updateView(dateNavigator->getSelected());
}

void TopWidget::goNext()
{
	// adapt this to work for other views
	agendaView->slotNextDates();
}

void TopWidget::goPrevious()
{
	// adapt this to work for other views
	agendaView->slotPrevDates();
}

void TopWidget::setupRollover()
{
  // right now, this is a single shot (because I am too lazy to code a
  // real one using a real qtimer object).  It will only work for a single
  // day rollover.  I should fix this. :)
  QDate tmpDate = QDate::currentDate().addDays(1);
  QTime tmpTime = QTime(00, 1);
  QDateTime tomorrow(tmpDate, tmpTime);

  QTimer::singleShot(QDateTime::currentDateTime().secsTo(tomorrow)*1000,
		     dateNavigator, SLOT(updateView()));
}

void TopWidget::initMenus()
{
  QPixmap pixmap;
  //  int itemId;
  KStdAccel stdAccel;

  fileMenu = new QPopupMenu;
  pixmap = Icon("mini/korganizer.xpm");
  fileMenu->insertItem(pixmap,i18n("&New Window"), this,
		       SLOT(newTopWidget()), stdAccel.openNew());
  fileMenu->insertSeparator();
  pixmap = Icon("filenew.xpm");
  fileMenu->insertItem(pixmap, i18n("&Open..."), this,
		       SLOT(file_open()), stdAccel.open());
  recentPop = new QPopupMenu;
  fileMenu->insertItem(i18n("Open &Recent"), recentPop);
  connect( recentPop, SIGNAL(activated(int)), SLOT(file_openRecent(int)));

  // setup the list of recently used files
  recentPop->clear();
  for (int i = 0; i < (int) recentFileList.count(); i++)
    recentPop->insertItem(recentFileList.at(i));
  add_recent_file(fileName);

  fileMenu->insertItem(i18n("&Close"), this,
		       SLOT(file_close()), stdAccel.close());
  fileMenu->insertSeparator();

  pixmap = Icon("filefloppy.xpm");
  fileMenu->insertItem(pixmap, i18n("&Save"), this,
		       SLOT(file_save()), stdAccel.save());
  fileMenu->insertItem(i18n("Save &As..."), this,
		       SLOT(file_saveas()));

  fileMenu->insertSeparator();

  fileMenu->insertItem(i18n("&Import From Ical..."), this,
  		       SLOT(file_import()));
  fileMenu->insertItem(i18n("&Merge Calendar..."), this,
		       SLOT(file_merge()));
  
  fileMenu->setItemEnabled(fileMenu->insertItem(i18n("Archive Old Entries..."),
						this,
						SLOT(file_archive())),
			   FALSE);
  

  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("Print Setup..."), this,
		       SLOT(printSetup()));

  pixmap = Icon("fileprint.xpm");
  fileMenu->insertItem(pixmap, i18n("&Print..."), this,
		       SLOT(print()), stdAccel.print());
  fileMenu->insertItem(i18n("Print Pre&view..."), this,
		       SLOT(printPreview()));
  
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("&Quit"), this,
		       SLOT(file_quit()), stdAccel.quit());

  editMenu = new QPopupMenu;
  //put stuff for editing here 
  pixmap = Icon("editcut.xpm");
  editMenu->insertItem(pixmap, i18n("C&ut"), this,
		       SLOT(edit_cut()), stdAccel.cut());
  pixmap = Icon("editcopy.xpm");
  editMenu->insertItem(pixmap, i18n("&Copy"), this,
		       SLOT(edit_copy()), stdAccel.copy());
  pixmap = Icon("editpaste.xpm");
  editMenu->insertItem(pixmap, i18n("&Paste"), this,
		       SLOT(edit_paste()), stdAccel.paste());

  viewMenu = new QPopupMenu;

  pixmap = Icon("dayicon.xpm");
  viewMenu->insertItem(pixmap, i18n("&List"), this,
			SLOT( view_list() ) );

  viewMenu->insertItem(pixmap, i18n("&Day"), this,
			SLOT( view_day() ) );
  pixmap = Icon("5dayicon.xpm");
  viewMenu->insertItem(pixmap, i18n("W&ork Week"), this,
			SLOT( view_workweek() ) );
  pixmap = Icon("weekicon.xpm");
  viewMenu->insertItem(pixmap, i18n("&Week"), this,
			SLOT( view_week() ) );
  pixmap = Icon("monthicon.xpm");
  viewMenu->insertItem(pixmap, i18n("&Month"), this,
			SLOT( view_month() ) );
  pixmap = Icon("checkedbox.xpm");
  viewMenu->insertItem(pixmap,i18n("&Todo list"), this,
			SLOT( view_todolist()), FALSE );
  viewMenu->insertSeparator();
  viewMenu->insertItem( i18n("&Update"), this,
			SLOT( update() ) );
	
  actionMenu = new QPopupMenu;
  pixmap = Icon("newevent.xpm");
  actionMenu->insertItem(pixmap, i18n("New &Appointment"), 
			 this, SLOT(apptmnt_new()));
  actionMenu->insertItem(i18n("New E&vent"),
			 this, SLOT(allday_new()));
  actionMenu->insertItem(i18n("&Edit Appointment"), this,
			  SLOT(apptmnt_edit()));
  pixmap = Icon("delete.xpm");
  actionMenu->insertItem(pixmap, i18n("&Delete Appointment"),
			 this, SLOT(apptmnt_delete()));
  actionMenu->insertItem(i18n("Delete Todo"),
			 this, SLOT(action_deleteTodo()));

  actionMenu->insertSeparator();
  pixmap = Icon("search.xpm");
  actionMenu->insertItem(pixmap,i18n("&Search..."), this,
			 SLOT(action_search()), stdAccel.find());

  pixmap = Icon("send.xpm");
  actionMenu->insertItem(pixmap, i18n("&Mail Appointment..."), this,
				  SLOT(action_mail()));

  optionsMenu = new QPopupMenu;
  toolBarMenuId = optionsMenu->insertItem(i18n("Show &Tool Bar"), this,
				   SLOT(toggleToolBar()));
  optionsMenu->setItemChecked(toolBarMenuId, TRUE);

  statusBarMenuId = optionsMenu->insertItem(i18n("Show St&atus Bar"), this,
				   SLOT(toggleStatusBar()));
  optionsMenu->setItemChecked(statusBarMenuId, TRUE);

  optionsMenu->insertSeparator();			  
  optionsMenu->insertItem(i18n("&Edit Options"), 
			  this, SLOT(edit_options()));

  helpMenu = new QPopupMenu;
  helpMenu->insertItem(i18n("&Contents"), this,
		       SLOT(help_contents()),
		       stdAccel.help());
  helpMenu->insertSeparator();
  helpMenu->insertItem(i18n("&About"), this,
		       SLOT(help_about())); 
  helpMenu->insertItem(i18n("Send &Bug Report"), this,
		       SLOT(help_postcard()));
    
  // construct a basic menu
  menubar = new KMenuBar(this, "menubar_0");
  menubar->insertItem(i18n("&File"), fileMenu);
  menubar->insertItem(i18n("&Edit"), editMenu);
  menubar->insertItem(i18n("&View"), viewMenu);
  menubar->insertItem(i18n("&Actions"), actionMenu);
  menubar->insertItem(i18n("&Options"), optionsMenu);
  menubar->insertSeparator();
  menubar->insertItem(i18n("&Help"), helpMenu);

  setMenu(menubar);
}

void TopWidget::initToolBar()
{
  QPixmap pixmap;
  QString dirName;

  tb = new KToolBar(this);
  pixmap = Icon("fileopen.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(file_open()), TRUE, 
		   i18n("Open A Calendar"));
	
  pixmap = Icon("filefloppy.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(file_save()), TRUE, 
		   i18n("Save This Calendar"));

  pixmap = Icon("fileprint.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(print()), TRUE, 
		   i18n("Print"));


  QFrame *sepFrame = new QFrame(tb);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  tb->insertWidget(0, 10, sepFrame);

  pixmap = Icon("newevent.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(apptmnt_new()), TRUE, 
		   i18n("New Appointment"));
  pixmap = Icon("delete.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(apptmnt_delete()), TRUE,
		   i18n("Delete Appointment"));

  pixmap = Icon("findf.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(action_search()), TRUE,
		   i18n("Search For an Appointment"));

  pixmap = Icon("send.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(action_mail()), TRUE,
		   i18n("Mail Appointment"));

  sepFrame = new QFrame(tb);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  tb->insertWidget(0, 10, sepFrame);

  KPTButton *bt = new KPTButton(tb);
  bt->setText(i18n("Go to Today"));
  //  bt->setPixmap(Icon("todayIcon.xpm"));
  connect(bt, SIGNAL(clicked()), SLOT(goToday()));
  tb->insertWidget(0, bt->sizeHint().width(), bt);

  pixmap = Icon("1leftarrow.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()),
		   this, SLOT(goPrevious()), TRUE, i18n("Previous"));


  pixmap = Icon("1rightarrow.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()),
		   this, SLOT(goNext()), TRUE, i18n("Next"));

  sepFrame = new QFrame(tb);
  sepFrame->setFrameStyle(QFrame::VLine|QFrame::Raised);
  tb->insertWidget(0, 10, sepFrame);

  QPopupMenu *agendaViewPopup = new QPopupMenu();
  agendaViewPopup->insertItem( Icon("dayicon.xpm"), "Show one day", KAgenda::DAY );
  agendaViewPopup->insertItem( Icon("5dayicon.xpm"), "Show a work week", KAgenda::WORKWEEK );
  agendaViewPopup->insertItem( Icon("weekicon.xpm"), "Show a week", KAgenda::WEEK );
  connect( agendaViewPopup, SIGNAL( activated(int) ), this , SLOT( changeAgendaView(int) ) );
  
  pixmap = Icon("agenda.xpm");
  tb->insertButton(pixmap, AGENDABUTTON, SIGNAL(clicked()),
  		this, SLOT( nextAgendaView()), TRUE,
		i18n("Schedule View"));
  tb->setDelayedPopup( AGENDABUTTON, agendaViewPopup );

  pixmap = Icon("monthicon.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(view_month()), TRUE,
		   i18n("Month View"));
  
  pixmap = Icon("listicon.xpm");
  tb->insertButton(pixmap, 0, SIGNAL(clicked()), this,
		   SLOT(view_list()), TRUE,
		   i18n("List View"));

  pixmap = Icon("checkedclipboard.xpm");
  tb->insertButton(pixmap, 0,
		   SIGNAL(clicked()), this,
		   SLOT(view_todolist()), TRUE,
		   i18n("Todo list view"));

  addToolBar(tb);
}

void TopWidget::hookupSignals()
{
  // SIGNAL/SLOTS FOR DATE SYNCHRO
    
  connect(listView, SIGNAL(datesSelected(const QDateList)),
	  dateNavigator, SLOT(selectDates(const QDateList)));
  connect(agendaView, SIGNAL(datesSelected(const QDateList)),
	  dateNavigator, SLOT(selectDates(const QDateList)));
  connect(monthView, SIGNAL(datesSelected(const QDateList)),
	  dateNavigator, SLOT(selectDates(const QDateList)));    
  connect(dateNavigator, SIGNAL(datesSelected(const QDateList)),
	  this, SLOT(updateView(const QDateList)));
    
  // SIGNALS/SLOTS FOR LIST VIEW
  connect(listView, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));
  connect(listView, SIGNAL(deleteEventSignal(KDPEvent *)), 
	  this, SLOT(deleteEvent(KDPEvent *)));

  // SIGNALS/SLOTS FOR DAY/WEEK VIEW
  connect(agendaView,SIGNAL(newEventSignal(QDateTime)),
		this, SLOT(newEvent(QDateTime)));
  connect(agendaView, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));
  connect(agendaView, SIGNAL(deleteEventSignal(KDPEvent *)), 
	  this, SLOT(deleteEvent(KDPEvent *)));

  // SIGNALS/SLOTS FOR MONTH VIEW
  connect(monthView, SIGNAL(newEventSignal(QDate)),
	  this, SLOT(newEvent(QDate)));
  connect(monthView, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));
  connect(monthView, SIGNAL(deleteEventSignal(KDPEvent *)),
	  this, SLOT(deleteEvent(KDPEvent *)));


  // SIGNALS/SLOTS FOR TODO LIST
  connect(todoList, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));
  connect(todoView, SIGNAL(editEventSignal(KDPEvent *)),
	  this, SLOT(editEvent(KDPEvent *)));

  // CONFIGURATION SIGNALS/SLOTS

  // Notification list. Add a connection to widgets when they
  // need to know about changed in configuration.
  connect(this, SIGNAL(configChanged()), calendar, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), agendaView, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), monthView, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), listView, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), calPrinter, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), dateNavigator, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), todoView, SLOT(updateConfig()));
  connect(this, SIGNAL(configChanged()), todoList, SLOT(updateConfig()));

  // MISC. SIGNALS/SLOTS
  connect(calendar, SIGNAL(calUpdated(KDPEvent *)), this, SLOT(setModified()));
}

// most of the changeEventDisplays() right now just call the view's
// total update mode, but they SHOULD be recoded to be more refresh-efficient.
void TopWidget::changeEventDisplay(KDPEvent *which, int action)
{
  dateNavigator->updateView();
  if (searchDlg)
    searchDlg->updateView();

  switch(viewMode) {
  case LISTVIEW:
    listView->changeEventDisplay(which, action);
    break;
  case AGENDAVIEW:
    agendaView->changeEventDisplay(which, action);
    break;
  case MONTHVIEW:
    monthView->changeEventDisplay(which, action);
    break;
  default:
    debug(i18n("we don't handle updates for that view, yet.\n"));
    qApp->beep();
    return;
  }
}

void TopWidget::changeAgendaView( int newView )
{
  QPixmap px;
  
  switch( newView ) {
  case KAgenda::DAY: {
    QDateList tmpList(FALSE);
    tmpList = dateNavigator->getSelected();
    if (saveSingleDate != *tmpList.first()) {
      dateNavigator->selectDates(saveSingleDate);
      updateView(dateNavigator->getSelected());
    }    
    px = Icon("dayicon.xpm" );
    tb->setButtonPixmap( AGENDABUTTON, px);
    break;
  }
  // if its a workweek view, calculate the dates and emit
  case KAgenda::WORKWEEK:
    px = Icon("5dayicon.xpm" );
    tb->setButtonPixmap( AGENDABUTTON, px);
    break;
    // if its a week view, calculate the dates and emit
  case KAgenda::WEEK:
    px = Icon("weekicon.xpm" );
    tb->setButtonPixmap( AGENDABUTTON, Icon("weekicon.xpm" ) );
    break;
    // if its a list view, update the list properties.
  case KAgenda::LIST:
    // we want to make sure that this is up to date.
    px = Icon("agenda.xpm" );
    tb->setButtonPixmap( AGENDABUTTON, Icon("agenda.xpm" ) );
    break;
  }
  agendaView->slotViewChange( newView );
}

void TopWidget::nextAgendaView()
{
  int view;
  
  if( viewMode == AGENDAVIEW ) {
    view = agendaView->currentView() + 1;
    if ((view >= KAgenda::DAY) && ( view < KAgenda::LIST))
      changeAgendaView( view );
    else 
      changeAgendaView( KAgenda::DAY );
  } else {
    changeAgendaView( agendaView->currentView() );
    changeView( AGENDAVIEW );
  }

}

void TopWidget::changeView(int newViewMode)
{
  viewMode = newViewMode;
  QPixmap px;
  
  updateView(dateNavigator->getSelected());

  switch(viewMode) {
  case AGENDAVIEW:
    todoView->hide();  
    monthView->hide();
    listView->hide();
    agendaView->show();
    panner->show();    
    break;
  case LISTVIEW:
    todoView->hide();  
    monthView->hide();
    agendaView->hide();
    listView->show();
    panner->show();
    break;
  case MONTHVIEW:
    todoView->hide();  
    panner->hide();
    monthView->show();
    break; 
  case TODOVIEW:
    panner->hide();
    monthView->hide();
    todoView->show();
    break;
  default:
    debug(i18n("we don't handle that view yet.\n"));
    break;
  }
  updateView();
}

void TopWidget::updateView(const QDateList selectedDates)
{
  QDateList tmpList(FALSE);
  tmpList = selectedDates;
  int numView;
  KConfig *config = kapp->getConfig();
  config->setGroup("Time & Date");
  bool weekStartsMonday = config->readBoolEntry("Week Starts Monday", TRUE);
  QPixmap px;

  if ((tmpList.count() == 5) && 
      (tmpList.first()->dayOfWeek() == (weekStartsMonday ? 1 : 7)) &&
      (tmpList.first()->daysTo(*tmpList.last()) == 4))
    numView = KAgenda::WORKWEEK;
  // if there are 7 dates and the first date is a monday, we have a regular week.
  else if ((tmpList.count() == 7) && 
	   (tmpList.first()->dayOfWeek() == (weekStartsMonday ? 1 : 7)) &&
	   (tmpList.first()->daysTo(*tmpList.last()) == 6))
    numView = KAgenda::WEEK;
  else if (tmpList.count() == 1) {
    numView = KAgenda::DAY;
    saveSingleDate = *tmpList.first();
  } else
    // for sanity, set viewtype to LIST for now...
    numView = KAgenda::LIST;
    
  switch( numView )
    {
    case KAgenda::DAY:
      px = Icon("dayicon.xpm" );
      tb->setButtonPixmap( AGENDABUTTON, px);
      break;
      // if its a workweek view, calculate the dates and emit
    case KAgenda::WORKWEEK:
      px = Icon("5dayicon.xpm" );
      tb->setButtonPixmap( AGENDABUTTON, px);
      break;
      // if its a week view, calculate the dates and emit
    case KAgenda::WEEK:
      px = Icon("weekicon.xpm" );
      tb->setButtonPixmap( AGENDABUTTON, Icon("weekicon.xpm" ) );
      break;
      // if its a list view, update the list properties.
    case KAgenda::LIST:
      // we want to make sure that this is up to date.
      px = Icon("agenda.xpm" );
      tb->setButtonPixmap( AGENDABUTTON, Icon("agenda.xpm" ) );
      break;
    }
  
  switch (viewMode) {
  case AGENDAVIEW:
      agendaView->selectDates(selectedDates);
    break;
  case LISTVIEW:
      listView->selectDates(selectedDates);
    break;
  case MONTHVIEW:
      monthView->selectDates(selectedDates);
    break;
  default:
    break;
  }
  todoList->updateView();
  todoView->updateView();
}

void TopWidget::updateView()
{
  // update the current view with the current dates from the date navigator
  QDateList tmpList(FALSE); // we want a shallow copy
  tmpList = dateNavigator->getSelected();

  // if no dates are supplied, we should refresh the dateNavigator too...
  dateNavigator->updateView();
  updateView(tmpList);
}

void TopWidget::file_open()
{
  QString defaultPath;
  
  // be nice and tidy and have all the files in a nice easy to find place
  defaultPath = KApplication::localkdedir().data();
  defaultPath += "/share/apps/korganizer/";
  
  if (isModified() && !calendar->autoSave()) {
    switch(QMessageBox::warning(this, 
				i18n("Modified Calendar Warning"),
				i18n("You are opening a new calendar, but the current"
				     "one has been modified.\nAre you sure you want to"
				     "do this? All changes will be lost!\n\n"),
				i18n("&OK"), 
				i18n("&Save"),
				i18n("&Cancel"),
				2)) {
    case 0: { // OK
      
      //      fileName = QFileDialog::getOpenFileName(defaultPath, "*.vcs", 
      //					      this);
      fileName = KFileDialog::getOpenFileName(defaultPath, "*.vcs",
					     this);
            
      // We sanity check the returned filename...
      if (!fileName.isEmpty()) {
	QFileInfo tf("",fileName);
	if(!tf.isDir()) {
	  QApplication::setOverrideCursor(waitCursor);
	  calendar->close();
	  if (calendar->load(fileName)) {
	    unsetModified();
	    add_recent_file(fileName);
	    updateView();
	    QApplication::restoreOverrideCursor();
	    return;
	  } else {
	    QApplication::restoreOverrideCursor();
	  }
	}
      }
      
      // Spit out an indication that the file was not opened,
      // either a bad file name was specified or the operation
      // was cancelled.
      QString msgStr;
      msgStr += i18n("KOrganizer had a problem opening your file:\n");
      msgStr += fileName + "\n";
      msgStr += i18n("Please check that the directory and/or file exists,\n"
		     "that you have permissions to read it, and try again.");
      
      QMessageBox::warning(this,i18n("KOrganizer Error"), msgStr);

      break;
    }
    case 1: // save
      if (file_save()) // bail on error
	return;
      file_open();
      break;

    case 2: // Cancel
      // exit
      break;
			
    } // switch

  } else if (isModified()) {
    if (file_save()) // bail on error
      return;
    file_open();
  } else {

    //    fileName = QFileDialog::getOpenFileName(defaultPath, "*.vcs",
    //					    this);
    QString newFileName =  KFileDialog::getOpenFileName(defaultPath, "*.vcs",
						       this);

    // We sanity check the returned filename
    if (newFileName.isEmpty())
      return;
    else {
      fileName = newFileName;
      QFileInfo tf("",fileName);
      if(!tf.isDir()) {
	QApplication::setOverrideCursor(waitCursor);
	calendar->close();
	if(calendar->load(fileName)) {
	  unsetModified();
	  add_recent_file(fileName);
	  updateView();
	  QApplication::restoreOverrideCursor();
	  return;
	} else {
	  QApplication::restoreOverrideCursor();
	}
      }
    }
    
    // Spit out an indication that the file was not opened,
    // either a bad file name was specified or the operation
    // was cancelled.
      QString msgStr;
      msgStr += i18n("KOrganizer had a problem opening your file:\n");
      msgStr += fileName + "\n";
      msgStr += i18n("Please check that the directory and/or file exists,\n"
		     "that you have permissions to read it, and try again.");
      
      QMessageBox::warning(this,i18n("KOrganizer Error"), msgStr);
  } 
}

void TopWidget::file_openRecent(int i)
{
  if (isModified() && !calendar->autoSave()) {
		
    switch(QMessageBox::warning(this, 
				i18n("KOrganizer Modified Calendar Warning"),
				i18n("You are opening a new calendar, but the current one "
				     "has been modified.\nAre you sure you want to do this?"
				     " All changes will be lost!\n\n"),
				i18n("&OK"),
				i18n("&Save"),
				i18n("&Cancel"),
				2)) {
    case 0: { // OK

      fileName = recentFileList.at(i);
      
      if (!fileName.isEmpty()) {
	QFileInfo tf("",fileName);
	// sanity check
	if(!tf.isDir()) {
	  QApplication::setOverrideCursor(waitCursor);
	  calendar->close();
	  if(calendar->load(fileName)) {
	    unsetModified();
	    add_recent_file(fileName);
	    updateView();
	    QApplication::restoreOverrideCursor();
	    return;
	  } else {
	    QApplication::restoreOverrideCursor();
	  }
	}
      }
      
      // Spit out an indication that the file was not opened,
      // either a bad file name was specified or the operation
      // was cancelled.
      QString msgStr;
      msgStr += i18n("KOrganizer had a problem opening your file:\n");
      msgStr += fileName + "\n";
      msgStr += i18n("Please check that the directory and/or file exists,\n"
		     "that you have permissions to read it, and try again.");
      
      QMessageBox::warning(this,i18n("KOrganizer Error"), msgStr);
    
      break;
    }
    case 1: // save

      if (file_save()) {
	// some sort of error saving.  bail.
	break;
      } else {
	calendar->close();
	fileName = "";
	file_openRecent(i);
      }
      break;

    case 2: // Cancel
      // exit
      break;
			
    } // switch
  } else if (isModified()) {
    if (file_save()) {
      // some sort of error, bail
      return;
    } else {
      QApplication::setOverrideCursor(waitCursor);
      calendar->close();
      QApplication::restoreOverrideCursor();
      fileName = "";
      file_openRecent(i);
    }
  } else {
    fileName = recentFileList.at(i);
    
    if (!fileName.isEmpty()) {
      QFileInfo tf("",fileName);
      // sanity check
      if(!tf.isDir()) {
	QApplication::setOverrideCursor(waitCursor);	
	calendar->close();
	if(calendar->load(fileName)) {
	  unsetModified();
	  add_recent_file(fileName);
	  updateView();
	  QApplication::restoreOverrideCursor();
	  return;
	} else {
	  QApplication::restoreOverrideCursor();
	}
      }
    }
    
    // Spit out an indication that the file was not opened,
    // either a bad file name was specified or the operation
    // was cancelled.
    QString msgStr;
    msgStr += i18n("KOrganizer had a problem opening your file:\n");
    msgStr += fileName + "\n";
    msgStr += i18n("Please check that the directory and/or file exists,\n"
		   "that you have permissions to read it, and try again.");
    
    QMessageBox::warning(this,i18n("KOrganizer Error"), msgStr);
  }
}					


void TopWidget::file_import()
{
  // eventually, we will need a dialog box to select import type, etc.
  // for now, hard-coded to ical file, $HOME/.calendar.
  int retVal;
  QString progPath;
  char *tmpFn;

  QString homeDir;
  homeDir.sprintf("%s/.calendar",getenv("HOME"));
		  
  if (!QFile::exists(homeDir.data())) {
    QMessageBox::critical(this, i18n("KOrganizer Error"),
			  i18n("You have no ical file in your home directory.\n"
			       "Import cannot proceed.\n"));
    return;
  }
  tmpFn = tmpnam(0);
  progPath.sprintf("%s/korganizer/ical2vcal %s",
		   KApplication::kde_datadir().data(), tmpFn);
  retVal = system(progPath.data());

  if (retVal >= 0 && retVal <= 2) {
    // now we need to MERGE what is in the iCal to the current calendar.
    calendar->load(tmpFn);
    if (!retVal)
      QMessageBox::information(this, i18n("KOrganizer Info"),
			       i18n("KOrganizer succesfully imported and merged your\n"
			       ".calendar file from ical into the currently\n"
			       "opened calendar.\n"),
			       QMessageBox::Ok);
    else
          QMessageBox::warning(this, i18n("ICal Import Successful With Warning"),
			       i18n("KOrganizer encountered some unknown fields while\n"
			       "parsing your .calendar ical file, and had to\n"
			       "discard them.  Please check to see that all\n"
			       "your relevant data was correctly imported.\n"));
  } else if (retVal == -1) {
    QMessageBox::warning(this, i18n("KOrganizer Error"),
			 i18n("KOrganizer encountered some error parsing your\n"
			 ".calendar file from ical.  Import has failed.\n"));
  } else if (retVal == -2) {
    QMessageBox::warning(this, i18n("KOrganizer Error"),
			 i18n("KOrganizer doesn't think that your .calendar\n"
			 "file is a valid ical calendar. Import has failed.\n"));
  }
  updateView();
}

void TopWidget::file_merge()
{
  QString defaultPath;
  QString mergeFileName;

  // be nice and tidy and have all the files in a nice easy to find place
  defaultPath = KApplication::localkdedir().data();
  defaultPath += "/share/apps/korganizer/";
  
  
  //  mergeFileName = QFileDialog::getOpenFileName(defaultPath, "*.vcs",
  //					       this);
  mergeFileName = KFileDialog::getOpenFileName(defaultPath, "*.vcs",
					      this);
  
  // If file dialog box was cancelled (trap for null) 
  if(mergeFileName.isEmpty()) {
    return;
  }
    
  // We sanity check the returned filename
  if (!mergeFileName.isEmpty()) {
    QFileInfo tf("",mergeFileName);
    if(!tf.isDir()) {
      if(calendar->load(mergeFileName)) {
	setModified();
	updateView();
	return;
      } 
    }
  }
  
  // Spit out an indication that the file was not opened,
  // either a bad file name was specified or the operation
  // was cancelled.
  QString msgStr;
  msgStr += i18n("KOrganizer had a problem opening your file:\n");
  msgStr += fileName + "\n";
  msgStr += i18n("Please check that the directory and/or file exists,\n"
		 "that you have permissions to read it, and try again.");
  
  QMessageBox::warning(this,i18n("KOrganizer Error"), msgStr);

}

void TopWidget::file_archive()
{
  ArchiveDialog ad;

  if (ad.exec()) {
    // ok was pressed.
  }
}

int TopWidget::file_saveas()
{
  QString defaultPath;
  QString saveFilename = fileName;

  saveFilename.detach();
  // Be nice and tidy and have all the files in a nice easy to find place.
  defaultPath = KApplication::localkdedir();

  defaultPath += "/share/apps/korganizer/";

  //  fileName = QFileDialog::getSaveFileName(defaultPath, "*.vcs", this);
  fileName = KFileDialog::getSaveFileName(defaultPath, "*.vcs", this);

  // If file dialog box was cancelled (trap for empty string)
  if(fileName.isEmpty()) {

    fileName = saveFilename;
    return 1;
  }
  
  QFileInfo tf("",fileName);
  if(!tf.isDir()) {
    
    // Force the extension for consistency.
    if(fileName.size() > 3) {
      QString e = fileName.right(4);
      // Extension ending in '.vcs' or anything else '.???' is cool.
      if(e != ".vcs" && e.right(1) != ".")
	// Otherwise, force the default extension.
	fileName += ".vcs";
    }
    
    calendar->save(fileName);
    unsetModified();
    return 0;
  }
  
  // Spit out an indication that the file was not saved
  // either a bad file name was specified or the operation
  // was cancelled.
  QString msgStr;
  msgStr += i18n("KOrganizer had a problem saving your file:\n");
  msgStr += fileName + "\n";
  msgStr += i18n("Please check that the directory exists and you have\n"
		 "permission to write to it, and try again.");
  
  QMessageBox::warning(this,i18n("KOrganizer Error"), msgStr);

  fileName = saveFilename;
  
  // recursive call; prompt for a proper filename.
  return file_saveas();
}



void TopWidget::file_close()
{
  int error = 0;
  if (isModified() && !calendar->autoSave()) {
    
    switch(QMessageBox::warning(this, i18n("KOrganizer Warning"),
				i18n("The calendar you are closing has been modified.\n"
				     "Do you want to save your changes?"),
				i18n("&Save"),
				i18n("&Don't Save"),
				i18n("&Cancel"),
				0,2)) {
      // Save changes...
    case 0:
      if ((error = file_save())) {
	// some sort of error saving, bail.
	break;
      } else {
	calendar->close();
	fileName = "";
      }
      break;
      // Don't save changes...
    case 1:
      calendar->close();
      fileName = "";
      unsetModified();
      // return;
      // break;
      // Cancel operation...
    case 2: 
      return;
      break;

    } // switch

  } else if (isModified()) {
    if ((error = file_save())) {
      // some sort of error saving, bail.
      return;
    } else {
      calendar->close();
      fileName = "";
    }
  } else {
    // Calendar not modified, close without confirmation...
    calendar->close();
    fileName = "";
    unsetModified();
    
  }
  
  if (error) {
    // do nothing else, some sort of error saving.
    return;
  }

  updateView();
  
  // Should we really remove the current window when we close something?
  // right now we don't, but that isn't exactly consistent.
  if( windowList.count() > 1 )
    close();
}

int TopWidget::file_save()
{
  if (calendar->autoSave() && fileName.isEmpty()) {
    QString fn = KApplication::localkdedir().copy();
    
    fn += "/share/apps/korganizer/default.vcs";
    int i = 1;
    if (QFile::exists(fn.data()))
      do {
	QString nStr;
	nStr.setNum(i++);
	fn = KApplication::localkdedir().copy();
	fn += "/share/apps/korganizer/default";
	fn += nStr;
	fn += ".vcs";
      } while (QFile::exists(fn.data()));

    fileName = fn;
  }

  // has this calendar been saved before? 
  if (!fileName.isEmpty()) {
    add_recent_file(fileName);
    if (calendar->save(fileName)) {
      QString msgStr;
      msgStr += i18n("KOrganizer had a problem saving your file:\n");
      msgStr += fileName + "\n";
      msgStr += i18n("Please check that the directory exists and you have\n"
		     "permission to write to it, and try again.");

      QMessageBox::warning(this,i18n("KOrganizer Error"), msgStr);
      return 1;
    } else {
      unsetModified();
      return 0;
    }
  }
  // force it to 'save as'
  else
    return file_saveas();
}

void TopWidget::file_quit()
{
  QString tmpfn;

  TopWidget *ptopwidget;
  for (ptopwidget=windowList.first(); ptopwidget != 0; ptopwidget=windowList.next()) {
    if (ptopwidget->isModified()) 
      // if any other windows have modified calendars, we need to
      // set the dirty bit.
      setModified();	
  }
  
  if (!isModified()) {
    writeSettings();
    emit closingDown();
  } else { // modified
    // work around the fact that file_close loses the file name
    tmpfn = fileName;
    file_close(); // see if they want to save the file they are losing
    if (fileName != "")
      // they hit cancel
      return;
    fileName = tmpfn;
    writeSettings();
    emit closingDown();
  }
  if (windowList.count() == 1)
    kapp->quit();
  else
    emit closed(this); // suicide
}

void TopWidget::edit_cut()
{
  KDPEvent *anEvent;
  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  case MONTHVIEW:
    anEvent= monthView->getSelected();
    break;
  default:
    QMessageBox::warning(this,i18n("KOrganizer error"),
			 i18n("Unfortunately, we don't handle cut/paste for\n"
			      "that view yet.\n"));
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }
  calendar->cutEvent(anEvent);
  changeEventDisplay(anEvent, EVENTDELETED);
}

void TopWidget::edit_copy()
{
  KDPEvent *anEvent;
  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  case MONTHVIEW:
    anEvent = monthView->getSelected();
    break;
  default:
    QMessageBox::warning(this,i18n("KOrganizer error"),
			 i18n("Unfortunately, we don't handle cut/paste for\n"
			      "that view yet.\n"));
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }
  calendar->copyEvent(anEvent);
}

void TopWidget::edit_paste()
{
  KDPEvent *pastedEvent;
  QDateList tmpList(FALSE);

  tmpList = dateNavigator->getSelected();
  pastedEvent = calendar->pasteEvent(tmpList.first());
  changeEventDisplay(pastedEvent, EVENTADDED);
}

void TopWidget::edit_options()
{
//  OptionsDialog *optionsDlg;
  
  optionsDlg->exec();
}

// Configuration changed as a result of the options dialog.
// I wanted to break this up, in order to avoid inefficiency 
// introduced as we were ALWAYS updating configuration
// in multiple widgets regardless of changed in information.
void TopWidget::updateConfig()
{
  emit configChanged();
  readSettings(); // this is the best way to assure that we have them back
  if (calendar->autoSave() && !autoSaveTimer->isActive()) {
    checkAutoSave();
    autoSaveTimer->start(1000*60);
  }
  if (!calendar->autoSave())
    autoSaveTimer->stop();

  // static slot calls here
  KDPEvent::updateConfig();
}

/****************************************************************************/

void TopWidget::newEvent(QDateTime fromHint, QDateTime toHint)
{
  // create empty event win
  EditEventWin *eventWin = new EditEventWin( calendar );

  // put in date hint
  eventWin->newEvent(fromHint, toHint);

  // connect the win for changed events
  connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)), 
  	  SLOT(changeEventDisplay(KDPEvent *, int)));
  connect(eventWin, SIGNAL(closed(QWidget *)),
	  SLOT(cleanWindow(QWidget *)));
  connect(this, SIGNAL(closingDown()),
	  eventWin, SLOT(cancel()));
  // show win
  eventWin->show();
}

void TopWidget::newEvent(QDateTime fromHint, QDateTime toHint, bool allDay)
{
  // create empty event win
  EditEventWin *eventWin = new EditEventWin( calendar );

  // put in date hint
  eventWin->newEvent(fromHint, toHint, allDay);

  // connect the win for changed events
  connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)), 
  	  SLOT(changeEventDisplay(KDPEvent *, int)));
  connect(eventWin, SIGNAL(closed(QWidget *)),
	  SLOT(cleanWindow(QWidget *)));
  connect(this, SIGNAL(closingDown()),
	  eventWin, SLOT(cancel()));

  // show win
  eventWin->show();
}

void TopWidget::apptmnt_new()
{
  QDate from, to;

  QDateList tmpList(FALSE);
  tmpList = dateNavigator->getSelected();
  from = *tmpList.first();
  to = *tmpList.last();
  
  ASSERT(from.isValid());
  if (!from.isValid()) { // dateNavigator sometimes returns GARBAGE!
    from = QDate::currentDate();
    to = from;
  }

  config = korganizer->getConfig();
  config->setGroup("Time & Date");
  QString confStr(config->readEntry("Default Start Time", "12:00"));
  int pos = confStr.find(':');
  if (pos >= 0)
    confStr.truncate(pos);
  int fmt = confStr.toUInt();

  newEvent(QDateTime(from, QTime(fmt,0,0)),
	   QDateTime(to, QTime(fmt+1,0,0)));
}

void TopWidget::allday_new()
{

  QDate from, to;
  QDateList tmpList(FALSE);
  tmpList = dateNavigator->getSelected();

  from = *tmpList.first();
  to = *tmpList.last();

  ASSERT(from.isValid());
  if (!from.isValid()) {
    from = QDate::currentDate();
    to = from;
  }
  
  newEvent(QDateTime(from, QTime(12,0,0)),
	   QDateTime(to, QTime(12,0,0)), TRUE);
}

void TopWidget::editEvent(KDPEvent *anEvent)
{
  QDateList tmpList(FALSE);
  QDate qd;

  tmpList = dateNavigator->getSelected();
  if(anEvent!=0)
    {
      qd = *tmpList.first();
      EditEventWin *eventWin = new EditEventWin(calendar, 
						anEvent->getTodoStatus());
      eventWin->editEvent(anEvent, qd);

      // connect for changed events
      if (anEvent->getTodoStatus())
	connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)),
		todoList, SLOT(updateView()));
	connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)),
		todoView, SLOT(updateView()));

      connect(eventWin, SIGNAL(eventChanged(KDPEvent *, int)), 
      	      SLOT(changeEventDisplay(KDPEvent *, int)));
      connect(eventWin, SIGNAL(closed(QWidget *)),
	      SLOT(cleanWindow(QWidget *)));
      connect(this, SIGNAL(closingDown()),
	      eventWin, SLOT(cancel()));
      eventWin->show();
    } else {
      qApp->beep();
    }
}

void TopWidget::apptmnt_edit()
{
  KDPEvent *anEvent;
  
  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  case MONTHVIEW:
    anEvent = monthView->getSelected();
    break;
  default:
    debug("we don't handle that view yet.");
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }

  editEvent(anEvent);
}

void TopWidget::apptmnt_delete()
{
  KDPEvent *anEvent;

  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  case MONTHVIEW:
    anEvent = monthView->getSelected();
  default:
    debug(i18n("don't handle deletes from that view, yet.\n"));
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }

  deleteEvent(anEvent);
}

void TopWidget::action_deleteTodo()
{
  KDPEvent *aTodo;
  TodoView *todoList2 = (viewMode == TODOVIEW ? todoView : todoList); 

  aTodo = todoList2->getSelected();
  if (!aTodo) {
    qApp->beep();
    return;
  }
  
  KConfig *config(kapp->getConfig());
  config->setGroup("General");
  if (config->readBoolEntry("Confirm Deletes") == TRUE) {
    switch(QMessageBox::warning(this, i18n("KOrganizer Confirmation"),
				i18n("Are you sure you want to delete\n"
				     "the selected to do item?\n\n"),
				i18n("&OK"),
				i18n("&Cancel"), 0, 
				0, 1)) {
    case 0: // OK
      calendar->deleteTodo(aTodo);
      todoList2->changeEventDisplay(aTodo, EVENTDELETED);
      break;
    case 1: // Cancel
      break;
    }
  } else {
    calendar->deleteTodo(aTodo);
    todoList2->changeEventDisplay(aTodo, EVENTDELETED);
  }
}

void TopWidget::deleteEvent(KDPEvent *anEvent)
{
  if (!anEvent) {
    qApp->beep();
    return;
  }

  if (anEvent->doesRecur()) {
    switch(QMessageBox::warning(this, i18n("KOrganizer Confirmation"),
				i18n("This event recurs over multiple dates.\n"
				"Are you sure you want to delete the\n"
				"selected event, or just this instance?\n\n"),
				i18n("&All"),
				i18n("&This"),
				i18n("&Cancel"),
				0,2)) {
    case 0: // all
      calendar->deleteEvent(anEvent);
      changeEventDisplay(anEvent, EVENTDELETED);
      break;
    case 1: // add an exception
	{
	    QDate qd;
	    QDateList tmpList(FALSE);
	    tmpList = dateNavigator->getSelected();
	    qd = *(tmpList.first());
	    if (!qd.isValid()) {
		debug(i18n("no date selected, or invalid date\n"));
		qApp->beep();
		return;
	    }
	    anEvent->addExDate(qd);
	    changeEventDisplay(anEvent, EVENTDELETED);
	    break;
	}
    case -1: // escape pressed
      // ^^^ Mirko --> maybe an default would be better
    case 2: // cancel
      break;
    } // switch
  } else {
    KConfig *config(kapp->getConfig());
    config->setGroup("General");
    if (config->readBoolEntry("Confirm Deletes") == TRUE) {
      switch(QMessageBox::warning(this, i18n("KOrganizer Confirmation"),
				  i18n("Are you sure you want to delete\n"
				       "the selected event?\n\n"),
				  i18n("&OK"),
				  i18n("&Cancel"), 0, 
				  0, 1)) {
      case 0: // OK
	calendar->deleteEvent(anEvent);
	changeEventDisplay(anEvent, EVENTDELETED);
	break;
      case 1: // Cancel
	break;
      } // switch
    } else {
      calendar->deleteEvent(anEvent);
      changeEventDisplay(anEvent, EVENTDELETED);
    }
  } // if-else
}

/*****************************************************************************/

void TopWidget::action_search()
{
  if (!searchDlg) {
    searchDlg = new SearchDialog(calendar);
    connect(searchDlg, SIGNAL(closed(QWidget *)),
	    this, SLOT(cleanWindow(QWidget *)));
    connect(searchDlg, SIGNAL(editEventSignal(KDPEvent *)),
	    this, SLOT(editEvent(KDPEvent *)));
    connect(searchDlg, SIGNAL(deleteEventSignal(KDPEvent *)), 
	    this, SLOT(deleteEvent(KDPEvent *)));
  }
  searchDlg->show();
  
}

void TopWidget::action_mail()
{
  KDPEvent *anEvent;
  KoMailClient mailobject(calendar);

  switch(viewMode) {
  case LISTVIEW:
    anEvent = listView->getSelected();
    break;
  case AGENDAVIEW:
    anEvent = agendaView->getSelected();
    break;
  case MONTHVIEW:
    anEvent= monthView->getSelected();
    break;
  default:
    QMessageBox::warning(this,i18n("KOrganizer error"),
			 i18n("Can't generate mail from this view\n"));
    return;
  }
  if (!anEvent) {
    qApp->beep();
    return;
  }
  if(anEvent->attendeeCount() == 0 ) {
    qApp->beep();
    QMessageBox::warning(this,i18n("KOrganizer error"),
			 i18n("Can't generate mail:\n No attendees defined!\n"));
    return;
  }
  mailobject.emailEvent(anEvent,this);
}

void TopWidget::add_recent_file(QString recentFileName)
{
  const char *rf;

  // sanity check
  if (recentFileName.isEmpty())
    return;

  // sanity check that it exists
  if (!QFile::exists(recentFileName.data()))
    return;

  // check to see if it is already in the list.
  for (rf = recentFileList.first(); rf;
       rf = recentFileList.next()) {
    if (strcmp(rf, recentFileName.data()) == 0)
      return;  
  }

  if( recentFileList.count() < 5)
    recentFileList.insert(0,recentFileName.data());
  else {
    recentFileList.remove(4);
    recentFileList.insert(0,recentFileName.data());
  }
  if (recentPop) {
    recentPop->clear();
    for ( int i =0 ; i < (int)recentFileList.count(); i++)
      {
	recentPop->insertItem(recentFileList.at(i));
      }
    if (recentFileList.count() == 0) {
      // disable the "Open Recent" option on the file menu if necessary
      fileMenu->setItemEnabled(1, FALSE);
    }
  }
}

void TopWidget::help_contents()
{
  korganizer->invokeHTMLHelp("korganizer/korganizer.html","");
}

void TopWidget::help_about()
{
  AboutDialog *ad = new AboutDialog(this);
  ad->show();
  delete ad;
}

void TopWidget::help_postcard()
{
  PostcardDialog *pcd = new PostcardDialog(this);
  pcd->show();
  delete pcd;
}

void TopWidget::newTopWidget()
{
  TopWidget *t = new TopWidget(new CalObject, QString(""));
  connect(t, SIGNAL(closed(QWidget *)),
	  this, SLOT(cleanWindow(QWidget *)));
  t->show();
}

void TopWidget::view_list()
{
  changeView(LISTVIEW);
}

void TopWidget::view_day()
{
  QDateList tmpList(FALSE);
  tmpList = dateNavigator->getSelected();
  if (saveSingleDate != *tmpList.first()) {
    dateNavigator->selectDates(saveSingleDate);
    updateView(dateNavigator->getSelected());
  }

  agendaView->slotViewChange( KAgenda::DAY );
  changeView(AGENDAVIEW);
}

void TopWidget::view_workweek()
{
  agendaView->slotViewChange(KAgenda::WORKWEEK);
  changeView(AGENDAVIEW);
}

void TopWidget::view_week()
{
  agendaView->slotViewChange(KAgenda::WEEK );
  changeView(AGENDAVIEW);
}

void TopWidget::view_month()
{
  changeView(MONTHVIEW);
}

void TopWidget::view_todolist()
{
  changeView(TODOVIEW);
}

void TopWidget::setModified()
{
  QString tmpStr;

  // quick exit if already set
  if (modifiedFlag)
    return;

  if (! fileName.isEmpty() )
    tmpStr.sprintf(i18n("%s%s- KOrganizer"), 
		   (fileName.mid(fileName.findRev('/')+1,
				 fileName.length())).data(),
		   (calendar->autoSave() ? " " : " (modified) "));
  else
    tmpStr.sprintf(i18n("New Calendar%s- KOrganizer"),
		   (calendar->autoSave() ? " " : " (modified) "));
  
  modifiedFlag = TRUE;
  setCaption(tmpStr);
}

void TopWidget::unsetModified()
{
  QString tmpStr;

  if (!fileName.isEmpty())
    tmpStr.sprintf(i18n("%s - KOrganizer"), 
		   (fileName.mid(fileName.findRev('/')+1, 
				 fileName.length())).data());
  else
    tmpStr.sprintf(i18n("New Calendar - KOrganizer"));
  modifiedFlag = FALSE;
  setCaption(tmpStr);

  signalAlarmDaemon();
}

bool TopWidget::isModified()
{
  return modifiedFlag;
}

void TopWidget::closeEvent( QCloseEvent * )
{
  file_quit();
  emit closed(this);
}

void TopWidget::signalAlarmDaemon()
{
  QFile pidFile;
  QString tmpStr;
  pid_t pid;
  char pidStr[25];

  tmpStr.sprintf("%s/share/apps/korganizer/alarmd.pid",
		 KApplication::localkdedir().data());
  pidFile.setName(tmpStr.data());

  // only necessary if the file actually is opened
  if(pidFile.open(IO_ReadOnly)) {
    pidFile.readLine(pidStr, 24);
    pidFile.close();
    pid = atoi(pidStr);
    if (pid > 0)
      kill(pid, SIGHUP);
  }
}

inline void TopWidget::cleanWindow(QWidget *window)
{
  //window->hide(); // to make sure it's not visible.
  delete window;
  window = 0L;
}

void TopWidget::checkAutoSave()
{
  // has this calendar been saved before? 
  if (calendar->autoSave() && !fileName.isEmpty()) {
    add_recent_file(fileName);
    calendar->save(fileName);
    unsetModified();
  }
}

void TopWidget::printSetup()
{
  calPrinter->setupPrinter();
}

void TopWidget::print() 
{
  QDateList tmpDateList(FALSE);

  tmpDateList = dateNavigator->getSelected();
  calPrinter->print(CalPrinter::Month, 
		    *tmpDateList.first(), *tmpDateList.last());
}

void TopWidget::printPreview()
{
  QDateList tmpDateList(FALSE);

  tmpDateList = dateNavigator->getSelected();

  switch(viewMode) {
  case AGENDAVIEW:
    if (tmpDateList.count() == 1)
      calPrinter->preview(CalPrinter::Day,
			  *tmpDateList.first(), *tmpDateList.last());
    else if (tmpDateList.count() <= 7)
      calPrinter->preview(CalPrinter::Week,
			  *tmpDateList.first(), *tmpDateList.last());
    else
      calPrinter->preview(CalPrinter::Month,
			  *tmpDateList.first(), *tmpDateList.last());
    break;
  case LISTVIEW:
    calPrinter->preview(CalPrinter::Day,
			*tmpDateList.first(), *tmpDateList.last());
    break;
  case MONTHVIEW:
  default:
    calPrinter->preview(CalPrinter::Month, 
			*tmpDateList.first(), *tmpDateList.last());
    break;
  }
}
