/*
    $Id: kticker.cpp,v 1.20 1999/02/07 03:35:31 dwerryh Exp $

    kTicker, a news headlines display program.
    
    Copyright (C) 1998 Paul Dwerryhouse   paul@xenu.ee.mu.oz.au

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// Define this if links don't come up properly when you click on them.
// Thanks to Chi-hsuan Lai (lai@physics.utexas.edu) for this patch.

#undef STRANGE_DESKTOP_PROBLEM

#include <kfm.h>
#include <kapp.h>
#include <kconfig.h>
#include <kcontainer.h>  
#include <kbutton.h>  


#include <qlabel.h>
#include <qpushbt.h>
#include <qtimer.h>
#include <qpopmenu.h>
#include <qpalette.h> 
#include <qdatetm.h> 
#include "kticker.h"
#include "kticker.moc"
#include "NewsSevenAM.h"
#include "NewsBBC.h"
#include "NewsSlashDot.h"
#include "NewsFreshmeat.h"
#include "NewsITN.h"
#include "NewsCEO.h"
#include "ItemList.h"
#include "TickerItem.h"
#include "OptBox.h"
#include "TickWidget.h"

extern "C" {
#include <stdio.h>
#include <unistd.h>
} 

KApplication *mykapp;
item *getheadlines(char *);
char *getline(FILE *);
KConfig *myconfig;
QString newsfeed;
QTime currtime;
int changetime;
KButton *button;

int main(int argc, char **argv)
{
    int retval;

    mykapp = new KApplication(argc, argv, "kticker");
    myconfig = mykapp->getConfig();
    newsfeed = myconfig->readEntry("newsfeed","bbc");
    changetime = myconfig->readNumEntry("changetime",3000);
    if (changetime<3000) {
    	changetime=3000;
	myconfig->writeEntry("changetime",changetime);
    }

    KTicker *s = new KTicker();

    mykapp->setMainWidget(s);
    s->show();
    retval = mykapp->exec();

    return retval;
}

KTicker::KTicker() : QWidget () {
    	kfm=0;
	havedata=0;
	headlines=0; current=0;
	il=0;
	currentItem=0;

	QColorGroup qcg ( QColor(255,255,255), 
		QColor(0,0,0), QColor(0,0,0), 
		QColor(0,0,0), QColor(0,0,0),
		QColor(255,255,255), QColor(0,0,0));

	QPalette qp (qcg,qcg,qcg);

	feedpopmenu = new QPopupMenu(0, "FeedPop");
	feedpopmenu->setCheckable( TRUE );
	feedpopmenu->insertItem("BBC");
	feedpopmenu->insertItem("7am.com");
	feedpopmenu->insertItem("SlashDot.Org");
	feedpopmenu->insertItem("Freshmeat.net");
	feedpopmenu->insertItem("ITN");
	feedpopmenu->insertItem("Central Europe Online");

	feedpopmenu->connectItem(0,this,SLOT(restartBBC()));
	feedpopmenu->connectItem(1,this,SLOT(restartSevenAM()));
	feedpopmenu->connectItem(2,this,SLOT(restartSlashDot()));
	feedpopmenu->connectItem(3,this,SLOT(restartFreshmeat()));
	feedpopmenu->connectItem(4,this,SLOT(restartITN()));
	feedpopmenu->connectItem(5,this,SLOT(restartCEO()));

	popmenu = new QPopupMenu(0,"Pop");
	popmenu->insertItem(i18n("Open"));
	popmenu->insertSeparator();
	popmenu->insertItem(i18n("News Feed"), feedpopmenu);
	popmenu->insertSeparator();
	popmenu->insertItem(i18n("Options..."));
	popmenu->insertItem(i18n("Save Options"));
	popmenu->insertItem(i18n("Help"));
	popmenu->insertItem(i18n("About"));
	popmenu->insertSeparator();
	popmenu->insertItem(i18n("Quit"));
	
	kttest = new OptBox(myconfig,0,"Options");
	connect(kttest,SIGNAL(changedopts(KConfig *)),this
		,SLOT(slotChangeOpts(KConfig *)));


	popmenu->connectItem(0,this,SLOT(slotOpenLink()));
	// News Feed Menu is Here
	popmenu->connectItem(4,this,SLOT(configOptions()));
	popmenu->connectItem(5,this,SLOT(saveOptions()));
	popmenu->connectItem(6,this,SLOT(getHelp()));
	popmenu->connectItem(7,this,SLOT(aboutDlg()));
	popmenu->connectItem(9,this,SLOT(quitApp()));

	setMaximumSize(420,40);
	tw = new TickWidget(this);
	tw->setText(i18n("Loading data..."));
	connect(tw,SIGNAL(clicked(QMouseEvent *)),this
		,SLOT(slotClickLink(QMouseEvent *)));
	connect(tw,SIGNAL(stopTicker()),this
		,SLOT(slotStopTicker()));
	connect(tw,SIGNAL(startTicker()),this
		,SLOT(slotStartTicker()));
	connect(tw,SIGNAL(fwdTicker()),this
		,SLOT(slotFwdItem()));
	tw->show();

	currtime.start();
	
	if (newsfeed == "7am") {
		news = new NewsSevenAM();
		((NewsSevenAM *) news)->getNews();
		setCheckedFeed(1);
	} else if (newsfeed == "slashdot") {
		news = new NewsSlashDot();
		((NewsSlashDot *) news)->getNews();
		setCheckedFeed(2);
	} else if (newsfeed == "freshmeat") {
	        news = new NewsFreshmeat();
	        ((NewsFreshmeat *) news)->getNews();
		setCheckedFeed(3);
	} else if (newsfeed == "itn") {
		news = new NewsITN();
		((NewsITN *) news)->getNews();
		setCheckedFeed(4);
	} else if (newsfeed == "ceo") {
		news = new NewsCEO();
		((NewsCEO *) news)->getNews();
		setCheckedFeed(5);
	} else {
		news = new NewsBBC();
		((NewsBBC *) news)->getNews();
		setCheckedFeed(0);
	}
	setSiteCaption();
	
	timer = new QTimer(this);
	connect(timer,SIGNAL(timeout()),this,SLOT(slotTimeSignal()));
	timer->start(changetime,0);
}

void KTicker::setText(char *s) {
	tw->setText(s);
}

void KTicker::slotTimeSignal() {
	
	if (!news) return;

	// If data hasn't been loaded, then just exit.
	if (!news->isReady()) return;

	// If we haven't read the data yet, read it.
	if (!il) il=news->getItemList();

	if (!il->getNum()) {
		tw->setText(i18n("No connection"));
		//setText(i18n("No connection"));
		return;
	}

	// Get the next item.
	currentItem=il->next();

	// Check to see if we're at the end of the list of items.
	// If yes, then load up new data.
	if (!currentItem) {
		il = news->getItemList();;	

		// Rob Malda of SlashDot fame requests that tickers
		// suck down the SlashDot news no more often than once
		// every thirty minutes. I've made it once every hour,
		// to be on the safe side. Be nice and leave it like
		// this -- otherwise he might take it away from us.
		// (Slashdot never seems to get updated more than once
		// an hour anyway).
		if (newsfeed != "slashdot" || (currtime.elapsed())>3600000) {
			il = 0;	
			news->getNews();
			tw->setText(i18n("Loading new data..."));
			//setText(i18n("Loading new data..."));
			currtime.restart();
		}
		return;
	}

	// Write the current item's headline onto the display
	tw->setText(currentItem->getHeadline());
	//setText(currentItem->getHeadline());
}

// This routine is called when the kticker is clicked on.
void KTicker::slotClickLink(QMouseEvent *me) {

#ifndef STRANGE_DESKTOP_PROBLEM
	KFM *webkfm;
#endif

	// If the right button was used, pop up a menu
	if (me && me->button()==RightButton) {
		popmenu->popup( mapToGlobal( me->pos() ) );
	}

	// Otherwise go get the relevent web page
	else if (currentItem && currentItem->getUrl()) {
#ifndef STRANGE_DESKTOP_PROBLEM
    		webkfm = new KFM;
    		webkfm->openURL(currentItem->getUrl());
		delete webkfm;
#else
		char cmd[160];
		sprintf(cmd, "kfmclient folder %s", currentItem->getUrl() );
		system(cmd);
#endif
	}
}

void KTicker::slotOpenLink() {
	slotClickLink(0);
}

void KTicker::slotStopTicker() {
	timer->stop();
	tw->setStopped();
}

void KTicker::slotStartTicker() {
	timer->start(changetime,0);
	tw->setPlay();
}

void KTicker::configOptions() {
	kttest->setMinimumSize(50,50);
	kttest->show();
}

void KTicker::getHelp() {
        mykapp->invokeHTMLHelp("kticker/index.html","");
}

void KTicker::aboutDlg() {
        QString msg;
        msg.sprintf(i18n("kTicker Ver 0.2.7\n\n"
                         "Author: Paul Dwerryhouse\n"
                         "paul@xenu.ee.mu.oz.au\n\n"
			 "Freshmeat support added by:\n"
			 "Boris Wedl\n"
			 "boris.wedl@kfunigraz.ac.at\n"
			 "Other code contributed by:\n"
			 "Mike Pilone\n"
			 "mpilone@gnosis.slac.com"));

        about = new KMsgBox(this,"kticker",msg,1,i18n("OK"));
	about->show();
}

void KTicker::slotChangeOpts(KConfig *cfg) {
    	changetime = cfg->readNumEntry("changetime",3000);
	timer->start(changetime,0);
}

void KTicker::saveOptions() {
	myconfig->sync();
}

void KTicker::quitApp() {
	exit(0);
}

void KTicker::restartBBC() {
	timer->stop();
	newsfeed="bbc";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	setCheckedFeed(0);
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsBBC();
	setSiteCaption();
	((NewsBBC *) news)->getNews();
	timer->start(changetime,0);
}

void KTicker::restartSevenAM() {
	timer->stop();
	newsfeed="7am";
	setCheckedFeed(1);
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsSevenAM();
	setSiteCaption();
	((NewsSevenAM *) news)->getNews();
	timer->start(3000,0);
}

void KTicker::restartSlashDot() {
	timer->stop();
	newsfeed="slashdot";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	setCheckedFeed(2);
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsSlashDot();
	setSiteCaption();
	((NewsSlashDot *) news)->getNews();
	timer->start(changetime,0);
}

void KTicker::restartFreshmeat() {
	timer->stop();
	newsfeed="freshmeat";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	setCheckedFeed(3);
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsFreshmeat();
	setSiteCaption();
	((NewsFreshmeat *) news)->getNews();
	timer->start(changetime,0);
}

void KTicker::restartITN() {
	timer->stop();
	newsfeed="itn";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	setCheckedFeed(4);
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsITN();
	setSiteCaption();
	((NewsITN *) news)->getNews();
	timer->start(3000,0);
}

void KTicker::restartCEO() {
	timer->stop();
	newsfeed="ceo";
	myconfig->writeEntry("newsfeed",newsfeed);
	myconfig->sync();
	setCheckedFeed(5);
	currentItem = NULL;
	delete news;
	il = 0;
	news = new NewsCEO();
	setSiteCaption();
	((NewsCEO *) news)->getNews();
	timer->start(3000,0);
}

void KTicker::enterEvent(QEvent *) {

	if (currentItem && currentItem->getUrl()) {
		tw->setHiColour();
	}
}

void KTicker::leaveEvent(QEvent *) {

	tw->setLoColour();
}

void KTicker::slotFwdItem() {
	
	if (!news) return;

	// If data hasn't been loaded, then just exit.
	if (!news->isReady()) return;

	// If we haven't read the data yet, read it.
	if (!il) il=news->getItemList();

	if (!il->getNum()) {
		tw->setText(i18n("No connection"));
		//setText(i18n("No connection"));
		return;
	}

	// Get the next item.
	currentItem=il->next();

	// Check to see if we're at the end of the list of items.
	// If yes, then load up new data.
	if (!currentItem) {
		il = news->getItemList();;	
		return;
	}

	// Write the current item's headline onto the display
	tw->setText(currentItem->getHeadline());
	//setText(currentItem->getHeadline());
}

void KTicker::setCheckedFeed(int id)
{
  // Uncheck all items
  feedpopmenu->setItemChecked(0, FALSE);
  feedpopmenu->setItemChecked(1, FALSE); 
  feedpopmenu->setItemChecked(2, FALSE);
  feedpopmenu->setItemChecked(3, FALSE);
  feedpopmenu->setItemChecked(4, FALSE);
  feedpopmenu->setItemChecked(5, FALSE);

  // Check the desired item
  feedpopmenu->setItemChecked(id, TRUE);
}

void KTicker::setSiteCaption() {
	
	QString cptn = "KTicker";
	if (news) {	// Safety first
		if (news->getSiteName()) {
			cptn += (" - " + news->getSiteName());
		}
	}
	setCaption((const char *)cptn);
}
