#include <kapp.h>
#include <kcolorbtn.h>

#include "OptBox.h"
#include "OptBox.moc"

KColorButton *fgcolorbox;
KColorButton *hifgcolorbox;
KColorButton *bgcolorbox;
KColorButton *hibgcolorbox;
QLabel *fglabel,*hifglabel,*bglabel,*hibglabel;

OptBox::OptBox(KConfig *cfg,QWidget *parent, const char *name) : KConfigWidget( parent, name )
{
	int start;
	config = cfg;
	QColor fgcol= config->readColorEntry("fgcol",new QColor(255,255,255));
	QColor hifgcol= config->readColorEntry("hifgcol",new QColor(255,255,255));
	QColor bgcol= config->readColorEntry("bgcol",new QColor(255,255,255));
	QColor hibgcol= config->readColorEntry("hibgcol",new QColor(255,255,255));

        changetime=config->readNumEntry("changetime",3000);
	start=((changetime/1000-3)*2000/597);

	layout = new QVBoxLayout(this, 3);
	optionsbox = new QGroupBox(i18n("Options"),this);
	slider = new KSlider(0,2000,10,start,KSlider::Horizontal,optionsbox);
	slider->setSteps ( 125, 125 );
	slider->setTickmarks ( static_cast<QSlider::TickSetting>(QSlider::Below) );
	slider->setTickInterval ( 250 );
	slider->setTracking( true );

	slider_label = new QLabel(slider,i18n("Delay"),optionsbox);

	slider->adjustSize();
	slider_label->adjustSize();
	connect( slider, SIGNAL(sliderMoved(int)), SLOT(ChangeTime(int)));

        fgcolorbox = new KColorButton(fgcol,optionsbox,0);
	fglabel = new QLabel(slider,i18n("Text"),optionsbox);
        hifgcolorbox = new KColorButton(hifgcol,optionsbox,0);
	hifglabel = new QLabel(slider,i18n("Text Hi"),optionsbox);
        bgcolorbox = new KColorButton(bgcol,optionsbox,0);
	bglabel = new QLabel(slider,i18n("Background"),optionsbox);
        hibgcolorbox = new KColorButton(hibgcol,optionsbox,0);
	hibglabel = new QLabel(slider,i18n("Background Hi"),optionsbox);

	optionsbox->adjustSize();
	optionsbox->setMinimumSize(100, slider->sizeHint().height()+
			slider_label->sizeHint().height()+38);

	optionsbox->setMinimumSize(100, slider->sizeHint().height()+ 30 +
	 	fgcolorbox->sizeHint().height() + 30 +
	 	hifgcolorbox->sizeHint().height() + 10 +
	 	bgcolorbox->sizeHint().height() + 10 +
		hibgcolorbox->sizeHint().height() + 10);


	layout->addWidget(optionsbox, 3);

	buttonbox = new QFrame(this);
	hlayout = new QHBoxLayout(buttonbox, 3);

	ok = new QPushButton(buttonbox); ok->setText(i18n("OK"));
	hlayout->addWidget(ok, 3);
	connect( ok, SIGNAL(clicked()), SLOT(okclicked()) );

	apply = new QPushButton(buttonbox); apply->setText(i18n("Apply"));
	hlayout->addWidget(apply, 3);
	connect( apply, SIGNAL(clicked()), SLOT(applyclicked()) );

	cancel = new QPushButton(buttonbox); cancel->setText(i18n("Cancel"));
	hlayout->addWidget(cancel, 3);
	connect( cancel, SIGNAL(clicked()), SLOT(cancelclicked()) );

	layout->addWidget(buttonbox, 3);

	layout->activate();
	setGeometry(0,0,210,optionsbox->height()+ok->height());
};

/*
void main(int argc,char **argv) {
	int retval;
	KApplication *mykapp;
	mykapp = new KApplication(argc, argv, "ktest");	
	OptBox *ob = new OptBox(NULL,"test");
	mykapp->setMainWidget(ob);
	ob->setMinimumSize(50,50);
	ob->show();
	retval = mykapp->exec();
	return retval;
}
*/

void OptBox::loadSettings() {};
void OptBox::applySettings() {};

void OptBox::resizeEvent(QResizeEvent *e) {
	KConfigWidget::resizeEvent(e);

	slider_label->adjustSize();
	slider->move(30,30);
	slider_label->move(30+slider->width()+5,30);

	fgcolorbox->adjustSize();
	fgcolorbox->move(slider->x(),slider->y()+40);
	fglabel->adjustSize();
	fglabel->move(fgcolorbox->x()+fgcolorbox->width()+5,slider->y()+45);

	hifgcolorbox->adjustSize();
	hifgcolorbox->move(slider->x(),fgcolorbox->y()+40);
	hifglabel->adjustSize();
	hifglabel->move(hifgcolorbox->x()+hifgcolorbox->width()+5,fgcolorbox->y()+45);

	bgcolorbox->adjustSize();
	bgcolorbox->move(slider->x(),hifgcolorbox->y()+40);
	bglabel->adjustSize();
	bglabel->move(bgcolorbox->x()+bgcolorbox->width()+5,hifgcolorbox->y()+45);

	hibgcolorbox->adjustSize();
	hibgcolorbox->move(slider->x(),bgcolorbox->y()+40);
	hibglabel->adjustSize();
	hibglabel->move(hibgcolorbox->x()+hibgcolorbox->width()+5,bgcolorbox->y()+45);
}

void OptBox::ChangeTime(int value)
{
	int t;
	t=1000*((value*597)/2000 + 3);
	changetime=t;
}

void OptBox::okclicked() {
        /* config->writeEntry("changetime",changetime);
        emit(changedopts(config)); */
	applyclicked();
        close();
}			

void OptBox::applyclicked() {
        config->writeEntry("changetime",changetime);
	config->writeEntry("fgcol",fgcolorbox->color());
	config->writeEntry("hifgcol",hifgcolorbox->color());
	config->writeEntry("bgcol",bgcolorbox->color());
	config->writeEntry("hibgcol",hibgcolorbox->color());
        emit(changedopts(config));
}
