
/***************************************************************

     Requires the Qt and KDE widget libraries, available at no cost at
     http://www.troll.no and http://www.kde.org respectively

     Copyright (C) 1997, 1998 Fester Zigterman ( fzr@dds.nl )

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************/

#include "kpropspin.h"
#include "kpropspin.moc"
#include <kapp.h>

KPropSpin::KPropSpin( QWidget *parent, const char *text, int perc, const char *key, const char *group, KConfig *config, const char *name )
	: QLabel( text, parent,name )
{
	setKConfig( config );
	setGroup( group );
	setKey( key );

	setFontPropagation( QWidget::SameFont );
	spinBox = new QSpinBox( this );
	percentage = perc;

	sizeUpdate();
}

KPropSpin::~KPropSpin()
{
	delete spinBox;
}

void KPropSpin::sizeUpdate()
{
	QFontMetrics fm( font() );
	int h=fm.height();
	
	setFixedHeight( h + 8 );
	spinBox->setFixedHeight( h + 4 );
}

void KPropSpin::resizeEvent( QResizeEvent *rev )
{
	int w = width()*percentage/100;
	spinBox->setGeometry( width() - w - 2, 2, w - 4, 100 );
}

void KPropSpin::fontChange( const QFont & )
{
	sizeUpdate();
}

QSpinBox *KPropSpin::getSpinBox()
{
	return spinBox;
}

const char *KPropSpin::getContents()
{
	return spinBox->text();
}

void KPropSpin::setContents( int value )
{
	spinBox->setValue( value );
}

void KPropSpin::setContents( const char *s )
{
	spinBox->setValue( QString( s ).toInt() );
}

void KPropSpin::setConfig()
{
	if( ConfigObject )
	{
		ConfigObject->setGroup( Group );
		//		printf("kpropspin: group=%s key=%s\n",ConfigObject->group(), Key.data() );
		if( Key.data() != 0 )
			ConfigObject->writeEntry( Key.data(), getContents() );
		else printf("kpropspin: Null key not allowed\n");
	}
}

void KPropSpin::getConfig()
{
	ConfigObject->setGroup( Group );
	int value = ConfigObject->readNumEntry( Key.data() );
	setContents( value );
	//	printf("kpropspin: reading config %s = %s\n",Key.data(), s.data() );

}

void KPropSpin::setKey( const char *key )
{
	Key=key;
}

void KPropSpin::setGroup( const char *group )
{
	Group= group;
}

void KPropSpin::setKConfig( KConfig *config )
{
	if( config == 0 )
		ConfigObject = kapp->getConfig();
	else 
		ConfigObject=config;
}

KConfig *KPropSpin::getKConfig()
{
	return ConfigObject;
}

const char *KPropSpin::getKey()
{
	return Key.data();
}

const char *KPropSpin::getGroup()
{
	return Group.data();
}
