// QaThumbnailListview.cpp
//
// Jan Borsodi <jbfdata@online.no>
// Last change: Time-stamp: <08-Feb-1999 05:30:49 amos>
//
//

#include "QaThumbnailListview.hpp"
#include <qframe.h>
#include <iostream.h>
#include <qapplication.h>
#include <qdir.h>
#include <qlabel.h>
#include <qprogressbar.h>
#include <time.h>


#include "qmimageviewbrowser.h"

extern QProgressBar		*g_pProgress;
extern QLabel			*g_pSmallInfo;
extern QLabel			*g_pLargeInfo;
extern QmImageViewBrowser	*g_pImageViewBrowser;
extern QPixmap			*folderIcon; // from qminfoitem.cpp


/*!
  \class QaThumbnailItem QaThumbnailItem.hpp
  \brief

*/

/*!
  Default constructor
*/

QaThumbnailItem::QaThumbnailItem( const QaThumbnailListview *parent )
	: Selected( false )
{
	Parent = parent;
}

QaThumbnailItem::~QaThumbnailItem()
{
}

void QaThumbnailItem::setSelected( bool s )
{
	if ( s == Selected )
		return;
	Selected = s;
}

void QaThumbnailItem::setText( const char *t )
{
	Text = t;
}


/*!
  \class QaThumbnailPreview QaThumbnailPreview.hpp
  \brief

*/

/*!
  Default constructor
*/


QaThumbnailPreview::QaThumbnailPreview( const QaThumbnailListview *parent, QFileInfo *info )
	: QaThumbnailItem( parent ), Size( 0, 0 )
{
	LWidth = 1;
	TSpace = 1;
	TFSpace = 1;
	FInfo = info;
}

QaThumbnailPreview::QaThumbnailPreview( const QaThumbnailListview *parent, QFileInfo *info, const QPixmap &p )
	: QaThumbnailItem( parent ), Size( 0, 0 ), Pix( p )
{
	LWidth = 1;
	TSpace = 1;
	TFSpace = 1;
	FInfo = info;
}

QaThumbnailPreview::~QaThumbnailPreview()
{
}

void QaThumbnailPreview::setSize( const QSize &s )
{
	if ( Size == s )
		return;
	Size = s;
}

void QaThumbnailPreview::setLineWidth( const int s )
{
	if ( LWidth == s )
		return;
	LWidth = s;
}

void QaThumbnailPreview::setTextFramespace( const int s )
{
	if ( s == TSpace )
		return;
	TSpace = s;
}

void QaThumbnailPreview::setTextSpace( const int s )
{
	if ( s == TFSpace )
		return;
	TFSpace = s;
}

QPoint QaThumbnailPreview::pixmapPos() const
{
	int w, h;
	int wd, hg;
	QRect r = pixmapRect();
	w = Pix.width();
	h = Pix.height();
	wd = r.width();
	hg = r.height();
	return QPoint( r.x() + (wd - w)/2, r.y() + (hg - h)/2 );
}

QRect QaThumbnailPreview::pixmapRect() const
{
	int fsize = filenameFrameRect().size().height();
	return QRect( QPoint( lineWidth(), lineWidth() ),
				   QSize( width() - 2*lineWidth(), height() - fsize - 2*lineWidth() - 1 - textFramespace() ) );
}

QRect QaThumbnailPreview::pixmapFrameRect() const
{
	int fsize = filenameFrameRect().size().height();
	return QRect( QPoint( 0, 0 ),
				   QSize( width(), height() - fsize - 1 - textFramespace() ) );
}

QRect QaThumbnailPreview::filenameRect() const
{
	int fsize = parent()->fontMetrics().height();
	return QRect( QPoint( textFramespace() + lineWidth() + textSpace(), height() - lineWidth() - textFramespace() - fsize ),
				  QSize( width() - textFramespace()*2 - lineWidth()*2 - textSpace()*2, fsize ) );
}

QRect QaThumbnailPreview::filenameFrameRect() const
{
	int fsize = parent()->fontMetrics().height() + textSpace()*2 + lineWidth()*2;
	return QRect( QPoint( textFramespace(), height() - fsize ),
				  QSize( width() - textFramespace()*2, fsize ) );
}

void QaThumbnailPreview::setPixmap( const QPixmap &p )
{
	ASSERT( ! p.isNull());

			// -2 to make a little border around the scaled pixmap.
    
	double x = static_cast<double>(pixmapRect().width() - 2) / p.width();
	double y = static_cast<double>(pixmapRect().height() - 2) / p.height();
	double Zoom;

	Zoom = (x > y) ? y : x;
    
	QWMatrix m;
	m.scale(Zoom, Zoom);
	Pix = p.xForm(m);
}

void QaThumbnailPreview::paint( QPainter *p )
{
	QRect textrec = filenameFrameRect();

	p->fillRect( textrec, QColor( 156, 153, 255 ) );
	qDrawShadePanel( p, textrec, qApp->palette()->active(), true, lineWidth() );
	p->drawText( filenameRect(), AlignHCenter | AlignBottom, text() );

	p->fillRect( pixmapRect(), qApp->palette()->active().background() );
	p->drawPixmap( pixmapPos(), Pix );
	qDrawShadePanel( p, pixmapFrameRect(), qApp->palette()->active(), isSelected(), lineWidth() );
}

/*!
  \class QaThumbnailListview QaThumbnailListview.hpp
  \brief

*/

/*!
  Default constructor
*/

QaThumbnailListview::QaThumbnailListview( QWidget *parent, const char *name )
	: QScrollView( parent, name ), ItemSpace( 5, 5 ), ItemSize( 50, 50 ), ItemMatrix( 1, 1 ), dirty( false )
{
	setFrameStyle( QFrame::Sunken | QFrame::Panel );
	resizeContents( viewportSize().width(), 500 );
	viewport()->setBackgroundMode( PaletteMidlight );
	ItemList.setAutoDelete( true );
	SelectList = new QList<QaThumbnailItem>;
	SelectList->setAutoDelete( false );
}

void QaThumbnailListview::setDirectory( const QFileInfoList &f )
{
	DirList  = f;
	DirCached = false;
}

void QaThumbnailListview::scanDirectory()
{
	if ( !directoryCached() )
	{
		DirCached = true;
		ItemList.clear();
		viewport()->update();
		QPixmap pix;
		QPixmap DirImage(*folderIcon);
		QFileInfoListIterator iter( DirList );
		QFileInfo *fi;

		QString info;
		int ImageCount = 0;
		// Init the progress bar
		g_pProgress->setTotalSteps(iter.count());
		g_pProgress->reset();
		time_t starttime = time(0);
		for(fi = iter.toFirst(); fi != 0; fi = ++iter)
		{
			info.sprintf("Generating thumbnail picture for %s",
					  (const char *)fi->fileName());
			g_pLargeInfo->setText((const char *)info);

			g_pProgress->setProgress(++ImageCount);
			info.sprintf("%d/%d", ImageCount, iter.count());
			g_pSmallInfo->setText((const char *)info);

			QString s = fi->fileName();
			QaThumbnailPreview *item = new QaThumbnailPreview( this, fi );
			item->setSize( QSize( 80, 80 ) );
			item->setText( s );
			if ( !fi->isDir() )
			{
				pix.load( fi->filePath() );
				item->setPixmap( pix );
			}
			else
				item->setPixmap( DirImage );
			insertItem( item );
			qApp->processEvents();
		}
		time_t stoptime = time(0);
		info.sprintf( "%d image(s) loaded in %d secs",
					  ImageCount, stoptime - starttime );
		g_pLargeInfo->setText((const char *)info);
	}
}

bool QaThumbnailListview::directoryDiffers( const QDir & )
{
//  	const QFileInfoList *silist = d.entryInfoList();
//  	const QFileInfoList *dilist = Dir.entryInfoList();
//  	if ( silist->count() == dilist->count() )
//  	{
//  		QFileInfoListIterator siter( *silist );
//  		QFileInfoListIterator diter( *dilist );
//  		QFileInfo *sfi = diter.toFirst();
//  		QFileInfo *sfi = siter.toFirst();
//  		while ( sfi != 0 )
//  		{
//  			if ( *sfi != *dfi )
//  				return true;
//  			sfi = ++siter;
//  			dfi = ++diter;
//  		}
//  		return false;
//  	}
//  	else
		return true;
}


/*!
  Destroys the object
*/

QaThumbnailListview::~QaThumbnailListview()
{
}


/*!
  Returns the size of the portion of the contents you really see.
*/

QSize QaThumbnailListview::viewportSize()
{
	int x, y;
	x = verticalScrollBar()->isVisible() ? verticalScrollBar()->width() : 0;
	x += frameStyle() != QFrame::NoFrame ? lineWidth()*2 : 0;
	x += leftMargin() + rightMargin();

	y = horizontalScrollBar()->isVisible() ? horizontalScrollBar()->width() : 0;
	y += frameStyle() != QFrame::NoFrame ? lineWidth()*2 : 0;
	y += topMargin() + bottomMargin();
	return QSize( width()  - x, height() - y );
}

/*!
  Sets the size of the space between items
*/

void QaThumbnailListview::setItemSpaceSize( const QSize &s )
{
	ItemSpace = s;
	update();
}

void QaThumbnailListview::recalcMatrix()
{
	int num = 1;
	// Calculate max number of items on a line
	while ( (num+2)*itemSpaceSize().width() + (num+1)*maxItemSize().width() < viewportSize().width() )
		num++;
	if ( ItemMatrix.width() != num )
	{
		ItemMatrix.setWidth( num );
		dirty = true;
	}
	int num2 = ( ItemList.count() + num - 1 ) / num;
	if ( ItemMatrix.height() != num2 )
	{
		ItemMatrix.setHeight( num2 );
		dirty = true;
	}
}


void QaThumbnailListview::insertItem( QaThumbnailItem *item, int index )
{
	if ( index < 0 )
		ItemList.append( item );
	else
		ItemList.insert( index, item );

	ItemSize = ItemSize.expandedTo( item->size() );
	recalcMatrix();

	QSize maxsize = maxItemSize();
	QSize isize = itemSpaceSize();

	int ypos = isize.height();
	int xpos = isize.width();

	// viewport rectangle
	QRect cr = QRect( QPoint( 0, 0 ), viewportSize() );
	// item rectangle
	QRect ir;
	// intersected rectangle
	QRect dr;

	int num = ItemList.count() - 1;
	int xm = num % ItemMatrix.width();
	int ym = num / ItemMatrix.width();
	xpos = xm*maxsize.width() + (xm+1)*isize.width();
	ypos = ym*maxsize.height() + (ym+1)*isize.height();

	ir = QRect( QPoint( xpos - contentsX(), ypos - contentsY() ), maxsize );
	dr = cr.intersect( ir );

	if ( !dr.isEmpty() )
	{
		viewport()->update( ir );
	}

	resizeContents( viewportSize().width(),
					(ItemMatrix.height() + 1)*itemSpaceSize().height() + ItemMatrix.height()*maxItemSize().height() );
}

QaThumbnailItem *QaThumbnailListview::itemAt( int x, int y )
{
	unsigned int num = y*ItemMatrix.width() + x;
	if ( num < ItemList.count() )
		return ItemList.at( num );
	else
		return 0;
}


/*!
  Overidden function from QScrollView, takes care of the drawing
*/

void QaThumbnailListview::drawContentsOffset( QPainter *p, int ox, int oy, int cx, int cy, int cw, int ch )
{
	int xpos, ypos;

	QSize maxsize = maxItemSize();
	QSize isize = itemSpaceSize();

	ypos = isize.height();
	xpos = isize.width();

	// viewport rectangle
	QRect cr = QRect( QPoint( cx - ox, cy - oy ), QSize( cw, ch ) );
	// item rectangle
	QRect ir;
	// intersected rectangle
	QRect dr;

	unsigned int maxitems = ItemList.count();
	QListIterator<QaThumbnailItem> itemi( ItemList );
	for ( unsigned int y = 0; maxitems > 0 && ypos - oy < viewportSize().height(); y++ )
	{
			for ( unsigned int x = 0; x < static_cast<unsigned int>(itemMatrix().width()) && x < maxitems; x++, ++itemi )
			{
				if ( ypos - oy + maxsize.height() + isize.height() > 0 )
				{
					ir = QRect( QPoint( xpos - ox, ypos - oy ), maxsize );
					dr = cr.intersect( ir );

					if ( !dr.isEmpty() )
					{
						p->save();
						p->setClipRegion( p->clipRegion().intersect( QRegion( ir ) ) );
						p->translate( ir.left(), ir.top() );
						itemi.current()->paint( p );
						p->restore();
					}
				}
				xpos += maxsize.width() + isize.width();
			}
			xpos = isize.width();
		ypos += maxsize.height() + isize.height();
		if ( maxitems > static_cast<unsigned int>(itemMatrix().width()) )
			maxitems -= static_cast<unsigned int>(itemMatrix().width());
		else
			maxitems = 0;
	}
}

void QaThumbnailListview::viewportMousePressEvent( QMouseEvent *e )
{
	if ( e->button() != LeftButton )
		return;
	int xpos = e->x() + contentsX();
	int ypos = e->y() + contentsY();
	int xm = ( xpos - itemSpaceSize().width())/(maxItemSize().width() + itemSpaceSize().width());
	int ym = ( ypos - itemSpaceSize().height())/(maxItemSize().height() + itemSpaceSize().height());
	if ( xm >= itemMatrix().width() || ym >= itemMatrix().height() )
		return;
	xpos = xm*maxItemSize().width() + (xm+1)*itemSpaceSize().width();
	ypos = ym*maxItemSize().height() + (ym+1)*itemSpaceSize().height();
	QaThumbnailItem *i = itemAt( xm, ym );
	if ( i == 0 )
		return;
	QRect cr = QRect( QPoint( 0, 0 ), viewportSize() );
	for ( QListIterator<QaThumbnailItem> iter( *SelectList ); iter.current(); ++iter )
	{
		iter.current()->setSelected( false );
		viewport()->update();
	}
	SelectList->clear();
	SelectList->append( i );
	QRect item( QPoint( xpos - contentsX(), ypos - contentsY() ), i->size() );
	if ( item.contains( e->pos() ) )
	{
		i->setSelected( true );
		QRect dr = cr.intersect( item );

		if ( !dr.isEmpty() )
			viewport()->update( item );
	}
}

void QaThumbnailListview::viewportMouseDoubleClickEvent( QMouseEvent *e )
{
	if ( e->button() != LeftButton )
		return;
	int xpos = e->x() + contentsX();
	int ypos = e->y() + contentsY();
	int xm = ( xpos - itemSpaceSize().width())/(maxItemSize().width() + itemSpaceSize().width());
	int ym = ( ypos - itemSpaceSize().height())/(maxItemSize().height() + itemSpaceSize().height());
	if ( xm >= itemMatrix().width() || ym >= itemMatrix().height() )
		return;
	xpos = xm*maxItemSize().width() + (xm+1)*itemSpaceSize().width();
	ypos = ym*maxItemSize().height() + (ym+1)*itemSpaceSize().height();
	QaThumbnailItem *i = itemAt( xm, ym );
	if ( i == 0 )
		return;
	QRect item( QPoint( xpos - contentsX(), ypos - contentsY() ), i->size() );
	if ( item.contains( e->pos() ) )
	{
		SelectList->clear();
		emit doubleClicked( i );
	}
}


/*!
  Makes sure the size of the contents is correct.
*/

void QaThumbnailListview::resizeEvent( QResizeEvent *e )
{
	QScrollView::resizeEvent( e );

	recalcMatrix();
	resizeContents( viewportSize().width(),
					(ItemMatrix.height() + 1)*itemSpaceSize().height() + ItemMatrix.height()*maxItemSize().height() );
}
