/* ;-*-c++-*-
 * 
 * qmbrowser.h
 * 
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <08-Feb-1999 06:52:43 amos>
 */

#ifndef QMBROWSER_H_INCLUDED
#define QMBROWSER_H_INCLUDED

#include <qimage.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qwidget.h>

#include "qmcachedpreview.h"
#include "qmfilelist.h"
#include "qmhotlist.h"
#include "qmlistview.h"
#include "qmtabbar.h"

class QmBrowser : public QWidget
{
    Q_OBJECT
    
public:
    QmBrowser(QWidget *Parent = 0,
	      const char *Name = 0);
    virtual ~QmBrowser();

	void setPath( QString dir );
signals:
    /*!
      \fn void QmBrowser::fileSelected(const char *Filename)

      This signal is emitted whenever a file is selected.
    */

    void		fileSelected(const char *Filename);

    /*!
      \fn void QmBrowser::imageSelected(const QPixmap &Pixmap)

      This signal is emitted whenever an image is selected.
    */
    
    void		imageSelected(const QPixmap &Pixmap);

    /*!
      \fn void QmBrowser::message(const char *Message)

      Emits the text \a Message for use in e.g. a statusbar.
      The signal is emitted whenever the class feels like it ;-)
    */
    
    void		message(const char *Message);

public slots:
    void		queryNextImage();
    void		queryPrevImage();
    void		setPreview(bool Enable);

// protected:
//     virtual void	focusInEvent(QFocusEvent *e);
//     virtual void	focusOutEvent(QFocusEvent *e);
    
private:
    QPixmap		ThePixmap;
    QString		CurrentDirSelection;
    QString		CurrentFileSelection;
    QTimer		*pDirTimer;
    QTimer		*pPreviewTimer;
    QWidget		*pLastActive;
    QmCachedPreview	*pPreviewImage;
    QmFileList		*pFileList;
    QmHotList		*pHotList;
    QmListView		*pDirList;
    QmTabBar		*pTabBar;
    bool		ShowPreview;

    int			DirTreeID;
    int			HotListID;
    static const int	ListDirDelay		= 350;
    static const int	ShowPreviewDelay	= 350;

private slots:
    void showPreview(const char *Filename);
    void previewTimer();
    void showImage(const QPixmap &ImageClicked);
    void selected(const char *Filename);
    void hotlistSelected(QListViewItem *pItem);
    void dirInTreeSelected(QListViewItem *pItem);
    void dirInListSelected(const char *Dirname);
    void dirInTreeHighlighted(QListViewItem *pItem);
    void dirTimer();
};

#endif // QMBROWSER_H_INCLUDED
