/* qmcachedpreview.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <12-Dec-1998 21:05:43 marius>
 */

#include "qmcachedpreview.h"

/*!
  \class QmCachedPreview qmcachedpreview.h
  \brief The QmCachedPreview class provides a resizable thumbnail widget.

  This class is similar to QmPreview, but this one caches the original pixmap passed,
  so that it can be scaled sensibly afterwards.
*/


/*!
  Constructs a cached preview.

  The \a Parent and \a Name arguments are sent to the QmPreview constructor.
*/

QmCachedPreview::QmCachedPreview(
    QWidget *Parent,
    const char *Name)
    : QmPreview(Parent, Name)
{
}


/*!

*/

QmCachedPreview::~QmCachedPreview()
{
}


/*!
  Set the preview pixmap to show \a PixmapToPreview. The pixmap is scaled
  to fit the widget's size. The aspect ratio is kept.
*/

void
QmCachedPreview::setPixmap(
    const QPixmap &PixmapToPreview)
{
    ThePixmap = PixmapToPreview;
    QmPreview::setPixmap(ThePixmap);
}


/*!
  Rescales the widget. If any pixmap is shown, it will be rescaled.
*/

void
QmCachedPreview::resizeEvent(
    QResizeEvent *)
{
    QmPreview::setPixmap(ThePixmap);
}




