/* qmdetaillist.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <01-Nov-1998 15:18:13 marius>
 */

#include <iostream.h>
#include <math.h>

#include <qdragobject.h>
#include <qpixmap.h>

#include "qmdetaillist.h"
#include "qminfoitem.h"
#include "qmdirdrag.h"

extern QPixmap *folderIcon; // from qminfoitem.cpp
extern const char *picture_xpm[]; // from qmimageview.cpp

/*!
  \class QmDetailList qmdetaillist.h
  \brief The QmDetailList provides specialized drag and drop, and context
  \brief sensitive popup menus.
*/


/*!
  Constructs the listview.
*/

QmDetailList::QmDetailList(
    QWidget *Parent,
    const char *Name)
    : QmListView(Parent, Name)
{
}


/*!

*/

QmDetailList::~QmDetailList()
{
}


/*!

 */

void
QmDetailList::mousePressEvent(
    QMouseEvent *e)
{
    QmListView::mousePressEvent(e);
    
    LastX = e->x();
    LastY = e->y();
}


/*!

 */

void
QmDetailList::mouseReleaseEvent(
    QMouseEvent *e)
{
    QmListView::mouseReleaseEvent(e);
    
    LastX = e->x();
    LastY = e->y();
}


/*!
  
*/

void
QmDetailList::mouseMoveEvent(
    QMouseEvent *e)
{
//    QListView::mouseMoveEvent(e);

//     cout << "QmDetailList::mouseMoveEvent():\n"
// 	 << " LastX = " << LastX << endl
// 	 << " LastY = " << LastY << endl
// 	 << " x()= " << e->x() << endl
// 	 << " y()= " << e->y() << endl;
    
    if(e->state() == LeftButton && currentItem() != 0)
    {
	// We must have a little "slack" before accepting a drag.
	
	if((int)fabs(LastX - e->x()) >= DragAcceptX ||
	   (int)fabs(LastY - e->y()) >= DragAcceptY)
	{
//	    cout << "QmDetailList::mouseMoveEvent(): Spawn drag\n";
	    
// 	    LastX = e->x();
// 	    LastY = e->y();

	    QmInfoItem *i = dynamic_cast<QmInfoItem *>(currentItem());

	    if(i == 0)
		return;
	    
	    const QFileInfo *fi = i->fileInfo();

	    if(fi->isDir())
	    {
		QmDirDrag *d = new QmDirDrag(fi->filePath(), viewport());
		d->setPixmap(*folderIcon);
		d->dragCopy();
	    }
	    else
	    {
		// Only drag files which we recognize as images
		
		QString s = i->text(QmInfoItem::Type);

		if( ! s.isEmpty())
		{
		    QTextDrag *d = new QTextDrag(fi->filePath(), viewport());
		    d->setPixmap(QPixmap(picture_xpm));
		    d->dragCopy();
		}
	    }
	}
    }
}







