/* qmdirtree.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <08-Feb-1999 03:51:09 amos>
 */

#include <iostream.h>

#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qapplication.h>

#include "qmdirtree.h"

/*!
  Based heavily on the dirview example by Troll Tech.
*/

/*!
*/

QmDirectory::QmDirectory(
    QmDirectory * parent,
    const char * filename)
    : QmListViewItem( parent ),
      f(filename )
{
    pTheDir = parent;
    readable = true;
}


/*!

 */

QmDirectory::QmDirectory(
    QListView * parent)
    : QmListViewItem( parent ),
      f("/")
{
    pTheDir = 0;
    readable = true;
}


/*!

 */

void
QmDirectory::setOpen(
    bool o )
{
    QListViewItem::setOpen(o);
    if(o && ! childCount())
    {
	QString s(fullName());
	QDir thisDir(s);
    thisDir.setFilter(QDir::Dirs);

	if ( thisDir.count() <= 2 )
		setExpandable( false );
	
	if( ! thisDir.isReadable())
	{
	    readable = false;
	    return;
	}

	const QFileInfoList *pFileList = thisDir.entryInfoList();
	
	if(pFileList)
	{
	    QFileInfoListIterator it(*pFileList);
	    QFileInfo *fi;
	    
	    while((fi = it.current()))
	    {
		++it;
		
		if(fi->fileName() != "." &&
		   fi->fileName() != ".." &&
		   fi->isDir())
		{
		    (void) new QmDirectory(this, fi->fileName());
		}
		qApp->processEvents();
	    }
	}
    }
}


/*!
  If a directory does not contain any directories, it will not be expandable.
*/

void
QmDirectory::setup()
{
//      QDir d(fullName());
//      d.setFilter(QDir::Dirs);

//      // If there are no directories, 2 will be returned (for `.' and `..').
    
//      if(d.count() == 2)
//  	setExpandable(false);
//      else
	setExpandable(true);
    
    QListViewItem::setup();
}


/*!

 */

QString
QmDirectory::fullName()
{
    QString s;

    if(pTheDir)
    {
	s = pTheDir->fullName();
	s.append(f.name());
	s.append("/");
    }
    else
    {
	s = "/";
    }
    
    return s;
}


/*!

 */

const char *
QmDirectory::text(
    int column) const
{
    if ( column == 0 )
	return f.name();
    else if ( readable )
	return "Directory";
    else
	return "Unreadable Directory";
}
