/* qmfilelist.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <08-Feb-1999 05:24:29 amos>
 */

#include <iostream.h>

#include <qapplication.h>
#include <qcombobox.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qkeycode.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qtabbar.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "qmfilelist.h"
#include "qminfoitem.h"
#include "qmmainwindow.h"
#include "qmpixmapbutton.h"
#include "qmpreview.h"
#include "qmtabbar.h"
#include "strings.h"

const unsigned int  folder_len = 110;
const unsigned char folder_data[] =
{
    0x47,0x49,0x46,0x38,0x39,0x61,0x0f,0x00,0x0d,0x00,0xc2,0x00,0x00,0x00,
    0x00,0x00,0x99,0x99,0x99,0xcc,0xcc,0xcc,0xff,0xff,0xff,0xff,0xff,0x00,
    0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,0x01,0x00,
    0x00,0x03,0x00,0x2c,0x00,0x00,0x00,0x00,0x0f,0x00,0x0d,0x00,0x00,0x03,
    0x33,0x38,0x13,0xcc,0xfa,0x30,0x08,0x32,0x45,0x80,0x4f,0xd2,0x6d,0x9b,
    0x0b,0x45,0x28,0x8e,0x61,0x00,0x80,0x5c,0x3a,0x99,0x60,0xa5,0x52,0x6c,
    0xf1,0xba,0xb1,0x6b,0xc3,0xa7,0x7c,0xd3,0xf9,0xce,0xc5,0xb3,0x0d,0xcb,
    0x43,0xf4,0x00,0x06,0x80,0xa4,0x72,0xb9,0x4c,0x00,0x00,0x3b
};

/*!
  \class QmFileList qmfilelist.h
  \brief The QmFileList class provides a detail (textual) list and a thumbnail
  \brief list.

  The class provides a path entering widget, a list filter combobox and
  a file lister. The file lister is either a textual display, or a thumbnail
  browser.

  The reason for this class is primarily due to the QSplitter limitation of
  just two "parents", i.e. it can only split between two widgets. Once
  this limitation is removed, this class is likely to be removed and
  merged with QmBrowser.
*/


/*!

*/

QmFileList::QmFileList(
    QWidget *Parent,
    const char *Name)
    : QWidget(Parent, Name)
{
    QBoxLayout *pMainLayout	= new QVBoxLayout(this, 2);
    QBoxLayout *pTopLayout	= new QHBoxLayout;

    // Add drag support for QmPixmapButton. Then, derive from
    // it a widget which knows which path it has "attached" to it.
    QmPixmapButton *pFolderIcon = new QmPixmapButton(this);
    {
	FolderIcon.loadFromData(folder_data, folder_len);
	pFolderIcon->setPixmap(FolderIcon);
	pFolderIcon->setFixedSize(FolderIcon.size());
	
	QToolTip::add(pFolderIcon, TT_FOLDER_ICON);
	QWhatsThis::add(pFolderIcon, WT_FOLDER_ICON);
    }
  
    pPath = new QLineEdit(this);
    {
	pPath->setFixedHeight(pPath->sizeHint().height());
	pPath->setMinimumWidth(70);

 	connect(pPath, SIGNAL(returnPressed()),
 		this, SLOT(pathChanged()));

	QToolTip::add(pPath, TT_PATH);
	QWhatsThis::add(pPath, WT_PATH);
    }

    pFilter = new QComboBox(true, this);
    {
	pFilter->setFixedHeight(pFilter->sizeHint().height());
	pFilter->setMaximumWidth(100);
	pFilter->setMinimumWidth(70);
	pFilter->insertItem("*");
	pFilter->insertItem("*.jpg");
	pFilter->insertItem("*.gif");
	pFilter->insertItem("*.xpm");

 	connect(pFilter, SIGNAL(activated(const char *)),
 		this, SLOT(pathChanged()));

	QToolTip::add(pFilter, TT_FILTER);
	QWhatsThis::add(pFilter, WT_FILTER);
    }

    pTabBar = new QmTabBar(QTabBar::RoundedBelow, this);
    {
	pFileList = new QmDetailList(pTabBar);
	{
	    pFileList->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
	
	    pFileList->addColumn("Filename", 170);
	    pFileList->setColumnWidthMode(0, QListView::Manual);
	    pFileList->addColumn("Size");
	    pFileList->setColumnAlignment(1, AlignRight);
	    pFileList->addColumn("Format");
	    
  	    connect(pFileList, SIGNAL(selectionChanged(QListViewItem *)),
  		    this, SLOT(itemHighlighted(QListViewItem *)));
	    
 	    connect(pFileList, SIGNAL(doubleClicked(QListViewItem *)),
 		    this, SLOT(itemSelected(QListViewItem *)));

	    // Let us take care of the Delete key, and not the listview:
	    
	    pFileList->installEventFilter(this);

	    QWhatsThis::add(pFileList, WT_FILE_LIST);
	}

	FileListID = pTabBar->addWidget(pFileList, "F&ilelist");

//  	pThumbnails = new QmThumbnailList(pTabBar);
	pThumbnails = new QaThumbnailListview(pTabBar);
	{
	    pThumbnails->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);

 	    connect(pThumbnails, SIGNAL(doubleClicked(QaThumbnailItem *)),
 		    this, SLOT(itemSelected(QaThumbnailItem *)));
	}
	connect( pTabBar, SIGNAL( aboutToShow( QWidget * ) ),
			 this, SLOT( showing( QWidget * ) ) );

	ThumbnailsID = pTabBar->addWidget(pThumbnails, "&Thumbnails");
    }
    
    pMainLayout->addLayout(pTopLayout);
    {
	pTopLayout->addWidget(pFolderIcon);
	pTopLayout->addWidget(pPath);
	pTopLayout->addWidget(pFilter);
    }

    pMainLayout->addWidget(pTabBar, 2);

    pMainLayout->activate();
}


/*!

*/

QmFileList::~QmFileList()
{
}


/*!
  Returns a pointer to the widget in the currently active tab.
*/

QWidget *
QmFileList::visibleWidget(void) const
{
    return pTabBar->visibleWidget();
}


/*!
  Sets the path to \a NewPath, and shows the directory contents.
*/

void
QmFileList::setPath(
    const char *NewPath)
{
    pPath->setText(NewPath);

    pathChanged();
}


/*!
  Emit the fileSelected() signal containing the next image.
*/

void
QmFileList::queryNextImage()
{
    QListViewItem *pNextItem;

    pNextItem = highlightNextImage(pFileList->currentItem());

    if(pNextItem == 0)
	return;
    
    pFileList->setCurrentItem(pNextItem);
    pFileList->setSelected(pNextItem, true);
    pFileList->ensureItemVisible(pNextItem);
    itemSelected(pNextItem);
}


/*!
  Emit the fileSelected() signal containing the previous image.
*/

void
QmFileList::queryPrevImage()
{
    QListViewItem *pPrevItem;

    pPrevItem = highlightPrevImage(pFileList->currentItem());

    if(pPrevItem == 0)
	return;
    
    pFileList->setCurrentItem(pPrevItem);
    pFileList->setSelected(pPrevItem, true);
    pFileList->ensureItemVisible(pPrevItem);
    itemSelected(pPrevItem);
}


/*!
  Handle keys like Return, Enter and Backspace.
*/

void
QmFileList::keyReleaseEvent(
    QKeyEvent *e)
{
    CHECK_PTR(pFileList);

    QmInfoItem *pi = dynamic_cast<QmInfoItem *>(pFileList->currentItem());

    if(pFileList->currentItem() == 0 || pPath->hasFocus() || pi == 0)
	return;

    switch(e->key())
    {
    case Key_Return:
    case Key_Enter:
	
	// NB: Fix this to take care of thumbnail view if visible
	
	cout << "QmFileList::keyReleaseEvent(): Select "
	     << pi->fileInfo()->filePath() << endl;
	
	itemSelected(pFileList->currentItem());
	break;
	
    case Key_Backspace:

	// Don't mind about parenting if we're in the root.

	if(CurrentDir == QDir::root().path() || pPath->hasFocus())
	    break;
	
	setPath(QDir::cleanDirPath(CurrentDir + "/.."));
	break;

    default:
	e->ignore();
	break;
    }
}


/*!
  Let us take care of the delete key, and not the listview.
*/

bool
QmFileList::eventFilter(
    QObject *,
    QEvent *e)
{
    if(e->type() == Event_KeyPress)
    {
	QKeyEvent *k = static_cast<QKeyEvent *>(e);

	if(k->key() == Key_Delete)
	{
	    // Eat the event

	    CHECK_PTR(pFileList);

	    if(pFileList->currentItem() == 0)
		return true;

	    QString s;
	    s.sprintf("%s/%s",
		      (const char *)CurrentDir,
		      pFileList->currentItem()->text(0));

	    if(QmMainWindow::deleteItem((const char *)s))
	    {
		QListViewItem *i = pFileList->currentItem()->itemBelow();
		delete pFileList->currentItem();
		pFileList->setSelected(i, true);
	    }
	    
	    return true;
	}
    }

    // Standard event processing
    
    return false;
}


/*!
  Highlights the next \e image in the current directory. The next image
  means the next image from the currently highlighted file. If no file is
  highlighted, the first image is highlighted. If the directory contains no
  supported images, 0 will be returned.
*/

QListViewItem *
QmFileList::highlightNextImage(
    QListViewItem *pStartItem)
{
    CHECK_PTR(pFileList);

    QListViewItem *pItem;

    // Start at the top? ...
    
    if(pStartItem == 0)
	pItem = pFileList->firstChild();
    else
    {
	// ... no. Get the item below. If the item below is null, we're at the
	// bottom, so go to the top.
	
	pItem = pStartItem->itemBelow();

	if(pItem == 0)
	    pItem = pFileList->firstChild();
    }

    // Go through all the items and find the first _image_.

    QString Format;
    
    for(; pItem != 0; pItem = pItem->itemBelow())
    {
	Format = pItem->text(QmInfoItem::Type);
	
	if( ! Format.isEmpty())
	{
	    pFileList->setCurrentItem(pItem);
	    break;
	}
    }

    return pItem;
}


/*!
  Highlights the previous \e image in the current directory. The previous image
  means the previous image from the currently highlighted file. If no file is
  highlighted, the first image is highlighted. If the directory contains no
  supported images, 0 will be returned.
*/

QListViewItem *
QmFileList::highlightPrevImage(
    QListViewItem *pStartItem)
{
    CHECK_PTR(pFileList);

    QListViewItem *pItem;

    // Start at the top? ...
    
    if(pStartItem == 0)
	pItem = pFileList->firstChild();
    else
    {
	// ... no. Get the item above. If the item above is null, we're at the
	// top, so go to the bottom.

	pItem = pStartItem->itemAbove();

	// At the top?
	
	if(pItem == 0)
	{
	    // ... yes
	    
	    pItem = pFileList->firstChild();

	    for(QListViewItem *p = pItem; p != 0;)
	    {
		p = pItem->itemBelow();
		if(p != 0)
		    pItem = p;
	    }
	}
	else
	{
	    // ... no. If the item above is a directory, go to the bottom
	    // item (as all directories are at the top)

	    QmInfoItem *pi = dynamic_cast<QmInfoItem *>(pItem);

	    if(pi != 0)
	    {
		if(pi->fileInfo()->isDir())
		{
		    pItem = pFileList->firstChild();
		    
		    for(QListViewItem *p = pItem; p != 0;)
		    {
			p = pItem->itemBelow();
			if(p != 0)
			    pItem = p;
		    }
		}
	    }
	}
	
	QString Format;
    
	Format = pItem->text(QmInfoItem::Type);
	
	if(Format.isEmpty())
	{
	    for(; pItem != 0; pItem = pItem->itemAbove())
	    {
		Format = pItem->text(QmInfoItem::Type);
	
		if( ! Format.isEmpty())
		{
		    pFileList->setCurrentItem(pItem);
		    break;
		}
	    }
	}
    }

    return pItem;
}


void QmFileList::showing( QWidget *w )
{
	if ( w == pThumbnails )
		pThumbnails->scanDirectory();
}

/*!
  Lists the directory.

  Merge this with setPath(), but last time I got weird compile errors
  due to passing QDir as an argument (since that's better than just
  a string, IMO).

  Or maybe not. pathChanged() is a private slot...
*/

void
QmFileList::pathChanged()
{
    Directory = pPath->text();

    if( ! Directory.exists())
    {
		QApplication::beep();
		return;
    }

    CurrentDir = Directory.absPath();

    CHECK_PTR(pFilter);
    CHECK_PTR(pTabBar);

	Directory.setNameFilter(pFilter->currentText());
	QFileInfoListIterator it( *Directory.entryInfoList() );
	QFileInfo *fi;

    QFileInfoList		ImageFiles;
    QFileInfoList		Directories;

	pFileList->clear();
    
	while((fi = it.current()))
	{
		if ( fi->isDir() )
		{
			if(CurrentDir == "/")
			{
				// We're listing the root. Don't include `..' as it makes
				// no sense.
		
				if(fi->fileName() != "." && fi->fileName() != "..")
					Directories.append( new QFileInfo( *fi ) );
			}
			else
			{
				// Listing non-root directory. Include `..'
		
				if(fi->fileName() != ".")
					Directories.append( new QFileInfo( *fi ) );
			}
		}
		else if ( QImage::imageFormat(fi->filePath()) )
		{
			ImageFiles.append( new QFileInfo( *fi ) );
		}
		++it;
	}

	QFileInfoListIterator it1( Directories );
	QFileInfoListIterator it2( ImageFiles );
	DirList.clear();
	while ( (fi = it1.current()) )
	{
		DirList.append( new QFileInfo( *fi ) );
		++it1;
	}
	while ( (fi = it2.current()) )
	{
		DirList.append( new QFileInfo( *fi ) );
		++it2;
	}

	pThumbnails->setDirectory( DirList );
  	if( pTabBar->visibleWidget() == pThumbnails )
		pThumbnails->scanDirectory();
//    else if(pTabBar->visibleWidget() == pFileList)
//      else
//      {
//	d.setFilter(QDir::Files | QDir::Hidden);
//  		d.setNameFilter(pFilter->currentText());
		it = QFileInfoListIterator( DirList );
//  		QFileInfo *fi;

		pFileList->clear();

		while((fi = it.current()))
		{
			// For a snappy user interface:
	    
// 	    qApp->processEvents();

// 	    if(CurrentDir != d.absPath())
// 		return;

	    // ... but unfortunately, this doesn't work well because
	    // if a new dir is highlighted in the dir browser before
	    // listing of the previous is done, we'll get these unfinished
	    // dirs listed in one chunk in the same list.
	    //
	    // The problem is solved with the if(CurrentDir...) above,
	    // but it's still sluggish and the highlighting in the dir
	    // browser fucks up. Without processEvents(), it's even
	    // more sluggish, but it's more of a block of the UI,
	    // which it's also is, so it feels a bit better.

			if(CurrentDir == "/")
			{
				// We're listing the root. Don't include `..' as it makes
				// no sense.
		
				if(fi->fileName() != "." && fi->fileName() != "..")
					(void) new QmInfoItem(fi, pFileList);
			}
			else
			{
				// Listing non-root directory. Include `..'
		
				if(fi->fileName() != ".")
					(void) new QmInfoItem(fi, pFileList);
			}
			qApp->processEvents();

			++it;
		}
//      }
}


/*!
  If the highlighted item is a file, the fileHighlighted() signal is
  emitted.
*/

void
QmFileList::itemHighlighted(
    QListViewItem *pItem)
{
//    pFileList->setFocus();

    CHECK_PTR(pItem);
    
    const QFileInfo *fi = ((QmInfoItem *)pItem)->fileInfo();

    CHECK_PTR(fi);
    
    if( ! fi->isDir())
	emit fileHighlighted(fi->filePath());
}


/*!
  If the selected item is a directory, that directory is listed. If the
  selected item is a file, the fileSelected() signal is emitted.
*/

void
QmFileList::itemSelected(
    QListViewItem *pItem)
{
    CHECK_PTR(pItem);
    
    QmInfoItem *pi = dynamic_cast<QmInfoItem *>(pItem);

    if(pi == 0)
	return;
    
    const QFileInfo *fi = pi->fileInfo();

    CHECK_PTR(fi);
    
    if(fi->isDir())
    {
	QString NewDir = QDir::cleanDirPath(fi->filePath());
	
	emit dirSelected((const char *)NewDir);
	setPath(NewDir);
    }
    else
	emit fileSelected(fi->filePath());
}

void
QmFileList::itemSelected(
    QaThumbnailItem *pItem)
{
    CHECK_PTR(pItem);
    
    QaThumbnailPreview *pi = dynamic_cast<QaThumbnailPreview *>(pItem);

    if(pi == 0)
	return;
    
    const QFileInfo *fi = pi->fileInfo();

    CHECK_PTR(fi);
    
    if(fi->isDir())
    {
	QString NewDir = QDir::cleanDirPath(fi->filePath());
	
	emit dirSelected((const char *)NewDir);
	setPath(NewDir);
    }
    else
	emit fileSelected(fi->filePath());
}


void
QmFileList::itemSelected(
    const char *Filename)
{
    emit fileSelected(Filename);
}

