/* ;-*-c++-*-
 * 
 * qmimageview.h
 * 
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <31-Oct-1998 20:21:30 marius>
 */

#ifndef QMIMAGEVIEW_H_INCLUDED
#define QMIMAGEVIEW_H_INCLUDED

#include <qbrush.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qscrollview.h>
#include <qtimer.h>
#include <qdropsite.h>

class QmImageView : public QScrollView, QDropSite
{
    Q_OBJECT
    
public:
    QmImageView(QWidget *Parent = 0,
		const char *Name = 0);
    virtual ~QmImageView();

    int			currentZoom(void) const;
    
public slots:    
    bool		loadImage(const char *Filename);
    void		zoom(int Percent);
    void		setPixmap(const QPixmap &Pixmap);
    

signals:
    /*!
      \fn void QmImageView::message(const char *Message)

      Emits the text \a Message for use in e.g. a statusbar.
      The signal is emitted whenever the class feels like it ;-)
    */
    
    void		message(const char *Message);

    
    /*!
      \fn void QmImageView::returnPressed()

      This signal is emitted whenever Return or Enter is pressed.
    */

    void		returnPressed();
    

    /*!
      \fn void QmImageView::queryNextImage()

      This signal is emitted whenever a key for the next image is pressed.
    */

    void		queryNextImage();

    
    /*!
      \fn void QmImageView::queryPrevImage()

      This signal is emitted whenever a key for the previous image is pressed.
    */

    void		queryPrevImage();
    
protected:
    virtual void	keyPressEvent(QKeyEvent *e);
    virtual void	keyReleaseEvent(QKeyEvent *e);
    virtual void	leaveEvent(QEvent *e);
    virtual void	viewportMousePressEvent(QMouseEvent *e);
    virtual void	viewportMouseReleaseEvent(QMouseEvent *e);
    virtual void	viewportMouseMoveEvent(QMouseEvent *e);
    virtual void	drawContentsOffset(QPainter *p,
					   int OffsetX, int OffsetY,
					   int ClipX, int ClipY,
					   int ClipW, int ClipH);
    void		dragEnterEvent(QDragEnterEvent *e);
    void		dragMoveEvent(QDragMoveEvent *e);
    void		dragLeaveEvent(QDragLeaveEvent *e);
    void		dropEvent(QDropEvent *e);

private slots:
    void		timerDone();
 
private:
    bool		directionPressed(void) const;
    void		deleteImage(void);

    QString		ImageFilename;
    QBrush		BlackBackground;
    QPixmap		ThePixmap;
    QPixmap		PixmapToShow;
    QTimer		*pTimer;
    int			CurrentZoom;
    
    bool		ButtonReleased;
    bool	        DirectionPressed[4];
    bool		ShiftPressed;

    enum VerticalScrollDirection
    {
		VDown,
		VUp,
		VNone
    };
    
    enum HorizontalScrollDirection
    {
	HLeft,
	HRight,
	HNone
    };

    VerticalScrollDirection	VScrollDir;
    HorizontalScrollDirection	HScrollDir;

    static const int	ScrollDelay	= 1;
    static const int	PixelsToScroll	= 1;
};

#endif // QMIMAGEVIEW_H_INCLUDED
