/* qminfoitem.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <01-Nov-1998 15:22:04 marius>
 */

#include <iostream.h>

#include <qapplication.h>
#include <qbitmap.h>
#include <qfiledialog.h>
#include <qfileinfo.h>
#include <qimage.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qstring.h>

#include "qminfoitem.h"

extern const char *picture_xpm[]; // from qmimageview.cpp

/*!
  \class QmInfoItem qminfoitem.h
  \brief The QmInfoItem class is used in the detailed list.

  It provides icons and background coloring of images. That is,
  a JPEG image has a different background than a GIF image, and so on.
  The idea is inspired by ACDSee by ACD Systems.
*/

/*
  This class is inspired by the QFileDialog source.
  Some code is stolen from qfiledialog.cpp
*/

QFileIconProvider *fileIconProvider = 0;

const unsigned int  closed_gif_len = 110;
const unsigned char closed_gif_data[] =
{
    0x47,0x49,0x46,0x38,0x39,0x61,0x0f,0x00,0x0d,0x00,0xc2,0x00,0x00,0x00,
    0x00,0x00,0x99,0x99,0x99,0xcc,0xcc,0xcc,0xff,0xff,0xff,0xff,0xff,0x00,
    0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x21,0xf9,0x04,0x01,0x00,
    0x00,0x03,0x00,0x2c,0x00,0x00,0x00,0x00,0x0f,0x00,0x0d,0x00,0x00,0x03,
    0x33,0x38,0x13,0xcc,0xfa,0x30,0x08,0x32,0x45,0x80,0x4f,0xd2,0x6d,0x9b,
    0x0b,0x45,0x28,0x8e,0x61,0x00,0x80,0x5c,0x3a,0x99,0x60,0xa5,0x52,0x6c,
    0xf1,0xba,0xb1,0x6b,0xc3,0xa7,0x7c,0xd3,0xf9,0xce,0xc5,0xb3,0x0d,0xcb,
    0x43,0xf4,0x00,0x06,0x80,0xa4,0x72,0xb9,0x4c,0x00,0x00,0x3b
};

QPixmap *folderIcon = 0;
QPixmap *imageIcon = 0;
QPixmap *fifteenTransparentPixels = 0;

QString *tmpString;
bool VariablesMade = false;

QColor JpgColor;
QColor GifColor;
QColor XpmColor;
QColor PngColor;

QColorGroup *pJpgCg;
QColorGroup *pGifCg;
QColorGroup *pXpmCg;
QColorGroup *pPngCg;


/*
  Called when the program is quitting.
*/

void cleanup()
{
    delete folderIcon;
    folderIcon = 0;

    delete imageIcon;
    imageIcon = 0;

    delete tmpString;
    tmpString = 0;

    delete pJpgCg;
    pJpgCg = 0;
    
    delete pGifCg;
    pJpgCg = 0;
    
    delete pXpmCg;
    pJpgCg = 0;
    
    delete pPngCg;
    pJpgCg = 0;
}


/*
  Initializes pixmaps and colors, common to all/many listview items.
*/

void makeVariables()
{
    if( ! VariablesMade) // I could check the tmpString pointer instead
    {
	VariablesMade = true;
	qAddPostRoutine(cleanup);

	tmpString = new QString;
	
	folderIcon = new QPixmap();
	folderIcon->loadFromData(closed_gif_data, closed_gif_len);

	imageIcon = new QPixmap(picture_xpm);
//	imageIcon->loadFromData(closed_gif_data, closed_gif_len);
	
	fifteenTransparentPixels = new QPixmap(folderIcon->width(), 1);
	QBitmap m(fifteenTransparentPixels->width(), 1);
	m.fill(color0);
	fifteenTransparentPixels->setMask(m);

	JpgColor.setRgb(92, 229, 224);
	GifColor.setRgb(120, 169, 255);
	XpmColor.setRgb(255, 252, 190);
	PngColor.setRgb(250, 128, 114);
	
	QColorGroup cg = QApplication::palette()->normal();

	// Note that not all supported formats gets colorized!
	
	pJpgCg = new QColorGroup(cg.foreground(), cg.background(), cg.light(),
				 cg.dark(), cg.mid(), cg.text(),
				 JpgColor);
	pGifCg = new QColorGroup(cg.foreground(), cg.background(), cg.light(),
				 cg.dark(), cg.mid(), cg.text(),
				 GifColor);
	pXpmCg = new QColorGroup(cg.foreground(), cg.background(), cg.light(),
				 cg.dark(), cg.mid(), cg.text(),
				 XpmColor);
	pPngCg = new QColorGroup(cg.foreground(), cg.background(), cg.light(),
				 cg.dark(), cg.mid(), cg.text(),
				 PngColor);
    }
}


/*!

*/

QmInfoItem::QmInfoItem(
    QFileInfo *pNewFileInfo,
    QListView *Parent)
    : QmListViewItem(Parent),
      FileInfo(*pNewFileInfo)
{
    makeVariables();

    initialize();
}

QmInfoItem::QmInfoItem(
    QFileInfo *pNewFileInfo,
    QmListViewItem *Parent)
    : QmListViewItem(Parent),
      FileInfo(*pNewFileInfo)
{
    makeVariables();

    initialize();
}


/*!

*/

QmInfoItem::~QmInfoItem()
{
}


/*!
  Returns a key for sorting.
*/

/*
  This function is necessary so that directories will be listed
  first, then files.
*/

const char *
QmInfoItem::key(
    int Column,
    bool Ascending) const
{
    QDateTime epoch( QDate( 1968, 6, 19 ) );

    char MajorKey = Ascending == FileInfo.isDir() ? '0' : '1';

    if(Column == Size)
    {
	tmpString->sprintf("%c%08d",
			   MajorKey,
			   FileInfo.size());
    }
    else if(Column == Modified)
    {
	tmpString->sprintf("%c%08d",
			   MajorKey,
			   epoch.secsTo(FileInfo.lastModified()));
    }
    else
    {
	*tmpString = text(Column);
	tmpString->insert(0, MajorKey);
    }

    return *tmpString;
}


/*!
  Returns the text of the item.
*/

/*
  NOTE:
  This function is called very often. See if it can be optimized.
*/

const char *
QmInfoItem::text(
    int Column) const
{
    QDate d;
    QTime t;

    switch(Column)
    {
    case Name:
	*tmpString = FileInfo.fileName();
	break;

    case Size:
	if( ! FileInfo.isDir())
	    *tmpString = friendlySize(FileInfo.size());
	else
	    *tmpString = "";
	break;

    case Type:
	if(FileInfo.isFile())
	{
	    *tmpString = QImage::imageFormat(FileInfo.filePath());

	    if(*tmpString == 0)
		*tmpString = "";
	}
	else
	    *tmpString = "";
	
	break;

    case Modified:
	d = FileInfo.lastModified().date();
	t = FileInfo.lastModified().time();
	tmpString->sprintf("%02d/%02d/%d  %02d:%02d:%02d",
			   d.day(),
			   d.month(),
			   d.year(),
			   t.hour(),
			   t.minute(),
			   t.second());
	
	break;
	
    case Permission:
	if(FileInfo.isReadable())
	    *tmpString = FileInfo.isWritable() ? "Read-write" : "Read-only";
	else
	    *tmpString = FileInfo.isWritable() ? "Write-only" : "Inaccessible";
	break;

    default:
	*tmpString = "<--->";
	break;
    }

//    cout << "QmInfoItem::text(" << Column << "): " << *tmpString << endl;

    tmpString->detach();
    return *tmpString;
}


/*!
  Returns an appropriate icon.
*/

const QPixmap *
QmInfoItem::pixmap(
    int Column) const
{
    // The QFileIconProvider is neat. See if it can be used sometime ;-)
    
    if(Column)
	return 0;
//    else if(fileIconProvider)
//	return fileIconProvider->pixmap(FileInfo);
    else if(ImageType != Unknown)
	return imageIcon;
    else if(FileInfo.isDir())
	return folderIcon;
    else
	return fifteenTransparentPixels;
}

void
QmInfoItem::paintCell(
    QPainter *p,
    const QColorGroup &/*cg*/,
    int col, int width,
    int align)
{
    if( ! p)
	return;

    QmListViewItem::paintCell(p, ColorGroup, col, width, align);
}


/*!
  Converts \a Value into a friendly, human readable string. Passing
  921 will return `921 B', 124566 will return `121.6 KB' and 6527885
  will return `6.2 MB'.

  As you see, a kilobyte is 1024 bytes, and a megabyte is 1024 kilobytes.
*/

QString
QmInfoItem::friendlySize(
    long Value)
{
    static QString ValueString;

    if(Value < 1024)
    {
	ValueString.sprintf("%d B", Value);
    }
    else if(Value < 1024 * 1024)
    {
	ValueString.sprintf("%.1f KB", Value / 1024.0);
    }
    else
    {
	ValueString.sprintf("%.1f MB", Value / (1024.0 * 1024.0));
    }

    return ValueString;
}


/*!

 */

const QFileInfo *
QmInfoItem::fileInfo(void) const
{
    return &FileInfo;
}


/*!
  \internal
*/

void
QmInfoItem::initialize(void)
{
    *tmpString = QImage::imageFormat(FileInfo.filePath());

    if(FileInfo.isFile())
    {
	if(*tmpString == "JPEG")
	{
	    ImageType = Jpg;
	    ColorGroup = *pJpgCg;
	}
	else if(*tmpString == "GIF")
	{
	    ImageType = Gif;
	    ColorGroup = *pGifCg;
	}
    
	else if(*tmpString == "XPM")
	{
	    ImageType = Xpm;
	    ColorGroup = *pXpmCg;
	}
	else if(*tmpString == "PNG")
	{
	    ImageType = Png;
	    ColorGroup = *pPngCg;
	}
	else
	{
	    ImageType = Unknown;
	    ColorGroup = QApplication::palette()->normal();
	}
    }
    else
    {
	ImageType = Unknown;
	ColorGroup = QApplication::palette()->normal();
    }
}
