/* qmlistdialog.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <08-Jan-1999 16:50:43 marius>
 */

#include <qimage.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmultilinedit.h>
#include <qpushbutton.h>
#include <qstrlist.h>

#include "qmlistdialog.h"
#include "version.h"

/*!
  \class QmListDialog qmlistdialog.h
  \brief The QmListDialog shows a list of the formats supported.

*/


/*!

*/

QmListDialog::QmListDialog(
    QWidget *Parent,
    const char *Name)
    : QDialog(Parent, Name)
{
    setCaption(APP_NAME);
    
    QBoxLayout *pMainLayout	= new QVBoxLayout(this, 2);

    QLabel *pInfo = new QLabel(
	"The formats supported are listed below. Which formats that are\n"
	"supported depends on the QImageIO extension compilation.", this);
    {
//	pInfo->setFrameStyle(QFrame::Sunken);
	pInfo->setFixedHeight(pInfo->sizeHint().height());
	pInfo->setMinimumWidth(pInfo->sizeHint().width());
	pInfo->setAlignment(AlignCenter);
    }

    QStrList l = QImageIO::inputFormats();

    QMultiLineEdit *pFormats = new QMultiLineEdit(this);
    {
	pFormats->setMinimumHeight(70);
	pFormats->setReadOnly(true);

	for(uint i = 0; i < l.count(); i++)
	{
	    pFormats->insertLine(l.at(i));
	}
    }
    
    QString s;
    s.sprintf("%d formats supported", l.count());
    
    QLabel *pFormatsCount = new QLabel((const char *)s, this);
    {
//	pFormatsCount->setFrameStyle(QFrame::Sunken);
	pFormatsCount->setFixedHeight(pFormatsCount->sizeHint().height());
	pFormatsCount->setMinimumWidth(pFormatsCount->sizeHint().width());
    }

    QPushButton *pClose = new QPushButton("&Close", this);
    {
	pClose->setFixedSize(pClose->sizeHint());
	pClose->setDefault(true);
	pClose->setAutoDefault(true);
	pClose->setFocus();

	connect(pClose, SIGNAL(clicked()), this, SLOT(hide()));
    }

    pMainLayout->addWidget(pInfo);
    pMainLayout->addWidget(pFormats, 2);
    pMainLayout->addWidget(pFormatsCount);
    pMainLayout->addWidget(pClose);

    pMainLayout->activate();
}


/*!

*/

QmListDialog::~QmListDialog()
{
}
