/* qmlistview.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <19-Dec-1998 13:36:10 marius>
 */

#include <iostream.h>

#include <qdragobject.h>

#include "qmlistview.h"

/* XPM */
static const char * DragPic_XPM[] = {
"13 9 3 1",
" 	c None",
".	c #000000",
"X	c #FFFFFF",
".X.X.X.X.X.X.",
"X.X.X.X.X.X.X",
".X         X.",
"X.         .X",
".X         X.",
"X.         .X",
".X         X.",
"X.X.X.X.X.X.X",
".X.X.X.X.X.X."};

/*!
  \class QmListView qmlistview.h
  \brief The QmListView class is a slight customization of QListView.

  It differs in that the listview will get focus when clicked in with
  the mouse. This means that you can conveniently click on the listview
  with the mouse, then start using it with the keyboard.
*/


/*!
  Creates a new empty list view. \a Parent and \a Name are sent to the
  QListView constructor.
*/

/*
  Notice that the QDropSite constructor must be given the viewport widget
  as parent. Choosing `this' as parent will result in that drops are only
  accepted in the tiny frame around the listview.

  This goes for any QDragObjects aswell; They must have the viewport widget
  as parent.
*/

QmListView::QmListView(
    QWidget *Parent,
    const char *Name)
    : QListView(Parent, Name),
      QDropSite(this->viewport())
{
}


QmListView::~QmListView()
{
}


/*!
  Pressing a printable character will highlight the first
  item in the list starting this character, if available.
  It's column 0 that's being searched.
*/

void
QmListView::keyPressEvent(
    QKeyEvent *e)
{
    QString s;
    
    for(QListViewItem *i = firstChild(); i != 0; i = i->itemBelow())
    {
	s = i->text(0);

	if(s.find(e->ascii()) == 0)
	{
	    setCurrentItem(i);
	    break;
	}
    }

    QListView::keyPressEvent(e);
}


/*!
  
*/

void
QmListView::mousePressEvent(
    QMouseEvent *e)
{
    setFocus();
    
    QListView::mousePressEvent(e);
}


/*!

 */

// void
// QmListView::mouseReleaseEvent(
//     QMouseEvent *e)
// {
//     if((e->state() & LeftButton) == 0)
// 	LeftButtonDown = false;
// }


/*!

 */

// void
// QmListView::leaveEvent(
//     QEvent *e)
// {
//     cout << "QmListView::leaveEvent(): " << e->type() << endl;
    
//     if(currentItem() != 0 && LeftButtonDown == true)
//     {
// 	QTextDrag *d = new QTextDrag(currentItem()->text(0), this);
// 	d->setPixmap(QPixmap(DragPic_XPM), QPoint(8, 8));
// 	d->dragCopy();
//     }
//}


/*!

 */

void
QmListView::dragEnterEvent(
    QDragEnterEvent *e)
{
//    cout << "QmListView::dragEnterEvent()\n";
    
    if(QTextDrag::canDecode(e))
    {
	e->accept();
    }

    // Give the user some feedback
    
    QString t;
    for(int i = 0; e->format(i); i++)
    {
	if(*(e->format(i)))
	{
	    if( ! t.isEmpty())
		t += "\n";
	    t += e->format(i);

	    cout << t << endl;
	}
    }
}


/*!

 */

void
QmListView::dragMoveEvent(
    QDragMoveEvent * /*e*/)
{
//    cout << "QmListView::dragMoveEvent()\n";
}


/*!

 */

void
QmListView::dragLeaveEvent(
    QDragLeaveEvent * /*e*/)
{
//     if(currentItem() != 0)
//     {
//  	QTextDrag *d = new QTextDrag(currentItem()->text(0), viewport());
//  	d->setPixmap(QPixmap(DragPic_XPM), QPoint(8, 8));
//  	d->dragCopy();
//     }
    
//    cout << "QmListView::dragLeaveEvent()\n";
}


/*!

 */

void
QmListView::dropEvent(
    QDropEvent *e)
{
    cout << "QmListView::dropEvent(): ";
    
    // Try to decode to the data

    QString str;
    if(QTextDrag::decode(e, str))
    {
	cout << str << endl;
	return;
    }

    cout << endl;
}







