/* qmmainwindow.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <08-Feb-1999 07:28:04 amos>
 */

#include <iostream.h>

#include <qaccel.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qcombobox.h>
#include <qfileinfo.h>
#include <qimage.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qprogressbar.h>
#include <qsplitter.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qvalidator.h>
#include <qwhatsthis.h>
#include <qwidgetlist.h>
#include <fstream.h>

#include "qmbrowser.h"
#include "qmimageview.h"
#include "qmimageviewbrowser.h"
#include "qmlistdialog.h"
#include "qmmainwindow.h"
#include "strings.h"
#include "version.h"



QList<QmMainWindow>	QmMainWindow::WindowList;
QPopupMenu		*QmMainWindow::pWindowMenu	= 0;

/* Global variables:
 *
 * g_pProgress		is the progress bar in the statusbar.
 * g_pSmallInfo		is a tiny text field in the statusbar. Use it for
 *			filecounts etc.
 * g_pLargeInfo		is the large text field in the statusbar. Use it
 *			for longer descriptions.
 * g_pImageViewBrowser	is the fullsize image viewer.
 */

QProgressBar		*g_pProgress			= 0;
QLabel			*g_pSmallInfo			= 0;
QLabel			*g_pLargeInfo			= 0;
QmImageViewBrowser	*g_pImageViewBrowser		= 0;

/*!
  \class QmMainWindow qmmainwindow.h
  \brief The QMainWindow is the main window.

  This window is the main working window.

  \warning
  
  It is \e vital that you do \e not create a new window on
  the stack. This is due to the multi-window feature. \e All objects of
  this class \e must be created with \c new. If you create a object of this
  class on the stack, the application \e will crash when the window is closed.
*/

/*!
  Creates the window.

  The \a Parent and \a Name arguments are sent to the QMainWindow constructor.

  \warning
  
  Read the detailed information!
*/

QmMainWindow::QmMainWindow(
    QWidget *Parent,
    const char *Name)
    : QMainWindow(Parent, Name)
{
    // If this is the very first window created, at it to the list here.
    // Otherwise, it has already been added in the newWindow() function.
    
//    if(WindowList.count() == 0)
    {
	WindowList.setAutoDelete(false);
	WindowList.append(this);
    }

    QString Caption;
    Caption.sprintf("%s - Window %d", APP_NAME, WindowList.count());
    setCaption((const char *)Caption);

    QAccel *pAccels = 0;

    pStack = new QWidgetStack(this);

    pBrowser = new QmBrowser(pStack);
    {
	connect(pBrowser, SIGNAL(fileSelected(const char *)),
		this, SLOT(showImage(const char *)));

	connect(pBrowser, SIGNAL(imageSelected(const QPixmap &)),
		this, SLOT(showImage(const QPixmap &)));

	connect(pBrowser, SIGNAL(message(const char *)),
		statusBar(), SLOT(message(const char *)));
    }

    /*
    pImageView = new QmImageView(pStack);
    {
	pImageView->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);

	connect(pImageView, SIGNAL(message(const char *)),
		statusBar(), SLOT(message(const char *)));

	connect(pImageView, SIGNAL(returnPressed()),
		this, SLOT(toggleView()));

	connect(pImageView, SIGNAL(queryNextImage()),
		this, SLOT(nextImage()));

	connect(pImageView, SIGNAL(queryPrevImage()),
		this, SLOT(prevImage()));

	QWhatsThis::add(pImageView, WT_IMAGE_VIEW);
    }
    */

    pImageView = 0;
    g_pImageViewBrowser = new QmImageViewBrowser(pStack);

    pStack->addWidget(pBrowser, BrowserID);
//    pStack->addWidget(pImageView, ImageViewID);
    pStack->addWidget(g_pImageViewBrowser, ImageViewID);
    pStack->raiseWidget(BrowserID);

    setCentralWidget(pStack);

    pToolBar = new QToolBar(this, "Main toolbar");
    {
	// Unfortunately, we can not simply set the toolbar's focus policy
	// to NoFocus and hope that the toolbuttons in it won't be able
	// to "tab to". Bug? We need to set NoFocus on all widgets in a toolbar
	// instead.
	
//	pToolBar->setFocusPolicy(QWidget::NoFocus);

		QString file = getIconPath( "fileopen.xpm" );
		cout << "Opening: " << file << endl;
		QPixmap openIcon(file);
		QToolButton * fileOpen = new QToolButton(
			openIcon,
			"Tooltip",
			"Statusbar",
			this, SLOT(toggleView()),
			pToolBar);

		fileOpen->setToggleButton(true);
		fileOpen->setFocusPolicy(QWidget::NoFocus);

		file = getIconPath( "back.xpm" );
		cout << "Opening: " << file << endl;
		QPixmap BackIcon( file );
		pBack = new QToolButton(
			BackIcon,
			"Previous image",
			"Shows the previous image.",
			pBrowser, SLOT(queryPrevImage()),
			pToolBar);

		pBack->setFocusPolicy(QWidget::NoFocus);
    
		file = getIconPath( "forward.xpm" );
		cout << "Opening: " << file << endl;
		QPixmap forwardIcon( file );
		pForward = new QToolButton(
			forwardIcon,
			"Next image (Ctrl+F)",
			"Shows the next image.",
			pBrowser, SLOT(queryNextImage()),
			pToolBar);

		pForward->setFocusPolicy(QWidget::NoFocus);
    
		pSlideshowTimer = new QTimer(this);
		connect(pSlideshowTimer, SIGNAL(timeout()),
				pBrowser, SLOT(queryNextImage()));
	
		file = getIconPath( "slide.xpm" );
		cout << "Opening: " << file << endl;
		QPixmap slideIcon( file );
		pSlide = new QToolButton(
			slideIcon,
			"Slideshow",
			"Shows the images in a slideshow.",
			this, SLOT(toggleSlideshow()),
			pToolBar);

		pSlide->setToggleButton(true);
		pSlide->setFocusPolicy(QWidget::NoFocus);

		QString s;
		s.sprintf("Zooms up the picture by %d%%", ZoomStep);
		file = getIconPath( "zoomin.xpm" );
		cout << "Opening: " << file << endl;
		QPixmap zoomInIcon( file);
		pZoomIn = new QToolButton(
			zoomInIcon,
			(const char *)s,
			(const char *)s,
			this, SLOT(zoomUp()),
			pToolBar);

		pZoomIn->setFocusPolicy(QWidget::NoFocus);
	
		s.sprintf("Zooms down the picture by %d%%", ZoomStep);
		file = getIconPath( "zoomout.xpm" );
		cout << "Opening: " << file << endl;
		QPixmap zoomOutIcon( file );
		pZoomOut = new QToolButton(
			zoomOutIcon,
			(const char *)s,
			(const char *)s,
			this, SLOT(zoomDown()),
			pToolBar);

		pZoomOut->setFocusPolicy(QWidget::NoFocus);

		/*
		  pZoom = new QComboBox(false, pToolBar);
		  {
		  pZoom->insertItem("20");
		  pZoom->insertItem("40");
		  pZoom->insertItem("60");
		  pZoom->insertItem("80");
		  pZoom->insertItem("100");
		  pZoom->insertItem("120");
		  pZoom->insertItem("140");
		  pZoom->insertItem("160");
		  pZoom->insertItem("180");
		  pZoom->insertItem("200");

		  pZoom->setFixedWidth(pZoom->sizeHint().width());
//	    pZoom->setValidator(new QIntValidator(20, 300, pZoom));

connect(pZoom, SIGNAL(activated(const char *)),
this, SLOT(zoomEntered(const char *)));
}
	*/

	pToolBar->addSeparator();
    
	QToolButton *pWhatsThis = QWhatsThis::whatsThisButton(pToolBar);
	pWhatsThis->setFocusPolicy(QWidget::NoFocus);
	
//	QWhatsThis::add(fileOpen, fileOpenText, false);
	QWhatsThis::add(pBack,		WT_BACK,	false);
	QWhatsThis::add(pForward,	WT_FORWARD,	false);
	QWhatsThis::add(pZoomIn,	WT_ZOOM_IN,	false);
	QWhatsThis::add(pZoomOut,	WT_ZOOM_OUT,	false);
	QWhatsThis::add(pSlide,		WT_SLIDE_SHOW,	false);
    }

    QPopupMenu *pFileMenu = new QPopupMenu;
    {
	menuBar()->insertItem("&File", pFileMenu);

	pFileMenu->insertItem("&New",
			      this, SLOT(newMainWindow()), CTRL + Key_N);
	pFileMenu->insertSeparator();
	pFileMenu->insertItem("&Close",
			      this, SLOT(closeWindow()), CTRL + Key_W);
	pFileMenu->insertSeparator();
	pFileMenu->insertItem("&Quit", qApp, SLOT(quit()), CTRL + Key_Q);
    }

    pControlsMenu = new QPopupMenu;
    {
	menuBar()->insertItem("&Settings", pControlsMenu);

	connect(pControlsMenu, SIGNAL(highlighted(int)),
		this, SLOT(menuInfo(int)));

	MenuBarToggleID = pControlsMenu->insertItem(
	    "&Menu bar", this,
	    SLOT(toggleMenuBar()),
	    CTRL + Key_M);

	// Now an accelerator for when the menubar is invisible!

	pAccels = new QAccel(this);
	pAccels->connectItem(pAccels->insertItem(CTRL + Key_M),
			     this, SLOT(toggleMenuBar()));

	ToolBarToggleID = pControlsMenu->insertItem(
	    "&Tool bar",
	    this, SLOT(toggleToolBar()),
	    CTRL + Key_T);
	
	StatusBarToggleID = pControlsMenu->insertItem(
	    "&Status bar",
	    this, SLOT(toggleStatusBar()),
	    CTRL + Key_B);
	
	PreviewToggleID = pControlsMenu->insertItem(
	    "&Preview",
	    this, SLOT(togglePreview()),
	    CTRL + Key_P);

	pControlsMenu->insertSeparator();

	pControlsMenu->insertItem(
	    "&Options...",
	    this, SLOT(showOptions()),
	    CTRL + Key_O);
	
	pControlsMenu->setCheckable(true);
	pControlsMenu->setItemChecked(MenuBarToggleID, true);
	pControlsMenu->setItemChecked(ToolBarToggleID, true);
	pControlsMenu->setItemChecked(StatusBarToggleID, true);
	pControlsMenu->setItemChecked(PreviewToggleID, true);
	ShowPreview = true;
    }

    // pWindowMenu is static
    // Make the items "bulleted" so that the active window has a "bullet"
    // next to it.
    
    if(pWindowMenu == 0)
    {
	pWindowMenu = new QPopupMenu;
    }

    // Why won't the window menu appear in any but the first window?
    // Is it because it's allocated in the first window and thus it
    // belongs to that object? But it IS a static variable, so it should
    // belong to ANY object of the class, right?
    //
    // The first time, insertItem() will return 2. All other times, it'll
    // return 0.
    //
    // See Thinking in C++ page 376-378 about copy constructors and static
    // members.
    //
    // There was some posting on the Qt mailinglist that you cannot
    // create static/external Qt objects. Maybe that's why it doesn't work?
    
    menuBar()->insertItem("&Windows", pWindowMenu);
    updateWindowMenu();

    menuBar()->insertSeparator();
    
    QPopupMenu *pHelpMenu = new QPopupMenu;
    {
	menuBar()->insertItem("&Help", pHelpMenu);

	pHelpMenu->insertItem("Help...",
			      this, SLOT(showHelp()));
	pHelpMenu->insertSeparator();
	pHelpMenu->insertItem("Supported formats...",
			      this, SLOT(showFormats()));
	pHelpMenu->insertItem("About...",
			      this, SLOT(showAbout()));
	pHelpMenu->insertItem("About Qt...",
			      this, SLOT(showQtInfo()));
    }

    g_pProgress = new QProgressBar(statusBar());
    {
	g_pProgress->setFixedSize(80, 19);
//	g_pProgress->setBackgroundMode(QWidget::PaletteDark);
    }

    g_pLargeInfo = new QLabel(statusBar());
    
    g_pSmallInfo = new QLabel(statusBar());
    {
	g_pSmallInfo->setFixedSize(60, 19);
	g_pSmallInfo->setAlignment(AlignCenter);
    }
    
    statusBar()->addWidget(g_pProgress, 0);
    statusBar()->addWidget(g_pSmallInfo, 0);
    statusBar()->addWidget(g_pLargeInfo, 2);

//     QToolTipGroup *tg = new QToolTipGroup(this);

//     connect(tg, SIGNAL(showTip(const char *)),
// 	    pInfo, SLOT(setText(const char *)));
//     connect(tg, SIGNAL(removeTip()),
// 	    pInfo, SLOT(clear()));
//     QToolTip::add(pBrowser, "Short text", tg, "Looooong text");

    // Note that since the toolbuttons have a border when the
    // mouse button is placed on top of it, the animateClick() function
    // will have no visual effect if the mouse button is not placed
    // on the button as it's only the border that gets recessed.
    //
    // Move the mouse button on top of a toolbutton that has a hotkey,
    // and press that hotkey, and you'll see that it gets recessed.
    
    pAccels->connectItem(pAccels->insertItem(CTRL + Key_F),
			 pForward, SLOT(animateClick()));

    pAccels->connectItem(pAccels->insertItem(CTRL + Key_U),
			 this, SLOT(toggleFullScreen()));

	ifstream config;
	config.open( QDir::homeDirPath() + "/.qtvurc" );
	if ( config.fail() )
	{
		cout << "Didn't find a config file, creating....\n";
		ofstream conf( QDir::homeDirPath() + "/.qtvurc" );
		conf << "QtVu Config File\n\n";
		conf << "400 400\n";
		conf.close();
		resize(400, 400);
	}
	else
	{
		char buf[200];
		config.getline( buf, 200 );
		int w,h;
		if ( strcmp( buf, "QtVu Config File" ) == 0 )
		{
			config >> w >> h;
			resize( w, h );
		}
		else
		{
			cout << "Error reading config file\n";
			resize(400, 400);
		}
	}

    QString s;
    s.sprintf("Welcome to %s.", APP_NAME);
    statusBar()->message((const char *)s, 1000 * 5);
}


/*!
  Destroys the window.
*/

QmMainWindow::~QmMainWindow()
{
		ofstream conf( QDir::homeDirPath() + "/.qtvurc" );
		conf << "QtVu Config File\n\n";
		conf << width() << " " << height() << endl;
		conf.close();
}


/*!
  Update the window menu to show all windows.
*/

/*
  The menu variable is static and is thusly shared by all objects of this
  class.
*/

void
QmMainWindow::updateWindowMenu(void)
{
    pWindowMenu->clear();
    
    QListIterator<QmMainWindow> wit(WindowList);

    // Build the menu

    for(QmMainWindow *w = wit.toFirst(); w != 0; w = ++wit)
    {
	// Place the mouse over the new window, maybe? Can be done
	// like this:
	// QCursor::setPos(100, 100);
    
	pWindowMenu->insertItem(w->caption(), w, SLOT(raise()));
    }
}


/*!
  Toggles between the image view and the file browser.
*/

void
QmMainWindow::toggleView()
{
    if(pStack->visibleWidget() == pBrowser)
    {
	// Disable/enable toolbuttons appropriate to this view

	/*
	cout << pForward << endl
	     << pBack << endl
	     << pSlide << endl
	     << pZoom << endl
	     << pZoomIn << endl;
	*/

//	pBack->setEnabled(true);
//	pForward->setEnabled(true);
	pSlide->setEnabled(true);
//	pZoom->setEnabled(true);
//	pZoomIn->setEnabled(true);

	/*
	pStack->raiseWidget(pImageView);
	pImageView->setFocus();
	*/

	pStack->raiseWidget(g_pImageViewBrowser);
	g_pImageViewBrowser->setFocus();
    }
    else
    {
	// Disable/enable toolbuttons appropriate to this view

	// DDD (gdb) says:
	//
	// Program received signal SIGSEGV, Segmentation fault.
	// QPixmap::isNull (this=0x0) at kernel/qpixmap.h:173
	//
	// and shows:
	//
	// #if defined(_WS_X11_)
	//    return hd == 0;
	// #else
	//    return data->hbm == 0;
	// #endif
	//
	// with a pointer to the first return
//	pBack->setEnabled(false);
//	pForward->setEnabled(false);
	pSlide->setEnabled(false);
//	pZoom->setEnabled(false);
//	pZoomIn->setEnabled(false);
	
	pStack->raiseWidget(pBrowser);
	pBrowser->setFocus();
    }
}


/*!
  Starts/stops slideshow.
*/

void
QmMainWindow::toggleSlideshow()
{
    CHECK_PTR(pSlideshowTimer);

    static bool SlideshowRunning = false;

    if(pSlideshowTimer->isActive())
	pSlideshowTimer->stop();
    
    if(SlideshowRunning)
	SlideshowRunning = false;
    else
    {
	pSlideshowTimer->start(SlideshowWait);
	SlideshowRunning = true;
    }
}


/*!

*/

void
QmMainWindow::zoomUp()
{
    CHECK_PTR(pImageView);
    CHECK_PTR(pZoomOut);
    CHECK_PTR(pZoomIn);
    
    int Zoom = pImageView->currentZoom() + ZoomStep;

//    pZoomIn->setEnabled(true);
    
    if(Zoom > 200)
	return;
//	pZoomOut->setEnabled(false); // Crashes!! Why?
    
    pImageView->zoom(Zoom);
}


/*!

*/

void
QmMainWindow::zoomDown()
{
    CHECK_PTR(pImageView);
    CHECK_PTR(pZoomOut);
    CHECK_PTR(pZoomIn);
    
    int Zoom = pImageView->currentZoom() - ZoomStep;
    
//    pZoomOut->setEnabled(true);
    
    if(Zoom < 20)
	return;
//	pZoomIn->setEnabled(false);
    
    pImageView->zoom(Zoom);
}


/*!

*/

void
QmMainWindow::zoomEntered(
    const char *ZoomFactor)
{
    CHECK_PTR(pImageView);
    
    QString s(ZoomFactor);
    pImageView->zoom(s.toInt());
}


/*!
  Shows the next image.
*/

void
QmMainWindow::nextImage()
{
    if(pStack->visibleWidget() == pImageView)
	pBrowser->queryNextImage();
}


/*!
  Shows the previous image.
*/

void
QmMainWindow::prevImage()
{
    if(pStack->visibleWidget() == pImageView)
	pBrowser->queryPrevImage();
}


/*!
  Loads the image specified by \a Filename. The image will be show in the
  image view.

  \sa showImage(const QPixmap &)
*/

void
QmMainWindow::showImage(
    const char *Filename)
{
    g_pSmallInfo->clear();
    
    QFileInfo fi(Filename);
    QString s;
    s.sprintf("Loading image %s...", (const char *)fi.fileName());
    g_pLargeInfo->setText((const char *)s);
    
    pStack->raiseWidget(g_pImageViewBrowser);
    g_pImageViewBrowser->setFocus();

    QApplication::setOverrideCursor(waitCursor);
    
    bool ImageLoaded = g_pImageViewBrowser->loadImage(Filename);
    
    QApplication::restoreOverrideCursor();
    
    if( ! ImageLoaded)
    {
	s.sprintf("Failed to load the image %s", (const char *)fi.fileName());
	statusBar()->message((const char *)s, 1000 * 5);
    }
}


/*!
  Shows the pixmap specified by \a Pixmap in the image view.

  \sa showImage(const char *Filename)
*/

void
QmMainWindow::showImage(
    const QPixmap &/*Pixmap*/)
{
    /*
    pStack->raiseWidget(pImageView);
    pImageView->setFocus();

    QApplication::setOverrideCursor(waitCursor);
    pImageView->setPixmap(Pixmap);
    QApplication::restoreOverrideCursor();
    */
}


/*!
  Toggles the menu bar on/off.
*/

void
QmMainWindow::toggleMenuBar()
{
    if(menuBar()->isVisible())
    {
        menuBar()->hide();
        pControlsMenu->setItemChecked(MenuBarToggleID, false);
    }
    else
    {
        menuBar()->show();
        pControlsMenu->setItemChecked(MenuBarToggleID, true);
    }
}


/*!
  Toggles the toolbar on/off.
*/

void
QmMainWindow::toggleToolBar()
{
    if(pToolBar->isVisible())
    {
        pToolBar->hide();
        pControlsMenu->setItemChecked(ToolBarToggleID, false);
    }
    else
    {
        pToolBar->show();
        pControlsMenu->setItemChecked(ToolBarToggleID, true);
    }
}


/*!
  Toggles the status bar on/off.
*/

void
QmMainWindow::toggleStatusBar()
{
    if(statusBar()->isVisible())
    {
        statusBar()->hide();
        pControlsMenu->setItemChecked(StatusBarToggleID, false);
    }
    else
    {
        statusBar()->show();
        pControlsMenu->setItemChecked(StatusBarToggleID, true);
    }
}


/*!
  Toggles the preview on/off.
*/

void
QmMainWindow::togglePreview()
{
    if(ShowPreview == true)
    {
	pBrowser->setPreview(false);
        pControlsMenu->setItemChecked(PreviewToggleID, false);
	ShowPreview = false;
    }
    else
    {
	pBrowser->setPreview(true);
        pControlsMenu->setItemChecked(PreviewToggleID, true);
	ShowPreview = true;
    }
}


/*!
  Toggles full screen mode.
*/

void
QmMainWindow::toggleFullScreen()
{
    static bool		FullScreen		= false;
    static bool		ToolBarVisible		= pToolBar->isVisible();
    static bool		StatusBarVisible	= statusBar()->isVisible();
    static bool		MenuBarVisible		= menuBar()->isVisible();
    static QPoint	OldPosition		= pos();
    static QSize	OldSize			= size();

    if( ! FullScreen)
    {
	// Save current settings
	
	OldPosition		= pos();
	OldSize			= size();
	ToolBarVisible		= pToolBar->isVisible();
	StatusBarVisible	= statusBar()->isVisible();
	MenuBarVisible		= menuBar()->isVisible();
	
	hide();
	pToolBar->hide();
	statusBar()->hide();
	menuBar()->hide();

	// Get the desktop widget to examine the screen width and height
	
	QWidget *d = QApplication::desktop();
	resize(d->width(), d->height());

	// Damn! I can't get the window to only have a border around,
	// but no title bar. It doesn't work by clearing the WStyle_Title
	// bit (clearing the bit work, but Qt doesn't care).
	
 	WFlags f = WStyle_Customize | WStyle_DialogBorder;
 	f &= ~WStyle_Title; // Clear the WStyle_Title bit

	recreate(parentWidget(), f, QPoint(0, 0), true);
	FullScreen = true;
    }
    else
    {
	// Lets hide the window. It gets less "painful" for the user to
	// see all the resizing, adding of toolbars etc. ;-)
	
	hide();

	// Load last saved settings
	
	if(ToolBarVisible)
	    pToolBar->show();

	if(StatusBarVisible)
	    statusBar()->show();

	if(MenuBarVisible)
	    menuBar()->show();
	
	resize(OldSize);
	
	recreate(parentWidget(),
		 WStyle_Customize | WStyle_NormalBorder,
		 OldPosition, true);
	
	FullScreen = false;
    }
}


/*!
  Shows long description of a highlighted menu item in the status bar.
*/

void
QmMainWindow::menuInfo(
    int MenuID)
{
    if(MenuID == MenuBarToggleID)
	g_pLargeInfo->setText("Toggles the menubar on/off.");
    else if(MenuID == PreviewToggleID)
	g_pLargeInfo->setText("Toggles the preview on/off.");
    else if(MenuID == StatusBarToggleID)
	g_pLargeInfo->setText("Toggles the statusbar on/off.");
    else if(MenuID == ToolBarToggleID)
	g_pLargeInfo->setText("Toggles the toolbar on/off.");
    else
	g_pLargeInfo->setText("");
}


/*!
  Shows the Options dialog.
*/

void
QmMainWindow::showOptions()
{
	QString s;
	s.sprintf( "QmMainWindow::showOptions(): Not implemented yet.\n" );
    QMessageBox::information(this, "Options", (const char *)s);
}


/*!
  Shows the formats supported.

  The formats supported depends on the compilation of the QImageIO
  extension. It can be compiled to support various formats.
*/

void
QmMainWindow::showFormats()
{
    static QmListDialog *pDialog = new QmListDialog;

    pDialog->show();
}


/*!
  Shows the Help system.
*/

void
QmMainWindow::showHelp()
{
	QString s;
	s.sprintf( "QmMainWindow::showHelp(): Not implemented yet.\n" );
    QMessageBox::information(this, "Help", (const char *)s);
}


/*!
  Shows some short information about the program.
*/

void
QmMainWindow::showAbout()
{
    QString s;
    s.sprintf(
	"%s %s\n"
	"\n"
	"This application is distributed under the\n"
	"GNU General Public License (GPL), version 2 or later,\n"
	"as defined by the Free Software Foundation.\n"
	"\n"
	"\n"
	"(C) 1998-1999 Marius Sundbakken <marius@fix.no>\n",
	APP_NAME,
	APP_VERSION);
    
    QMessageBox::about(this, APP_NAME, (const char *)s);
}


/*!
  Shows the standard Qt information dialog box.
*/

void
QmMainWindow::showQtInfo()
{
    QMessageBox::aboutQt(this, APP_NAME);
}


/*!
  Opens a new main window.
*/

void
QmMainWindow::newMainWindow()
{
    QmMainWindow *pNewWindow = new QmMainWindow;
    pNewWindow->show();
}


/*!
  Closes the window.
*/

void
QmMainWindow::closeWindow()
{
    if(WindowList.count() == 1)
    {
	delete this;
	qApp->quit();
    }
    else
    {
	WindowList.removeRef(this);
	delete this;
    }

    updateWindowMenu();
}


/*!
  Deletes the file \a FileToDelete if the user wants to.
*/

/*
  Write some replacement for QMessageBox::warning() which disables the Delete button
  for half a second or so to get rid of "multiple return key press out of habit" problems.
*/

bool
QmMainWindow::deleteItem(
    const char *FileToDelete)
{
    QFileInfo ItemToDelete(FileToDelete);

    return deleteItem(ItemToDelete);
}

/*!
  Deletes the file/dir if the user want so. Returns true if the file was
  deleted, false otherwise.
 */

bool
QmMainWindow::deleteItem(
    const QFileInfo &ItemToDelete)
{
    QString s;
	    
    if(ItemToDelete.isDir())
    {
	s.sprintf("-- NOT IMPLEMENTED YET --\n"
		  "Are you sure you want to DELETE the directory\n"
		  "%s\n"
		  "and all of its contents?",
		  ItemToDelete.filePath());
    }
    else
    {
	cout << "File path: " << ItemToDelete.filePath() << endl
	     << "File name: " << (const char *)ItemToDelete.fileName() << endl;

	s.sprintf("Are you sure you want to DELETE the file\n"
		  "%s?",
		  (const char *)ItemToDelete.fileName());

	int rc = QMessageBox::warning(0, APP_NAME,
				      (const char *)s,
				      "Delete", "Cancel", 0, 1, 1);

	if(rc == 0)
	{
	    // Delete file

	    if( ! QFile::remove(ItemToDelete.filePath()))
	    {
		s.sprintf("Failed to delete the file\n"
			  "%s",
			  ItemToDelete.filePath());

		QMessageBox::warning(0, APP_NAME,
				     (const char *)s);

		return false;
	    }
	}
    }

    return true;
    
//    cout << "QmMainWindow::deleteItem(): rc = " << rc << endl;
}


void QmMainWindow::setPictureFile( QString fname )
{
	showImage( fname );
}

void QmMainWindow::setPath( QString dir )
{
	pBrowser->setPath( dir );
}

QString QmMainWindow::getIconPath( QString name )
{
	QFileInfo f;
	f.setFile( name );
	if ( f.exists() )
		return f.absFilePath();
	f.setFile( QDir::homeDirPath() + "/." + QString( APP_NAME ) + "/icons/" + name );
	if ( f.exists() )
		return f.absFilePath();
	f.setFile( "/usr/share/icons/" + name );
	if ( f.exists() )
		return f.absFilePath();
	return QString();
}
