/* qmpreview.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <10-Jan-1999 12:24:41 marius>
 */

#include <iostream.h>
#include <math.h>

#include <qdrawutil.h>
#include <qfileinfo.h>
#include <qpainter.h>
#include <qwmatrix.h>

#include "qmpreview.h"

/*!
  \class QmPreview qmpreview.h
  \brief The QmPreview class provides a thumbnail widget.

  It can be used to show a smoothly scaled down version of a larger picture.
  The thumbnail will keep the aspect ratio. It will be scaled so that the
  entire picture fits into the widget with some margin.
*/

/*
  TODO:
  
  QmPreview should probably be derived from QmPixmapButton
*/

/*!
  Construct a preview widget.

  The \a Parent and \a Name arguments are sent to the QButton constructor.
*/

QmPreview::QmPreview(
    QWidget *Parent,
    const char *Name)
    : QButton(Parent, Name)
{
    connect(this, SIGNAL(clicked()), this, SLOT(previewClicked()));

    // No background! This speeds up thumbnail scrolling considerably!
    
    setBackgroundMode(NoBackground);

    LabelHeight = fontMetrics().height();
}


/*!

*/

QmPreview::~QmPreview()
{
}


/*!
  Draws the button frame and fills the background correctly.
*/

void
QmPreview::drawButton(
    QPainter *p)
{
    if(isDown())
    {
//	QBrush Fill(colorGroup().background().dark());
	QBrush Fill(darkBlue);

	qDrawShadePanel(p, 0, 0, width(), height(), colorGroup(), true,
			1, &Fill);
    }
    else
    {
	QBrush Fill(colorGroup().background());
    
	qDrawShadePanel(p, 0, 0, width(), height(), colorGroup(), false,
			1, &Fill);
    }

    drawButtonLabel(p);
}


/*!
  Draws the thumbnail pixmap and the pixmap filename.
*/

void
QmPreview::drawButtonLabel(
    QPainter *p)
{
    qDrawItem(p, style(),
	      width() / 2 - PixmapToShow.width() / 2,
	      (height() - LabelHeight) / 2 - PixmapToShow.height() / 2,
	      PixmapToShow.width(),
	      PixmapToShow.height(),
	      AlignCenter, colorGroup(), true, &PixmapToShow, 0);

    p->setPen(colorGroup().text());
    p->drawText(4, height() - 4 /*p->fontMetrics().ascent()*/,
		(const char *)Label);
}


/*!
  Set the preview pixmap to show \a PixmapToPreview. The pixmap is scaled
  to fit the widget's size. The aspect ratio is kept.
*/

void
QmPreview::setPixmap(
    const QPixmap &PixmapToPreview)
{
    if( ! PixmapToPreview.isNull() /*&& isVisibleToTLW()*/)
    {
	PixmapToShow = scale(PixmapToPreview);
	QButton::setPixmap(PixmapToShow);
//	repaint();
    }
}


/*!
  Set the filename associated with the preview image.
*/

void
QmPreview::setFilename(
    const QString &NewFilename)
{
    Filename = NewFilename;

    QFileInfo fi(Filename);
    Label = fi.fileName();
}

/*!
  The pixmap \a OriginalPixmap is scaled to fit the widget's size, and the resulting
  pixmap is returned. The aspect ratio is kept.
*/

QPixmap
QmPreview::scale(
    const QPixmap &OriginalPixmap) const
{
    ASSERT( ! OriginalPixmap.isNull());

    // -4 to make a little border around the scaled pixmap so it doesn't
    // overlap the button border.
    
    double x = static_cast<double>(width() - 4) / OriginalPixmap.width();
    double y = static_cast<double>(height() - LabelHeight - 4) / OriginalPixmap.height();
    double Zoom;

    Zoom = (x > y) ? y : x;
    
    QWMatrix m;
    m.scale(Zoom, Zoom);
    return OriginalPixmap.xForm(m);
}


/*!
  \internal
*/

void
QmPreview::previewClicked()
{
//    if( ! ThePixmap.isNull())
//	emit clicked(ThePixmap);
    emit clicked((const char *)Filename);
}
