/* qmtabbar.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <08-Feb-1999 03:19:35 amos>
 */

#include <iostream.h>

#include <qlayout.h>

#include "qmtabbar.h"

/*!
  \class QmTabBar qmtabbar.h
  \brief The QmTabBar is a combination of QTabBar and QTabDialog.

  This is what QTabBar should have been :-) It has the possibility
  to add widgets to it, and it can have to tabbar where you want it,
  and in which form.
*/

/*
  Based on the QTabDialog code.
*/

/*!
  
*/

QmTabBar::QmTabBar(
    QTabBar::Shape TabBarShape,
    QWidget *Parent,
    const char *Name)
    : QWidget(Parent, Name)
{
    // Taken from the QTabDialog source
    
    const int TopMargin		= 6;
    const int LeftMargin	= 6;
    const int RightMargin	= 6;
    const int BottomMargin	= 6;
    
    TabCount = 0;
    pTabBar = new QTabBar(this);
    pTabBar->setShape(TabBarShape);

    pStack = new QWidgetStack(this);

    connect(pTabBar, SIGNAL(selected(int)),
			this, SLOT(raiseWidget(int)));

    QBoxLayout *pMainLayout	= new QVBoxLayout(this, 0, -1,
						  "QmTabBar::pMainLayout");
    QBoxLayout *pTabBarLayout	= new QHBoxLayout(-1,
						  "QmTabBar::pTabBarLayout");
    QBoxLayout *pStackLayout	= new QHBoxLayout(-1,
						  "QmTabBar::pStackLayout");

    if(pTabBar->shape() == QTabBar::RoundedAbove)
    {
	pMainLayout->addLayout(pTabBarLayout, 0);
	{
//	    pTabBarLayout->addSpacing(LeftMargin);
	    pTabBarLayout->addWidget(pTabBar, 0);
	    pTabBarLayout->addStretch(1);
	    pTabBarLayout->addSpacing(RightMargin + 2);
	}

	pMainLayout->addSpacing(TopMargin);
	
	pMainLayout->addLayout(pStackLayout, 1);
	{
	    pStackLayout->addSpacing(LeftMargin);
	    pStackLayout->addWidget(pStack, 1);
	    pStackLayout->addSpacing(RightMargin);
	}

	pMainLayout->addSpacing(BottomMargin);
    }
    else if(pTabBar->shape() == QTabBar::RoundedBelow)
    {
	pMainLayout->addSpacing(TopMargin);
	
	pMainLayout->addLayout(pStackLayout, 1);
	{
	    pStackLayout->addSpacing(LeftMargin);
	    pStackLayout->addWidget(pStack, 1);
	    pStackLayout->addSpacing(RightMargin);
	}

	pMainLayout->addSpacing(BottomMargin);
	
	pMainLayout->addLayout(pTabBarLayout);
	{
//	    pTabBarLayout->addSpacing(LeftMargin);
	    pTabBarLayout->addWidget(pTabBar);
	    pTabBarLayout->addStretch(1);
	    pTabBarLayout->addSpacing(RightMargin + 2);
	}
    }
    else
	warning("QmTabBar::QmTabBar(): Unsupported shape\n");

    pMainLayout->activate();
}


/*!

*/

QmTabBar::~QmTabBar()
{
}


/*!
  Creates a new tab with label \a Label containing the widget \a pNewWidget.

  Returns the ID of the new tab.
*/

int
QmTabBar::addWidget(
    QWidget *pNewWidget,
    const char *Label)
{
    TabCount++;
    
    QTab *pNewTab = new QTab;
    pNewTab->label = Label;
    int ID = pTabBar->addTab(pNewTab);
    pTabBar->setMinimumSize(pTabBar->sizeHint());
    
    pStack->addWidget(pNewWidget, ID);

    return ID;
}


/*!
  Returns a pointer to the currently visible widget.
*/

QWidget *
QmTabBar::visibleWidget(void) const
{
    return pStack->visibleWidget();
}


/*!
  Returns the ID of the current visible tab.

  DOESN'T WORK!
*/

int
QmTabBar::currentTab(void) const
{
//     cout << "pStack->visibleWidget(): " << pStack->visibleWidget() << endl;
//     cout << "pTabBar->currentTab(): " << pTabBar->currentTab() << endl;
//     cout << "pStack->id(...): " << pStack->id(pStack->visibleWidget()) << endl;
    
//     ASSERT(pTabBar->currentTab() == pStack->id(pStack->visibleWidget()));
    
    return pTabBar->currentTab();
}


/*!
  Paints the tab page border.
*/

void
QmTabBar::paintEvent(
    QPaintEvent *)
{
    if(TabCount == 0)
	return;
    
    QPainter p;
    p.begin( this );

    QCOORD t = 0;
    QCOORD b = height() - 1;
    QCOORD l = 0;
    QCOORD r = width() - 1;

    p.setPen(colorGroup().light());
    
    // note - this line overlaps the bottom/top line drawn by QTabBar

    if(pTabBar->shape() == QTabBar::RoundedAbove)
    {
	t += pTabBar->height() - 1;
	
	p.drawLine( l + pTabBar->sizeHint().width(), t, r - 1, t );
	p.drawLine( l, t + 1, l, b );
	p.setPen( black );
	p.drawLine( r, b, l,b );
	p.drawLine( r, b-1, r, t );
	p.setPen( colorGroup().dark() );
	p.drawLine( l+1, b-1, r-1, b-1 );
	p.drawLine( r-1, b-2, r-1, t+1 );
    }
    else
    {
	b -= pTabBar->height() - 2;
	
	p.drawLine( l, t, r - 1, t );
	p.drawLine( l, t + 1, l, b );
	p.setPen( black );
	p.drawLine( r, b, l + pTabBar->sizeHint().width(), b );
	p.drawLine( r, b-1, r, t );
	p.setPen( colorGroup().dark() );
	p.drawLine( l + pTabBar->sizeHint().width(), b-1, r-1, b-1 );
	p.drawLine( r-1, b-2, r-1, t+1 );
    }
    
    p.end();    
}

void QmTabBar::raiseWidget( int w )
{
	pStack->raiseWidget( w );
	emit aboutToShow( w );
	emit aboutToShow( pStack->widget( w ) );
}

