/* qmthumbnaillist.cpp
 *
 * Marius Sundbakken <marius@fix.no>
 *
 * Last modified: Time-stamp: <10-Jan-1999 11:49:41 marius>
 */

#include <iostream.h>

#include <qapplication.h>
#include <qbrush.h>
#include <qdir.h>
#include <qimage.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qprogressbar.h>
#include <qtooltip.h>

#include "qmimageviewbrowser.h"
#include "qmpreview.h"
#include "qmthumbnaillist.h"

extern QProgressBar		*g_pProgress;
extern QLabel			*g_pSmallInfo;
extern QLabel			*g_pLargeInfo;
extern QmImageViewBrowser	*g_pImageViewBrowser;
extern QPixmap			*folderIcon; // from qminfoitem.cpp

/*!
  \class QmThumbnailList qmthumbnaillist.h
  \brief The QmThumbnailList class provides a thumbnail browser.

  It can be used to list a directory. All directories will be listed
  and all files recognized as images will get a thumbnail (a miniature
  of the image).
*/

/*!
  Construct a thumbnail widget.

  The \a Parent and \a Name arguments are sent to the QScrollView constructor.
*/

QmThumbnailList::QmThumbnailList(
    QWidget *Parent,
    const char *Name)
    : QScrollView(Parent, Name)
{
    ChildList.setAutoDelete(true);

    setFocusPolicy(StrongFocus);
    viewport()->setBackgroundMode(PaletteBase);
}


/*!

*/

QmThumbnailList::~QmThumbnailList()
{
}


/*!

*/

void
QmThumbnailList::drawContentsOffset(
    QPainter *p,
    int OffsetX, int OffsetY,
    int ClipX, int ClipY,
    int ClipW, int ClipH)
{
    p->setClipRect(ClipX - OffsetX, ClipY - OffsetY, ClipW, ClipH);

//    QBrush Fill(black, SolidPattern);
//    p->fillRect(ClipX-OffsetX, ClipY-OffsetY, ClipW, ClipH, Fill);

    QListIterator<QWidget> wit(ChildList);

    for(QWidget *w = wit.toFirst(); w != 0; w = ++wit)
    {
// 	cout << w << " " << w->name() << " " << w->className() << " is ";

// 	if(childIsVisible(w))
// 	    cout << "visible" << endl;
// 	else
// 	    cout << "not visible" << endl;

	// Flickers alot!
	
// 	if(childIsVisible(w))
// 	    w->repaint();
//	    w->show();

    }
}


/*!
  Rearranges the icons/thumbnails.
*/

void
QmThumbnailList::resizeEvent(
    QResizeEvent *e)
{
    int x = HSpacing;
    int y = VSpacing;
    
    QListIterator<QWidget> it(ChildList);

    for(QWidget *w = it.toFirst(); w != 0; w = ++it)
    {
	moveChild(w, x, y);
	
	x += w->width() + HSpacing;
	
	if(x > width() - (w->width() + HSpacing))
	{
	    y += w->height() + VSpacing;
	    x = HSpacing;
	}
    }

    QWidget *w = it.toFirst();
    if(w != 0)
	y += w->height() + VSpacing;
    
    resizeContents(x, y);
    QScrollView::resizeEvent(e);
    viewport()->repaint();
    repaint();
}

/*!
  Creates thumbnails for all pictures in the given path \a Path.
*/

/*
  At the moment, this code is far from efficient, and it's not
  meant to be either. At least not yet. You've been warned ;-)

  TODO:
  -  `..' will be listed when path is root.
  - There are some bugs with the final size of the scrollview. Also when
    resizing during progress.

  Note that the sorting is different from the textual list.
  The difference is in capital letters, it seems.
*/

void
QmThumbnailList::setPath(
    QString Path)
{
    CHECK_PTR(g_pLargeInfo);
    CHECK_PTR(g_pSmallInfo);
    CHECK_PTR(g_pProgress);
    CHECK_PTR(folderIcon);

    cout << "QmThumbnailList::setPath(): Path = " << Path << endl;
    
    // Remove all the previous thumbnails from the scrollview.
    
    QListIterator<QWidget> wit(ChildList);

    for(QWidget *w = wit.toFirst(); w != 0; w = ++wit)
    {
	removeChild(w);
    }
    
    ChildList.clear();

    // Reset position counters
    
    xPos = HSpacing;
    yPos = VSpacing;
    
    resizeContents(0, 0);
    qApp->processEvents();
    viewport()->repaint();

    QDir d(Path);
    d.setFilter(QDir::Files | QDir::Dirs | QDir::Hidden);
//    d.setNameFilter("*.jpg");
//    d.setNameFilter(pFilter->currentText());
    const QFileInfoList		*list = d.entryInfoList();
    QFileInfoListIterator	it(*list);
    QFileInfo			*fi;
    QFileInfoList		ImageFiles;
    QFileInfoList		Directories;

    // Build directory list and image file list.
    
    for(fi = it.toFirst(); fi != 0; fi = ++it)
    {
	if(fi->isDir() && fi->fileName() != ".")
	{
	    Directories.append(new QFileInfo(*fi));
	}
	else if(QImage::imageFormat(fi->filePath()))
	{
	    ImageFiles.append(new QFileInfo(*fi));
	}
    }

    
    // Insert directories into browser (as long as the browser is visible).

    QFileInfoListIterator	dit(Directories); // Dir ITerator
    QmPreview			*pDirBox = 0;
    QPixmap			DirImage(*folderIcon);
    
    for(fi = dit.toFirst(); fi != 0 && isVisibleToTLW(); fi = ++dit)
    {
	pDirBox = new QmPreview(this);
	pDirBox->setFixedSize(HSize, VSize);
	pDirBox->setFocusPolicy(QWidget::NoFocus);
	pDirBox->setPixmap(DirImage);
	pDirBox->setFilename(fi->filePath());
	QToolTip::add(pDirBox, fi->fileName());

	appendIconBox(pDirBox);

	connect(pDirBox, SIGNAL(clicked(const char *)),
		this, SLOT(directoryClicked(const char *)));
    }

    qApp->processEvents();
    viewport()->repaint();


    // Insert images into browser (as long as the browser is visible).

    QFileInfoListIterator	iit(ImageFiles); // Image ITerator
    QmPreview			*pImageBox;
    QPixmap			Pixmap;
    int				ImageCount = 0;
    QString			s;

    g_pProgress->setTotalSteps(iit.count());
    g_pProgress->reset();
    
    for(fi = iit.toFirst(); fi != 0 && isVisibleToTLW(); fi = ++iit)
    {
	s.sprintf("Generating thumbnail picture for %s",
		  (const char *)fi->fileName());
	g_pLargeInfo->setText((const char *)s);
	
	if(Pixmap.load(fi->filePath()))
	{
	    g_pProgress->setProgress(++ImageCount);
	    s.sprintf("%d/%d", ImageCount, iit.count());
	    g_pSmallInfo->setText((const char *)s);
	    
	    pImageBox = new QmPreview(this);
	    pImageBox->setFixedSize(HSize, VSize);
	    pImageBox->setFocusPolicy(QWidget::NoFocus);
	    pImageBox->setPixmap(Pixmap);
	    pImageBox->setFilename(fi->filePath());
	    QToolTip::add(pImageBox, fi->fileName());

	    appendIconBox(pImageBox);

	    connect(pImageBox, SIGNAL(clicked(const char *)),
		    this, SLOT(thumbnailClicked(const char *)));
	}
	
	qApp->processEvents();
    }
    
    g_pProgress->reset();
    g_pSmallInfo->clear();
    g_pLargeInfo->clear();
    
    resizeContents(xPos, yPos + VSize + VSpacing);
}


/*!
  Adds a new iconbox (i.e. a placeholder for a directory/image).
*/

void
QmThumbnailList::appendIconBox(
    QmPreview *pNewIconBox)
{
    ChildList.append(pNewIconBox);
    addChild(pNewIconBox, xPos, yPos);
		
    pNewIconBox->show();
		
    // The placement will go "wrong" when the scrollview
    // is resized during thumbnail creation.
		
    xPos += pNewIconBox->width() + HSpacing;

    if(xPos > width() - (pNewIconBox->width() + HSpacing))
    {
	yPos += pNewIconBox->height() + VSpacing;
	xPos = HSpacing;
    }

    // Resize the scollview, and handle any waiting events

    resizeContents(xPos, yPos);
}


/*!
  Emits the selected() signal whenever a thumbnail is clicked.
  The signal contains the (full) filename associated to the thumbnail
  being clicked.
*/

void
QmThumbnailList::thumbnailClicked(
    const char *Filename)
{
    emit selected(Filename);
}


/*!
  Emits the dirSelected() signal whenever a directory is clicked.
  The signal contains the full path of the directory.
*/

void
QmThumbnailList::directoryClicked(
    const char *Directory)
{
    QString s = QDir::cleanDirPath(Directory);
    emit dirSelected((const char *)s);
}


/*!
  \internal
*/

void
QmThumbnailList::debug(void)
{
    cout << "\nDEBUG START ------------------------------------------------\n";
    cout << "ChildList:\n";

    QWidget *w = ChildList.first();
    
    for(uint i = 0; i < ChildList.count(); i++)
    {
	cout << "i: " << i << " = " << w;

	if(w != 0)
	    cout << " Name: " << w->name() << " of class " << w->className();

	cout << endl;
	
	w = ChildList.next();
    }

    cout << "\nDEBUG END --------------------------------------------------\n";
}

