/*
   libmikmod for x11amp

   Modified mikmod lib 2/1/99 for x11amp by J. Nick Koston (BlueDraco)

   Current Version 0.9.6

   Changes: (1) 2/1/99 Inital Build / Release
   (2) 2/2/99 Psychotron fixed a small problem w/interfacing
   (3) 2/2/99 Shows Channels in spectrum analyzer
   (4) 2/2/99 Fixed the infinate loop problem, and the absured channel
   value problem, removed the going stuff
   (5) 2/2/99 Added back in going (it is needed)
   (6) 2/2/99 Repaired SEGV on playlist goes to next song, bad
   printf when the mod cannot be loaded and x11amp is in repeat mode.
   Also made the absured channel display code work 

 */

#include "mikmod-plugin.h"

pthread_t decode_thread;

/* current module */
MODULE *mf = NULL;
extern MODULE *mf;

/* module parameters */
BOOL cfg_extspd = 1;		/* Extended Speed enable */
BOOL cfg_panning = 1;		/* DMP panning enable (8xx effects) */
BOOL cfg_wrap = 0;		/* auto song-wrapping disable */
BOOL cfg_loop = 0;		/* allow module to loop */
BOOL cfg_fade = 0;		/* allow volume fade during last pattern */
BOOL cfg_curious = 0;		/* look for extra patterns */
BOOL going = 1;



InputPlugin ip =
{
  NULL,
  NULL,
  "MikMod Player",
  init,
  NULL,
  NULL,
  is_our_file,
  play_file,
  stop,
  mod_pause,
  seek,
  NULL,
  get_time,
  NULL,
  NULL,
  NULL,
};

void
stop (void)
{
  if (going)
    {
      going = 0;
      pthread_join (decode_thread, NULL);
    }
}

void
seek (int time)
{
  /*
     We need to seek in pattrens somehow
     can't seek by time only by X pattrens on way or the other

     Player_NextPosition ();
     Player_PrevPosition ();

   */
  return;
}

void
mod_pause (short p)
{
  ip.output->pause (p);
}


int
get_time (void)
{
  if (!Player_Active ())
    return -1;
  return ip.output->output_time ();
}

InputPlugin *
get_iplugin_info (void)
{
  return &ip;
}

void
init (void)
{
  md_mixfreq = 44100;		/* standard mixing freq */
  md_device = 0;		/* standard device: autodetect */
  md_pansep = 128;		/* panning separation */
  md_reverb = 0;		/* Reverb (max 15) */

  md_mode = DMODE_16BITS | DMODE_STEREO | DMODE_SOFT_MUSIC;

  MikMod_RegisterAllLoaders ();
  MikMod_RegisterAllDrivers ();
  MikMod_Init ();

}

int
is_our_file (char *filename)
{
  char *ext;
  ext = strrchr (filename, '.');
  if (ext)
    {
      if (!strcasecmp (ext, ".669"))
	return 1;
      if (!strcasecmp (ext, ".amf"))
	return 1;
      if (!strcasecmp (ext, ".dsm"))
	return 1;
      if (!strcasecmp (ext, ".far"))
	return 1;
      if (!strcasecmp (ext, ".it"))
	return 1;
      if (!strcasecmp (ext, ".m15"))
	return 1;
      if (!strcasecmp (ext, ".med"))
	return 1;
      if (!strcasecmp (ext, ".mod"))
	return 1;
      if (!strcasecmp (ext, ".mtm"))
	return 1;
      if (!strcasecmp (ext, ".s3m"))
	return 1;
      if (!strcasecmp (ext, ".stm"))
	return 1;
      if (!strcasecmp (ext, ".ult"))
	return 1;
      if (!strcasecmp (ext, ".xm"))
	return 1;
    }
  return 0;
}



void
play_file (char *filename)
{
  char archive[PATH_MAX];
  char *playfile;
  int cfg_maxchn = 128, c;
  int tolerant = 0;
  int uservolume = 128;


  if (!(mf = Player_Load (filename, cfg_maxchn, cfg_curious)))
    {
      printf ("Couldn't load mod\n");
      going = 0;
      return;
    }

  mf->extspd = cfg_extspd;
  mf->panflag = cfg_panning;
  mf->wrap = cfg_wrap;
  mf->loop = cfg_loop;
  mf->fadeout = cfg_fade;


  if (!(ip.output->open_audio (FMT_S16_NE, md_mixfreq, DMODE_STEREO)))
    {
      printf ("Couldn't open Audio\n");
      going = 0;
      return;
    }

  Player_Start (mf);
  going = 1;
  if (mf->volume > uservolume)
    Player_SetVolume (uservolume);


/* mods are in pattrens .. you need to be able to seek
   back the forth from pattrens */

  ip.set_info (mf->songname, -1, 0, md_mixfreq, DMODE_STEREO);
  pthread_create (&decode_thread, NULL, play_loop, NULL);
  return;

}

void *
play_loop (void *arg)
{
  int channel;			/* Number of channels in the mod */
  int cps;			/* How many channels per spectrum line */
  int speccount;
  int specchn = 0;
  UWORD specval;
  UWORD maxperiod = 1;		/* Something nice ... above zero */
  unsigned char spectrum[75];

  cps = (75 / mf->numchn);
  if (((cps * mf->numchn) > 74))
    {
      cps -= 1;
    }

  while (Player_Active () && going)
    {
      memset (spectrum, 0, 75);

      specchn = 0;
      for (channel = 0; channel < mf->numchn; channel++)
	{
	  if (mf->control[channel].period > maxperiod)
	    {
	      /* Need to account for absurd values in the
	         channel so the whole thing does disappear */
	      if ((((mf->control[channel].period < (maxperiod * 4)) && (maxperiod < mf->control[channel].period)) || (maxperiod == 1)))
		{
		  maxperiod = mf->control[channel].period;
		}
	    }

	  specval = ((((mf->control[channel].period * 100) / (maxperiod * 10)) * 15) / 10);


	  for (speccount = 0; speccount <= cps; speccount++)
	    {
	      spectrum[specchn] = specval;
	      spectrum[specchn] = spectrum[specchn] > 15 ? 15 : spectrum[specchn];
	      specchn++;
	    }
	}
      MikMod_Update ();
      ip.set_spectrum (ip.output->written_time (), spectrum);

    }
  Player_Stop ();		/* stop playing */
  while (Player_Active ())
    {
      usleep (10000);
    }
  Player_Free (mf);		/* and free the module */
  ip.output->close_audio ();
  going = 0;
  pthread_exit (NULL);
}
