/*  x11amp - graphically mp3 player..
 *  Copyright (C) 1998-1999  Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "x11amp.h"
#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <getopt.h>
#include "libx11amp/configfile.h"


		
GtkWidget *mainwin,*mainwin_filesel=NULL;
GtkItemFactory *mainwin_options_menu,*mainwin_songname_menu,*mainwin_vis_menu;
GdkPixmap *mainwin_bg=NULL,*mainwin_bg_dblsize;
GdkGC	  *mainwin_gc;
GdkWindow *root_window;

GtkAccelGroup *mainwin_accel;

gboolean	mainwin_moving=FALSE,mainwin_focus=FALSE,mainwin_move_playlist=FALSE,mainwin_move_equalizer=FALSE;
gboolean start_playing=FALSE;
gint		mainwin_move_x,mainwin_move_y;
gint		mainwin_timeout_tag;

PButton		*mainwin_minimize,*mainwin_shade,*mainwin_close;
PButton 	*mainwin_rew,*mainwin_play,*mainwin_pause,*mainwin_stop,*mainwin_fwd,*mainwin_eject;
TButton 	*mainwin_shuffle,*mainwin_repeat,*mainwin_eq,*mainwin_pl;
TextBox 	*mainwin_info,*mainwin_rate_text,*mainwin_freq_text;
MenuRow		*mainwin_menurow;
HSlider 	*mainwin_volume,*mainwin_balance,*mainwin_position;
MonoStereo	*mainwin_monostereo;
PlayStatus	*mainwin_playstatus;
Number		*mainwin_minus_num,*mainwin_10min_num,*mainwin_min_num,*mainwin_10sec_num,*mainwin_sec_num;
Vis		*mainwin_vis;

LList		*mainwin_wlist;


Config cfg;

static GtkTargetEntry drop_types [] = {
	{ "text/plain", 0, 1 }
};

#define MAINWIN_OPTIONS_MENU_ENTRIES 14

void mainwin_options_menu_callback(gpointer cb_data,guint action,GtkWidget *w);

enum {MAINWIN_OPT_PREFS,MAINWIN_OPT_SKIN,MAINWIN_OPT_REPEAT,MAINWIN_OPT_SHUFFLE,
	MAINWIN_OPT_NPA,MAINWIN_OPT_TELAPSED,MAINWIN_OPT_TREMAINING,MAINWIN_OPT_ALWAYS,MAINWIN_OPT_WS,
	MAINWIN_OPT_PWS,MAINWIN_OPT_DOUBLESIZE};


GtkItemFactoryEntry	mainwin_options_menu_entries[]=
{
	"/Preferences",			"<control>P",mainwin_options_menu_callback,MAINWIN_OPT_PREFS,"<Item>",
	"/Skin Browser",		"<alt>S",mainwin_options_menu_callback,MAINWIN_OPT_SKIN,"<Item>",
	"/-",				NULL,NULL,0,"<Separator>",
	"/Repeat",			"R",mainwin_options_menu_callback,MAINWIN_OPT_REPEAT,"<ToggleItem>",
	"/Shuffle",			"S",mainwin_options_menu_callback,MAINWIN_OPT_SHUFFLE,"<ToggleItem>",
	"/No Playlist Advance",		NULL,mainwin_options_menu_callback,MAINWIN_OPT_NPA,"<ToggleItem>",
	"/-",				NULL,NULL,0,"<Separator>",
	"/Time Elapsed",		"<control>E",mainwin_options_menu_callback,MAINWIN_OPT_TELAPSED,"<RadioItem>",
	"/Time Remaining",		"<control>R",mainwin_options_menu_callback,MAINWIN_OPT_TREMAINING,"/Time Elapsed",
	"/-",				NULL,NULL,0,"<Separator>",
	"/Always On Top",		"<control>A",mainwin_options_menu_callback,MAINWIN_OPT_ALWAYS,"<ToggleItem>",
	"/WindowShade Mode",		"<control>W",mainwin_options_menu_callback,MAINWIN_OPT_WS,"<ToggleItem>",
	"/Playlist WindowShade Mode",	"<control><shift>W",mainwin_options_menu_callback,MAINWIN_OPT_PWS,"<ToggleItem>",
	"/DoubleSize",			"<control>D",mainwin_options_menu_callback,MAINWIN_OPT_DOUBLESIZE,"<ToggleItem>",
};

#define MAINWIN_SONGNAME_MENU_ENTRIES 4

void mainwin_songname_menu_callback(gpointer cb_data,guint action,GtkWidget *w);

enum {MAINWIN_SONGNAME_FILEINFO,MAINWIN_SONGNAME_JTF,MAINWIN_SONGNAME_JTT,MAINWIN_SONGNAME_SCROLL};

GtkItemFactoryEntry mainwin_songname_menu_entries[]=
{
	"/File Info",	"<alt>3",mainwin_songname_menu_callback,MAINWIN_SONGNAME_FILEINFO,"<Item>",
	"/Jump To File","J",mainwin_songname_menu_callback,MAINWIN_SONGNAME_JTF,"<Item>",
	"/Jump To Time","<control>J",mainwin_songname_menu_callback,MAINWIN_SONGNAME_JTT,"<Item>",
	"/Autoscroll Songname",NULL,mainwin_songname_menu_callback,MAINWIN_SONGNAME_SCROLL,"<ToggleItem>",
};
	
#define MAINWIN_VIS_MENU_ENTRIES 37

void mainwin_vis_menu_callback(gpointer cb_data,guint action,GtkWidget *w);	
		
enum
{
	MAINWIN_VIS_ANALYZER,MAINWIN_VIS_SCOPE,MAINWIN_VIS_OFF,
	MAINWIN_VIS_ANALYZER_NORMAL,MAINWIN_VIS_ANALYZER_FIRE,MAINWIN_VIS_ANALYZER_VLINES,
	MAINWIN_VIS_ANALYZER_LINES,MAINWIN_VIS_ANALYZER_BARS,MAINWIN_VIS_ANALYZER_PEAKS,
	MAINWIN_VIS_SCOPE_DOT,MAINWIN_VIS_SCOPE_LINE,MAINWIN_VIS_SCOPE_SOLID,
	MAINWIN_VIS_VU_NORMAL,MAINWIN_VIS_VU_SMOOTH,
	MAINWIN_VIS_REFRESH_FULL,MAINWIN_VIS_REFRESH_HALF,MAINWIN_VIS_REFRESH_QUARTER,MAINWIN_VIS_REFRESH_EIGTH,
	MAINWIN_VIS_AFALLOFF_SLOWEST,MAINWIN_VIS_AFALLOFF_SLOW,MAINWIN_VIS_AFALLOFF_MEDIUM,MAINWIN_VIS_AFALLOFF_FAST,MAINWIN_VIS_AFALLOFF_FASTEST,
	MAINWIN_VIS_PFALLOFF_SLOWEST,MAINWIN_VIS_PFALLOFF_SLOW,MAINWIN_VIS_PFALLOFF_MEDIUM,MAINWIN_VIS_PFALLOFF_FAST,MAINWIN_VIS_PFALLOFF_FASTEST		
};
			
	
GtkItemFactoryEntry mainwin_vis_menu_entries[]=
{
	"/Visualization Mode",NULL,NULL,0,"<Branch>",
	"/Visualization Mode/Analyzer",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_ANALYZER,"<RadioItem>",
	"/Visualization Mode/Scope",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_SCOPE,"/Visualization Mode/Analyzer",
	"/Visualization Mode/Off",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_OFF,"/Visualization Mode/Analyzer",
	"/Analyzer Mode",NULL,NULL,0,"<Branch>",
	"/Analyzer Mode/Normal",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_ANALYZER_NORMAL,"<RadioItem>",
	"/Analyzer Mode/Fire",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_ANALYZER_FIRE,"/Analyzer Mode/Normal",
	"/Analyzer Mode/Vertical Lines",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_ANALYZER_VLINES,"/Analyzer Mode/Normal",
	"/Analyzer Mode/-",NULL,NULL,0,"<Separator>",
	"/Analyzer Mode/Lines",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_ANALYZER_LINES,"<RadioItem>",
	"/Analyzer Mode/Bars",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_ANALYZER_BARS,"/Analyzer Mode/Lines",
	"/Analyzer Mode/-",NULL,NULL,0,"<Separator>",
	"/Analyzer Mode/Peaks",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_ANALYZER_PEAKS,"<ToggleItem>",
	"/Scope Mode",NULL,NULL,0,"<Branch>",
	"/Scope Mode/Dot Scope",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_SCOPE_DOT,"<RadioItem>",
	"/Scope Mode/Line Scope",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_SCOPE_LINE,"/Scope Mode/Dot Scope",
	"/Scope Mode/Solid Scope",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_SCOPE_SOLID,"/Scope Mode/Dot Scope",
	"/WindowShade VU Mode",NULL,NULL,0,"<Branch>",
	"/WindowShade VU Mode/Normal",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_VU_NORMAL,"<RadioItem>",
	"/WindowShade VU Mode/Smooth",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_VU_SMOOTH,"/WindowShade VU Mode/Normal",
	"/Refresh Rate",NULL,NULL,0,"<Branch>",
	"/Refresh Rate/Full (~50 fps)",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_REFRESH_FULL,"<RadioItem>",
	"/Refresh Rate/Half (~25 fps)",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_REFRESH_HALF,"/Refresh Rate/Full (~50 fps)",
	"/Refresh Rate/Quarter (~13 fps)",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_REFRESH_QUARTER,"/Refresh Rate/Full (~50 fps)",
	"/Refresh Rate/Eigth (~6 fps)",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_REFRESH_EIGTH,"/Refresh Rate/Full (~50 fps)",
	"/Analyzer Falloff",NULL,NULL,0,"<Branch>",
	"/Analyzer Falloff/Slowest",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_AFALLOFF_SLOWEST,"<RadioItem>",
	"/Analyzer Falloff/Slow",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_AFALLOFF_SLOW,"/Analyzer Falloff/Slowest",
	"/Analyzer Falloff/Medium",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_AFALLOFF_MEDIUM,"/Analyzer Falloff/Slowest",
	"/Analyzer Falloff/Fast",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_AFALLOFF_FAST,"/Analyzer Falloff/Slowest",
	"/Analyzer Falloff/Fastest",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_AFALLOFF_FASTEST,"/Analyzer Falloff/Slowest",
	"/Peaks Falloff",NULL,NULL,0,"<Branch>",
	"/Peaks Falloff/Slowest",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_PFALLOFF_SLOWEST,"<RadioItem>",
	"/Peaks Falloff/Slow",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_PFALLOFF_SLOW,"/Peaks Falloff/Slowest",
	"/Peaks Falloff/Medium",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_PFALLOFF_MEDIUM,"/Peaks Falloff/Slowest",
	"/Peaks Falloff/Fast",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_PFALLOFF_FAST,"/Peaks Falloff/Slowest",
	"/Peaks Falloff/Fastest",NULL,mainwin_vis_menu_callback,MAINWIN_VIS_PFALLOFF_FASTEST,"/Peaks Falloff/Slowest"
};

/*
 * If you change the menu above change these defines also
 */

#define MAINWIN_VIS_MENU_VIS_MODE		1
#define MAINWIN_VIS_MENU_NUM_VIS_MODE		3
#define MAINWIN_VIS_MENU_ANALYZER_MODE		5
#define MAINWIN_VIS_MENU_NUM_ANALYZER_MODE	3
#define MAINWIN_VIS_MENU_ANALYZER_TYPE	    	9
#define MAINWIN_VIS_MENU_NUM_ANALYZER_TYPE  	2
#define MAINWIN_VIS_MENU_ANALYZER_PEAKS		12
#define MAINWIN_VIS_MENU_SCOPE_MODE		14
#define MAINWIN_VIS_MENU_NUM_SCOPE_MODE		3
#define MAINWIN_VIS_MENU_WSHADEVU_MODE		18
#define MAINWIN_VIS_MENU_NUM_WSHADEVU_MODE	2
#define MAINWIN_VIS_MENU_REFRESH_RATE		21
#define MAINWIN_VIS_MENU_NUM_REFRESH_RATE	4
#define MAINWIN_VIS_MENU_AFALLOFF		26
#define MAINWIN_VIS_MENU_NUM_AFALLOFF		5
#define MAINWIN_VIS_MENU_PFALLOFF		32
#define MAINWIN_VIS_MENU_NUM_PFALLOFF		5

static void read_config(void)
{
	ConfigFile *cfgfile;
	gchar *tmp,*filename,eqtext[18];
	gint i,tmpint;
	
	memset(&cfg,0,sizeof(Config));
	cfg.outputplugin="";
	cfg.autoscroll=TRUE;
	cfg.player_x=20;
	cfg.player_y=20;
	cfg.always_show_cb=TRUE;
	cfg.convert_underscore=TRUE;
	cfg.convert_twenty=TRUE;
	cfg.show_numbers_in_pl=TRUE;
	cfg.snap_windows=TRUE;
	
	cfg.playlist_x=295;
	cfg.playlist_y=20;
	cfg.playlist_width=300;
	cfg.playlist_height=232;
	
	
	cfg.equalizer_x=20;
	cfg.equalizer_y=136;
			
	cfg.vis_type=VIS_ANALYZER;
	cfg.analyzer_mode=ANALYZER_FIRE;
	cfg.analyzer_type=ANALYZER_LINES;
	cfg.analyzer_peaks=TRUE;
	cfg.scope_mode=SCOPE_DOT;
	cfg.vu_mode=VU_NORMAL;
	cfg.vis_refresh=REFRESH_FULL;
	cfg.analyzer_falloff=FALLOFF_FAST;
	cfg.peaks_falloff=FALLOFF_FAST;
	
	filename=g_strconcat(g_get_home_dir(),"/.x11amp",NULL);
	mkdir(filename,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	g_free(filename);
	filename=g_strconcat(g_get_home_dir(),"/.x11amp/Skins",NULL);
	mkdir(filename,S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
	g_free(filename);
	
	filename=g_strconcat(g_get_home_dir(),"/.x11amp/config",NULL);
	
	cfgfile=x11amp_cfg_open_file(filename);
	if(cfgfile)
	{
		x11amp_cfg_read_int(cfgfile,"x11amp","player_x",&cfg.player_x);
		x11amp_cfg_read_int(cfgfile,"x11amp","player_y",&cfg.player_y);
		x11amp_cfg_read_boolean(cfgfile,"x11amp","shuffle",&cfg.shuffle);
		x11amp_cfg_read_boolean(cfgfile,"x11amp","repeat",&cfg.repeat);
		x11amp_cfg_read_boolean(cfgfile,"x11amp","doublesize",&cfg.doublesize);
		x11amp_cfg_read_boolean(cfgfile,"x11amp","autoscroll_songname",&cfg.autoscroll);
		x11amp_cfg_read_int(cfgfile,"x11amp","timer_mode",&cfg.timer_mode);
		x11amp_cfg_read_int(cfgfile,"x11amp","vis_type",&cfg.vis_type);
		x11amp_cfg_read_int(cfgfile,"x11amp","analyzer_mode",&cfg.analyzer_mode);
		x11amp_cfg_read_int(cfgfile,"x11amp","analyzer_type",&cfg.analyzer_type);
		x11amp_cfg_read_boolean(cfgfile,"x11amp","analyzer_peaks",&cfg.analyzer_peaks);
		x11amp_cfg_read_int(cfgfile,"x11amp","scope_mode",&cfg.scope_mode);
		x11amp_cfg_read_int(cfgfile,"x11amp","vu_mode",&cfg.vu_mode);
		x11amp_cfg_read_int(cfgfile,"x11amp","vis_refresh_rate",&cfg.vis_refresh);
		x11amp_cfg_read_int(cfgfile,"x11amp","analyzer_falloff",&cfg.analyzer_falloff);
		x11amp_cfg_read_int(cfgfile,"x11amp","peaks_falloff",&cfg.peaks_falloff);
		x11amp_cfg_read_int(cfgfile,"x11amp","playlist_x",&cfg.playlist_x);
		x11amp_cfg_read_int(cfgfile,"x11amp","playlist_y",&cfg.playlist_y);
		x11amp_cfg_read_int(cfgfile,"x11amp","playlist_width",&cfg.playlist_width);
		x11amp_cfg_read_int(cfgfile,"x11amp","playlist_height",&cfg.playlist_height);
		x11amp_cfg_read_boolean(cfgfile,"x11amp","playlist_visible",&cfg.playlist_visible);
		if(!x11amp_cfg_read_string(cfgfile,"x11amp","playlist_font",&cfg.playlist_font))
			cfg.playlist_font=g_strdup("-*-helvetica-bold-r-*-*-10-*");
		x11amp_cfg_read_int(cfgfile,"x11amp","equalizer_x",&cfg.equalizer_x);	
		x11amp_cfg_read_int(cfgfile,"x11amp","equalizer_y",&cfg.equalizer_y);	
		x11amp_cfg_read_boolean(cfgfile,"x11amp","equalizer_visible",&cfg.equalizer_visible);	
		x11amp_cfg_read_boolean(cfgfile,"x11amp","equalizer_active",&cfg.equalizer_active);	
		x11amp_cfg_read_float(cfgfile,"x11amp","equalizer_preamp",&cfg.equalizer_preamp);	
		for(i=0;i<10;i++)
		{
			sprintf(eqtext,"equalizer_band%d",i);
			x11amp_cfg_read_float(cfgfile,"x11amp",eqtext,&cfg.equalizer_bands[i]);
		}
		x11amp_cfg_read_string(cfgfile,"x11amp","skin",&cfg.skin);
		x11amp_cfg_read_string(cfgfile,"x11amp","output_plugin",&cfg.outputplugin);
		x11amp_cfg_read_string(cfgfile,"x11amp","filesel_path",&cfg.filesel_path);
		x11amp_cfg_free(cfgfile);
	}
	else
	{
		cfg.playlist_font=g_strdup("-*-helvetica-bold-r-*-*-10-*");
	}
	g_free(filename);
}

static void save_config(void)
{
	gchar *filename,*str;
	int i;
	ConfigFile *cfgfile;
	
	
	
	filename=g_strconcat(g_get_home_dir(),"/.x11amp/config",NULL);
	cfgfile=x11amp_cfg_open_file(filename);
	if(!cfgfile)
		cfgfile=x11amp_cfg_new();
	x11amp_cfg_write_int(cfgfile,"x11amp","player_x",cfg.player_x);
	x11amp_cfg_write_int(cfgfile,"x11amp","player_y",cfg.player_y);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","shuffle",mainwin_shuffle->tb_selected);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","repeat",mainwin_repeat->tb_selected);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","doublesize",cfg.doublesize);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","autoscroll_songname",cfg.autoscroll);
	x11amp_cfg_write_int(cfgfile,"x11amp","timer_mode",cfg.timer_mode);
	x11amp_cfg_write_int(cfgfile,"x11amp","vis_type",cfg.vis_type);
	x11amp_cfg_write_int(cfgfile,"x11amp","analyzer_mode",cfg.analyzer_mode);
	x11amp_cfg_write_int(cfgfile,"x11amp","analyzer_type",cfg.analyzer_type);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","analyzer_peaks",cfg.analyzer_peaks);
	x11amp_cfg_write_int(cfgfile,"x11amp","scope_mode",cfg.scope_mode);
	x11amp_cfg_write_int(cfgfile,"x11amp","vu_mode",cfg.vu_mode);
	x11amp_cfg_write_int(cfgfile,"x11amp","vis_refresh_rate",cfg.vis_refresh);
	x11amp_cfg_write_int(cfgfile,"x11amp","analyzer_falloff",cfg.analyzer_falloff);
	x11amp_cfg_write_int(cfgfile,"x11amp","peaks_falloff",cfg.peaks_falloff);
	x11amp_cfg_write_int(cfgfile,"x11amp","playlist_x",cfg.playlist_x);
	x11amp_cfg_write_int(cfgfile,"x11amp","playlist_y",cfg.playlist_y);
	x11amp_cfg_write_int(cfgfile,"x11amp","playlist_width",cfg.playlist_width);
	x11amp_cfg_write_int(cfgfile,"x11amp","playlist_height",cfg.playlist_height);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","playlist_visible",cfg.playlist_visible);
	x11amp_cfg_write_string(cfgfile,"x11amp","playlist_font",cfg.playlist_font);
	x11amp_cfg_write_int(cfgfile,"x11amp","equalizer_x",cfg.equalizer_x);
	x11amp_cfg_write_int(cfgfile,"x11amp","equalizer_y",cfg.equalizer_y);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","equalizer_visible",cfg.equalizer_visible);
	x11amp_cfg_write_boolean(cfgfile,"x11amp","equalizer_active",cfg.equalizer_active);
	x11amp_cfg_write_float(cfgfile,"x11amp","equalizer_preamp",cfg.equalizer_preamp);
	for(i=0;i<10;i++)
	{
		str=g_strdup_printf("equalizer_band%d",i);
		x11amp_cfg_write_float(cfgfile,"x11amp",str,cfg.equalizer_bands[i]);
		g_free(str);
	}
	if(skin->path)
		x11amp_cfg_write_string(cfgfile,"x11amp","skin",skin->path);
	else
		x11amp_cfg_remove_key(cfgfile,"x11amp","skin");
	x11amp_cfg_write_string(cfgfile,"x11amp","output_plugin",get_current_output_plugin()->filename);
	if(cfg.filesel_path)
		x11amp_cfg_write_string(cfgfile,"x11amp","filesel_path",cfg.filesel_path);
	x11amp_cfg_write_file(cfgfile,filename);
	x11amp_cfg_free(cfgfile);
	
	g_free(filename);
	filename=g_strconcat(g_get_home_dir(),"/.x11amp/x11amp.m3u",NULL);
	playlist_save(filename);
	g_free(filename);
}

void set_doublesize(gboolean ds)
{
	gint ox,oy;
	
	cfg.doublesize=ds;
	if(cfg.doublesize)
	{
		gdk_window_set_hints(mainwin->window,0,0,550,232,550,232,GDK_HINT_MIN_SIZE|GDK_HINT_MAX_SIZE);
		gdk_window_resize(mainwin->window,550,232);
		gdk_window_set_back_pixmap(mainwin->window,mainwin_bg_dblsize,0);
		draw_main_window(TRUE);
		if(is_docked(cfg.player_x,cfg.player_y,275,116,cfg.playlist_x,cfg.playlist_y,cfg.playlist_width,cfg.playlist_height))
		{
			ox=(cfg.playlist_x-cfg.player_x);
			oy=(cfg.playlist_y-cfg.player_y);
			if(ox>0) ox*=2;
			if(oy>0) oy*=2;
			if(ox>0||oy>0)
				playlistwin_move(cfg.player_x+ox,cfg.player_y+oy);
		}
		if(is_docked(cfg.player_x,cfg.player_y,275,116,cfg.equalizer_x,cfg.equalizer_y,275,116))
		{
			ox=(cfg.equalizer_x-cfg.player_x);
			oy=(cfg.equalizer_y-cfg.player_y);
			if(ox>0) ox*=2;
			if(oy>0) oy*=2;
			if(ox>0||oy>0)
				equalizerwin_move(cfg.player_x+ox,cfg.player_y+oy);
		}
		
	}
	else
	{
		gdk_window_set_hints(mainwin->window,0,0,275,116,275,116,GDK_HINT_MIN_SIZE|GDK_HINT_MAX_SIZE);
		gdk_window_resize(mainwin->window,275,116);
		gdk_window_set_back_pixmap(mainwin->window,mainwin_bg,0);
		gdk_window_clear(mainwin->window);
		if(is_docked(cfg.player_x,cfg.player_y,550,232,cfg.playlist_x,cfg.playlist_y,cfg.playlist_width,cfg.playlist_height))
		{
			ox=(cfg.playlist_x-cfg.player_x);
			oy=(cfg.playlist_y-cfg.player_y);
			if(ox>0) ox/=2;
			if(oy>0) oy/=2;
			if(ox>0||oy>0)
				playlistwin_move(cfg.player_x+ox,cfg.player_y+oy);
		}
		if(is_docked(cfg.player_x,cfg.player_y,550,232,cfg.equalizer_x,cfg.equalizer_y,550,232))
		{
			ox=(cfg.equalizer_x-cfg.player_x);
			oy=(cfg.equalizer_y-cfg.player_y);
			if(ox>0) ox/=2;
			if(oy>0) oy/=2;
			if(ox>0||oy>0)
				equalizerwin_move(cfg.player_x+ox,cfg.player_y+oy);
		}
	}
	equalizerwin_set_doublesize(ds);
}

void mainwin_vis_set_refresh(RefreshRate rate)
{
	cfg.vis_refresh=rate;
	vis_set_refresh(mainwin_vis,rate);
}

void mainwin_vis_set_afalloff(FalloffSpeed speed)
{
	cfg.analyzer_falloff=speed;
	vis_set_afalloff(mainwin_vis,speed);
}

void mainwin_vis_set_pfalloff(FalloffSpeed speed)
{
	cfg.peaks_falloff=speed;
	vis_set_pfalloff(mainwin_vis,speed);
}

void mainwin_vis_set_analyzer_mode(AnalyzerMode mode)
{
	cfg.analyzer_mode=mode;
}

void mainwin_vis_set_analyzer_type(AnalyzerType mode)
{
	cfg.analyzer_type=mode;
}

gboolean is_docked(gint x,gint y,gint w,gint h,gint ox,gint oy,gint ow,gint oh)
{
	if((x==ox||x==ox+ow||x+w==ox||x+w==ox+ow)&&oy>=y-oh&&oy<=y+h) return TRUE;
	if((y==oy||y==oy+oh||y+h==oy||y+h==oy+oh)&&ox>=x-ow&&ox<=x+w) return TRUE;
	return FALSE;
}

void dock(gint *x,gint *y,gint w,gint h,gint ox,gint oy,gint ow,gint oh)
{
	if(*x+w>ox-10 && *x+w<ox+10 && *y>oy-h && *y<oy+oh)
	{
		*x=ox-w;
		if(*y>oy-10 && *y<oy+10) *y=oy;
		if(*y+h>oy+oh-10 && *y+h<oy+oh+10) *y=oy+oh-h;
	}
	
	if(*x>ox+ow-10 && *x<ox+ow+10 && *y>oy-h && *y<oy+oh) 
	{
		*x=ox+ow;
		if(*y>oy-10 && *y<oy+10) *y=oy;
		if(*y+h>oy+oh-10 && *y+h<oy+oh+10) *y=oy+oh-h;
	}
	if(*y+h>oy-10 && *y+h<oy+10 && *x>ox-w && *x<ox+ow) 
	{
		*y=oy-h;
		if(*x>ox-10 && *x<ox+10) *x=ox;
		if(*x+w>ox+ow-10 && *x+w<ox+ow+10) *x=ox+ow-w;
	}
	if(*y>oy+oh-10 && *y<oy+oh+10 && *x>ox-w && *x<ox+ow) 
	{
		*y=oy+oh;
		if(*x>ox-10 && *x<ox+10) *x=ox;
		if(*x+w>ox+ow-10 && *x+w<ox+ow+10) *x=ox+ow-w;
	}
}
	

void mainwin_minimize_cb(void)
{
	Window xwindow;

	if (!mainwin->window) return;

	xwindow = GDK_WINDOW_XWINDOW(mainwin->window);
	XIconifyWindow(GDK_DISPLAY(), xwindow, DefaultScreen(GDK_DISPLAY()));
}


void mainwin_quit_cb(void)
{
	gtk_timeout_remove(mainwin_timeout_tag);
	save_config();
	cleanup_input_plugins();
	cleanup_output_plugins();
	cleanup_ctrlsocket();
	gtk_main_quit();
}

void draw_mainwin_titlebar(int focus)
{
	GdkImage *img,*img2;
	if(focus)
		gdk_draw_pixmap(mainwin_bg,mainwin_gc,get_skin_pixmap(SKIN_TITLEBAR),27,0,0,0,275,14);
	else
		gdk_draw_pixmap(mainwin_bg,mainwin_gc,get_skin_pixmap(SKIN_TITLEBAR),27,15,0,0,275,14);
}
	
void draw_main_window(gboolean force)
{
	GdkImage *img,*img2;
	WidgetNode *wn;
	Widget *w;
	gboolean redraw;
	
	
	if(force)
	{
		gdk_draw_pixmap(mainwin_bg,mainwin_gc,get_skin_pixmap(SKIN_MAIN),0,0,0,0,275,116);
		draw_mainwin_titlebar(mainwin_focus);
		draw_widget_list(mainwin_wlist,&redraw,TRUE);
	}
	else
		draw_widget_list(mainwin_wlist,&redraw,FALSE);
	
	if(redraw||force)
	{
		
		if(force)
		{
			if(cfg.doublesize)
			{
				img=gdk_image_get(mainwin_bg,0,0,275,116);
				img2=create_dblsize_image(img);
				gdk_draw_image(mainwin_bg_dblsize,mainwin_gc,img2,0,0,0,0,550,232);
				gdk_image_destroy(img2);
				gdk_image_destroy(img);
			}
		}
		else
		{
			wn=(WidgetNode *)mainwin_wlist->li_first;
			while(wn)
			{
				w=wn->wn_widget;
				if(w->redraw&&w->visible)
				{
					if(cfg.doublesize)
					{
						img=gdk_image_get(mainwin_bg,w->x,w->y,w->width,w->height);
						img2=create_dblsize_image(img);
						gdk_draw_image(mainwin_bg_dblsize,mainwin_gc,img2,0,0,w->x<<1,w->y<<1,w->width<<1,w->height<<1);
						gdk_image_destroy(img2);
						gdk_image_destroy(img);
						gdk_window_clear_area(mainwin->window,w->x<<1,w->y<<1,w->width<<1,w->height<<1);
					}
					else
						gdk_window_clear_area(mainwin->window,w->x,w->y,w->width,w->height);
					w->redraw=FALSE;
					
				}
				wn=(WidgetNode *)wn->wn_node.no_next;
			}
		}
		if(force) gdk_window_clear(mainwin->window);
		gdk_flush();			
	}
}

void mainwin_set_info_text(void)
{
	gchar *text;
	
	if(mainwin_volume->hs_pressed||mainwin_balance->hs_pressed||mainwin_position->hs_pressed||mainwin_menurow->mr_bpushed)
		return;
	
	if(text=input_get_info_text())
	{
		textbox_set_text(mainwin_info,text);
	}
	else if(text=playlist_get_info_text())
	{
		textbox_set_text(mainwin_info,text);
		g_free(text);
	}
	else
	{
		text=g_strdup_printf("%s %s",PACKAGE,VERSION);
		textbox_set_text(mainwin_info,text);
		g_free(text);
	}
}

void mainwin_set_song_info(int rate,int freq,int nch)
{
	gchar text[10];
	gchar *tmp,*title;
	if(rate==0&&freq==0&&nch==0)
	{
		textbox_set_text(mainwin_rate_text,"   ");
		textbox_set_text(mainwin_freq_text,"  ");
		monostereo_set_num_channels(mainwin_monostereo,0);
		playstatus_set_status(mainwin_playstatus,STATUS_STOP);
		hide_widget(mainwin_minus_num);
		hide_widget(mainwin_10min_num);
		hide_widget(mainwin_min_num);
		hide_widget(mainwin_10sec_num);
		hide_widget(mainwin_sec_num);
		hide_widget(mainwin_position);
		draw_main_window(TRUE);
		gtk_window_set_title(GTK_WINDOW(mainwin),"X11Amp");
		return;
	}
	
	rate/=1000; 
	if(rate<1000)
	{
		sprintf(text,"%3d",rate);
		textbox_set_text(mainwin_rate_text,text);
	}
	else
	{
		rate/=100;
		sprintf(text,"%2dH",rate);
		textbox_set_text(mainwin_rate_text,text);
	}	
	sprintf(text,"%2d",freq);
	textbox_set_text(mainwin_freq_text,text);
	monostereo_set_num_channels(mainwin_monostereo,nch);
	
	show_widget(mainwin_minus_num);
	show_widget(mainwin_10min_num);
	show_widget(mainwin_min_num);
	show_widget(mainwin_10sec_num);
	show_widget(mainwin_sec_num);
	playstatus_set_status(mainwin_playstatus,STATUS_PLAY);
	if(playlist_get_current_length()!=-1)
		show_widget(mainwin_position);
	mainwin_position->hs_pressed=FALSE;
	if(tmp=playlist_get_info_text())
	{
		title=g_malloc0(strlen(tmp)+10);
		sprintf(title,"X11Amp - %s",tmp);
		gtk_window_set_title(GTK_WINDOW(mainwin),title);
		g_free(title);
		g_free(tmp);
	}
}

void mainwin_release(GtkWidget *widget,GdkEventButton *event,gpointer callback_data)
{
	gdk_pointer_ungrab(GDK_CURRENT_TIME);
	
	/*
	 *
	 * The gdk_flush() is just for making sure that the pointer really
	 * gets ungrabbed before calling any button callbacks
	 *
	 */ 
	
	gdk_flush();
	
	if(mainwin_moving)
	{
		mainwin_moving=FALSE;
	}
	if(mainwin_menurow->mr_doublesize_selected)
	{
		event->x/=2;
		event->y/=2;
	}
	handle_release_cb(mainwin_wlist,widget,event);
		
	draw_main_window(FALSE);

}

void mainwin_motion(GtkWidget *widget,GdkEventMotion *event,gpointer callback_data)
{
	XEvent ev;
	
	while(XCheckMaskEvent(GDK_DISPLAY(),ButtonMotionMask,&ev));
	
	if(cfg.doublesize)
	{
		event->x/=2;
		event->y/=2;
	}
	if(mainwin_moving)
	{
		gint mx,my,newx,newy,eqw,eqh,min_offsetx=0,min_offsety=0;
		gint sw,sh,w,h;
		GdkModifierType modmask;
		
		gdk_window_get_pointer(NULL, &mx, &my, &modmask);
		newx=mx-mainwin_move_x;
		newy=my-mainwin_move_y;
		sw=gdk_screen_width();
		sh=gdk_screen_height();
		gdk_window_get_size(mainwin->window,&w,&h);
		if(mainwin_move_playlist)
		{
			if(cfg.playlist_x+cfg.playlist_width>cfg.player_x+w) w=cfg.playlist_x+cfg.playlist_width-cfg.player_x;
			if(cfg.playlist_y+cfg.playlist_height>cfg.player_y+h) h=cfg.playlist_y+cfg.playlist_height-cfg.player_y;
			if(cfg.playlist_x<cfg.player_x) min_offsetx=cfg.player_x-cfg.playlist_x;
			if(cfg.playlist_y<cfg.player_y) min_offsety=cfg.player_y-cfg.playlist_y;
		}
		if(mainwin_move_equalizer)
		{
			if(cfg.doublesize)
			{
				eqw=550;
				eqh=232;
			}
			else
			{
				eqw=275;
				eqh=116;
			}
			if(cfg.equalizer_x+eqw>cfg.player_x+w) w=cfg.equalizer_x+eqw-cfg.player_x;
			if(cfg.equalizer_y+eqh>cfg.player_y+h) h=cfg.equalizer_y+eqh-cfg.player_y;
			if(cfg.equalizer_x<cfg.player_x-min_offsetx) min_offsetx=cfg.player_x-cfg.equalizer_x;
			if(cfg.equalizer_y<cfg.player_y-min_offsety) min_offsety=cfg.player_y-cfg.equalizer_y;
		}
		
		if(newx>min_offsetx-10&&newx<min_offsetx+10) newx=min_offsetx;
		if(newx+w>sw-10&&newx+w<sw+10) newx=sw-w;
		if(newy>min_offsety-10&&newy<min_offsety+10) newy=min_offsety;
		if(newy+h>sh-10&&newy+h<sh+10) newy=sh-h;
		
		if(cfg.playlist_visible&&!mainwin_move_playlist) dock(&newx,&newy,w,h,cfg.playlist_x,cfg.playlist_y,cfg.playlist_width,cfg.playlist_height);
		if(cfg.equalizer_visible&&!mainwin_move_equalizer)
			if(cfg.doublesize)
				dock(&newx,&newy,w,h,cfg.equalizer_x,cfg.equalizer_y,550,232);
			else
				dock(&newx,&newy,w,h,cfg.equalizer_x,cfg.equalizer_y,275,116);
		if(mainwin_move_playlist) playlistwin_move((cfg.playlist_x-cfg.player_x)+newx,(cfg.playlist_y-cfg.player_y)+newy);
		if(mainwin_move_equalizer) equalizerwin_move((cfg.equalizer_x-cfg.player_x)+newx,(cfg.equalizer_y-cfg.player_y)+newy);
		cfg.player_x=newx;
		cfg.player_y=newy;
		gdk_window_move(mainwin->window,newx,newy);
	}
	else
	{
		handle_motion_cb(mainwin_wlist,widget,event);
		draw_main_window(FALSE);
	}
	gdk_flush();
	

}
		
void mainwin_press(GtkWidget *widget,GdkEventButton *event,gpointer callback_data)
{
	gint mx,my,w,h;
	gboolean grab=TRUE;
	
	mx=event->x;
	my=event->y;
	if(cfg.doublesize)
	{
		event->x/=2;
		event->y/=2;
	}

	if(event->y<14&&event->x<244&&event->button==1)
	{
		gdk_window_raise(mainwin->window);
		equalizerwin_raise();
		playlistwin_raise();

		mainwin_moving=TRUE;
		mainwin_move_x=mx;
		mainwin_move_y=my;
		gdk_window_get_size(mainwin->window,&w,&h);
		if(is_docked(cfg.player_x,cfg.player_y,w,h,cfg.playlist_x,cfg.playlist_y,cfg.playlist_width,cfg.playlist_height))  mainwin_move_playlist=TRUE;
		else mainwin_move_playlist=FALSE;
		if(cfg.doublesize)
		{
			if(is_docked(cfg.player_x,cfg.player_y,w,h,cfg.equalizer_x,cfg.equalizer_y,550,232))  mainwin_move_equalizer=TRUE;
			else mainwin_move_equalizer=FALSE;
			if(mainwin_move_equalizer||mainwin_move_playlist)
			{
				if(is_docked(cfg.playlist_x,cfg.playlist_y,cfg.playlist_width,cfg.playlist_height,cfg.equalizer_x,cfg.equalizer_y,550,232))
				{
					mainwin_move_equalizer=TRUE;
					mainwin_move_playlist=TRUE;
				}
			}
		}
		else
		{
			if(is_docked(cfg.player_x,cfg.player_y,w,h,cfg.equalizer_x,cfg.equalizer_y,275,116))  mainwin_move_equalizer=TRUE;
			else mainwin_move_equalizer=FALSE;	
			if(mainwin_move_equalizer||mainwin_move_playlist)
			{
				if(is_docked(cfg.playlist_x,cfg.playlist_y,cfg.playlist_width,cfg.playlist_height,cfg.equalizer_x,cfg.equalizer_y,275,116))
				{
					mainwin_move_equalizer=TRUE;
					mainwin_move_playlist=TRUE;
				}
			}

		}		
	}
	else	
	{
		handle_press_cb(mainwin_wlist,widget,event);
		draw_main_window(FALSE);
	}
	if(event->button==3)
	{
		GdkModifierType modmask;
		if(inside_widget(event->x,event->y,mainwin_info))
		{
			gdk_window_get_pointer(NULL,&mx,&my, &modmask);
			gtk_item_factory_popup(mainwin_songname_menu,mx,my,3,GDK_CURRENT_TIME);
			grab=FALSE;
		}
		if(inside_widget(event->x,event->y,mainwin_vis))
		{
			gdk_window_get_pointer(NULL,&mx,&my, &modmask);
			gtk_item_factory_popup(mainwin_vis_menu,mx,my,3,GDK_CURRENT_TIME);
			grab=FALSE;
		}
	}
	if(event->button==1)
	{
		if(event->x>35&&event->x<100&&event->y>25&&event->y<40)
		{
			if(cfg.timer_mode==TIMER_ELAPSED) set_timer_mode(TIMER_REMAINING);
			else set_timer_mode(TIMER_ELAPSED);
		}
		
	}
	if(grab) gdk_pointer_grab(mainwin->window,FALSE,GDK_BUTTON_MOTION_MASK|GDK_BUTTON_RELEASE_MASK,GDK_NONE,GDK_NONE,GDK_CURRENT_TIME);
	
}

void mainwin_focus_in(GtkWidget *widget,GdkEvent *event,gpointer callback_data)
{
	mainwin_focus=1;
	show_widget(mainwin_minimize);
	show_widget(mainwin_shade);
	show_widget(mainwin_close);
	draw_main_window(TRUE);
}

void mainwin_focus_out(GtkWidget *widget,GdkEventButton *event,gpointer callback_data)
{
	mainwin_focus=0;
	hide_widget(mainwin_minimize);
	hide_widget(mainwin_shade);
	hide_widget(mainwin_close);
	draw_main_window(TRUE);
}

gint mainwin_configure(GtkWidget *window)
{
	/*
	 * On some WM's gdk_window_get_root_origin return weird values so..
	 */
	
	Window dummy;
	if(!mainwin_moving)
		XTranslateCoordinates(GDK_DISPLAY(),GDK_WINDOW_XWINDOW(window->window),GDK_ROOT_WINDOW(),0,0,&cfg.player_x,&cfg.player_y,&dummy);
	return FALSE;
}

static void mainwin_drag_data_received(GtkWidget *widget,
					GdkDragContext   *context,
					gint              x,
					gint              y,
					GtkSelectionData *selection_data,
					guint             info,
					guint             time)
{
	if(selection_data->data)
	{
		playlist_clear();
		playlist_add_url_string((gchar *)selection_data->data);
		playlistwin_update_list();
		playlist_play();
	}
}

void mainwin_filesel_cancel(GtkWidget *w,GtkWidget *filesel)
{
	gtk_widget_destroy(GTK_WIDGET(filesel));
	mainwin_filesel=NULL;
}

void mainwin_filesel_ok(GtkWidget *w,GtkWidget *filesel)
{
	GList *node;
	gchar *text,*text2;
	GList *sel_list=NULL;
	
	/*
	 * This is *REAL* ugly and needs to be fixed
	 */
	
	gtk_widget_hide(filesel);
	
	if(cfg.filesel_path)
		g_free(cfg.filesel_path);
	cfg.filesel_path=g_strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
	text=strrchr(cfg.filesel_path,'/');
	if(text)
		*(text+1)='\0';
	
	node=GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list)->selection;
	while(node)
	{
		gtk_clist_get_text(GTK_CLIST(GTK_FILE_SELECTION(filesel)->file_list),(gint)node->data,0,&text);
		text2=g_malloc(strlen(text)+1);
		strcpy(text2,text);
		if(!sel_list)
			sel_list=g_list_append(NULL,text2);
		else
			sel_list=g_list_append(sel_list,text2);



		node=node->next;
	}
	node=sel_list;
	if(node)
	{
		if(get_input_playing()) input_stop();
		playlist_clear();
		while(node)
		{
			text=(gchar *)node->data;
			gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel),text);
			playlist_add(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
			g_free(text);
			node=node->next;
		}
		g_list_free(sel_list);
		playlist_generate_shuffle_list();
		playlist_play();
	}
	else
	{
		text=gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
		if(text[strlen(text)-1]!='/')
		{
			if(get_input_playing()) input_stop();
			playlist_clear();
			playlist_add(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel)));
			playlist_generate_shuffle_list();
			playlist_play();
		}
		g_free(text);
	}
	gtk_widget_destroy(GTK_WIDGET(filesel));
	mainwin_filesel=NULL;
	
}

void mainwin_eject_pushed(void)
{
	if(!mainwin_filesel)
	{
		mainwin_filesel=gtk_file_selection_new("Load file(s)");
		gtk_clist_set_selection_mode(GTK_CLIST(GTK_FILE_SELECTION(mainwin_filesel)->file_list),GTK_SELECTION_EXTENDED);
		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(mainwin_filesel)->ok_button),"clicked",GTK_SIGNAL_FUNC(mainwin_filesel_ok),mainwin_filesel);
		gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(mainwin_filesel)->cancel_button),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),GTK_OBJECT(mainwin_filesel));
     		gtk_signal_connect(GTK_OBJECT(mainwin_filesel),"destroy",GTK_SIGNAL_FUNC(gtk_widget_destroyed),&mainwin_filesel);
		if(cfg.filesel_path)
			gtk_file_selection_set_filename(GTK_FILE_SELECTION(mainwin_filesel),cfg.filesel_path);
		gtk_widget_show(mainwin_filesel);
	}
	if(!GTK_WIDGET_VISIBLE(mainwin_filesel))
		gtk_widget_show(mainwin_filesel);
}

void mainwin_play_pushed(void)
{
	if(get_playlist_length())
		playlist_play();
	else
		mainwin_eject_pushed();
}

void mainwin_stop_pushed(void)
{
	mainwin_set_song_info(0,0,0);
	input_stop();
}


		
void mainwin_shuffle_pushed(gboolean toggled)
{
	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Shuffle"))->active=toggled;
	cfg.shuffle=toggled;
}

void mainwin_repeat_pushed(gboolean toggled)
{
	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Repeat"))->active=toggled;
	cfg.repeat=toggled;
}

void mainwin_eq_pushed(gboolean toggled)
{
	if(toggled) equalizerwin_show();
	else equalizerwin_hide();
}

void mainwin_pl_pushed(gboolean toggled)
{
	if(toggled) playlistwin_show();
	else playlistwin_hide();
}

void mainwin_position_motioncb(gint pos)
{
	char buf[50];
	int length,time;
	length=playlist_get_current_length()/1000;
	time=(length*pos)/219;
	sprintf(buf,"SEEK TO: %d:%-2.2d/%d:%-2.2d (%d%%)",time/60,time%60,length/60,length%60,(time*100)/length);
	textbox_set_text(mainwin_info,buf);
}

void mainwin_position_releasecb(gint pos)
{
	int length,time;
	length=playlist_get_current_length()/1000;
	time=(length*pos)/219;
	input_seek(time);
	mainwin_set_info_text();
}	

gint mainwin_volume_framecb(gint pos)
{
	return (pos*28)/52;
}

void mainwin_volume_motioncb(gint pos)
{
	char tmp[20];	
	int v,b;
	
	v=(pos*100)/51;
	sprintf(tmp,"VOLUME: %d%%",v);
	textbox_set_text(mainwin_info,tmp);
	if(mainwin_balance->hs_position<12)
	{
		b=(mainwin_balance->hs_position*100)/12;
		output_set_volume(v,(v*b)/100);
	}
	else if(mainwin_balance->hs_position>12)
	{
		b=100-(((mainwin_balance->hs_position-12)*100)/12);
		output_set_volume((v*b)/100,v);
	}
	else
	{
		output_set_volume(v,v);
	}
}

void mainwin_volume_releasecb(gint pos)
{
	mainwin_set_info_text();
}

gint mainwin_balance_framecb(gint pos)
{
	return ((abs(pos-12)*28)/13);
}	

void mainwin_balance_motioncb(gint pos)
{
	char tmp[20];
	int b,v;
	v=(mainwin_volume->hs_position*100)/51;
	if(pos<12)
	{
		b=((12-pos)*100)/12;
		sprintf(tmp,"BALANCE: %d%% LEFT",b);
		output_set_volume(v,(v*(100-b))/100);
	}
	else if(pos==12)
	{
		sprintf(tmp,"BALANCE: CENTER");		
		output_set_volume(v,v);
	}
	else if(pos>12)
	{
		b=((pos-12)*100)/12;
		sprintf(tmp,"BALANCE: %d%% RIGHT",b);
		output_set_volume((v*(100-b))/100,v);
	}
	textbox_set_text(mainwin_info,tmp);
}

void mainwin_balance_releasecb(gint pos)
{
	mainwin_set_info_text();
}

void mainwin_songname_menu_callback(gpointer cb_data,guint action,GtkWidget *w)
{
	switch(action)
	{
		case	MAINWIN_SONGNAME_FILEINFO:
			break;
		case	MAINWIN_SONGNAME_JTF:
			break;
		case	MAINWIN_SONGNAME_JTT:
			break;
		case	MAINWIN_SONGNAME_SCROLL:
			cfg.autoscroll=GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_songname_menu,"/Autoscroll Songname"))->active;
			textbox_set_scroll(mainwin_info,cfg.autoscroll);
			break;
	}
}

void mainwin_options_menu_callback(gpointer cb_data,guint action,GtkWidget *w)
{
	switch(action)
	{
		case	MAINWIN_OPT_PREFS:
			show_prefs_window();
			break;
		case	MAINWIN_OPT_SKIN:
			show_skin_window();
			break;
		case	MAINWIN_OPT_SHUFFLE:
			tbutton_set_toggled(mainwin_shuffle,GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Shuffle"))->active);
			cfg.shuffle=GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Shuffle"))->active;
			break;
		case	MAINWIN_OPT_REPEAT:
			tbutton_set_toggled(mainwin_repeat,GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Repeat"))->active);
			cfg.repeat=GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Repeat"))->active;
			break;
		case	MAINWIN_OPT_TELAPSED:
			set_timer_mode(TIMER_ELAPSED);
			break;
		case	MAINWIN_OPT_TREMAINING:
			set_timer_mode(TIMER_REMAINING);
			break;
		case	MAINWIN_OPT_ALWAYS:
			mainwin_menurow->mr_always_selected=GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Always On Top"))->active;
			draw_widget(mainwin_menurow);
			break;
		case	MAINWIN_OPT_DOUBLESIZE:
			mainwin_menurow->mr_doublesize_selected=GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/DoubleSize"))->active;
			draw_widget(mainwin_menurow);
			set_doublesize(mainwin_menurow->mr_doublesize_selected);
			gdk_flush();
			break;
	}
}

void mainwin_vis_menu_callback(gpointer cb_data,guint action,GtkWidget *w)
{
	switch(action)
	{
		case	MAINWIN_VIS_ANALYZER_NORMAL:
		case	MAINWIN_VIS_ANALYZER_FIRE:
		case	MAINWIN_VIS_ANALYZER_VLINES:
			mainwin_vis_set_analyzer_mode(action-MAINWIN_VIS_ANALYZER_NORMAL);
			break;
		case    MAINWIN_VIS_ANALYZER_LINES:
		case    MAINWIN_VIS_ANALYZER_BARS:
			mainwin_vis_set_analyzer_type(action-MAINWIN_VIS_ANALYZER_LINES);
			break;
		case	MAINWIN_VIS_ANALYZER_PEAKS:
			cfg.analyzer_peaks=GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_ANALYZER_PEAKS].path))->active;
			break;
		case	MAINWIN_VIS_REFRESH_FULL:
		case	MAINWIN_VIS_REFRESH_HALF:
		case	MAINWIN_VIS_REFRESH_QUARTER:
		case	MAINWIN_VIS_REFRESH_EIGTH:
			mainwin_vis_set_refresh(action-MAINWIN_VIS_REFRESH_FULL);
			break;
		case	MAINWIN_VIS_AFALLOFF_SLOWEST:
		case	MAINWIN_VIS_AFALLOFF_SLOW:
		case	MAINWIN_VIS_AFALLOFF_MEDIUM:
		case	MAINWIN_VIS_AFALLOFF_FAST:			
		case	MAINWIN_VIS_AFALLOFF_FASTEST:
			mainwin_vis_set_afalloff(action-MAINWIN_VIS_AFALLOFF_SLOWEST);
			break;
		case	MAINWIN_VIS_PFALLOFF_SLOWEST:
		case	MAINWIN_VIS_PFALLOFF_SLOW:
		case	MAINWIN_VIS_PFALLOFF_MEDIUM:
		case	MAINWIN_VIS_PFALLOFF_FAST:			
		case	MAINWIN_VIS_PFALLOFF_FASTEST:
			mainwin_vis_set_pfalloff(action-MAINWIN_VIS_PFALLOFF_SLOWEST);
			break;

	}
}


void mainwin_mr_change(MenuRowItem i)
{
	switch(i)
	{
		case	MENUROW_NONE:
			mainwin_set_info_text();
			break;
		case	MENUROW_OPTIONS:
			textbox_set_text(mainwin_info,"OPTIONS MENU");
			break;
		case	MENUROW_ALWAYS:
			if(mainwin_menurow->mr_always_selected)
				textbox_set_text(mainwin_info,"DISABLE ALWAYS ON TOP (N/A)");
			else
				textbox_set_text(mainwin_info,"ENABLE ALWAYS ON TOP (N/A)");
			break;
		case	MENUROW_FILEINFOBOX:
			textbox_set_text(mainwin_info,"FILE INFO BOX");
			break;
		case	MENUROW_DOUBLESIZE:
			if(mainwin_menurow->mr_doublesize_selected)
				textbox_set_text(mainwin_info,"DISABLE DOUBLESIZE");
			else	
				textbox_set_text(mainwin_info,"ENABLE DOUBLESIZE");
			break;
		case	MENUROW_VISUALIZATION:
			textbox_set_text(mainwin_info,"VISUALIZATION MENU");
			break;
	}
}

void mainwin_mr_release(MenuRowItem i)
{
	GdkModifierType modmask;
	gint x,y;

	switch(i)
	{
		case	MENUROW_OPTIONS:
			gdk_window_get_pointer(NULL,&x,&y, &modmask);
			gtk_item_factory_popup(mainwin_options_menu,x,y,1,GDK_CURRENT_TIME);
			break;
		case	MENUROW_ALWAYS:
			GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Always On Top"))->active=mainwin_menurow->mr_always_selected;
			break;
		case	MENUROW_FILEINFOBOX:
			break;
		case	MENUROW_DOUBLESIZE:
			GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/DoubleSize"))->active=mainwin_menurow->mr_doublesize_selected;
			set_doublesize(mainwin_menurow->mr_doublesize_selected);
			break;
		case	MENUROW_VISUALIZATION:
			gdk_window_get_pointer(NULL,&x,&y, &modmask);
			gtk_item_factory_popup(mainwin_vis_menu,x,y,1,GDK_CURRENT_TIME);
			break;
	}
	mainwin_set_info_text();
}


void read_volume(void)
{
	int vl,vr;
	output_get_volume(&vl,&vr);
	if(vl>vr)
	{
		hslider_set_position(mainwin_volume,(vl*51)/100);
		hslider_set_position(mainwin_balance,(vr*12)/vl);
	}
	else if(vr>vl)
	{
		hslider_set_position(mainwin_volume,(vr*51)/100);
		hslider_set_position(mainwin_balance,(vl*12)/vr);
	}
	else
	{
		hslider_set_position(mainwin_volume,(vl*51)/100);
		hslider_set_position(mainwin_balance,12);
	}
}

void create_popups(void)
{
	GtkMenuItem *item;
	int i;
	
	mainwin_options_menu=gtk_item_factory_new(GTK_TYPE_MENU,"<Main>",mainwin_accel);
	gtk_item_factory_create_items(mainwin_options_menu,MAINWIN_OPTIONS_MENU_ENTRIES,mainwin_options_menu_entries,NULL);
	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Shuffle"))->active=cfg.shuffle;
	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Repeat"))->active=cfg.repeat;
	if(cfg.timer_mode==TIMER_ELAPSED)
	{
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Elapsed"))->active=TRUE;
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Remaining"))->active=FALSE;
	}
	else
	{
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Elapsed"))->active=FALSE;
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Remaining"))->active=TRUE;
	}

	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/DoubleSize"))->active=cfg.doublesize;
	mainwin_songname_menu=gtk_item_factory_new(GTK_TYPE_MENU,"<Main>",mainwin_accel);
	gtk_item_factory_create_items(mainwin_songname_menu,MAINWIN_SONGNAME_MENU_ENTRIES,mainwin_songname_menu_entries,NULL);
	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_songname_menu,"/Autoscroll Songname"))->active=cfg.autoscroll;
	
	mainwin_vis_menu=gtk_item_factory_new(GTK_TYPE_MENU,"<Main>",mainwin_accel);
	gtk_item_factory_create_items(mainwin_vis_menu,MAINWIN_VIS_MENU_ENTRIES,mainwin_vis_menu_entries,NULL);
	for(i=0;i<MAINWIN_VIS_MENU_NUM_VIS_MODE;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_VIS_MODE+i].path))->active=(cfg.vis_type==i)?TRUE:FALSE;
	for(i=0;i<MAINWIN_VIS_MENU_NUM_ANALYZER_MODE;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_ANALYZER_MODE+i].path))->active=(cfg.analyzer_mode==i)?TRUE:FALSE;
	for(i=0;i<MAINWIN_VIS_MENU_NUM_ANALYZER_TYPE;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_ANALYZER_TYPE+i].path))->active=(cfg.analyzer_type==i)?TRUE:FALSE;
	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_ANALYZER_PEAKS].path))->active=cfg.analyzer_peaks;
	for(i=0;i<MAINWIN_VIS_MENU_NUM_SCOPE_MODE;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_SCOPE_MODE+i].path))->active=(cfg.scope_mode==i)?TRUE:FALSE;
	for(i=0;i<MAINWIN_VIS_MENU_NUM_WSHADEVU_MODE;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_WSHADEVU_MODE+i].path))->active=(cfg.vu_mode==i)?TRUE:FALSE;
	for(i=0;i<MAINWIN_VIS_MENU_NUM_REFRESH_RATE;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_REFRESH_RATE+i].path))->active=(cfg.vis_refresh==i)?TRUE:FALSE;
	for(i=0;i<MAINWIN_VIS_MENU_NUM_AFALLOFF;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_AFALLOFF+i].path))->active=(cfg.analyzer_falloff==i)?TRUE:FALSE;
	for(i=0;i<MAINWIN_VIS_MENU_NUM_AFALLOFF;i++)
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_vis_menu,mainwin_vis_menu_entries[MAINWIN_VIS_MENU_PFALLOFF+i].path))->active=(cfg.peaks_falloff==i)?TRUE:FALSE;
	
}

void setup_main_window(void)
{
	
	mainwin_bg=gdk_pixmap_new(mainwin->window,275,116,gdk_visual_get_best_depth());
	mainwin_bg_dblsize=gdk_pixmap_new(mainwin->window,550,232,gdk_visual_get_best_depth());
	if(cfg.doublesize)
		gdk_window_set_back_pixmap(mainwin->window,mainwin_bg_dblsize,0);
	else
		gdk_window_set_back_pixmap(mainwin->window,mainwin_bg,0);
	mainwin_gc=gdk_gc_new(mainwin->window);
	
	mainwin_wlist=alloc_list();
	mainwin_minimize=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,244,3,9,9,9,0,9,9,mainwin_minimize_cb,SKIN_TITLEBAR);
	mainwin_shade=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,254,3,9,9,0,18,9,18,NULL,SKIN_TITLEBAR);
	mainwin_close=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,264,3,9,9,18,0,18,9,mainwin_quit_cb,SKIN_TITLEBAR);
	mainwin_rew=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,16,88,22,18,0,0,0,18,playlist_prev,SKIN_CBUTTONS);
	mainwin_play=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,39,88,22,18,23,0,23,18,mainwin_play_pushed,SKIN_CBUTTONS);
	mainwin_pause=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,62,88,22,18,46,0,46,18,input_pause,SKIN_CBUTTONS);
	mainwin_stop=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,85,88,22,18,69,0,69,18,mainwin_stop_pushed,SKIN_CBUTTONS);
	mainwin_fwd=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,108,88,22,18,92,0,92,18,playlist_next,SKIN_CBUTTONS);
	mainwin_eject=create_pbutton(mainwin_wlist,mainwin_bg,mainwin_gc,136,89,22,16,114,0,114,16,mainwin_eject_pushed,SKIN_CBUTTONS);
	
	
	mainwin_shuffle=create_tbutton(mainwin_wlist,mainwin_bg,mainwin_gc,164,89,46,15,28,0,28,15,28,30,28,45,mainwin_shuffle_pushed,SKIN_SHUFREP);
	tbutton_set_toggled(mainwin_shuffle,cfg.shuffle);
	mainwin_repeat=create_tbutton(mainwin_wlist,mainwin_bg,mainwin_gc,210,89,28,15,0,0,0,15,0,30,0,45,mainwin_repeat_pushed,SKIN_SHUFREP);
	tbutton_set_toggled(mainwin_repeat,cfg.repeat);
	
	mainwin_eq=create_tbutton(mainwin_wlist,mainwin_bg,mainwin_gc,219,58,23,12,0,61,46,61,0,73,46,73,mainwin_eq_pushed,SKIN_SHUFREP);
	tbutton_set_toggled(mainwin_eq,cfg.equalizer_visible);
	mainwin_pl=create_tbutton(mainwin_wlist,mainwin_bg,mainwin_gc,242,58,23,12,23,61,69,61,23,73,69,73,mainwin_pl_pushed,SKIN_SHUFREP);
	tbutton_set_toggled(mainwin_pl,cfg.playlist_visible);
	
	mainwin_info=create_textbox(mainwin_wlist,mainwin_bg,mainwin_gc,112,27,153,1,SKIN_TEXT);
	textbox_set_scroll(mainwin_info,cfg.autoscroll);
	mainwin_rate_text=create_textbox(mainwin_wlist,mainwin_bg,mainwin_gc,111,43,15,0,SKIN_TEXT);
	mainwin_freq_text=create_textbox(mainwin_wlist,mainwin_bg,mainwin_gc,156,43,10,0,SKIN_TEXT);
	
	mainwin_menurow=create_menurow(mainwin_wlist,mainwin_bg,mainwin_gc,10,22,304,0,304,44,mainwin_mr_change,mainwin_mr_release,SKIN_TITLEBAR);
	mainwin_menurow->mr_doublesize_selected=cfg.doublesize;
	
	mainwin_volume=create_hslider(mainwin_wlist,mainwin_bg,mainwin_gc,107,57,68,13,15,422,0,422,14,11,15,0,0,51,mainwin_volume_framecb,mainwin_volume_motioncb,mainwin_volume_releasecb,SKIN_VOLUME);
	mainwin_balance=create_hslider(mainwin_wlist,mainwin_bg,mainwin_gc,177,57,38,13,15,422,0,422,14,11,15,9,0,24,mainwin_balance_framecb,mainwin_balance_motioncb,mainwin_balance_releasecb,SKIN_BALANCE);
	read_volume();
	
	mainwin_monostereo=create_monostereo(mainwin_wlist,mainwin_bg,mainwin_gc,212,41,SKIN_MONOSTEREO);
	
	mainwin_playstatus=create_playstatus(mainwin_wlist,mainwin_bg,mainwin_gc,24,28);
	
	mainwin_minus_num=create_number(mainwin_wlist,mainwin_bg,mainwin_gc,36,26,SKIN_NUMBERS);
	hide_widget(mainwin_minus_num);
	mainwin_10min_num=create_number(mainwin_wlist,mainwin_bg,mainwin_gc,48,26,SKIN_NUMBERS);
	hide_widget(mainwin_10min_num);
	mainwin_min_num=create_number(mainwin_wlist,mainwin_bg,mainwin_gc,60,26,SKIN_NUMBERS);
	hide_widget(mainwin_min_num);
	mainwin_10sec_num=create_number(mainwin_wlist,mainwin_bg,mainwin_gc,78,26,SKIN_NUMBERS);
	hide_widget(mainwin_10sec_num);
	mainwin_sec_num=create_number(mainwin_wlist,mainwin_bg,mainwin_gc,90,26,SKIN_NUMBERS);
	hide_widget(mainwin_sec_num);
	
	mainwin_vis=create_vis(mainwin_wlist,mainwin_bg,mainwin_gc,24,43);
	
	mainwin_position=create_hslider(mainwin_wlist,mainwin_bg,mainwin_gc,16,72,249,10,248,0,278,0,29,10,10,0,0,219,NULL,mainwin_position_motioncb,mainwin_position_releasecb,SKIN_POSBAR);
	hide_widget(mainwin_position);
	
}

void set_timer_mode(TimerMode mode)
{
	cfg.timer_mode=mode;
	if(mode==TIMER_ELAPSED)
	{
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Elapsed"))->active=TRUE;
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Remaining"))->active=FALSE;
	}
	else
	{
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Elapsed"))->active=FALSE;
		GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mainwin_options_menu,"/Time Remaining"))->active=TRUE;
	}
}
	
gint idle_func(gpointer data)
{		
	gint time,t,length;
	guchar *spec;	
			
	if(get_input_playing())
	{
		
		time=input_get_time();
		if(time==-1)
		{
			playlist_eof_reached();
		}
		else
		{
			length=playlist_get_current_length();
			playlistwin_set_time(time/1000,length/1000);
			if(spec=input_get_spectrum(time))
			{
				vis_set_spectrum(mainwin_vis,spec);
				g_free(spec);

			}

			if(cfg.timer_mode==TIMER_REMAINING)
			{
				if(length!=-1)
				{
					number_set_number(mainwin_minus_num,11);
					t=length-time;
				}
				else
				{
					number_set_number(mainwin_minus_num,10);
					t=time;
				}
			}
			else
			{
				number_set_number(mainwin_minus_num,10);
				t=time;
			}
			t/=1000;
			number_set_number(mainwin_10min_num,t/600);
			number_set_number(mainwin_min_num,(t/60)%10);
			number_set_number(mainwin_10sec_num,(t/10)%6);
			number_set_number(mainwin_sec_num,t%10);
			
			time/=1000;
			length/=1000;
			if(length>0)
				hslider_set_position(mainwin_position,(time*219)/length);
			else
				hslider_set_position(mainwin_position,0);
			
		}
		
	}		
	vis_timeout_func(mainwin_vis);
	check_ctrlsocket();	
	draw_main_window(FALSE);
	draw_playlist_window(FALSE);
	draw_equalizer_window(FALSE);
	return TRUE;
}

struct option long_options[]=
{
	{"help",0,0,0},
	{"session",1,0,0},
	{"rew",0,0,0},
	{"play",0,0,0},
	{"pause",0,0,0},
	{"stop",0,0,0},
	{"fwd",0,0,0},
	{"enqueue",0,0,0},
	{"version",0,0,0},
	{0,0,0,0}    
};


void display_usage(void)
{
    printf("\nUsage: x11amp [options] [files] ...\n\n\
Options:\n\
--------\n\n\
-h, --help              Display this text and exit.\n\
-n, --session		Select X11Amp session (Default: 0)\n\
-r, --rew               Skip backwards in playlist\n\
-p, --play		Start playing current playlist\n\
-u, --pause		Pause current song\n\
-s, --stop		Stop current song\n\
-f, --fwd               Skip forward in playlist\n\
-e, --enqueue		Don't clear the playlist\n\
-v, --version           Print version number and exit.\n\n");


	exit(0);
}
	
void parse_cmd_line(int argc,char **argv,gboolean remote)
{
	gint c,o,session=0,i,fd;
	gboolean do_rew=FALSE,do_play=FALSE,do_pause=FALSE,do_stop=FALSE,do_fwd=FALSE,enqueue=FALSE;
	gchar	**playlist,*filename,*tmp;
	struct stat statbuf;
	gchar *ext;
	
	
	while((c=getopt_long(argc,argv,"hn:rpusfev",long_options,&o))!=-1)
	{
		switch(c)
		{
			case	0:
				switch(o)
				{
					case	0:
						display_usage();
						break;
					case	1:
						session=atoi(optarg);
						break;
					case	2:
						do_rew=TRUE;
						break;
					case	3:
						do_play=TRUE;
						break;
					case	4:
						do_pause=TRUE;
						break;
					case	5:
						do_stop=TRUE;
						break;
					case	6:
						do_fwd=TRUE;
						break;
					case	7:
						enqueue=TRUE;
						break;
					case	8:
						printf("%s %s\n",PACKAGE,VERSION);
						exit(0);
						break;
				}
				break;
			case	'h':
				display_usage();
				break;
			case	'n':
				session=atoi(optarg);
				break;
			case	'r':
				do_rew=TRUE;
				break;
			case	'p':
				do_play=TRUE;
				break;
			case	'u':
				do_pause=TRUE;
				break;
			case	's':
				do_stop=TRUE;
				break;
			case	'f':
				do_fwd=TRUE;
				break;
			case	'e':
				enqueue=TRUE;
				break;
			case	'v':
				printf("%s %s\n",PACKAGE,VERSION);
				exit(0);
				break;
		}
	}
	if(remote)
		playlist=g_malloc0((argc-optind)*sizeof(gchar *));
	else if(!enqueue&&argc-optind>0)
		playlist_clear();
	for(i=optind;i<argc;i++)
	{
		if(argv[i][0]=='/'||strstr(argv[i],"://"))
			filename=g_strdup(argv[i]);
		else
		{
			filename=g_strconcat(tmp=g_get_current_dir(),"/",argv[i],NULL);
			g_free(tmp);
		}
		if(remote)
			playlist[i-optind]=filename;
		else
		{
			if(!stat(filename,&statbuf))
			{
				if(S_ISDIR(statbuf.st_mode))
					playlist_add_dir(filename);
				else
					if(ext=strrchr(filename,'.'))
						if(!strcasecmp(ext,".m3u")||!strcasecmp(ext,".pls"))
							playlist_load(filename);
						else
							playlist_add(filename);
					else
						playlist_add(filename);
			}								
			else
				if(ext=strrchr(filename,'.'))
					if(!strcasecmp(ext,".m3u")||!strcasecmp(ext,".pls"))
						playlist_load(filename);
					else
						playlist_add(filename);
				else
					playlist_add(filename);
			playlist_generate_shuffle_list();
			start_playing=TRUE;
			g_free(filename);
		}
	}
	if(remote)
	{
		if(argc-optind>0)
			if(fd=connect_to_session(session))
				remote_playlist(fd,playlist,argc-optind,enqueue);
		if(do_rew)
			if(fd=connect_to_session(session))
				remote_rew(fd);
		if(do_play)
			if(fd=connect_to_session(session))
				remote_play(fd);
		if(do_pause)
			if(fd=connect_to_session(session))
				remote_pause(fd);
		if(do_stop)
			if(fd=connect_to_session(session))
				remote_stop(fd);
		if(do_fwd)
			if(fd=connect_to_session(session))
				remote_fwd(fd);
	}	
}
			
void main(int argc,char **argv)
{
	gchar *filename,*ext;
	gint i,x,y;
	
	srand(time(NULL));
	
	read_config();
	init_output_plugins();
	init_input_plugins();
	
	
	gtk_init(&argc,&argv);
	gdk_rgb_init();
	
	/*
	 * Why doesn't GTK 1.1.5 have GTK_HAVE_FEATURES_1_1_5 defined?! :/
	 */
	if(!setup_ctrlsocket())
	{
		parse_cmd_line(argc,argv,TRUE);
		exit(0);
	}
	
	root_window=gdk_window_foreign_new(GDK_ROOT_WINDOW());
#if (GTK_MAJOR_VERSION>=1) && (GTK_MINOR_VERSION==1) && (GTK_MICRO_VERSION>=5) && (GTK_MICRO_VERSION<=7)
	mainwin=gtk_draw_window_new(GTK_WINDOW_TOPLEVEL);
#else
	mainwin=gtk_window_new(GTK_WINDOW_TOPLEVEL);
#ifdef GTK_HAVE_FEATURES_1_1_8
	gtk_widget_set_app_paintable(mainwin,TRUE);
#endif
#endif
	gtk_window_set_title(GTK_WINDOW(mainwin),"X11Amp");
	gtk_window_set_policy(GTK_WINDOW(mainwin),FALSE,FALSE,TRUE);
	gtk_window_set_wmclass(GTK_WINDOW(mainwin),"X11Amp_Player","x11amp");
	
	gtk_widget_set_events (mainwin,GDK_FOCUS_CHANGE_MASK|GDK_BUTTON_MOTION_MASK|GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK|GDK_STRUCTURE_MASK);
	gtk_widget_realize(mainwin);
	if(cfg.doublesize)
		gtk_widget_set_usize(mainwin,550,232);
	else
		gtk_widget_set_usize(mainwin,275,116);
	gdk_window_set_decorations(mainwin->window,0);
	mainwin_accel=gtk_accel_group_new();
	
	filename=g_strconcat(g_get_home_dir(),"/.x11amp/gtkrc",NULL);
	gtk_rc_init();
	gtk_rc_parse(filename);
	g_free(filename);
	
	
	setup_main_window();
	playlistwin_create();
	equalizerwin_create();
	init_skins();
	load_skin(cfg.skin);
	mainwin_set_info_text();
	create_popups();
	create_prefs_window();
	
	parse_cmd_line(argc,argv,FALSE);
	filename=g_strconcat(g_get_home_dir(),"/.x11amp/x11amp.m3u",NULL);
	playlist_load(filename);
	g_free(filename);

	
	
	gtk_window_add_accel_group(GTK_WINDOW(mainwin),mainwin_accel);
	gtk_signal_connect(GTK_OBJECT(mainwin),"button_press_event",GTK_SIGNAL_FUNC(mainwin_press),NULL);
	gtk_signal_connect(GTK_OBJECT(mainwin),"button_release_event",GTK_SIGNAL_FUNC(mainwin_release),NULL);
	gtk_signal_connect(GTK_OBJECT(mainwin),"motion_notify_event",GTK_SIGNAL_FUNC(mainwin_motion),NULL);
	gtk_signal_connect(GTK_OBJECT(mainwin),"focus_in_event",GTK_SIGNAL_FUNC(mainwin_focus_in),NULL);
	gtk_signal_connect(GTK_OBJECT(mainwin),"focus_out_event",GTK_SIGNAL_FUNC(mainwin_focus_out),NULL);
	gtk_signal_connect(GTK_OBJECT(mainwin),"configure_event",GTK_SIGNAL_FUNC(mainwin_configure),NULL);
	gtk_drag_dest_set(mainwin,GTK_DEST_DEFAULT_ALL,drop_types,1,GDK_ACTION_COPY);
	gtk_signal_connect(GTK_OBJECT(mainwin),"drag_data_received",GTK_SIGNAL_FUNC(mainwin_drag_data_received),NULL);
		
	
	if(cfg.playlist_visible) playlistwin_show();
	if(cfg.equalizer_visible) equalizerwin_show();
	gtk_widget_show(mainwin);
	if(cfg.player_x!=-1)
		/*
		gtk_widget_set_uposition(mainwin,cfg.player_x,cfg.player_y);
		*/
		gdk_window_move(mainwin->window,cfg.player_x,cfg.player_y);
	gdk_flush();
	draw_main_window(TRUE);
	/*
	 * There seems to be something strange going on with timeouts, if this is set to 20
	 * it starts running at 30 (Intel Pentium II, Kernel 2.2.0), if it is set to 10 it 
	 * runs at 20, this may cause problems on timing the visualization on other systems so
	 * there is a lame check in vis_update_func to make sure that the visualization don't get 
	 * updated to often
	 */
	
	mainwin_timeout_tag=gtk_timeout_add(10,idle_func,NULL); 
	if(start_playing)
		playlist_play();	
	gtk_main();

}
