/*  x11amp - graphically mp3 player..
 *  Copyright (C) 1998-1999  Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "x11amp.h"

static gfloat vis_afalloff_speeds[]={0.34,0.5,1.0,1.3,1.6};
static gfloat vis_pfalloff_speeds[]={1.1,1.16,1.23,1.3,1.4};
static gint vis_redraw_delays[]={1,2,4,8};

gint vis_timeout_func(gpointer data)
{
	static GTimer *timer=NULL;
	gulong micros=9999999;
	if(!timer)
	{
		timer=g_timer_new();
		g_timer_start(timer);
	}
	else
	{
		g_timer_elapsed(timer,&micros);
		if(micros>14000)
			g_timer_reset(timer);
		
	}
	if(micros>14000)
	{
		vis_afalloff_func(data);
		vis_pfalloff_func(data);
		vis_redraw_func(data);
	}
}

gint vis_afalloff_func(gpointer data)
{
	Vis *vis=(Vis *)data;
	gint i,j,max;
	
	for(i=0;i<75;i++)
	{
		if(vis->vs_spectrum[i]>0.0)
		{
			vis->vs_spectrum[i]-=vis_afalloff_speeds[cfg.analyzer_falloff];
			if(vis->vs_spectrum[i]<0.0) vis->vs_spectrum[i]=0.0;
			vis->vs_need_redraw=TRUE;
		}
	}

	return TRUE;
}

gint vis_pfalloff_func(gpointer data)
{
	Vis *vis=(Vis *)data;
	gint i,j;

	
	for(i=0;i<75;i++)
	{
		if(vis->vs_peak[i]>0.0)
		{
			vis->vs_peak[i]-=vis->vs_peak_speed[i];
			vis->vs_peak_speed[i]*=vis_pfalloff_speeds[cfg.peaks_falloff];
			vis->vs_need_redraw=TRUE;
			if(vis->vs_peak[i]<vis->vs_spectrum[i])
				vis->vs_peak[i]=vis->vs_spectrum[i];
			if(vis->vs_peak[i]<0.0)
				vis->vs_peak[i]=0.0;
		}
	}
	return TRUE;
}

gint vis_redraw_func(gpointer data)
{
	Vis *vis=(Vis *)data;
	
	if(!vis->vs_refresh_delay)
	{
		if(vis->vs_need_redraw)
			vis_draw(vis);
		vis->vs_need_redraw=FALSE;
		vis->vs_refresh_delay=vis_redraw_delays[cfg.vis_refresh];
		
	}
	vis->vs_refresh_delay--;
	return TRUE;
}

void vis_draw(Widget *w)
{
	Vis *vis=(Vis *)w;
	GdkColor c;
	gint i,x,y,h;
	guchar vis_color[24][3],*tmp;
	guchar rgb_data[150*32],*ptr;
	guint32 colors[24];
	static GdkImage *img=NULL;
	GdkRgbCmap *cmap;
	
	get_skin_viscolor(vis_color);
	for(y=0;y<24;y++)
	{
		colors[y]=vis_color[y][0]<<16|vis_color[y][1]<<8|vis_color[y][2];
	}
	cmap=gdk_rgb_cmap_new(colors,24);
	

	if(!cfg.doublesize)
	{
		memset(rgb_data,0,75*16);
		for(y=1;y<16;y+=2)
		{
			ptr=ptr=rgb_data+(y*75);
			for(x=0;x<75;x+=2,ptr+=2)
				*ptr=1;
		}		

		for(x=0;x<75;x++)
		{
			if((cfg.analyzer_type==ANALYZER_BARS&&(x%5)==0)||cfg.analyzer_type==ANALYZER_LINES)
				h=(gint)vis->vs_spectrum[x];

			if(h&&(cfg.analyzer_type==ANALYZER_LINES||(x%5)!=4))
			{
				ptr=rgb_data+((16-h)*75)+x;
				switch(cfg.analyzer_mode)
				{
					case	ANALYZER_NORMAL:		
						for(y=0;y<h;y++,ptr+=75)
							*ptr=18-h+y;
						break;
					case	ANALYZER_FIRE:
						for(y=0;y<h;y++,ptr+=75)
							*ptr=y+2;
						break;
					case	ANALYZER_VLINES:
						for(y=0;y<h;y++,ptr+=75)
							*ptr=18-h;
						break;
				}
			}
		}
		if(cfg.analyzer_peaks)
		{
			for(x=0;x<75;x++)
			{
				if((cfg.analyzer_type==ANALYZER_BARS&&(x%5)==0)||cfg.analyzer_type==ANALYZER_LINES)
					h=(gint)vis->vs_peak[x];
				if(h&&(cfg.analyzer_type==ANALYZER_LINES||(x%5)!=4))
					rgb_data[(16-h)*75+x]=23;
			}
		}
		gdk_draw_indexed_image(mainwin->window,mainwin_gc,vis->vs_widget.x,vis->vs_widget.y,vis->vs_widget.width,vis->vs_widget.height,GDK_RGB_DITHER_NONE,(guchar *)rgb_data,75,cmap);		
	}
	else
	{
		memset(rgb_data,0,150*32);
		for(y=1;y<16;y+=2)
		{
			ptr=rgb_data+(y*300);
			for(x=0;x<75;x+=2,ptr+=4)
			{
				*ptr=1;
				*(ptr+1)=1;
				*(ptr+150)=1;
				*(ptr+151)=1;
			}
		}		

		for(x=0;x<75;x++)
		{
			if((cfg.analyzer_type==ANALYZER_BARS&&(x%5)==0)||cfg.analyzer_type==ANALYZER_LINES)
				h=(gint)vis->vs_spectrum[x];
			if(h&&(cfg.analyzer_type==ANALYZER_LINES||(x%5)!=4))
			{
				ptr=rgb_data+((16-h)*300)+(x<<1);
				switch(cfg.analyzer_mode)
				{
					case	ANALYZER_NORMAL:		
						for(y=0;y<h;y++,ptr+=300)
						{
							*ptr=18-h+y;
							*(ptr+1)=18-h+y;
							*(ptr+150)=18-h+y;
							*(ptr+151)=18-h+y;
						}
						break;
					case	ANALYZER_FIRE:
						for(y=0;y<h;y++,ptr+=300)
						{
							*ptr=y+2;
							*(ptr+1)=y+2;
							*(ptr+150)=y+2;
							*(ptr+151)=y+2;
						}
						break;
					case	ANALYZER_VLINES:
						for(y=0;y<h;y++,ptr+=300)
						{
							*ptr=18-h;
							*(ptr+1)=18-h;
							*(ptr+150)=18-h;
							*(ptr+151)=18-h;
						}

						break;
				}
				
			}
		}
		if(cfg.analyzer_peaks)
		{
			
			for(x=0;x<75;x++)
			{
				if((cfg.analyzer_type==ANALYZER_BARS&&(x%5)==0)||cfg.analyzer_type==ANALYZER_LINES)
					h=(gint)vis->vs_peak[x];

				if(h&&(cfg.analyzer_type==ANALYZER_LINES||(x%5)!=4))
				{
					ptr=rgb_data+(16-h)*300+(x<<1);
					*ptr=23;
					*(ptr+1)=23;
					*(ptr+150)=23;
					*(ptr+151)=23;
				}
			}
		}
		
		gdk_draw_indexed_image(mainwin->window,mainwin_gc,vis->vs_widget.x<<1,vis->vs_widget.y<<1,vis->vs_widget.width<<1,vis->vs_widget.height<<1,GDK_RGB_DITHER_NONE,(guchar *)rgb_data,150,cmap);		
	}
	gdk_rgb_cmap_free(cmap);
	gdk_flush();
}

void vis_set_spectrum(Vis *vis,guchar *spectrum)
{
	gint i;
	for(i=0;i<75;i++)
	{
		if(spectrum[i]>vis->vs_spectrum[i]) 
		{
			vis->vs_spectrum[i]=spectrum[i];
			if(vis->vs_spectrum[i]>vis->vs_peak[i])
			{
				vis->vs_peak[i]=vis->vs_spectrum[i];
				vis->vs_peak_speed[i]=0.05;
				vis->vs_need_redraw=TRUE;
			}
		}
	}
}

void vis_set_refresh(Vis *vis,RefreshRate rate)
{
/*	if(vis->vs_redraw_tag)
		gtk_timeout_remove(vis->vs_redraw_tag);
	vis->vs_redraw_tag=gtk_timeout_add((rate+1)*20,vis_redraw_func,vis);*/
}

void vis_set_afalloff(Vis *vis,FalloffSpeed speed)
{
		
}
				
void vis_set_pfalloff(Vis *vis,FalloffSpeed speed)
{
}


Vis *create_vis(LList *wlist,GdkPixmap *parent,GdkGC *gc,gint x,gint y)
{
	Vis *vis;
	vis=(Vis *)g_malloc0(sizeof(Vis));
	vis->vs_widget.parent=parent;
	vis->vs_widget.gc=gc;
	vis->vs_widget.x=x;
	vis->vs_widget.y=y;
	vis->vs_widget.width=75;
	vis->vs_widget.height=16;
	vis->vs_widget.visible=1;
	vis->vs_widget.draw=vis_draw;
/*	vis->vs_afalloff_tag=gtk_timeout_add(20,vis_timeout_func,vis);
	vis->vs_pfalloff_tag=gtk_timeout_add(20,vis_pfalloff_func,vis);
	
	vis->vs_redraw_tag=gtk_timeout_add((cfg.vis_refresh+1)*20,vis_redraw_func,vis);*/
	add_widget(wlist,vis);
	return vis;
}


