/*
    WebMaker - KDE HTML Editor
    Copyright (C) 1998, 1999 Alexei Dets <dets@services.ru>
	
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
			
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
					
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/    

//#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <qlist.h>
#include <qobjcoll.h>
#include <qprinter.h>
#include <qprintdialog.h>
#include <qpaintdevicemetrics.h>
#include <kmsgbox.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include "qextfileinfo.h"
#include "htmledit.moc.h"
#include "html.h"
#include <kwm.h>


const int BORDER=3;

const char* ANCHOR_TAGS[]={"A", "LINK", "BASE"};
const char* STYLE_TAGS[]={"B", "BIG", "FONT", "I", "S", "SMALL", "STRIKE", "STYLE", "TT", "U", "BASEFONT"};
const char* IMAGE_TAGS[]={"IMG"};
const char* SCRIPT_TAGS[]={"APPLET", "OBJECT", "PARAM", "SCRIPT"};
const char* TABLE_TAGS[]={"CAPTION", "COL", "COLGROUP", "TABLE", "TBODY", "TD", "TFOOT", "TH", "THEAD", "TR"};

const uint ANCHOR_TAGS_COUNT=sizeof(ANCHOR_TAGS) / sizeof(char*);
const uint STYLE_TAGS_COUNT=sizeof(STYLE_TAGS) /sizeof(char*);
const uint IMAGE_TAGS_COUNT=sizeof(IMAGE_TAGS) /sizeof(char*);
const uint SCRIPT_TAGS_COUNT=sizeof(SCRIPT_TAGS) /sizeof(char*);
const uint TABLE_TAGS_COUNT=sizeof(TABLE_TAGS) /sizeof(char*);

static TagChooserDialog *chooser=0;
static uint editorCount=0;

HTMLEdit::HTMLEdit (KApplication *a,QWidget *parent, const char *name, const char *filename) : MyKEdit(a, parent, name, filename)
{
		initPalette();
		initTagDescriptions();
		cursorOn=FALSE;
		cursorTimer=0;
		connect(this, SIGNAL(CursorPositionChanged()), this, SLOT(makeCursorVisible()));
		externalBrowser="/usr/local/bin/netscape";
		tempFile=tempnam(0, "html");
		basePath=QDir::currentDirPath();
		connect(this, SIGNAL(fileChanged()), this, SLOT(setProjectRoot()));
		chooserConnected=FALSE;
		editorCount++;

		myLeftWords.append(  "[<>\"]" );
		myRightWords.append( "[<>\"]" );
}

HTMLEdit::~HTMLEdit()
{
		remove(tempFile);
		free(tempFile);
		delete tagDescriptions;
		editorCount--;
		if (chooser && (editorCount==0))
				{
						delete chooser;
						chooser=0;
				};
}

int HTMLEdit::getFile(QFile &file, QString mask)
{
		if (isModified())
				{
						int result=KMsgBox::yesNoCancel(this, i18n("Warning"), i18n("The current Document has been modified.\nWould you like to save it?"), KMsgBox::EXCLAMATION);
						if (result==1)
								{
										result=doSave();
										if (result==KEDIT_USER_CANCEL) return KEDIT_USER_CANCEL;
										if (result!=KEDIT_OK)
												{
														result=KMsgBox::yesNo(this, i18n("Sorry"), i18n("Could not save the document.\nOpen a new document anyways?"), KMsgBox::EXCLAMATION);
														if (result==2) return KEDIT_USER_CANCEL;
												};
								};
						if (result==3) return KEDIT_USER_CANCEL;
				};
	
		QString fileName=KFileDialog::getOpenFileName(basePath, mask, this);
		if (fileName.isEmpty()) return KEDIT_USER_CANCEL;
	
		QFileInfo info(fileName);
	
		if (!info.exists())
				{
						KMsgBox::message(this, i18n("Sorry"), i18n("The specified File does not exist."), KMsgBox::EXCLAMATION);
						return KEDIT_RETRY;
				};
		if (info.isDir())
				{
						KMsgBox::message(this, i18n("Sorry"), i18n("You have specificated a directory."), KMsgBox::EXCLAMATION);
						return KEDIT_RETRY;
				};
		if (!info.isReadable())
				{
						KMsgBox::message(this, i18n("Sorry"), i18n("You do not have read permission to this file."), KMsgBox::EXCLAMATION);
						return KEDIT_RETRY;
				};
	
		file.setName(fileName);
	
		if (!file.open(IO_ReadOnly))
				{
						KMsgBox::message(this, i18n("Sorry"), i18n("An Error occured while trying to open this Document."), KMsgBox::EXCLAMATION);
						return KEDIT_OS_ERROR;
				};
	
		emit loading();
		kapp->processEvents();
	
		return KEDIT_OK;
}

int HTMLEdit::getTagGroup(QString *tag)
{
		unsigned int i, start, stop;
		start=stop=0;
		for (i=0; i < tag->length(); i++)
				{
						if (isalpha(tag->data()[i]))
								{
										start=i;
										break;
								};
				};
		for (i=start+1; i < tag->length(); i++)
				{
						if (!isalnum(tag->data()[i]))
								{
										stop=i;
										break;
								};
				};
		QString element(tag->mid(start, stop-start));
		element=element.upper();
		for (i=0; i<TABLE_TAGS_COUNT; i++) if (element==TABLE_TAGS[i]) return TABLE;
		for (i=0; i<STYLE_TAGS_COUNT; i++) if (element==STYLE_TAGS[i]) return STYLE;
		for (i=0; i<IMAGE_TAGS_COUNT; i++) if (element==IMAGE_TAGS[i]) return IMAGE;
		for (i=0; i<ANCHOR_TAGS_COUNT; i++) if (element==ANCHOR_TAGS[i]) return ANCHOR;
		for (i=0; i<SCRIPT_TAGS_COUNT; i++) if (element==SCRIPT_TAGS[i]) return SCRIPT;
		return DEFAULT;
}

int HTMLEdit::textWidthWithTabs(const char *s, int nChars)
{
		if (!s) return 0;
		QFontMetrics fm=fontMetrics();
		if (nChars==-1) nChars = strlen(s);
		int dist=-fm.minLeftBearing();
		const char *tmp=s;
		if (!tmp) return 0;
		int tabDist = tabLength();
		while (*tmp && tmp-s < nChars )
				{
						if (*tmp == '\t')
								dist = ( dist/tabDist + 1 ) * tabDist;
						else
								dist += fm.width( tmp, 1 );
						tmp++;
				}
		return dist;
}

void HTMLEdit::paintCell (QPainter *painter, int row, int col)
{
		QColorGroup colors=colorGroup();
		QString s(textLine(row));
		QRect r = cellUpdateRect();
		QPixmap* buffer=new QPixmap(r.size());
		buffer->fill(colors.base());
		QPainter p( buffer );
		p.translate( -r.left(), -r.top() );
		p.setTabStops(tabLength());
		p.setFont( painter->font() );
	
		//Text parsing
		QList <QRect> fragments;
		QList <int> styles;
		int tagStart, tagEnd, startLength, endLength, parsedLength;
		tagStart=tagEnd=startLength=endLength=parsedLength=0;
		fragments.setAutoDelete(TRUE);
		styles.setAutoDelete(TRUE);
		while (parsedLength<cellWidth(col))
				{
						tagStart=s.find("<", tagEnd);
						if (tagStart==-1) break;
						else
								{
										tagEnd=s.find(">", tagStart);
										if (tagEnd==-1) break;
										startLength=textWidthWithTabs(s, tagStart);
										endLength=textWidthWithTabs(s, tagEnd+1);
										fragments.append(new QRect(BORDER+startLength-r.left(), -r.top(), endLength-startLength+1, cellHeight(row)));
										parsedLength=endLength;
			
										//Tag content parsing
										QString tagContent=s.mid(tagStart, tagEnd-tagStart+1);
										styles.append(new int(getTagGroup(&tagContent)));
								};
				};
	
		//Text drawing
		p.setPen(colors.text());
		p.drawText(BORDER, 0, cellWidth(col)-BORDER, cellHeight(row), ExpandTabs, s);
		p.setClipping(TRUE);
		for (unsigned int i=0; i<fragments.count(); i++)
				{
						p.setClipRect(*fragments.at(i));
						p.setPen(palette[*styles.at(i)]);
						p.drawText(BORDER, 0, cellWidth(col)-BORDER, cellHeight(row), ExpandTabs, s);
				};
		p.setClipping(FALSE);
	
		//Marked text
		int line1, line2, col1, col2;
		int markStart, markLength;
		if (getMarkedRegion(&line1, &col1, &line2, &col2))
				{
						if ((row>=line1) && (row<=line2))
								{
										markStart=markLength=0;
										if ((row>line1) && (row<line2))
												{
														markStart=BORDER;
														markLength=cellWidth(col)-BORDER;
												};
										if ((row==line1))
												{
														markStart=BORDER + textWidthWithTabs(s, col1);
														markLength=cellWidth(col)-markStart;
												};
										if ((row==line2))
												{
														markStart=BORDER;
														markLength=textWidthWithTabs(s, col2);
												};
										if ((line1==line2))
												{
														markStart=BORDER + textWidthWithTabs(s, col1);
														markLength=textWidthWithTabs(s, col2)-textWidthWithTabs(s, col1)+1;
												};
										p.setClipping(TRUE);
										p.setClipRect(markStart-r.left(), -r.top(), markLength, cellHeight(row));
										if ( style() == WindowsStyle )
												{
														p.fillRect(markStart, 0, markLength, cellHeight(row), KApplication::winStyleHighlightColor());
														p.setPen(white);
												}
										else
												{
														p.fillRect(markStart, 0, markLength, cellHeight(row), colors.text());
														p.setPen(colors.base());
												}
										p.drawText(BORDER, 0, cellWidth(col)-BORDER, cellHeight(row), ExpandTabs, s);
										p.setClipping(FALSE);
								};
				};
	
		//Cursor drawing
		int cursorX, cursorY;
		cursorPosition(&cursorY, &cursorX);
		if ( row == cursorY && cursorOn && !isReadOnly() )
				{
						QFontMetrics fm=painter->fontMetrics();
						int cursorPos = QMIN( (int)s.length(), cursorX );
						int x = BORDER + textWidthWithTabs(s, cursorPos ) - 1;
						int y = 0;
						if ( hasFocus() )
								{
										p.setPen(colors.text());
										p.drawLine(x - 2, y, x + 2, y);
										p.drawLine(x, y, x, y + fm.height() - 2);
										p.drawLine(x - 2, y + fm.height() - 2, x + 2, y + fm.height() - 2);
								}
				}
	
		p.end();
		painter->drawPixmap(r.left(), r.top(), *buffer);
		delete(buffer);
}

void HTMLEdit::setupPainter(QPainter *painter)
{
		painter->setBackgroundColor(colorGroup().base()); //why not in QMultiLineEdit???!
}

void HTMLEdit::closeEvent(QCloseEvent* e)
{
		if (isModified())
				{
						int result;
						QString message(i18n("File "));
						message+=QFileInfo(getName()).fileName();
						message+=i18n(" has been modified.\nWould you like to save it?");
						result=KMsgBox::yesNoCancel(this, i18n("Message"), message);
						switch (result)
								{
								case 1:
										result = doSave();
										if (result == KEDIT_USER_CANCEL) return;
										if (result != KEDIT_OK)
												{
														result=KMsgBox::yesNo(this, i18n("Error!"), i18n("Error saving file! Exit anyway?"), KMsgBox::EXCLAMATION);
														if (result != 1) return;
														break;
												};
								case 2:
										break;
								case 3:
										return;
								}
				};
		e->accept();
}

void HTMLEdit::focusInEvent(QFocusEvent *e )
{
		cursorTimer=startTimer(500);
		cursorOn=TRUE;
		KEdit::focusInEvent(e);
}

void HTMLEdit::focusOutEvent(QFocusEvent *e )
{
		killTimer(cursorTimer);
		cursorTimer=0;
		cursorOn=FALSE;
		KEdit::focusOutEvent(e);
}

void HTMLEdit::resizeEvent(QResizeEvent *e)
{
		KEdit::resizeEvent(e);
		//		setCellWidth(width());
}

void HTMLEdit::timerEvent (QTimerEvent *t)
{
		if (hasFocus() && t->timerId()==cursorTimer) cursorOn=!cursorOn;
		KEdit::timerEvent(t);
}

void HTMLEdit::makeCursorVisible()
{
		cursorOn=TRUE;
		int cursorX, cursorY;
		cursorPosition(&cursorY, &cursorX);
		updateCell(cursorY, 0, FALSE);
}

void HTMLEdit::print(const bool pixmapPrinting)
{
		QPrinter *printer=new QPrinter();
		if (QPrintDialog::getPrinterSetup(printer))
				{
						QPaintDeviceMetrics pm(printer);
						QPainter printerPainter;
						QPainter bufferPainter;
						QPainter *painter;
						QPixmap *buffer;

						printerPainter.begin(printer);

						if (pixmapPrinting)
								{
										buffer=new QPixmap(pm.width(), pm.height());
										buffer->fill(white);
										bufferPainter.begin(buffer);
										painter=&bufferPainter;
								}
						else
								{
										painter=&printerPainter;
								};
			
						QColorGroup colors=colorGroup();
						painter->setFont(font());
						painter->setPen(colors.text());
						painter->setTabStops(tabLength());
						QFontMetrics fm = painter->fontMetrics();
			
						int yPos=0;
						for (int i=0; i<numLines(); i++)
								{
										if (yPos + fm.lineSpacing() > pm.height())
												{
														if (pixmapPrinting)
																{
																		printerPainter.drawPixmap(0, 0, *buffer);
																		buffer->fill(white);
																};
														printer->newPage();
														yPos = 0;
												};
										painter->drawText(0, yPos,	pm.width(), fm.lineSpacing(),
																												ExpandTabs | DontClip, textLine(i));
										yPos = yPos + fm.lineSpacing();
								};
						if (pixmapPrinting)
								{
										bufferPainter.end();
										delete buffer;
								};

						printerPainter.end();
				};
		delete printer;
}

int HTMLEdit::startImport(const char *command)
{
		if (filter.isRunning())
				{
						KMsgBox::message(this, i18n("Error"),
																					  i18n("Other filter is running. Sorry..."),
																							KMsgBox::STOP);
						return KEDIT_OS_ERROR;
				};
		
		QFile file;
		int result=getFile(file, "*|All Files");
		if (result!=KEDIT_OK) return result;
	
		QString commandLine;
		commandLine.sprintf(command, file.name());

		filter.clearArguments();
		filter << commandLine.data();
		connect(&filter, SIGNAL(processExited(KProcess*)),
										this, SLOT(filterTerminated(KProcess*)));
		connect(&filter, SIGNAL(receivedStdout(KProcess*, char*, int)),
		  						this, SLOT(filterOutput(KProcess*, char*, int)));
		connect(&filter, SIGNAL(receivedStderr(KProcess*, char*, int)),
		  						this, SLOT(filterError(KProcess*, char*, int)));
		
		if (!filter.start(KProcess::NotifyOnExit, KProcess::AllOutput)) return KEDIT_OS_ERROR;
		setReadOnly(TRUE);
		setAutoUpdate(FALSE);
		clear();
		setModified();
		setName(file.name());
		emit fileChanged();
		return KEDIT_OK;
}

int HTMLEdit::startExport(const char *command)
{
		if (filter.isRunning())
				{
						KMsgBox::message(this, i18n("Error"),
																					  i18n("Other filter is running. Sorry..."),
																							KMsgBox::STOP);
						return KEDIT_OS_ERROR;
				};
		
		QString fileName=KFileDialog::getSaveFileName(basePath, "*|All Files", this);
		if (fileName.isEmpty()) return KEDIT_USER_CANCEL;

		QFileInfo info(fileName);

		if (info.exists())
				{
						int result=KMsgBox::yesNoCancel(this, i18n("Warning"), i18n("A Document with this Name exists already.\nDo you want to overwrite it ?"), KMsgBox::EXCLAMATION);
						if (result==2) return KEDIT_RETRY;
						if (result==3) return KEDIT_USER_CANCEL;
				};
		
		QString commandLine;
		commandLine.sprintf(command, fileName.data());

		filter.clearArguments();
		filter << commandLine.data();
		connect(&filter, SIGNAL(processExited(KProcess*)),
										this, SLOT(filterTerminated(KProcess*)));
		//		connect(&filter, SIGNAL(receivedStdout(KProcess*, char*, int)),
		//						this, SLOT(filterOutput(KProcess*, char*, int)));
		//	connect(&filter, SIGNAL(receivedStderr(KProcess*, char*, int)),
		//						this, SLOT(filterError(KProcess*, char*, int)));

		if (!filter.start(KProcess::NotifyOnExit, KProcess::Stdin)) return KEDIT_OS_ERROR;
		setReadOnly(TRUE);
		filter.writeStdin(text().data(), text().length());
		filter.closeStdin();
			
		return KEDIT_OK;
}

int HTMLEdit::startFilter(const char *command)
{
		if (filter.isRunning())
				{
						KMsgBox::message(this, i18n("Error"),
																					  i18n("Other filter is running. Sorry..."),
																							KMsgBox::STOP);
						return KEDIT_OS_ERROR;
				};
		
		filter.clearArguments();
		filter << command;
		connect(&filter, SIGNAL(processExited(KProcess*)),
										this, SLOT(filterTerminated(KProcess*)));
		connect(&filter, SIGNAL(receivedStdout(KProcess*, char*, int)),
		  						this, SLOT(filterOutput(KProcess*, char*, int)));
		connect(&filter, SIGNAL(receivedStderr(KProcess*, char*, int)),
		  						this, SLOT(filterError(KProcess*, char*, int)));
		filterBuffer=text();
		
		if (!filter.start(KProcess::NotifyOnExit, KProcess::All)) return KEDIT_OS_ERROR;
		setReadOnly(TRUE);
		setAutoUpdate(FALSE);
		clear();
		setModified();
		filter.writeStdin(filterBuffer.data(), filterBuffer.length());
		filter.closeStdin();
		return KEDIT_OK;
}

void HTMLEdit::filterOutput(KProcess *, char *buffer, int buflen)
{
		QString temp(buflen+1);
		memmove(temp.data(), buffer, buflen);
		int line, col;
		cursorPosition(&line, &col);
		insertAt(temp.data(), line, col);
}

void HTMLEdit::filterError(KProcess *proc, char *buffer, int buflen)
{
		//must be implemented in future
}

void HTMLEdit::filterTerminated(KProcess *)
{
		setAutoUpdate(TRUE);
		setReadOnly(FALSE);
		repaint();
		emit CursorPositionChanged();
		disconnect(&filter, SIGNAL(receivedStderr(KProcess*, char*, int)),
		  					   	this, SLOT(filterError(KProcess*, char*, int)));
		disconnect(&filter, SIGNAL(receivedStdout(KProcess*, char*, int)),
		  				   		this, SLOT(filterOutput(KProcess*, char*, int)));
		disconnect(&filter, SIGNAL(processExited(KProcess *)),
												 this, SLOT(filterTerminated(KProcess *)));
		filterBuffer=0;
		KMsgBox::message(this, i18n("Message"),
																   i18n("Filter job finished."),
																			KMsgBox::INFORMATION);
};

void HTMLEdit::createTableTemplate()
{
		int rows, cols, headRows, headCols;
		QString tableCaption;
		TableTemplateDialog *dialog=new TableTemplateDialog(this);
		if (!dialog) return;
		if (dialog->exec())
				{
						tableCaption=dialog->getCaption();
						cols=dialog->getColumns();
						rows=dialog->getRows();
						headCols=dialog->getHeaderColumns();
						headRows=dialog->getHeaderRows();
				}
		else
				{
						delete dialog;
						return;
				};
		delete dialog;
		if (cols<1 || rows<1 || headCols<0 || headRows<0) return;
		if (headRows>rows) headRows=rows;
		if (headCols>cols) headCols=cols;
		QString s;
		int i, j, k;
		s="\n<TABLE>\n";
		if (!tableCaption.isEmpty()) s+="<CAPTION>"+tableCaption+"</CAPTION>\n";
		if (headRows>0)
				{
						s+="<THEAD>\n";
						for (i=0; i<headRows; i++)
								{
										s+="<TR>\n";
										for (j=0; j<cols; j++) s+="<TH></TH>\n";
										s+="</TR>\n";
								};
						s+="</THEAD>\n";
						s+="<TBODY>\n";
				};
		for (i=0; i<rows-headRows; i++)
				{
						s+="<TR>\n";
						for (j=0; j<headCols; j++) s+="<TH></TH>\n";
						for (k=0; k<cols-headCols; k++) s+="<TD></TD>\n";
						s+="</TR>\n";
				};
		if (headRows>0) s+="</TBODY>\n";
		s+="</TABLE>\n";
		int cursorX, cursorY;
		cursorPosition(&cursorY, &cursorX);
		setAutoUpdate(FALSE);
		insertAt(s, cursorY, cursorX);
		setAutoUpdate(TRUE);
		repaint();
		setModified();
		emit CursorPositionChanged();
};

QString HTMLEdit::getTag()
{
		QString tag;
		int cursorX, cursorY;
		cursorPosition(&cursorY, &cursorX);
		QString s=textLine(cursorY);
		int tagStart, tagEnd;
		tagStart=tagEnd=0;
		tagStart=s.findRev("<", cursorX-1);
		if (tagStart==-1) return tag;
		tagEnd=s.findRev(">", cursorX-1);
		if (tagEnd>tagStart) return tag; //not inside tag
		tagEnd=s.find(">", cursorX);
		if (tagEnd==-1) return tag;
		tag=s.mid(tagStart+1, tagEnd-tagStart-1);
		if (tag.left(1)=="/") tag=""; //closing tag
		return tag;
}

QString HTMLEdit::getTagElement()
{
		QString tag=getTag();
		unsigned int i;
		int start, stop;
		start=0;
		stop=-1;
		for (i=0; i < tag.length(); i++)
				{
						if (!isspace(tag.data()[i]))
								{
										start=i;
										break;
								};
				};
		for (i=start+1; i < tag.length(); i++)
				{
						if (!isalnum(tag.data()[i]))
								{
										stop=i;
										break;
								};
				};
		if (stop==-1) stop=tag.length();
		QString element(tag.mid(start, stop-start));
		element=element.upper();
		return element;
}

QString HTMLEdit::getTagAttributes()
{
		QString tagAttributes;
		QString tag=getTag();
		QString tagElement=getTagElement();
		if (tag.isEmpty() || tagElement.isEmpty()) return tagAttributes;
		int start=tag.find(tagElement, 0, FALSE);
		tagAttributes=tag.right(tag.length()-(start+tagElement.length()));
		return tagAttributes;
}

void HTMLEdit::initPalette()
{
		palette[ANCHOR]	=QColor(0x00, 0x80, 0x00);
		palette[COMMENT]	=QColor(0x80, 0x80, 0x80);
		palette[STYLE]	=QColor(0x00, 0x00, 0xFF);
		palette[IMAGE]	=QColor(0x80, 0x00, 0x80);
		palette[SCRIPT]	=QColor(0x80, 0x00, 0x00);
		palette[TABLE]	=QColor(0x00, 0x80, 0x80);
		palette[QUOTED]	=QColor(0x00, 0x00, 0x00);
		palette[DEFAULT]	=QColor(0x00, 0x00, 0x80);
}

void HTMLEdit::initTagDescriptions()
{
		tagDescriptions=new TagDict(97);
		tagDescriptions->setAutoDelete(FALSE);
		int i=0;
		while (TAGS[i].elementName[0]!=0)
				{
						tagDescriptions->insert(TAGS[i].elementName, &TAGS[i]);
						i++;
				};
}

void HTMLEdit::openTagChooser()
{
		if (chooser)
				{
						if (chooser->isVisible())
								{
										KWM::activate(chooser->winId());
								}
						else
								{
										chooser->show();
								}
				}
		else
				{
						chooser=new TagChooserDialog(0, "TagChooser");
						chooser->setIcon(kapp->getMiniIcon());
						chooser->show();
				};
		if (!chooserConnected)
				{
						connect(chooser, SIGNAL(tagSelected()), this, SLOT(insertTagFromChooser()));
						chooserConnected=TRUE;
				};
}

void HTMLEdit::insertTagFromChooser()
{
		if (isVisible() && getTag().isEmpty()) insertTag(chooser->getTag());
}

void HTMLEdit::insertTag(const QString &tag, const QString &attributes)
{
		if (!getTag().isEmpty()) return;
		QString startTag;
		if (attributes.isEmpty())
				startTag="<"+tag+">";
		else
				startTag="<"+tag+attributes+">";
		QString endTag;
		HTMLTag *tagProperties=tagDescriptions->find(tag.data());
		if (tagProperties)
				{
						if (tagProperties->needClosing!=FORBIDDEN)
								endTag="</"+tag+">";
						else
								endTag="";
				}
		else
				endTag="</"+tag+">";
		int line, col;
		if (hasMarkedText())
				{
						int line1, col1;
						setUpdatesEnabled(FALSE);
						getMarkedRegion(&line, &col, &line1, &col1);
						insertAt(endTag, line1, col1);
						insertAt(startTag, line, col);
						setCursorPosition(line, col+startTag.length());
						if (line==line1)
								setCursorPosition(line1, col1+startTag.length(), TRUE);
						else
								setCursorPosition(line1, col1, TRUE);
						setUpdatesEnabled(TRUE);
						for (int i=line; i<=line1; i++) updateCell(i, 0, FALSE);
				}
		else
				{
						setUpdatesEnabled(FALSE);
						cursorPosition(&line, &col);
						insertAt(startTag, line, col);
						cursorPosition(&line, &col);
						insertAt(endTag, line, col);
						setCursorPosition(line, col);
						setUpdatesEnabled(TRUE);
						updateCell(line, 0, FALSE);
				};
		setModified();
		emit CursorPositionChanged();
}

void HTMLEdit::insertTagWithEdit(const QString &tag, const QString &attributes)
{
		if (!getTag().isEmpty()) return;
		int line, col, line1, col1;
		if (hasMarkedText())
				getMarkedRegion(&line, &col, &line1, &col1);
		else
				cursorPosition(&line, &col);
		insertTag(tag, attributes);
		setCursorPosition(line, col+1);
		emit CursorPositionChanged();
		editTag();
}
	 
void HTMLEdit::updateTag(const QString &tag)
{
		int line, col;
		cursorPosition(&line, &col);
		QString oldTag=getTag();
		QString text=textLine(line);
		int pos=text.findRev("<", col-1)+1; // we should be inside the tag actually
		text.replace(pos, oldTag.length(), tag.data());
		setAutoUpdate(FALSE);
		removeLine(line);
		insertLine(text, line);
		setCursorPosition(line, col);
		setAutoUpdate(TRUE);
		repaint();
		setModified();
		emit CursorPositionChanged();
}

void HTMLEdit::editTag()
{
		QString element=getTagElement();
		QString attributeStr=getTagAttributes();
		if (element.isEmpty()) return;
		HTMLTag *tagDescr=tagDescriptions->find(element.data());
		if (!tagDescr)
				{
						element="Unknown tag - <"+element+">!";
						KMsgBox::message(this, "Error!", element.data(), KMsgBox::EXCLAMATION);
						return;
				};
		TagEditDialog *dialog=new TagEditDialog(tagDescr, this);
		if (!dialog) return;
		StrDict *attributes=parseAttributes(&attributeStr);
		dialog->setParameters(attributes);
		if (dialog->exec())
				attributeStr=dialog->getParameters();
		else
				{
						delete dialog;
						return;
				};
		delete dialog;
		if (attributes->count()>0)
				{
						StrDictIterator *params = new StrDictIterator(*attributes);
						while (params->current())
								{
										attributeStr+=" ";
										attributeStr+=params->currentKey();
										attributeStr+="=\"" + *params->current() + "\"";
										++(*params);
								};
						delete params;
				};
		delete attributes;
		QString tag=element + attributeStr;
		updateTag(tag);
}

QString HTMLEdit::stripQuotes(const QString *s)
{
		QString temp=*s;
		if (!strncmp(temp.data(), "\"", 1)) temp.remove(0, 1);
		if (!strncmp(temp.data()+temp.length()-1, "\"", 1)) temp.truncate(temp.length()-1);
		return temp;
}

StrDict* HTMLEdit::parseAttributes(const QString *attributes)
{
		StrDict *parsedAttributes=new StrDict(17, FALSE);
		parsedAttributes->setAutoDelete(TRUE);
		QString attrs=attributes->stripWhiteSpace();
		unsigned int i=0;
		int parsedLength=0;
		char c;
		QString fragment, temp;
		QStrList fragmentList;
		bool inQuotes=FALSE;
		while (i<=attrs.length())
				{
						c=attrs.data()[i];
						if (c=='\"') inQuotes=!inQuotes;
						if ((isspace(c) && !inQuotes) || c==0)
								{
										fragment=attrs.mid(parsedLength, i-parsedLength);
										if (fragmentList.count())
												{
														temp=fragmentList.last();
														if ((temp.right(1)=="=") || (fragment.left(1)=="="))
																{
																		fragment=temp+fragment;
																		fragmentList.removeLast();
																};
												};
										if (!fragment.isEmpty()) fragmentList.append(fragment);
										parsedLength=i+1;
								};
						i++;
				};
		int pos;
		for (i=0; i<fragmentList.count(); i++)
				{
						fragment=fragmentList.at(i);
						if ((pos=fragment.find("="))!=-1)
								{
										parsedAttributes->insert(fragment.left(pos).data(), new QString(stripQuotes(&fragment.right(fragment.length()-pos-1))));
								}
						else
								{
										parsedAttributes->insert(fragment.data(), new QString(fragment));
								};
				};
		return parsedAttributes;
}

int HTMLEdit::browseExternal(bool browseTempFile)
{
		QString fileName;
		int status;
		if (browseTempFile)
				{
						if (!tempFile) return KEDIT_OS_ERROR;
						bool modified=isModified();
						toggleModified(TRUE);
						status=doSave(tempFile);
						if (status!=KEDIT_OK)
								{
										toggleModified(modified);
										return status;
								};
						toggleModified(modified);
						fileName=tempFile;
				}
		else
				{
						if (isModified()) 
								{
										status=doSave();
										if (status!=KEDIT_OK) return status;
								};
						fileName=getName();
				};
		QString command="openURL(" + fileName + ")";
		pid_t pid;
		pid=fork();
		if (pid==0)
				{
						execl(externalBrowser.data(), externalBrowser.data(), "-remote", command.data(), 0);
						_exit(EXIT_FAILURE);
				}
		else if (pid<0) return KEDIT_OS_ERROR; /* fork error */
		else
				{
						/* fork ok, wait for child */
						if (waitpid(pid, &status, 0)!=pid) return KEDIT_OS_ERROR;
						else	if (status!=0)
								{
										/* NN returns 1 - try again without -remote */
										pid=fork();
										if (pid==0)
												{
														execl(externalBrowser.data(), externalBrowser.data(), fileName.data(), 0);
														_exit(EXIT_FAILURE);
												}
										else if (pid<0) return KEDIT_OS_ERROR; /* fork error */
								}
				};
		return KEDIT_OK;
}

void HTMLEdit::setExternalBrowser(const char *browser)
{
		externalBrowser=browser;
}

void HTMLEdit::setProjectRoot()
{
		QFileInfo file(getName());
		basePath=file.dirPath(TRUE);
		QDir::setCurrent(basePath);
}

QString HTMLEdit::projectRoot()
{
		return basePath;
}

/*--------------------------------------TableTemplateDialog------------------------------------------*/

TableTemplateDialog::TableTemplateDialog(QWidget *parent, const char *name):QDialog(parent, name, TRUE)
{
		QLabel *label;
		QVBoxLayout *vbox=new QVBoxLayout(this, 5);
	
		tableCaption=new QLineEdit(this);
		tableCaption->setMinimumSize(tableCaption->sizeHint());
		label=new QLabel(tableCaption, i18n("C&aption"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(tableCaption);
	
		QGridLayout *grid=new QGridLayout(2, 4);
		vbox->addLayout(grid);
	
		headHeight=new QLineEdit(this);
		headHeight->setMinimumSize(headHeight->sizeHint());
		label=new QLabel(headHeight, i18n("Head rows"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(headHeight, 0, 1);
	
		headWidth=new QLineEdit(this);
		headWidth->setMinimumSize(headWidth->sizeHint());
		label=new QLabel(headWidth, i18n("Head columns"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(headWidth, 0, 3);
	
		height=new QLineEdit(this);
		height->setText("2");
		height->setMinimumSize(height->sizeHint());
		label=new QLabel(height, i18n("&Rows"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(height, 1, 1);
	
		width=new QLineEdit(this);
		width->setText("2");
		width->setMinimumSize(width->sizeHint());
		label=new QLabel(width, i18n("Co&lumns"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(width, 1, 3);
	
		QHBoxLayout *hbox=new QHBoxLayout();
		vbox->addLayout(hbox);
	
		hbox->addStretch(1);
		QPushButton *ok=new QPushButton(i18n("&OK"), this);
		ok->setDefault(TRUE);
		ok->setMinimumSize(ok->sizeHint());
		hbox->addWidget(ok);
		QPushButton *cancel=new QPushButton(i18n("&Cancel"), this);
		cancel->setMinimumSize(cancel->sizeHint());
		hbox->addWidget(cancel);
		vbox->activate();
		vbox->freeze();
	
		setCaption(i18n("Table Template"));
	
		connect(ok, SIGNAL(clicked()), this, SLOT(accept()));
		connect(cancel, SIGNAL(clicked()), this, SLOT(reject()));
}

QString TableTemplateDialog::getCaption()
{
		return tableCaption->text();
}

int TableTemplateDialog::getColumns()
{
		bool ok=TRUE;
		QString s(width->text());
		int cols=s.toInt(&ok);
		if (ok) return cols; else return 0;
}

int TableTemplateDialog::getRows()
{
		bool ok=TRUE;
		QString s(height->text());
		int rows=s.toInt(&ok);
		if (ok) return rows; else return 0;
}

int TableTemplateDialog::getHeaderColumns()
{
		bool ok=TRUE;
		QString s(headWidth->text());
		int cols=s.toInt(&ok);
		if (ok) return cols; else return 0;
}

int TableTemplateDialog::getHeaderRows()
{
		bool ok=TRUE;
		QString s(headHeight->text());
		int rows=s.toInt(&ok);
		if (ok) return rows; else return 0;
}

/*--------------------------------------TagChooserDialog------------------------------------------*/

TagChooserDialog::TagChooserDialog(QWidget *parent, const char *name):QDialog(parent, name, FALSE)
{
		QVBoxLayout *vbox=new QVBoxLayout(this, 5);
	
		QGridLayout *grid=new QGridLayout(2, 3, 0);
		vbox->addLayout(grid, 1);
	
		QListBox *groupList=new QListBox(this);
		for (int i=0; i<TAG_GROUPS_COUNT; i++) groupList->insertItem(TAG_GROUPS[i][0]);
		groupList->setCurrentItem(0);
		groupList->setMinimumSize(groupList->sizeHint().width(), 120);
		grid->addWidget(groupList, 1, 0);
		tagList=new QListBox(this);
		for (int i=1; i<TAG_GROUPS_SIZE[0]; i++) tagList->insertItem(TAG_GROUPS[0][i]);
		tagList->setCurrentItem(0);
		tagList->setMinimumSize(tagList->sizeHint().width(), 120);
		grid->addWidget(tagList, 1, 2);
		QLabel *gl=new QLabel(groupList, i18n("Tag &group"), this);
		gl->setMargin(3);
		gl->setMinimumSize(gl->sizeHint());
		grid->addWidget(gl, 0, 0);
		QLabel *tl=new QLabel(tagList, i18n("Tag &name"), this);
		tl->setMargin(3);
		tl->setMinimumSize(tl->sizeHint());
		grid->addWidget(tl, 0, 2);
		grid->setColStretch(0, 1);
		grid->addColSpacing(1, 5);
		grid->setColStretch(2, 1);
		grid->setRowStretch(1, 1);
	
		QHBoxLayout *hbox=new QHBoxLayout();
		vbox->addLayout(hbox, 0);
	
		hbox->addStretch(1);
		QPushButton *ok=new QPushButton(i18n("&Insert Tag"), this);
		ok->setDefault(TRUE);
		ok->setMinimumSize(ok->sizeHint());
		hbox->addWidget(ok);
		QPushButton *cancel=new QPushButton(i18n("&Close"), this);
		cancel->setMinimumSize(cancel->sizeHint());
		hbox->addWidget(cancel);
	
		vbox->activate();
		setCaption(i18n("WebMaker: TagChooser"));
		connect(ok, SIGNAL(clicked()), this, SIGNAL(tagSelected()));
		connect(cancel, SIGNAL(clicked()), this, SLOT(hide()));
		connect(tagList, SIGNAL(selected(int)), this, SLOT(tagSelected(int)));
		connect(groupList, SIGNAL(highlighted(int)), this, SLOT(groupHighlighted(int)));
}

const char* TagChooserDialog::getTag()
{
		return tagList->text(tagList->currentItem());
}

void TagChooserDialog::groupHighlighted(int index)
{
		tagList->setAutoUpdate(FALSE);
		tagList->clear();
		for (int i=1; i<TAG_GROUPS_SIZE[index]; i++) tagList->insertItem(TAG_GROUPS[index][i]);
		tagList->setCurrentItem(0);
		tagList->setAutoUpdate(TRUE);
		tagList->update();
}

void TagChooserDialog::tagSelected(int)
{
		emit tagSelected();
}

/*-----------------------------------------------TagEditDialog-------------------------------------------------*/

AttributesTab::AttributesTab(QWidget *parent, const char *name):QWidget(parent, name)
{
}

QString AttributesTab::getParameters()
{
		QObjectList *childs=queryList("QLineEdit");
		QObjectListIt childsIt(*childs);
		QString s;
		QObject *child;
		while ((child=childsIt.current())!=0)
				{
						if ((((QLineEdit*)child)->text()) && ((QLineEdit*)child)->text()[0])
								{
										s+=" ";
										s+=((QLineEdit*)child)->name();
										s+="=\"";
										s+=((QLineEdit*)child)->text();
										s+="\"";
								};
						++childsIt;
				};
		delete childs;
		return s;
}

void AttributesTab::setParameters(StrDict *parameters)
{
		QObjectList *childs=queryList("QLineEdit");
		QObjectListIt childsIt(*childs);
		QObject *child;
		while ((child=childsIt.current())!=0)
				{
						if (parameters->find(child->name()))
								{
										((QLineEdit*)child)->setText(parameters->find(child->name())->data());
										parameters->remove(child->name());
								};
						++childsIt;
				};
		delete childs;
}

CoreTab::CoreTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid = new QGridLayout(this, 5, 2, 5);
		QLabel *label;
		QLineEdit *lineEdit;
	
		lineEdit=new QLineEdit(this, "id");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Id"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "class");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Class"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "style");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Style"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		lineEdit=new QLineEdit(this, "title");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Title"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		grid->setRowStretch(4, 1);
		grid->setColStretch(1, 1);
		grid->activate();
}

EventsTab::EventsTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid = new QGridLayout(this, EVENTS_ENTITY_SIZE+1, 2, 5);
		QLabel *label;
		QLineEdit *lineEdit;
		for (int i=0; i<EVENTS_ENTITY_SIZE; i++)
				{
						lineEdit=new QLineEdit(this, EVENTS_ENTITY[i]);
						lineEdit->setMinimumSize(lineEdit->sizeHint());
						grid->addWidget(lineEdit, i, 1);
						label=new QLabel(lineEdit, i18n(EVENTS_ENTITY_NAME[i]), this);
						//   label->setMargin(3);
						label->setMinimumSize(label->sizeHint());
						grid->addWidget(label, i, 0);
				};
		grid->setRowStretch(EVENTS_ENTITY_SIZE, 1);
		grid->setColStretch(1, 1);
		grid->activate();
}

I18nTab::I18nTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid = new QGridLayout(this, 3, 2, 5);
		QLabel *label;
		lang=new QLineEdit(this, "lang");
		lang->setMinimumSize(lang->sizeHint());
		label=new QLabel(lang, i18n("Language"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lang, 0, 1);
	
		dir=new QComboBox(FALSE, this, "dir");
		dir->insertItem(i18n("Implied"));
		dir->insertItem(i18n("Left to Right"));
		dir->insertItem(i18n("Right to Left"));
		dir->setMinimumSize(dir->sizeHint());
		label=new QLabel(dir, i18n("Text direction"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(dir, 1, 1);
	
		grid->setRowStretch(2, 1);
		grid->setColStretch(1, 1);
		grid->activate();
}

QString I18nTab::getParameters()
{
		QString s;
		if ((lang->text()) && (lang->text()[0]))
				{
						s+=" lang=\"";
						s+=lang->text();
						s+="\"";
				};
		if (dir->currentItem())
				switch (dir->currentItem())
						{
						case 1: s+=" dir=\"ltr\"";break;
						case 2: s+=" dir=\"rtl\"";break;
						};
		return s;
}

void I18nTab::setParameters(StrDict *parameters)
{
		if (parameters->find("lang"))
				{
						lang->setText(parameters->find("lang")->data());
						parameters->remove("lang");
				};
		if (parameters->find("dir"))
				{
						if (*(parameters->find("dir"))=="ltr")
								dir->setCurrentItem(1);
						else
								if (*(parameters->find("dir"))=="rtl")
										dir->setCurrentItem(2);
								else
										dir->setCurrentItem(0);
						parameters->remove("dir");
				};
}

ImageTab::ImageTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QHBoxLayout *hbox = new QHBoxLayout(this, 5);
		QLabel *label;
		QLineEdit *lineEdit;
	
		QGridLayout *grid=new QGridLayout(15, 3, 5);
		hbox->addLayout(grid, 1);
	
		imgSrc=new QLineEdit(this, "src");
		imgSrc->setMinimumSize(imgSrc->sizeHint());
		imgSrc->installEventFilter(this);
		label=new QLabel(imgSrc, i18n("Image Source"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *imgSrcButton=new QPushButton(this);
		imgSrcButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		imgSrcButton->setMinimumSize(imgSrcButton->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(imgSrc, 1, 1, 0, 1);
		grid->addWidget(imgSrcButton, 1, 2);
	
		imgWidth=new QLineEdit(this, "width");
		imgWidth->setMinimumSize(imgWidth->sizeHint());
		label=new QLabel(imgWidth, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(imgWidth, 3, 0);
		imgHeight=new QLineEdit(this, "height");
		imgHeight->setMinimumSize(imgHeight->sizeHint());
		label=new QLabel(imgHeight, i18n("Height"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *recalcSize=new QPushButton(this);
		recalcSize->setPixmap(loader->loadIcon("reload.xpm"));
		recalcSize->setMinimumSize(recalcSize->sizeHint());
		grid->addWidget(label, 2, 1);
		grid->addWidget(imgHeight, 3, 1);
		grid->addWidget(recalcSize, 3, 2);
	
		lineEdit=new QLineEdit(this, "hspace");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("HSpace"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(lineEdit, 5, 0);
		lineEdit=new QLineEdit(this, "vspace");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("VSpace"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 1);
		grid->addWidget(lineEdit, 5, 1);
	
		lineEdit=new QLineEdit(this, "alt");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Alternate Text"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addMultiCellWidget(lineEdit, 7, 7, 0, 1);
	
		longDesc=new QLineEdit(this, "longdesc");
		longDesc->setMinimumSize(longDesc->sizeHint());
		label=new QLabel(longDesc, i18n("Link to Long Description"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *longDescButton=new QPushButton(this);
		longDescButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		longDescButton->setMinimumSize(longDescButton->sizeHint());
		grid->addWidget(label, 8, 0);
		grid->addMultiCellWidget(longDesc, 9, 9, 0, 1);
		grid->addWidget(longDescButton, 9, 2);
	
		lineEdit=new QLineEdit(this, "border");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Border"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 10, 0);
		grid->addWidget(lineEdit, 11, 0);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("aligntop.xpm"));
		align->insertItem(loader->loadIcon("alignmiddle.xpm"));
		align->insertItem(loader->loadIcon("alignbottom.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 10, 1);
		grid->addWidget(align, 11, 1);
	
		lineEdit=new QLineEdit(this, "usemap");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Image Map"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 12, 0);
		grid->addWidget(lineEdit, 13, 0);
		ismap=new QCheckBox("IsMap", this, "ismap");
		ismap->setMinimumSize(ismap->sizeHint());
		grid->addWidget(ismap, 13, 1);
	
		grid->setColStretch(0, 1);
		grid->setColStretch(1, 1);
		grid->setRowStretch(14, 1);
	
		QVBoxLayout *vbox=new QVBoxLayout();
		hbox->addLayout(vbox);
	
		label=new QLabel(lineEdit, i18n(" "), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		QPixmap pixmap(150, 150);
		QColorGroup colors=colorGroup();
		pixmap.fill(colors.background());
		preview=new QLabel(this);
		preview->setPixmap(pixmap);
		preview->setLineWidth(2);
		preview->setFrameStyle(QFrame::Panel | QFrame::Sunken);
		preview->setAlignment(AlignCenter);
		preview->setFixedSize(preview->sizeHint());
		vbox->addWidget(preview);
	
		vbox->addStretch(1);
	
		hbox->activate();
	
		connect(imgSrcButton, SIGNAL(clicked()), this, SLOT(imgSrcClicked()));
		connect(recalcSize, SIGNAL(clicked()), this, SLOT(recalcSizeClicked()));
		connect(longDescButton, SIGNAL(clicked()), this, SLOT(longDescClicked()));
	
		basePath=baseDir;
}

QString ImageTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"right\"";break;
						case 3: s+=" align=\"top\"";break;
						case 4: s+=" align=\"middle\"";break;
						case 5: s+=" align=\"bottom\"";break;
						};
		if (ismap->isChecked()) s+=" ismap";
		return s;
}

void ImageTab::setParameters(StrDict *parameters)
{
		if (parameters->find("src"))
				{
						QString fileName=(parameters->find("src"))->data();
						if (fileName.isEmpty()) return;
						QString oldPath=QDir::currentDirPath();
						QDir::setCurrent(basePath);
						QImage image(fileName);
						QDir::setCurrent(oldPath);
						if (!image.isNull())
								{
										float scaleFactor=(image.width()>image.height())?150.0/image.width():150.0/image.height();
										if (scaleFactor<1) image=image.smoothScale(int(image.width()*scaleFactor), int(image.height()*scaleFactor));
										QPixmap pixmap;
										pixmap.convertFromImage(image);
										preview->setPixmap(pixmap);
								};
				};
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="right")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="top")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="middle")
														align->setCurrentItem(4);
												else
														if (*(parameters->find("align"))=="bottom")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("ismap"))
				{
						ismap->setChecked(TRUE);
						parameters->remove("ismap");
				};
		AttributesTab::setParameters(parameters);
}

bool ImageTab::eventFilter(QObject*, QEvent *ev)
{
		if (ev->type() == Event_FocusOut) 
				{
						QString fileName=imgSrc->text();
						if (fileName.isEmpty()) return FALSE;
						QString oldPath=QDir::currentDirPath();
						QDir::setCurrent(basePath);
						QImage image(fileName);
						QDir::setCurrent(oldPath);
						if (image.isNull()) return FALSE;
						QString width, height;
						width.setNum(image.width());
						height.setNum(image.height());
						float scaleFactor=(image.width()>image.height())?150.0/image.width():150.0/image.height();
						if (scaleFactor<1) image=image.smoothScale(int(image.width()*scaleFactor), int(image.height()*scaleFactor));
						QPixmap pixmap;
						pixmap.convertFromImage(image);
						preview->setPixmap(pixmap);
						imgSrc->setText(fileName);
						imgWidth->setText(width);
						imgHeight->setText(height);
				};
		return FALSE; 
}

void ImageTab::imgSrcClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.gif *.jpg *.png|Image Files (*.gif *.jpg *.png)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QString oldPath=QDir::currentDirPath();
		QDir::setCurrent(basePath);
		QImage image(fileName);
		QDir::setCurrent(oldPath);
		if (image.isNull())
				{
						fileName="Can't open file \"" + fileName + "\"!";
						KMsgBox::message(this, "Error!", fileName, KMsgBox::EXCLAMATION);
						return;
				};
		QString width, height;
		width.setNum(image.width());
		height.setNum(image.height());
		float scaleFactor=(image.width()>image.height())?150.0/image.width():150.0/image.height();
		if (scaleFactor<1) image=image.smoothScale(int(image.width()*scaleFactor), int(image.height()*scaleFactor));
		QPixmap pixmap;
		pixmap.convertFromImage(image);
		preview->setPixmap(pixmap);
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		imgSrc->setText(fileName);
		imgWidth->setText(width);
		imgHeight->setText(height);
}

void ImageTab::recalcSizeClicked()
{
		QString fileName=imgSrc->text();
		if (fileName.isEmpty()) return;
		QString oldPath=QDir::currentDirPath();
		QDir::setCurrent(basePath);
		QImage image(fileName);
		QDir::setCurrent(oldPath);
		if (image.isNull()) return;
		QString width, height;
		width.setNum(image.width());
		height.setNum(image.height());
		imgWidth->setText(width);
		imgHeight->setText(height);
}

void ImageTab::longDescClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		longDesc->setText(fileName);
}

DocTypeTab::DocTypeTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QVBoxLayout *vbox=new QVBoxLayout(this, 5);
		dtd=new QComboBox(TRUE, this);
		dtd->insertItem("HTML 4.0 Transitional");
		dtd->insertItem("HTML 4.0 Frameset");
		dtd->insertItem("HTML 4.0 Strict");
		dtd->setMinimumSize(dtd->sizeHint());
		QLabel *label=new QLabel(dtd, i18n("HTML version"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(dtd);
		vbox->addStretch(1);
		vbox->activate();
}

QString DocTypeTab::getParameters()
{
		QString s(dtd->currentText());
		if (s=="HTML 4.0 Transitional") s="HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\" \"http://www.w3.org/TR/REC-html40/loose.dtd\"";
		else
				if (s=="HTML 4.0 Frameset") s="HTML PUBLIC \"-//W3C//DTD HTML 4.0 Frameset//EN\" \"http://www.w3.org/TR/REC-html40/frameset.dtd\"";
				else
						if (s=="HTML 4.0 Strict") s="HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\"";
		s.prepend(" ");
		return s; 
}

void DocTypeTab::setParameters(StrDict *parameters)
{
		parameters->clear();
}

MetaTab::MetaTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLineEdit *lineEdit;
		QLabel *label;
		QVBoxLayout *vbox=new QVBoxLayout(this, 5);
	
		lineEdit=new QLineEdit(this, "http-equiv");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Http-Equiv"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(lineEdit);
	
		lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(lineEdit);
	
		lineEdit=new QLineEdit(this, "content");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Content"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(lineEdit);
	
		lineEdit=new QLineEdit(this, "scheme");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Scheme"), this);
		label->setMinimumSize(label->sizeHint());
		vbox->addWidget(label);
		vbox->addWidget(lineEdit);
	
		vbox->addStretch(1);
		vbox->activate();
}

AlignTab::AlignTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 2, 2, 5);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("aligncenter.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("alignjustify.xpm"));
		align->setMinimumSize(align->sizeHint());
		QLabel *label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(align, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

QString AlignTab::getParameters()
{
		QString s;
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"center\"";break;
						case 3: s+=" align=\"right\"";break;
						case 4: s+=" align=\"justify\"";break;
						};
		return s;
}

void AlignTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="center")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="right")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="justify")
														align->setCurrentItem(4);
												else
														align->setCurrentItem(0);
						parameters->remove("align");
				};
}

FrameSetTab::FrameSetTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLineEdit *lineEdit;
		QLabel *label;
		QGridLayout *grid=new QGridLayout(this, 3, 4, 5);
	
		lineEdit=new QLineEdit(this, "rows");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Rows"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "cols");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Columns"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		lineEdit=new QLineEdit(this, "onload");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Load"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "onunload");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Unload"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(lineEdit, 1, 3);
	
		grid->setRowStretch(2, 1);
		grid->activate();
}

LabelTab::LabelTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLineEdit *lineEdit;
		QLabel *label;
		QGridLayout *grid=new QGridLayout(this, 4, 4, 5);
	
		lineEdit=new QLineEdit(this, "for");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("For"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "accesskey");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Access Key"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(lineEdit, 1, 1, 1, 3);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		grid->setRowStretch(3, 1);
		grid->activate();
}

BodyTab::BodyTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLineEdit *lineEdit;
		QLabel *label;
		QPushButton *button;
		KIconLoader *loader=kapp->getIconLoader();
	
		palette[BGCOLOR]=QColor(0xC0, 0xC0, 0xC0);
		palette[TEXT]=QColor(0, 0, 0);
		palette[LINK]=QColor(0, 0, 0xEE);
		palette[VLINK]=QColor(0x55, 0x1A, 0x8B);
		palette[ALINK]=QColor(0xEE, 0, 0);

		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 9, 4, 5);
	
		bgEdit=new QLineEdit(this, "background");
		bgEdit->setMinimumSize(bgEdit->sizeHint());
		label=new QLabel(bgEdit, i18n("Background Image"), this);
		label->setMinimumSize(label->sizeHint());
		button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(bgEdit, 0, 1);
		grid->addWidget(button, 0, 2);
		connect(button, SIGNAL(clicked()), this, SLOT(bgClicked()));
	
		bgcolorEdit=new QLineEdit(this, "bgcolor");
		bgcolorEdit->setMinimumSize(bgcolorEdit->sizeHint());
		label=new QLabel(bgcolorEdit, i18n("Background Color"), this);
		label->setMinimumSize(label->sizeHint());
		bgcolorButton=new KColorButton(palette[BGCOLOR], this);
		bgcolorButton->setMinimumSize(bgcolorButton->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(bgcolorEdit, 1, 1);
		grid->addWidget(bgcolorButton, 1, 2);
		
		textEdit=new QLineEdit(this, "text");
		textEdit->setMinimumSize(textEdit->sizeHint());
		label=new QLabel(textEdit, i18n("Text Color"), this);
		label->setMinimumSize(label->sizeHint());
		textButton=new KColorButton(palette[TEXT], this);
		textButton->setMinimumSize(textButton->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(textEdit, 2, 1);
		grid->addWidget(textButton, 2, 2);
	
		linkEdit=new QLineEdit(this, "link");
		linkEdit->setMinimumSize(linkEdit->sizeHint());
		label=new QLabel(linkEdit, i18n("Link Color"), this);
		label->setMinimumSize(label->sizeHint());
		linkButton=new KColorButton(palette[LINK], this);
		linkButton->setMinimumSize(linkButton->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(linkEdit, 3, 1);
		grid->addWidget(linkButton, 3, 2);
	
		vlinkEdit=new QLineEdit(this, "vlink");
		vlinkEdit->setMinimumSize(vlinkEdit->sizeHint());
		label=new QLabel(vlinkEdit, i18n("Visited Link Color"), this);
		label->setMinimumSize(label->sizeHint());
		vlinkButton=new KColorButton(palette[VLINK], this);
		vlinkButton->setMinimumSize(vlinkButton->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(vlinkEdit, 4, 1);
		grid->addWidget(vlinkButton, 4, 2);
	
		alinkEdit=new QLineEdit(this, "alink");
		alinkEdit->setMinimumSize(alinkEdit->sizeHint());
		label=new QLabel(alinkEdit, i18n("Active Link Color"), this);
		label->setMinimumSize(label->sizeHint());
		alinkButton=new KColorButton(palette[ALINK], this);
		alinkButton->setMinimumSize(alinkButton->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(alinkEdit, 5, 1);
		grid->addWidget(alinkButton, 5, 2);
	
		lineEdit=new QLineEdit(this, "onload");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Load"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addMultiCellWidget(lineEdit, 6, 6, 1, 2);
	
		lineEdit=new QLineEdit(this, "onunload");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Unload"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 0);
		grid->addMultiCellWidget(lineEdit, 7, 7, 1, 2);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(8, 1);
		grid->activate();
	
		connect(bgcolorButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		connect(textButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		connect(linkButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		connect(vlinkButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		connect(alinkButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
	
		bgcolorEdit->installEventFilter(this);
		textEdit->installEventFilter(this);
		linkEdit->installEventFilter(this);
		vlinkEdit->installEventFilter(this);
		alinkEdit->installEventFilter(this);
}

bool BodyTab::eventFilter(QObject *obj, QEvent *ev)
{
		if (ev->type() == Event_FocusOut) 
				{
						QString s=((QLineEdit*)obj)->text();
						if (s.isEmpty()) return FALSE;
						QColor color;
						color.setNamedColor(s);
						if (obj==bgcolorEdit)
								{
										bgcolorButton->setColor(color);
										return FALSE;
								};
						if (obj==textEdit)
								{
										textButton->setColor(color);
										return FALSE;
								};
						if (obj==linkEdit)
								{
										linkButton->setColor(color);
										return FALSE;
								};
						if (obj==vlinkEdit)
								{
										vlinkButton->setColor(color);
										return FALSE;
								};
						if (obj==alinkEdit)
								alinkButton->setColor(color);
				};
		return FALSE;
}

void BodyTab::setParameters(StrDict *parameters)
{
		if (parameters->find("bgcolor"))
				{
						bgcolorButton->setColor(QColor(parameters->find("bgcolor")->data()));
						palette[BGCOLOR]=QColor(parameters->find("bgcolor")->data());
				};
		if (parameters->find("text"))
				{
						textButton->setColor(QColor(parameters->find("text")->data()));
						palette[TEXT]=QColor(parameters->find("text")->data());
				};
		if (parameters->find("link"))
				{
						linkButton->setColor(QColor(parameters->find("link")->data()));
						palette[LINK]=QColor(parameters->find("link")->data());
				};
		if (parameters->find("vlink"))
				{
						vlinkButton->setColor(QColor(parameters->find("vlink")->data()));
						palette[VLINK]=QColor(parameters->find("vlink")->data());
				};
		if (parameters->find("alink"))
				{
						alinkButton->setColor(QColor(parameters->find("alink")->data()));
						palette[ALINK]=QColor(parameters->find("alink")->data());
				};
		AttributesTab::setParameters(parameters);
}

void BodyTab::bgClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.gif *.jpg *.png|Image Files (*.gif *.jpg *.png)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		bgEdit->setText(fileName);
}

void BodyTab::updateEdit(QLineEdit *edit, KColorButton *button)
{
		QColor color=button->color();
		QString s;
		s.sprintf("#%02X%02X%02X", color.red(), color.green(), color.blue());
		edit->setText(s);
}

void BodyTab::colorChanged(const QColor&)
{
		if (palette[BGCOLOR]!=bgcolorButton->color())
				{
						updateEdit(bgcolorEdit, bgcolorButton);
						palette[BGCOLOR]=bgcolorButton->color();
						return;
				}
		if (palette[TEXT]!=textButton->color())
				{
						updateEdit(textEdit, textButton);
						palette[TEXT]=textButton->color();
						return;
				}
		if (palette[LINK]!=linkButton->color())
				{
						updateEdit(linkEdit, linkButton);
						palette[LINK]=linkButton->color();
						return;
				}
		if (palette[VLINK]!=vlinkButton->color())
				{
						updateEdit(vlinkEdit, vlinkButton);
						palette[VLINK]=vlinkButton->color();
						return;
				}
		if (palette[ALINK]!=alinkButton->color())
				{
						updateEdit(alinkEdit, alinkButton);
						palette[ALINK]=alinkButton->color();
						return;
				}
}

QuoteTab::QuoteTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 2, 3, 5);
	
		cite=new QLineEdit(this, "cite");
		cite->setMinimumSize(cite->sizeHint());
		QLabel *label=new QLabel(cite, i18n("Cite"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(cite, 0, 1);
		grid->addWidget(button, 0, 2);
		connect(button, SIGNAL(clicked()), this, SLOT(citeClicked()));
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

void QuoteTab::citeClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		cite->setText(fileName);
}

BRTab::BRTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 2, 2, 5);
	
		clear=new QComboBox(FALSE, this, "clear");
		clear->insertItem(i18n("None"));
		clear->insertItem(loader->loadIcon("alignleft.xpm"));
		clear->insertItem(loader->loadIcon("alignright.xpm"));
		clear->insertItem(loader->loadIcon("alignjustify.xpm"));
		clear->setMinimumSize(clear->sizeHint());
		QLabel *label=new QLabel(clear, i18n("Clear"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(clear, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

QString BRTab::getParameters()
{
		QString s;
		if (clear->currentItem())
				switch (clear->currentItem())
						{
						case 1: s+=" clear=\"left\"";break;
						case 2: s+=" clear=\"right\"";break;
						case 3: s+=" clear=\"all\"";break;
						};
		return s;
}

void BRTab::setParameters(StrDict *parameters)
{
		if (parameters->find("clear"))
				{
						if (*(parameters->find("clear"))=="left")
								clear->setCurrentItem(1);
						else
								if (*(parameters->find("clear"))=="right")
										clear->setCurrentItem(2);
								else
										if (*(parameters->find("clear"))=="all")
												clear->setCurrentItem(3);
										else
												clear->setCurrentItem(0);
						parameters->remove("clear");
				};
}

PreTab::PreTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 2, 2, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

EditTab::EditTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 3, 3, 5);
	
		cite=new QLineEdit(this, "cite");
		cite->setMinimumSize(cite->sizeHint());
		QLabel *label=new QLabel(cite, i18n("Cite"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(cite, 0, 1);
		grid->addWidget(button, 0, 2);
		connect(button, SIGNAL(clicked()), this, SLOT(citeClicked()));
	
		QLineEdit *lineEdit=new QLineEdit(this, "datetime");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QDate date=QDate::currentDate();
		QTime time=QTime::currentTime();
		QString s;
		s.sprintf("%d-%d-%dT%s+03:00", date.year(), date.month(), date.day(),
												time.toString().data());
		lineEdit->setText(s);
		label=new QLabel(lineEdit, i18n("Date of change"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(2, 1);
		grid->activate();
}

void EditTab::citeClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		cite->setText(fileName);
}

ULTab::ULTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 3, 2, 5);
	
		type=new QComboBox(FALSE, this, "type");
		type->insertItem(i18n("Implied"));
		type->insertItem(loader->loadIcon("disc.xpm"));
		type->insertItem(loader->loadIcon("circle.xpm"));
		type->insertItem(loader->loadIcon("square.xpm"));
		type->setMinimumSize(type->sizeHint());
		QLabel *label=new QLabel(type, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(type, 0, 1);
	
		compact=new QCheckBox("Compact", this, "compact");
		compact->setMinimumSize(compact->sizeHint());
		grid->addWidget(compact, 1, 0);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(2, 1);
		grid->activate();
}

QString ULTab::getParameters()
{
		QString s;
		if (type->currentItem())
				switch (type->currentItem())
						{
						case 1: s+=" type=\"disc\"";break;
						case 2: s+=" type=\"circle\"";break;
						case 3: s+=" type=\"square\"";break;
						};
		if (compact->isChecked()) s+=" compact";
		return s;
}

void ULTab::setParameters(StrDict *parameters)
{
		if (parameters->find("type"))
				{
						if (*(parameters->find("type"))=="disc")
								type->setCurrentItem(1);
						else
								if (*(parameters->find("type"))=="circle")
										type->setCurrentItem(2);
								else
										if (*(parameters->find("type"))=="square")
												type->setCurrentItem(3);
										else
												type->setCurrentItem(0);
						parameters->remove("type");
				};
		if (parameters->find("compact"))
				{
						compact->setChecked(TRUE);
						parameters->remove("compact");
				};
}

OLTab::OLTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 4, 2, 5);
	
		type=new QComboBox(FALSE, this, "type");
		type->insertItem(i18n("Implied"));
		type->insertItem("1, 2, 3, ...");
		type->insertItem("a, b, c, ...");
		type->insertItem("A, B, C, ...");
		type->insertItem("i, ii, iii, ...");
		type->insertItem("I, II, III, ...");
		type->setMinimumSize(type->sizeHint());
		QLabel *label=new QLabel(type, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(type, 0, 1);
	
		start=new QLineEdit(this, "start");
		start->setMinimumSize(start->sizeHint());
		label=new QLabel(start, i18n("Start"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(start, 1, 1);

		compact=new QCheckBox("Compact", this, "compact");
		compact->setMinimumSize(compact->sizeHint());
		grid->addWidget(compact, 2, 0);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(3, 1);
		grid->activate();
}

QString OLTab::getParameters()
{
		QString s;
		if (type->currentItem())
				switch (type->currentItem())
						{
						case 1: s+=" type=\"1\"";break;
						case 2: s+=" type=\"a\"";break;
						case 3: s+=" type=\"A\"";break;
						case 4: s+=" type=\"i\"";break;
						case 5: s+=" type=\"I\"";break;
						};
		if (start->text() && start->text()[0])
				{
						s+=" start=\"";
						s+=start->text();
						s+="\"";
				};
		if (compact->isChecked()) s+=" compact";
		return s;
}

void OLTab::setParameters(StrDict *parameters)
{
		if (parameters->find("type"))
				{
						if (*(parameters->find("type"))=="1")
								type->setCurrentItem(1);
						else
								if (*(parameters->find("type"))=="a")
										type->setCurrentItem(2);
								else
										if (*(parameters->find("type"))=="A")
												type->setCurrentItem(3);
										else
												if (*(parameters->find("type"))=="i")
														type->setCurrentItem(4);
												else
														if (*(parameters->find("type"))=="I")
																type->setCurrentItem(5);
														else
																type->setCurrentItem(0);
						parameters->remove("type");
				};
		if (parameters->find("start"))
				{
						start->setText(parameters->find("start")->data());
						parameters->remove("start");
				};
		if (parameters->find("compact"))
				{
						compact->setChecked(TRUE);
						parameters->remove("compact");
				};
}

LITab::LITab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 3, 2, 5);
	
		type=new QComboBox(FALSE, this, "type");
		type->insertItem(i18n("Implied"));
		type->insertItem("1, 2, 3, ...");
		type->insertItem("a, b, c, ...");
		type->insertItem("A, B, C, ...");
		type->insertItem("i, ii, iii, ...");
		type->insertItem("I, II, III, ...");
		type->insertItem(loader->loadIcon("disc.xpm"));
		type->insertItem(loader->loadIcon("circle.xpm"));
		type->insertItem(loader->loadIcon("square.xpm"));
		type->setMinimumSize(type->sizeHint());
		QLabel *label=new QLabel(type, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(type, 0, 1);
	
		value=new QLineEdit(this, "value");
		value->setMinimumSize(value->sizeHint());
		label=new QLabel(value, i18n("Value"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(value, 1, 1);

		grid->setColStretch(1, 1);
		grid->setRowStretch(2, 1);
		grid->activate();
}

QString LITab::getParameters()
{
		QString s;
		if (type->currentItem())
				switch (type->currentItem())
						{
						case 1: s+=" type=\"1\"";break;
						case 2: s+=" type=\"a\"";break;
						case 3: s+=" type=\"A\"";break;
						case 4: s+=" type=\"i\"";break;
						case 5: s+=" type=\"I\"";break;
						case 6: s+=" type=\"disc\"";break;
						case 7: s+=" type=\"circle\"";break;
						case 8: s+=" type=\"square\"";break;
						};
		if (value->text() && value->text()[0])
				{
						s+=" value=\"";
						s+=value->text();
						s+="\"";
				};
		return s;
}

void LITab::setParameters(StrDict *parameters)
{
		if (parameters->find("type"))
				{
						QString &s=*(parameters->find("type"));
						if (s=="1")
								type->setCurrentItem(1);
						else
								if (s=="a")
										type->setCurrentItem(2);
								else
										if (s=="A")
												type->setCurrentItem(3);
										else
												if (s=="i")
														type->setCurrentItem(4);
												else
														if (s=="I")
																type->setCurrentItem(5);
														else
																if (s=="disc")
																		type->setCurrentItem(6);
																else
																		if (s=="circle")
																				type->setCurrentItem(7);
																		else
																				if (s=="square")
																						type->setCurrentItem(8);
																				else
																						type->setCurrentItem(0);
						parameters->remove("type");
				};
		if (parameters->find("value"))
				{
						value->setText(parameters->find("value")->data());
						parameters->remove("value");
				};
}

CompactTab::CompactTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 2, 2, 5);
	
		compact=new QCheckBox("Compact", this, "compact");
		compact->setMinimumSize(compact->sizeHint());
		grid->addWidget(compact, 0, 0);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

QString CompactTab::getParameters()
{
		QString s;
		if (compact->isChecked()) s+=" compact";
		return s;
}

void CompactTab::setParameters(StrDict *parameters)
{
		if (parameters->find("compact"))
				{
						compact->setChecked(TRUE);
						parameters->remove("compact");
				};
}

CaptionTab::CaptionTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 2, 2, 5);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("aligntop.xpm"));
		align->insertItem(loader->loadIcon("alignbottom.xpm"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->setMinimumSize(align->sizeHint());
		QLabel *label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(align, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

QString CaptionTab::getParameters()
{
		QString s;
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"top\"";break;
						case 2: s+=" align=\"bottom\"";break;
						case 3: s+=" align=\"left\"";break;
						case 4: s+=" align=\"right\"";break;
						};
		return s;
}

void CaptionTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="top")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="bottom")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="left")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="right")
														align->setCurrentItem(4);
												else
														align->setCurrentItem(0);
						parameters->remove("align");
				};
}

BaseTab::BaseTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 3, 3, 5);
	
		href=new QLineEdit(this, "href");
		href->setMinimumSize(href->sizeHint());
		QLabel *label=new QLabel(href, i18n("HREF"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(href, 0, 1);
		grid->addWidget(button, 0, 2);
		connect(button, SIGNAL(clicked()), this, SLOT(hrefClicked()));
	
		QLineEdit *lineEdit=new QLineEdit(this, "target");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Target"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(2, 1);
		grid->activate();
}

void BaseTab::hrefClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		href->setText(fileName);
}

MapTab::MapTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 2, 2, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

StyleTab::StyleTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 4, 2, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "type");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "media");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Media"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "title");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Title"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(3, 1);
		grid->activate();
}

HRTab::HRTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 5, 2, 5);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("aligncenter.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->setMinimumSize(align->sizeHint());
		QLabel *label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(align, 0, 1);

		QLineEdit *lineEdit=new QLineEdit(this, "size");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Size"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		noshade=new QCheckBox("No Shade", this, "noshade");
		noshade->setMinimumSize(noshade->sizeHint());
		grid->addWidget(noshade, 3, 0);

		grid->setColStretch(1, 1);
		grid->setRowStretch(4, 1);
		grid->activate();
}

QString HRTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"center\"";break;
						case 3: s+=" align=\"right\"";break;
						};
		if (noshade->isChecked()) s+=" noshade";
		return s;
}

void HRTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="center")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="right")
												align->setCurrentItem(3);
										else
												align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("noshade"))
				{
						noshade->setChecked(TRUE);
						parameters->remove("noshade");
				};
		AttributesTab::setParameters(parameters);
}

OptGroupTab::OptGroupTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 6, 2, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "label");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Label"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		lineEdit=new QLineEdit(this, "onchange");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Change"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		disabled=new QCheckBox("Disabled", this, "disabled");
		disabled->setMinimumSize(disabled->sizeHint());
		grid->addWidget(disabled, 4, 0);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(5, 1);
		grid->activate();
}

QString OptGroupTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (disabled->isChecked()) s+=" disabled";
		return s;
}

void OptGroupTab::setParameters(StrDict *parameters)
{
		if (parameters->find("disabled"))
				{
						disabled->setChecked(TRUE);
						parameters->remove("disabled");
				};
		AttributesTab::setParameters(parameters);
}

OptionTab::OptionTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 5, 2, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "label");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Label"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "value");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Value"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		selected=new QCheckBox("Selected", this, "selected");
		selected->setMinimumSize(selected->sizeHint());
		grid->addWidget(selected, 2, 0);
	
		disabled=new QCheckBox("Disabled", this, "disabled");
		disabled->setMinimumSize(disabled->sizeHint());
		grid->addWidget(disabled, 3, 0);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(4, 1);
		grid->activate();
}

QString OptionTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (selected->isChecked()) s+=" selected";
		if (disabled->isChecked()) s+=" disabled";
		return s;
}

void OptionTab::setParameters(StrDict *parameters)
{
		if (parameters->find("selected"))
				{
						selected->setChecked(TRUE);
						parameters->remove("selected");
				};
		if (parameters->find("disabled"))
				{
						disabled->setChecked(TRUE);
						parameters->remove("disabled");
				};
		AttributesTab::setParameters(parameters);
}

IsIndexTab::IsIndexTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 2, 2, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "prompt");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Prompt"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

LegendTab::LegendTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 2, 4, 5);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("aligntop.xpm"));
		align->insertItem(loader->loadIcon("alignbottom.xpm"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->setMinimumSize(align->sizeHint());
		QLabel *label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(align, 0, 1);
	
		QLineEdit *lineEdit=new QLineEdit(this, "accesskey");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Access Key"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

QString LegendTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"top\"";break;
						case 2: s+=" align=\"bottom\"";break;
						case 3: s+=" align=\"left\"";break;
						case 4: s+=" align=\"right\"";break;
						};
		return s;
}

void LegendTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="top")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="bottom")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="left")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="right")
														align->setCurrentItem(4);
												else
														align->setCurrentItem(0);
						parameters->remove("align");
				};
		AttributesTab::setParameters(parameters);
}

ScriptTab::ScriptTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 6, 3, 5);
	
		src=new QLineEdit(this, "src");
		src->setMinimumSize(src->sizeHint());
		QLabel *label=new QLabel(src, i18n("Script Source"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(src, 0, 1);
		grid->addWidget(button, 0, 2);
		connect(button, SIGNAL(clicked()), this, SLOT(srcClicked()));
	
		QLineEdit *lineEdit=new QLineEdit(this, "type");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Content Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "charset");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Character Set"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		lineEdit=new QLineEdit(this, "language");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Scripting Language"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		defer=new QCheckBox("Defer", this, "defer");
		defer->setMinimumSize(defer->sizeHint());
		grid->addWidget(defer, 4, 0);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(5, 1);
		grid->activate();
}

QString ScriptTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (defer->isChecked()) s+=" defer";
		return s;
}

void ScriptTab::setParameters(StrDict *parameters)
{
		if (parameters->find("defer"))
				{
						defer->setChecked(TRUE);
						parameters->remove("defer");
				};
		AttributesTab::setParameters(parameters);
}

void ScriptTab::srcClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.js|Javascript Files (*.js)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		src->setText(fileName);
}

ATab::ATab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 11, 5, 5);
	
		href=new QLineEdit(this, "href");
		href->setMinimumSize(href->sizeHint());
		QLabel *label=new QLabel(href, i18n("HREF"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(href, 0, 0, 1, 3);
		grid->addWidget(button, 0, 4);
		connect(button, SIGNAL(clicked()), this, SLOT(hrefClicked()));
	
		QLineEdit *lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(lineEdit, 1, 1, 1, 3);
	
		lineEdit=new QLineEdit(this, "target");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Target"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		lineEdit=new QLineEdit(this, "rel");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Rel"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		lineEdit=new QLineEdit(this, "rev");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Rev"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 2);
		grid->addWidget(lineEdit, 3, 3);
	
		lineEdit=new QLineEdit(this, "accesskey");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Access Key"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(lineEdit, 4, 1);
	
		lineEdit=new QLineEdit(this, "tabindex");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Tab Index"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 2);
		grid->addWidget(lineEdit, 4, 3);

		lineEdit=new QLineEdit(this, "type");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(lineEdit, 5, 1);
	
		lineEdit=new QLineEdit(this, "hreflang");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("HREF Language"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 2);
		grid->addWidget(lineEdit, 5, 3);
	
		lineEdit=new QLineEdit(this, "charset");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Character Set"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addMultiCellWidget(lineEdit, 6, 6, 1, 3);

		shape=new QComboBox(FALSE, this, "shape");
		shape->insertItem(i18n("Implied"));
		shape->insertItem(loader->loadIcon("square.xpm"));
		shape->insertItem(loader->loadIcon("circle.xpm"));
		shape->insertItem(loader->loadIcon("poly.xpm"));
		shape->setMinimumSize(shape->sizeHint());
		label=new QLabel(shape, i18n("Shape"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 0);
		grid->addWidget(shape, 7, 1);
	
		lineEdit=new QLineEdit(this, "coords");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Coordinates"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 2);
		grid->addWidget(lineEdit, 7, 3);
	
		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 8, 0);
		grid->addMultiCellWidget(lineEdit, 8, 8, 1, 3);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 9, 0);
		grid->addMultiCellWidget(lineEdit, 9, 9, 1, 3);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(10, 1);
		grid->activate();
}

QString ATab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (shape->currentItem())
				switch (shape->currentItem())
						{
						case 1: s+=" shape=\"rect\"";break;
						case 2: s+=" shape=\"circle\"";break;
						case 3: s+=" shape=\"poly\"";break;
						};
		return s;
}

void ATab::setParameters(StrDict *parameters)
{
		if (parameters->find("shape"))
				{
						if (*(parameters->find("shape"))=="rect")
								shape->setCurrentItem(1);
						else
								if (*(parameters->find("shape"))=="circle")
										shape->setCurrentItem(2);
								else
										if (*(parameters->find("shape"))=="poly")
												shape->setCurrentItem(3);
										else
												shape->setCurrentItem(0);
						parameters->remove("shape");
				};
		AttributesTab::setParameters(parameters);
}

void ATab::hrefClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		href->setText(fileName);
}

TableTab::TableTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLabel *label;
		QLineEdit *lineEdit;
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 6, 5, 5);
		
		lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "border");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Border"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		frame=new QComboBox(FALSE, this, "frame");
		frame->insertItem(i18n("Implied"));
		frame->insertItem("void");
		frame->insertItem("above");
		frame->insertItem("below");
		frame->insertItem("hsides");
		frame->insertItem("vsides");
		frame->insertItem("lhs");
		frame->insertItem("rhs");
		frame->insertItem("box");
		frame->insertItem("border");
		frame->setMinimumSize(frame->sizeHint());
		label=new QLabel(frame, i18n("Frame"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(frame, 1, 1);

		rules=new QComboBox(FALSE, this, "rules");
		rules->insertItem(i18n("Implied"));
		rules->insertItem("none");
		rules->insertItem("groups");
		rules->insertItem("rows");
		rules->insertItem("cols");
		rules->insertItem("all");
		rules->setMinimumSize(rules->sizeHint());
		label=new QLabel(rules, i18n("Rules"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(rules, 1, 3);

		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("aligncenter.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(align, 2, 1);
	
		bgcolorEdit=new QLineEdit(this, "bgcolor");
		bgcolorEdit->setMinimumSize(bgcolorEdit->sizeHint());
		bgcolorEdit->installEventFilter(this);
		label=new QLabel(bgcolorEdit, i18n("Background Color"), this);
		label->setMinimumSize(label->sizeHint());
		bgcolorButton=new KColorButton(QColor(0xC0, 0xC0, 0xC0), this);
		bgcolorButton->setMinimumHeight(bgcolorButton->sizeHint().height());
		bgcolorButton->setMinimumWidth(bgcolorButton->sizeHint().width()/2);
		connect(bgcolorButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		grid->addWidget(label, 2, 2);
		grid->addWidget(bgcolorEdit, 2, 3);
		grid->addWidget(bgcolorButton, 2, 4);
		
		lineEdit=new QLineEdit(this, "cellspacing");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Cell Spacing"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		lineEdit=new QLineEdit(this, "cellpadding");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Cell Padding"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 2);
		grid->addWidget(lineEdit, 3, 3);
	
		lineEdit=new QLineEdit(this, "summary");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Summary"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addMultiCellWidget(lineEdit, 4, 4, 1, 3);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(5, 1);
		grid->activate();
}

QString TableTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (frame->currentItem())
				{
						s+=" frame=\"";
						s+=frame->currentText();
						s+="\"";
				};
		if (rules->currentItem())
				{
						s+=" rules=\"";
						s+=rules->currentText();
						s+="\"";
				};
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"center\"";break;
						case 3: s+=" align=\"right\"";break;
						};
		return s;
}

void TableTab::setParameters(StrDict *parameters)
{
		if (parameters->find("frame"))
				{
						QString s=*(parameters->find("frame"));
						for (int i=1; i<10; i++)
								if (s==frame->text(i))
										{
												frame->setCurrentItem(i);
												break;
										};
						parameters->remove("frame");
				};
		if (parameters->find("rules"))
				{
						QString s=*(parameters->find("rules"));
						for (int i=1; i<10; i++)
								if (s==rules->text(i))
										{
												rules->setCurrentItem(i);
												break;
										};
						parameters->remove("rules");
				};
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="center")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="right")
												align->setCurrentItem(3);
										else
												align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("bgcolor"))
				bgcolorButton->setColor(QColor(parameters->find("bgcolor")->data()));
		AttributesTab::setParameters(parameters);
}

bool TableTab::eventFilter(QObject *obj, QEvent *ev)
{
		if (obj==bgcolorEdit && ev->type()==Event_FocusOut)
				{
						QString s=bgcolorEdit->text();
						if (s.isEmpty()) return FALSE;
						QColor color;
						color.setNamedColor(s);
						bgcolorButton->setColor(color);
				};
		return FALSE;
}
		
void TableTab::colorChanged(const QColor &color)
{
		QString s;
		s.sprintf("#%02X%02X%02X", color.red(), color.green(), color.blue());
		bgcolorEdit->setText(s);
}

THeadTab::THeadTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLabel *label;
		QLineEdit *lineEdit;
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 3, 4, 5);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("aligncenter.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("alignjustify.xpm"));
		align->insertItem(loader->loadIcon("alignchar.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Horizontal Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(align, 0, 1);
	
		valign=new QComboBox(FALSE, this, "valign");
		valign->insertItem(i18n("Implied"));
		valign->insertItem(loader->loadIcon("aligntop.xpm"));
		valign->insertItem(loader->loadIcon("alignmiddle.xpm"));
		valign->insertItem(loader->loadIcon("alignbottom.xpm"));
		valign->insertItem(loader->loadIcon("alignbaseline.xpm"));
		valign->setMinimumSize(valign->sizeHint());
		label=new QLabel(valign, i18n("Vertical Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(valign, 0, 3);
	
		lineEdit=new QLineEdit(this, "char");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "charoff");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char Offset"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(lineEdit, 1, 3);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(2, 1);
		grid->activate();
}

QString THeadTab::getParameters()
{
		QString s;
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"center\"";break;
						case 3: s+=" align=\"right\"";break;
						case 4: s+=" align=\"justify\"";break;
						case 5: s+=" align=\"char\"";break;
						};
		if (valign->currentItem())
				switch (valign->currentItem())
						{
						case 1: s+=" valign=\"top\"";break;
						case 2: s+=" valign=\"middle\"";break;
						case 3: s+=" valign=\"bottom\"";break;
						case 4: s+=" valign=\"baseline\"";break;
						};
		s+=AttributesTab::getParameters();
		return s;
}

void THeadTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="center")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="right")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="justify")
														align->setCurrentItem(4);
						      else
														if (*(parameters->find("align"))=="char")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("valign"))
				{
						if (*(parameters->find("valign"))=="top")
								valign->setCurrentItem(1);
						else
								if (*(parameters->find("valign"))=="middle")
										valign->setCurrentItem(2);
								else
										if (*(parameters->find("valign"))=="bottom")
												valign->setCurrentItem(3);
										else
												if (*(parameters->find("valign"))=="baseline")
														valign->setCurrentItem(4);
												else
														valign->setCurrentItem(0);
						parameters->remove("valign");
				};
		AttributesTab::setParameters(parameters);
}

TRTab::TRTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLabel *label;
		QLineEdit *lineEdit;
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 4, 4, 5);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("aligncenter.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("alignjustify.xpm"));
		align->insertItem(loader->loadIcon("alignchar.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Horizontal Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(align, 0, 1);
	
		valign=new QComboBox(FALSE, this, "valign");
		valign->insertItem(i18n("Implied"));
		valign->insertItem(loader->loadIcon("aligntop.xpm"));
		valign->insertItem(loader->loadIcon("alignmiddle.xpm"));
		valign->insertItem(loader->loadIcon("alignbottom.xpm"));
		valign->insertItem(loader->loadIcon("alignbaseline.xpm"));
		valign->setMinimumSize(valign->sizeHint());
		label=new QLabel(valign, i18n("Vertical Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(valign, 0, 3);
	
		lineEdit=new QLineEdit(this, "char");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "charoff");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char Offset"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(lineEdit, 1, 3);
	
		bgcolorEdit=new QLineEdit(this, "bgcolor");
		bgcolorEdit->setMinimumSize(bgcolorEdit->sizeHint());
		bgcolorEdit->installEventFilter(this);
		label=new QLabel(bgcolorEdit, i18n("Background Color"), this);
		label->setMinimumSize(label->sizeHint());
		bgcolorButton=new KColorButton(QColor(0xC0, 0xC0, 0xC0), this);
		bgcolorButton->setMinimumSize(bgcolorButton->sizeHint());
		connect(bgcolorButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		grid->addWidget(label, 2, 0);
		grid->addWidget(bgcolorEdit, 2, 1);
		grid->addWidget(bgcolorButton, 2, 2);
		
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(3, 1);
		grid->activate();
}

QString TRTab::getParameters()
{
		QString s;
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"center\"";break;
						case 3: s+=" align=\"right\"";break;
						case 4: s+=" align=\"justify\"";break;
						case 5: s+=" align=\"char\"";break;
						};
		if (valign->currentItem())
				switch (valign->currentItem())
						{
						case 1: s+=" valign=\"top\"";break;
						case 2: s+=" valign=\"middle\"";break;
						case 3: s+=" valign=\"bottom\"";break;
						case 4: s+=" valign=\"baseline\"";break;
						};
		s+=AttributesTab::getParameters();
		return s;
}

void TRTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="center")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="right")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="justify")
														align->setCurrentItem(4);
						      else
														if (*(parameters->find("align"))=="char")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("valign"))
				{
						if (*(parameters->find("valign"))=="top")
								valign->setCurrentItem(1);
						else
								if (*(parameters->find("valign"))=="middle")
										valign->setCurrentItem(2);
								else
										if (*(parameters->find("valign"))=="bottom")
												valign->setCurrentItem(3);
										else
												if (*(parameters->find("valign"))=="baseline")
														valign->setCurrentItem(4);
												else
														valign->setCurrentItem(0);
						parameters->remove("valign");
				};
		if (parameters->find("bgcolor"))
				bgcolorButton->setColor(QColor(parameters->find("bgcolor")->data()));
		AttributesTab::setParameters(parameters);
}

bool TRTab::eventFilter(QObject *obj, QEvent *ev)
{
		if (obj==bgcolorEdit && ev->type()==Event_FocusOut)
				{
						QString s=bgcolorEdit->text();
						if (s.isEmpty()) return FALSE;
						QColor color;
						color.setNamedColor(s);
						bgcolorButton->setColor(color);
				};
		return FALSE;
}
		
void TRTab::colorChanged(const QColor &color)
{
		QString s;
		s.sprintf("#%02X%02X%02X", color.red(), color.green(), color.blue());
		bgcolorEdit->setText(s);
}

TDTab::TDTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLabel *label;
		QLineEdit *lineEdit;
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 8, 4, 5);
	
		lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "height");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Height"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		lineEdit=new QLineEdit(this, "rowspan");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Rows Spanned"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "colspan");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Columns Spanned"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(lineEdit, 1, 3);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("aligncenter.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("alignjustify.xpm"));
		align->insertItem(loader->loadIcon("alignchar.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Horizontal Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(align, 2, 1);
	
		valign=new QComboBox(FALSE, this, "valign");
		valign->insertItem(i18n("Implied"));
		valign->insertItem(loader->loadIcon("aligntop.xpm"));
		valign->insertItem(loader->loadIcon("alignmiddle.xpm"));
		valign->insertItem(loader->loadIcon("alignbottom.xpm"));
		valign->insertItem(loader->loadIcon("alignbaseline.xpm"));
		valign->setMinimumSize(valign->sizeHint());
		label=new QLabel(valign, i18n("Vertical Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 2);
		grid->addWidget(valign, 2, 3);
	
		lineEdit=new QLineEdit(this, "char");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		lineEdit=new QLineEdit(this, "charoff");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char Offset"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 2);
		grid->addWidget(lineEdit, 3, 3);
	
		lineEdit=new QLineEdit(this, "headers");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Headers"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(lineEdit, 4, 1);
	
		scope=new QComboBox(FALSE, this, "scope");
		scope->insertItem(i18n("Implied"));
		scope->insertItem(i18n("row"));
		scope->insertItem(i18n("col"));
		scope->insertItem(i18n("rowgroup"));
		scope->insertItem(i18n("colgroup"));
		scope->setMinimumSize(scope->sizeHint());
		label=new QLabel(scope, i18n("Scope"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 2);
		grid->addWidget(scope, 4, 3);
	
		lineEdit=new QLineEdit(this, "abbr");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Abbreviation"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(lineEdit, 5, 1);
	
		lineEdit=new QLineEdit(this, "axis");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Axis"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 2);
		grid->addWidget(lineEdit, 5, 3);
	
		bgcolorEdit=new QLineEdit(this, "bgcolor");
		bgcolorEdit->setMinimumSize(bgcolorEdit->sizeHint());
		bgcolorEdit->installEventFilter(this);
		label=new QLabel(bgcolorEdit, i18n("Background Color"), this);
		label->setMinimumSize(label->sizeHint());
		bgcolorButton=new KColorButton(QColor(0xC0, 0xC0, 0xC0), this);
		bgcolorButton->setMinimumSize(bgcolorButton->sizeHint());
		connect(bgcolorButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		grid->addWidget(label, 6, 0);
		grid->addWidget(bgcolorEdit, 6, 1);
		grid->addWidget(bgcolorButton, 6, 2);
		
		nowrap=new QCheckBox("No Wrap", this, "nowrap");
		nowrap->setMinimumSize(nowrap->sizeHint());
		grid->addWidget(nowrap, 6, 3);
		
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(7, 1);
		grid->activate();
}

QString TDTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"center\"";break;
						case 3: s+=" align=\"right\"";break;
						case 4: s+=" align=\"justify\"";break;
						case 5: s+=" align=\"char\"";break;
						};
		if (valign->currentItem())
				switch (valign->currentItem())
						{
						case 1: s+=" valign=\"top\"";break;
						case 2: s+=" valign=\"middle\"";break;
						case 3: s+=" valign=\"bottom\"";break;
						case 4: s+=" valign=\"baseline\"";break;
						};
		if (scope->currentItem())
				{
						s+=" scope=\"";
						s+=scope->currentText();
						s+="\"";
				};
		if (nowrap->isChecked()) s+=" nowrap";
		return s;
}

void TDTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="center")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="right")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="justify")
														align->setCurrentItem(4);
						      else
														if (*(parameters->find("align"))=="char")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("valign"))
				{
						if (*(parameters->find("valign"))=="top")
								valign->setCurrentItem(1);
						else
								if (*(parameters->find("valign"))=="middle")
										valign->setCurrentItem(2);
								else
										if (*(parameters->find("valign"))=="bottom")
												valign->setCurrentItem(3);
										else
												if (*(parameters->find("valign"))=="baseline")
														valign->setCurrentItem(4);
												else
														valign->setCurrentItem(0);
						parameters->remove("valign");
				};
		if (parameters->find("scope"))
				{
						QString s=*(parameters->find("scope"));
						for (int i=1; i<5; i++)
								if (s==scope->text(i))
										{
												scope->setCurrentItem(i);
												break;
										};
						parameters->remove("scope");
				};
		if (parameters->find("bgcolor"))
				bgcolorButton->setColor(QColor(parameters->find("bgcolor")->data()));
		if (parameters->find("nowrap"))
				{
						nowrap->setChecked(TRUE);
						parameters->remove("nowrap");
				};
		AttributesTab::setParameters(parameters);
}

bool TDTab::eventFilter(QObject *obj, QEvent *ev)
{
		if (obj==bgcolorEdit && ev->type()==Event_FocusOut)
				{
						QString s=bgcolorEdit->text();
						if (s.isEmpty()) return FALSE;
						QColor color;
						color.setNamedColor(s);
						bgcolorButton->setColor(color);
				};
		return FALSE;
}
		
void TDTab::colorChanged(const QColor &color)
{
		QString s;
		s.sprintf("#%02X%02X%02X", color.red(), color.green(), color.blue());
		bgcolorEdit->setText(s);
}

ColGroupTab::ColGroupTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QLabel *label;
		QLineEdit *lineEdit;
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 4, 4, 5);
	
		lineEdit=new QLineEdit(this, "span");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Span"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("aligncenter.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("alignjustify.xpm"));
		align->insertItem(loader->loadIcon("alignchar.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Horizontal Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(align, 1, 1);
	
		valign=new QComboBox(FALSE, this, "valign");
		valign->insertItem(i18n("Implied"));
		valign->insertItem(loader->loadIcon("aligntop.xpm"));
		valign->insertItem(loader->loadIcon("alignmiddle.xpm"));
		valign->insertItem(loader->loadIcon("alignbottom.xpm"));
		valign->insertItem(loader->loadIcon("alignbaseline.xpm"));
		valign->setMinimumSize(valign->sizeHint());
		label=new QLabel(valign, i18n("Vertical Alignment"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(valign, 1, 3);
	
		lineEdit=new QLineEdit(this, "char");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		lineEdit=new QLineEdit(this, "charoff");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Char Offset"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 2);
		grid->addWidget(lineEdit, 2, 3);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(3, 1);
		grid->activate();
}

QString ColGroupTab::getParameters()
{
		QString s;
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"center\"";break;
						case 3: s+=" align=\"right\"";break;
						case 4: s+=" align=\"justify\"";break;
						case 5: s+=" align=\"char\"";break;
						};
		if (valign->currentItem())
				switch (valign->currentItem())
						{
						case 1: s+=" valign=\"top\"";break;
						case 2: s+=" valign=\"middle\"";break;
						case 3: s+=" valign=\"bottom\"";break;
						case 4: s+=" valign=\"baseline\"";break;
						};
		s+=AttributesTab::getParameters();
		return s;
}

void ColGroupTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="center")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="right")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="justify")
														align->setCurrentItem(4);
						      else
														if (*(parameters->find("align"))=="char")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("valign"))
				{
						if (*(parameters->find("valign"))=="top")
								valign->setCurrentItem(1);
						else
								if (*(parameters->find("valign"))=="middle")
										valign->setCurrentItem(2);
								else
										if (*(parameters->find("valign"))=="bottom")
												valign->setCurrentItem(3);
										else
												if (*(parameters->find("valign"))=="baseline")
														valign->setCurrentItem(4);
												else
														valign->setCurrentItem(0);
						parameters->remove("valign");
				};
		AttributesTab::setParameters(parameters);
}

AreaTab::AreaTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 9, 5, 5);
	
		href=new QLineEdit(this, "href");
		href->setMinimumSize(href->sizeHint());
		QLabel *label=new QLabel(href, i18n("HREF"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(href, 0, 0, 1, 3);
		grid->addWidget(button, 0, 4);
		connect(button, SIGNAL(clicked()), this, SLOT(hrefClicked()));
		
		QLineEdit *lineEdit=new QLineEdit(this, "target");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Target"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(lineEdit, 1, 1, 1, 3);
	
	
		lineEdit=new QLineEdit(this, "alt");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Alternate Text"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		lineEdit=new QLineEdit(this, "coords");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Coordinates"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addMultiCellWidget(lineEdit, 3, 3, 1, 3);
	
		shape=new QComboBox(FALSE, this, "shape");
		shape->insertItem(i18n("Implied"));
		shape->insertItem(loader->loadIcon("square.xpm"));
		shape->insertItem(loader->loadIcon("circle.xpm"));
		shape->insertItem(loader->loadIcon("poly.xpm"));
		shape->setMinimumSize(shape->sizeHint());
		label=new QLabel(shape, i18n("Shape"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(shape, 4, 1);
		
		nohref=new QCheckBox("No HREF", this, "nohref");
		nohref->setMinimumSize(nohref->sizeHint());
		grid->addMultiCellWidget(nohref, 4, 4, 2, 3);
	
		lineEdit=new QLineEdit(this, "accesskey");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Access Key"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(lineEdit, 5, 1);
	
		lineEdit=new QLineEdit(this, "tabindex");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Tab Index"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 2);
		grid->addWidget(lineEdit, 5, 3);

		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addMultiCellWidget(lineEdit, 6, 6, 1, 3);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 0);
		grid->addMultiCellWidget(lineEdit, 7, 7, 1, 3);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(8, 1);
		grid->activate();
}

QString AreaTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (shape->currentItem())
				switch (shape->currentItem())
						{
						case 1: s+=" shape=\"rect\"";break;
						case 2: s+=" shape=\"circle\"";break;
						case 3: s+=" shape=\"poly\"";break;
						};
		if (nohref->isChecked()) s+=" nohref";
		return s;
}

void AreaTab::setParameters(StrDict *parameters)
{
		if (parameters->find("shape"))
				{
						if (*(parameters->find("shape"))=="rect")
								shape->setCurrentItem(1);
						else
								if (*(parameters->find("shape"))=="circle")
										shape->setCurrentItem(2);
								else
										if (*(parameters->find("shape"))=="poly")
												shape->setCurrentItem(3);
										else
												shape->setCurrentItem(0);
						parameters->remove("shape");
				};
		if (parameters->find("nohref"))
				{
						nohref->setChecked(TRUE);
						parameters->remove("nohref");
				};
		AttributesTab::setParameters(parameters);
}

void AreaTab::hrefClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		href->setText(fileName);
}

BaseFontTab::BaseFontTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 4, 4, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "face");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Face"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "size");
		lineEdit->setMaxLength(2);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Size"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		colorEdit=new QLineEdit(this, "color");
		colorEdit->setMinimumSize(colorEdit->sizeHint());
		colorEdit->installEventFilter(this);
		label=new QLabel(colorEdit, i18n("Color"), this);
		label->setMinimumSize(label->sizeHint());
		colorButton=new KColorButton(black, this);
		colorButton->setMinimumSize(colorButton->sizeHint());
		connect(colorButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(colorEdit, 1, 1, 1, 2);
		grid->addWidget(colorButton, 1, 3);
		
		lineEdit=new QLineEdit(this, "id");
  lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Id"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(3, 1);
		grid->activate();
}

void BaseFontTab::setParameters(StrDict *parameters)
{
		if (parameters->find("color"))
				colorButton->setColor(QColor(parameters->find("color")->data()));
		AttributesTab::setParameters(parameters);
}

bool BaseFontTab::eventFilter(QObject *obj, QEvent *ev)
{
		if (obj==colorEdit && ev->type()==Event_FocusOut)
				{
						QString s=colorEdit->text();
						if (s.isEmpty()) return FALSE;
						QColor color;
						color.setNamedColor(s);
						colorButton->setColor(color);
				};
		return FALSE;
}
		
void BaseFontTab::colorChanged(const QColor &color)
{
		QString s;
		s.sprintf("#%02X%02X%02X", color.red(), color.green(), color.blue());
		colorEdit->setText(s);
}

FontTab::FontTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 3, 4, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "face");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Face"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(lineEdit, 0, 1);
	
		lineEdit=new QLineEdit(this, "size");
		lineEdit->setMaxLength(2);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Size"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 2);
		grid->addWidget(lineEdit, 0, 3);
	
		colorEdit=new QLineEdit(this, "color");
		colorEdit->setMinimumSize(colorEdit->sizeHint());
		colorEdit->installEventFilter(this);
		label=new QLabel(colorEdit, i18n("Color"), this);
		label->setMinimumSize(label->sizeHint());
		colorButton=new KColorButton(black, this);
		colorButton->setMinimumSize(colorButton->sizeHint());
		connect(colorButton, SIGNAL(changed(const QColor&)), this, SLOT(colorChanged(const QColor&)));
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(colorEdit, 1, 1, 1, 2);
		grid->addWidget(colorButton, 1, 3);

		grid->setColStretch(1, 1);
		grid->setRowStretch(2, 1);
		grid->activate();
}

void FontTab::setParameters(StrDict *parameters)
{
		if (parameters->find("color"))
				colorButton->setColor(QColor(parameters->find("color")->data()));
		AttributesTab::setParameters(parameters);
}

bool FontTab::eventFilter(QObject *obj, QEvent *ev)
{
		if (obj==colorEdit && ev->type()==Event_FocusOut)
				{
						QString s=colorEdit->text();
						if (s.isEmpty()) return FALSE;
						QColor color;
						color.setNamedColor(s);
						colorButton->setColor(color);
				};
		return FALSE;
}
		
void FontTab::colorChanged(const QColor &color)
{
		QString s;
		s.sprintf("#%02X%02X%02X", color.red(), color.green(), color.blue());
		colorEdit->setText(s);
}

FrameTab::FrameTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 12, 3, 5);
		QLabel *label;
		QLineEdit *lineEdit;
	
		src=new QLineEdit(this, "src");
		src->setMinimumSize(src->sizeHint());
		label=new QLabel(src, i18n("Frame Source"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *srcButton=new QPushButton(this);
		srcButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		srcButton->setMinimumSize(srcButton->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(src, 1, 1, 0, 1);
		grid->addWidget(srcButton, 1, 2);
	
		lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 3, 3, 0, 1);
	
		lineEdit=new QLineEdit(this, "marginwidth");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Margin Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(lineEdit, 5, 0);
	
		lineEdit=new QLineEdit(this, "marginheight");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Margin Height"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 1);
		grid->addWidget(lineEdit, 5, 1);
		
		longDesc=new QLineEdit(this, "longdesc");
		longDesc->setMinimumSize(longDesc->sizeHint());
		label=new QLabel(longDesc, i18n("Link to Long Description"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *longDescButton=new QPushButton(this);
		longDescButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		longDescButton->setMinimumSize(longDescButton->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addMultiCellWidget(longDesc, 7, 7, 0, 1);
		grid->addWidget(longDescButton, 7, 2);
	
		frameBorder=new QComboBox(FALSE, this, "frameborder");
		frameBorder->insertItem(i18n("Implied"));
		frameBorder->insertItem(i18n("1"));
		frameBorder->insertItem(i18n("0"));
		frameBorder->setMinimumSize(frameBorder->sizeHint());
		label=new QLabel(frameBorder, i18n("Frame Border"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 8, 0);
		grid->addWidget(frameBorder, 9, 0);
	
		scrolling=new QComboBox(FALSE, this, "scrolling");
		scrolling->insertItem(i18n("Implied"));
		scrolling->insertItem(i18n("Yes"));
		scrolling->insertItem(i18n("No"));
		scrolling->insertItem(i18n("Auto"));
		scrolling->setMinimumSize(scrolling->sizeHint());
		label=new QLabel(scrolling, i18n("Scrolling"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 8, 1);
		grid->addWidget(scrolling, 9, 1);
	
		noresize=new QCheckBox("No Resize", this, "noresize");
		noresize->setMinimumSize(noresize->sizeHint());
		grid->addWidget(noresize, 10, 0);
	
		grid->setColStretch(0, 1);
		grid->setColStretch(1, 1);
		grid->setRowStretch(11, 1);

		grid->activate();
	
		connect(srcButton, SIGNAL(clicked()), this, SLOT(srcClicked()));
		connect(longDescButton, SIGNAL(clicked()), this, SLOT(longDescClicked()));
	
		basePath=baseDir;
}

QString FrameTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (frameBorder->currentItem())
				switch (frameBorder->currentItem())
						{
						case 1: s+=" frameborder=\"1\"";break;
						case 2: s+=" frameborder=\"0\"";break;
						};
		if (scrolling->currentItem())
				switch (scrolling->currentItem())
						{
						case 1: s+=" scrolling=\"yes\"";break;
						case 2: s+=" scrolling=\"no\"";break;
						case 3: s+=" scrolling=\"auto\"";break;
						};
		if (noresize->isChecked()) s+=" noresize";
		return s;
}

void FrameTab::setParameters(StrDict *parameters)
{
		if (parameters->find("frameborder"))
				{
						if (*(parameters->find("frameborder"))=="1")
								frameBorder->setCurrentItem(1);
						else
								if (*(parameters->find("frameborder"))=="0")
										frameBorder->setCurrentItem(2);
								else
										frameBorder->setCurrentItem(0);
						parameters->remove("frameborder");
				};
		if (parameters->find("scrolling"))
				{
						if (*(parameters->find("scrolling"))=="yes")
								scrolling->setCurrentItem(1);
						else
								if (*(parameters->find("scrolling"))=="no")
										scrolling->setCurrentItem(2);
								else
										if (*(parameters->find("scrolling"))=="auto")
												scrolling->setCurrentItem(3);
										else
												scrolling->setCurrentItem(0);
						parameters->remove("scrolling");
				};
		if (parameters->find("noresize"))
				{
						noresize->setChecked(TRUE);
						parameters->remove("noresize");
				};
		AttributesTab::setParameters(parameters);
}

void FrameTab::srcClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		src->setText(fileName);
}

void FrameTab::longDescClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		longDesc->setText(fileName);
}

IFrameTab::IFrameTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader = kapp->getIconLoader();
		QGridLayout *grid=new QGridLayout(this, 15, 3, 5);
		QLabel *label;
		QLineEdit *lineEdit;
	
		src=new QLineEdit(this, "src");
		src->setMinimumSize(src->sizeHint());
		label=new QLabel(src, i18n("Frame Source"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *srcButton=new QPushButton(this);
		srcButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		srcButton->setMinimumSize(srcButton->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(src, 1, 1, 0, 1);
		grid->addWidget(srcButton, 1, 2);
	
		lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 3, 3, 0, 1);
	
		lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(lineEdit, 5, 0);
	
		lineEdit=new QLineEdit(this, "height");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Height"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 1);
		grid->addWidget(lineEdit, 5, 1);
		
		lineEdit=new QLineEdit(this, "marginwidth");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Margin Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addWidget(lineEdit, 7, 0);
	
		lineEdit=new QLineEdit(this, "marginheight");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Margin Height"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 1);
		grid->addWidget(lineEdit, 7, 1);
		
		longDesc=new QLineEdit(this, "longdesc");
		longDesc->setMinimumSize(longDesc->sizeHint());
		label=new QLabel(longDesc, i18n("Link to Long Description"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *longDescButton=new QPushButton(this);
		longDescButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		longDescButton->setMinimumSize(longDescButton->sizeHint());
		grid->addWidget(label, 8, 0);
		grid->addMultiCellWidget(longDesc, 9, 9, 0, 1);
		grid->addWidget(longDescButton, 9, 2);
	
		frameBorder=new QComboBox(FALSE, this, "frameborder");
		frameBorder->insertItem(i18n("Implied"));
		frameBorder->insertItem(i18n("1"));
		frameBorder->insertItem(i18n("0"));
		frameBorder->setMinimumSize(frameBorder->sizeHint());
		label=new QLabel(frameBorder, i18n("Frame Border"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 10, 0);
		grid->addWidget(frameBorder, 11, 0);
	
		scrolling=new QComboBox(FALSE, this, "scrolling");
		scrolling->insertItem(i18n("Implied"));
		scrolling->insertItem(i18n("Yes"));
		scrolling->insertItem(i18n("No"));
		scrolling->insertItem(i18n("Auto"));
		scrolling->setMinimumSize(scrolling->sizeHint());
		label=new QLabel(scrolling, i18n("Scrolling"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 10, 1);
		grid->addWidget(scrolling, 11, 1);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("aligntop.xpm"));
		align->insertItem(loader->loadIcon("alignmiddle.xpm"));
		align->insertItem(loader->loadIcon("alignbottom.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 12, 0);
		grid->addWidget(align, 13, 0);
	
		grid->setColStretch(0, 1);
		grid->setColStretch(1, 1);
		grid->setRowStretch(14, 1);

		grid->activate();
	
		connect(srcButton, SIGNAL(clicked()), this, SLOT(srcClicked()));
		connect(longDescButton, SIGNAL(clicked()), this, SLOT(longDescClicked()));
	
		basePath=baseDir;
}

QString IFrameTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"right\"";break;
						case 3: s+=" align=\"top\"";break;
						case 4: s+=" align=\"middle\"";break;
						case 5: s+=" align=\"bottom\"";break;
						};
		if (frameBorder->currentItem())
				switch (frameBorder->currentItem())
						{
						case 1: s+=" frameborder=\"1\"";break;
						case 2: s+=" frameborder=\"0\"";break;
						};
		if (scrolling->currentItem())
				switch (scrolling->currentItem())
						{
						case 1: s+=" scrolling=\"yes\"";break;
						case 2: s+=" scrolling=\"no\"";break;
						case 3: s+=" scrolling=\"auto\"";break;
						};
		return s;
}

void IFrameTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="right")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="top")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="middle")
														align->setCurrentItem(4);
												else
														if (*(parameters->find("align"))=="bottom")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("frameborder"))
				{
						if (*(parameters->find("frameborder"))=="1")
								frameBorder->setCurrentItem(1);
						else
								if (*(parameters->find("frameborder"))=="0")
										frameBorder->setCurrentItem(2);
								else
										frameBorder->setCurrentItem(0);
						parameters->remove("frameborder");
				};
		if (parameters->find("scrolling"))
				{
						if (*(parameters->find("scrolling"))=="yes")
								scrolling->setCurrentItem(1);
						else
								if (*(parameters->find("scrolling"))=="no")
										scrolling->setCurrentItem(2);
								else
										if (*(parameters->find("scrolling"))=="auto")
												scrolling->setCurrentItem(3);
										else
												scrolling->setCurrentItem(0);
						parameters->remove("scrolling");
				};
		AttributesTab::setParameters(parameters);
}

void IFrameTab::srcClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		src->setText(fileName);
}

void IFrameTab::longDescClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		longDesc->setText(fileName);
}

FormTab::FormTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 8, 3, 5);
	
		action=new QLineEdit(this, "action");
		action->setMinimumSize(action->sizeHint());
		QLabel *label=new QLabel(action, i18n("Action"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(action, 0, 1);
		grid->addWidget(button, 0, 2);
		connect(button, SIGNAL(clicked()), this, SLOT(actionClicked()));
	
		method=new QComboBox(FALSE, this, "method");
		method->insertItem(i18n("Implied"));
		method->insertItem("GET");
		method->insertItem("POST");
		method->setMinimumSize(method->sizeHint());
		label=new QLabel(method, i18n("Method"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(method, 1, 1);

		enctype=new QComboBox(TRUE, this, "enctype");
		enctype->insertItem("application/x-www-form-urlencoded");
		enctype->insertItem("multipart/form-data");
		enctype->setMinimumSize(enctype->sizeHint());
		label=new QLabel(enctype, i18n("Encoding Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(enctype, 2, 1);

		QLineEdit *lineEdit=new QLineEdit(this, "target");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Target"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		lineEdit=new QLineEdit(this, "accept-charset");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Accept-charset"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(lineEdit, 4, 1);
	
		lineEdit=new QLineEdit(this, "onsubmit");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Submit"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(lineEdit, 5, 1);
	
		lineEdit=new QLineEdit(this, "onreset");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Reset"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addWidget(lineEdit, 6, 1);
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(7, 1);
		grid->activate();
}

QString FormTab::getParameters()
{
		QString s;
		if (method->currentItem())
				switch (method->currentItem())
						{
						case 1: s+=" method=\"get\"";break;
						case 2: s+=" method=\"post\"";break;
						};
		s+=" enctype=\"";
		s+=enctype->currentText();
		s+="\"";
		enctype->clearEdit();
		s+=AttributesTab::getParameters();
		return s;
}

void FormTab::setParameters(StrDict *parameters)
{
		if (parameters->find("method"))
				{
						if (*(parameters->find("method"))=="get")
								method->setCurrentItem(1);
						else
								if (*(parameters->find("method"))=="post")
										method->setCurrentItem(2);
								else
										method->setCurrentItem(0);
						parameters->remove("method");
				};
		if (parameters->find("enctype"))
				{
						if (*(parameters->find("enctype"))=="application/x-www-form-urlencoded")
								enctype->setCurrentItem(0);
						else
								if (*(parameters->find("enctype"))=="multipart/form-data")
										enctype->setCurrentItem(1);
								else
										{
												enctype->insertItem((parameters->find("enctype"))->data(), 0);
												enctype->setCurrentItem(0);
										};
						parameters->remove("enctype");
				};
		AttributesTab::setParameters(parameters);
}

void FormTab::actionClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.cgi *.pl|CGI Scripts (*.cgi *.pl)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		action->setText(fileName);
}

SelectTab::SelectTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 7, 4, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(lineEdit, 0, 0, 1, 3);
	
		lineEdit=new QLineEdit(this, "size");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Size"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "tabindex");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Tab Index"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(lineEdit, 1, 3);
	
		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addMultiCellWidget(lineEdit, 3, 3, 1, 3);
	
		lineEdit=new QLineEdit(this, "onchange");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Change"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addMultiCellWidget(lineEdit, 4, 4, 1, 3);
	
		multiple=new QCheckBox("Multiple Selection", this, "multiple");
		multiple->setMinimumSize(multiple->sizeHint());
		grid->addMultiCellWidget(multiple, 5, 5, 0, 1);
	
		disabled=new QCheckBox("Disabled", this, "disabled");
		disabled->setMinimumSize(disabled->sizeHint());
		grid->addMultiCellWidget(disabled, 5, 5, 2, 3);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(6, 1);
		grid->activate();
}

QString SelectTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (multiple->isChecked()) s+=" multiple";
		if (disabled->isChecked()) s+=" disabled";
		return s;
}

void SelectTab::setParameters(StrDict *parameters)
{
		if (parameters->find("multiple"))
				{
						multiple->setChecked(TRUE);
						parameters->remove("multiple");
				};
		if (parameters->find("disabled"))
				{
						disabled->setChecked(TRUE);
						parameters->remove("disabled");
				};
		AttributesTab::setParameters(parameters);
}

TextAreaTab::TextAreaTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 9, 4, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(lineEdit, 0, 0, 1, 3);
	
		lineEdit=new QLineEdit(this, "rows");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Rows"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addWidget(lineEdit, 1, 1);
	
		lineEdit=new QLineEdit(this, "cols");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Columns"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 2);
		grid->addWidget(lineEdit, 1, 3);
	
		lineEdit=new QLineEdit(this, "accesskey");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Access Key"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		lineEdit=new QLineEdit(this, "tabindex");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Tab Index"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 2);
		grid->addWidget(lineEdit, 2, 3);
	
		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addMultiCellWidget(lineEdit, 3, 3, 1, 3);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addMultiCellWidget(lineEdit, 4, 4, 1, 3);
	
		lineEdit=new QLineEdit(this, "onselect");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Select"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addMultiCellWidget(lineEdit, 5, 5, 1, 3);
	
		lineEdit=new QLineEdit(this, "onchange");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Change"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addMultiCellWidget(lineEdit, 6, 6, 1, 3);
	
		readOnly=new QCheckBox("Read Only", this, "readonly");
		readOnly->setMinimumSize(readOnly->sizeHint());
		grid->addMultiCellWidget(readOnly, 7, 7, 0, 1);
	
		disabled=new QCheckBox("Disabled", this, "disabled");
		disabled->setMinimumSize(disabled->sizeHint());
		grid->addMultiCellWidget(disabled, 7, 7, 2, 3);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(8, 1);
		grid->activate();
}

QString TextAreaTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (readOnly->isChecked()) s+=" readonly";
		if (disabled->isChecked()) s+=" disabled";
		return s;
}

void TextAreaTab::setParameters(StrDict *parameters)
{
		if (parameters->find("readonly"))
				{
						readOnly->setChecked(TRUE);
						parameters->remove("readonly");
				};
		if (parameters->find("disabled"))
				{
						disabled->setChecked(TRUE);
						parameters->remove("disabled");
				};
		AttributesTab::setParameters(parameters);
}

ButtonTab::ButtonTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid=new QGridLayout(this, 8, 4, 5);
	
		QLineEdit *lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		QLabel *label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(lineEdit, 0, 0, 1, 3);
	
		lineEdit=new QLineEdit(this, "value");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Value"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(lineEdit, 1, 1, 1, 3);
	
		type=new QComboBox(FALSE, this, "type");
		type->insertItem(i18n("Implied"));
		type->insertItem(i18n("Button"));
		type->insertItem(i18n("Submit"));
		type->insertItem(i18n("Reset"));
		type->setMinimumSize(type->sizeHint());
		label=new QLabel(type, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(type, 2, 2, 1, 3);

		lineEdit=new QLineEdit(this, "accesskey");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Access Key"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		lineEdit=new QLineEdit(this, "tabindex");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Tab Index"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 2);
		grid->addWidget(lineEdit, 3, 3);
	
		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addMultiCellWidget(lineEdit, 4, 4, 1, 3);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addMultiCellWidget(lineEdit, 5, 5, 1, 3);
	
		disabled=new QCheckBox("Disabled", this, "disabled");
		disabled->setMinimumSize(disabled->sizeHint());
		grid->addMultiCellWidget(disabled, 6, 6, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(7, 1);
		grid->activate();
}

QString ButtonTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (type->currentItem())
				switch (type->currentItem())
						{
						case 1: s+=" type=\"button\"";break;
						case 2: s+=" type=\"submit\"";break;
						case 3: s+=" type=\"reset\"";break;
						};
		if (disabled->isChecked()) s+=" disabled";
		return s;
}

void ButtonTab::setParameters(StrDict *parameters)
{
		if (parameters->find("type"))
				{
						if (*(parameters->find("type"))=="button")
								type->setCurrentItem(1);
						else
								if (*(parameters->find("type"))=="submit")
										type->setCurrentItem(2);
								else
										if (*(parameters->find("type"))=="reset")
												type->setCurrentItem(3);
										else
												type->setCurrentItem(0);
						parameters->remove("type");
				};
		if (parameters->find("disabled"))
				{
						disabled->setChecked(TRUE);
						parameters->remove("disabled");
				};
		AttributesTab::setParameters(parameters);
}

InputTab::InputTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 16, 5, 5);
	
		type=new QComboBox(FALSE, this, "type");
		type->insertItem(i18n("Implied"));
		type->insertItem(i18n("Text"));
		type->insertItem(i18n("Password"));
		type->insertItem(i18n("Checkbox"));
		type->insertItem(i18n("Radio Button"));
		type->insertItem(i18n("Submit"));
		type->insertItem(i18n("Reset"));
		type->insertItem(i18n("File"));
		type->insertItem(i18n("Hidden"));
		type->insertItem(i18n("Image"));
		type->insertItem(i18n("Button"));
		type->setMinimumSize(type->sizeHint());
		QLabel *label=new QLabel(type, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(type, 0, 0, 1, 3);

		QLineEdit *lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(lineEdit, 1, 1, 1, 3);
	
		lineEdit=new QLineEdit(this, "value");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Value"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		src=new QLineEdit(this, "src");
		src->setMinimumSize(src->sizeHint());
		label=new QLabel(src, i18n("Image Source"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *srcButton=new QPushButton(this);
		srcButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		srcButton->setMinimumSize(srcButton->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addMultiCellWidget(src, 3, 3, 1, 3);
		grid->addWidget(srcButton, 3, 4);
	
		lineEdit=new QLineEdit(this, "alt");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Alternate Text"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addMultiCellWidget(lineEdit, 4, 4, 1, 3);
	
		usemap=new QLineEdit(this, "usemap");
		usemap->setMinimumSize(usemap->sizeHint());
		label=new QLabel(usemap, i18n("Image Map"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *mapButton=new QPushButton(this);
		mapButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		mapButton->setMinimumSize(mapButton->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addMultiCellWidget(usemap, 5, 5, 1, 3);
		grid->addWidget(mapButton, 5, 4);
	
		lineEdit=new QLineEdit(this, "size");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Size"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addWidget(lineEdit, 6, 1);
	
		lineEdit=new QLineEdit(this, "maxlength");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Maximum Length"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 2);
		grid->addWidget(lineEdit, 6, 3);
	
		lineEdit=new QLineEdit(this, "accesskey");
		lineEdit->setMaxLength(1);
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Access Key"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 0);
		grid->addWidget(lineEdit, 7, 1);
	
		lineEdit=new QLineEdit(this, "tabindex");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Tab Index"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 2);
		grid->addWidget(lineEdit, 7, 3);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("aligntop.xpm"));
		align->insertItem(loader->loadIcon("alignmiddle.xpm"));
		align->insertItem(loader->loadIcon("alignbottom.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 8, 0);
		grid->addWidget(align, 8, 1);
	
		lineEdit=new QLineEdit(this, "accept");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Accept Types"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 8, 2);
		grid->addWidget(lineEdit, 8, 3);
	
		lineEdit=new QLineEdit(this, "onfocus");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Focus"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 9, 0);
		grid->addMultiCellWidget(lineEdit, 9, 9, 1, 3);
	
		lineEdit=new QLineEdit(this, "onblur");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Blur"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 10, 0);
		grid->addMultiCellWidget(lineEdit, 10, 10, 1, 3);
	
		lineEdit=new QLineEdit(this, "onselect");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Select"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 11, 0);
		grid->addMultiCellWidget(lineEdit, 11, 11, 1, 3);
	
		lineEdit=new QLineEdit(this, "onchange");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("On Change"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 12, 0);
		grid->addMultiCellWidget(lineEdit, 12, 12, 1, 3);
	
		readOnly=new QCheckBox("Read Only", this, "readonly");
		readOnly->setMinimumSize(readOnly->sizeHint());
		grid->addMultiCellWidget(readOnly, 13, 13, 0, 1);
	
		checked=new QCheckBox("Checked", this, "checked");
		checked->setMinimumSize(checked->sizeHint());
		grid->addMultiCellWidget(checked, 13, 13, 2, 3);
	
		disabled=new QCheckBox("Disabled", this, "disabled");
		disabled->setMinimumSize(disabled->sizeHint());
		grid->addMultiCellWidget(disabled, 14, 14, 0, 1);
	
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(15, 1);
		grid->activate();
		
		connect(srcButton, SIGNAL(clicked()), this, SLOT(srcClicked()));
		connect(mapButton, SIGNAL(clicked()), this, SLOT(usemapClicked()));
}

QString InputTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (type->currentItem())
				switch (type->currentItem())
						{
						case 1: s+=" type=\"text\"";break;
						case 2: s+=" type=\"password\"";break;
						case 3: s+=" type=\"checkbox\"";break;
						case 4: s+=" type=\"radio\"";break;
						case 5: s+=" type=\"submit\"";break;
						case 6: s+=" type=\"reset\"";break;
						case 7: s+=" type=\"file\"";break;
						case 8: s+=" type=\"hidden\"";break;
						case 9: s+=" type=\"image\"";break;
						case 10: s+=" type=\"button\"";break;
						};
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"right\"";break;
						case 3: s+=" align=\"top\"";break;
						case 4: s+=" align=\"middle\"";break;
						case 5: s+=" align=\"bottom\"";break;
						};
		if (readOnly->isChecked()) s+=" readonly";
		if (checked->isChecked()) s+=" checked";
		if (disabled->isChecked()) s+=" disabled";
		return s;
}

void InputTab::setParameters(StrDict *parameters)
{
		if (parameters->find("type"))
				{
						if (*(parameters->find("type"))=="text")
								type->setCurrentItem(1);
						else
								if (*(parameters->find("type"))=="password")
										type->setCurrentItem(2);
								else
										if (*(parameters->find("type"))=="checkbox")
												type->setCurrentItem(3);
										else
												if (*(parameters->find("type"))=="radio")
														type->setCurrentItem(4);
												else
														if (*(parameters->find("type"))=="submit")
																type->setCurrentItem(5);
														else
																if (*(parameters->find("type"))=="reset")
																		type->setCurrentItem(6);
																else
																		if (*(parameters->find("type"))=="file")
																				type->setCurrentItem(7);
																		else
																				if (*(parameters->find("type"))=="hidden")
																						type->setCurrentItem(8);
																				else
																						if (*(parameters->find("type"))=="image")
																								type->setCurrentItem(9);
																						else
																								if (*(parameters->find("type"))=="button")
																										type->setCurrentItem(10);
																								else
																										type->setCurrentItem(0);
						parameters->remove("type");
				};
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="right")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="top")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="middle")
														align->setCurrentItem(4);
												else
														if (*(parameters->find("align"))=="bottom")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("readonly"))
				{
						readOnly->setChecked(TRUE);
						parameters->remove("readonly");
				};
		if (parameters->find("checked"))
				{
						checked->setChecked(TRUE);
						parameters->remove("checked");
				};
		if (parameters->find("disabled"))
				{
						disabled->setChecked(TRUE);
						parameters->remove("disabled");
				};
		AttributesTab::setParameters(parameters);
}

void InputTab::srcClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.gif *.jpg *.png|Image Files (*.gif *.jpg *.png)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		src->setText(fileName);
}

void InputTab::usemapClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		usemap->setText(fileName);
}

ParamTab::ParamTab(QWidget *parent, const char *name):AttributesTab(parent, name)
{
		QGridLayout *grid = new QGridLayout(this, 5, 4, 5);
		QLabel *label;
		QLineEdit *lineEdit;
	
		lineEdit=new QLineEdit(this, "id");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Id"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(lineEdit, 0, 0, 1, 3);
	
		lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(lineEdit, 1, 1, 1, 3);
	
		lineEdit=new QLineEdit(this, "value");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Value"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		valueType=new QComboBox(FALSE, this, "valuetype");
		valueType->insertItem(i18n("Implied"));
		valueType->insertItem("data");
		valueType->insertItem("ref");
		valueType->insertItem("object");
		valueType->setMinimumSize(valueType->sizeHint());
		label=new QLabel(valueType, i18n("Value Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(valueType, 3, 1);
	
		lineEdit=new QLineEdit(this, "type");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Content Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 2);
		grid->addWidget(lineEdit, 3, 3);
	
		grid->setRowStretch(4, 1);
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->activate();
}

QString ParamTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (valueType->currentItem())
				switch (valueType->currentItem())
						{
						case 1: s+=" valuetype=\"data\"";break;
						case 2: s+=" valuetype=\"ref\"";break;
						case 3: s+=" valuetype=\"object\"";break;
						};
		return s;
}

void ParamTab::setParameters(StrDict *parameters)
{
		if (parameters->find("valuetype"))
				{
						if (*(parameters->find("valuetype"))=="data")
								valueType->setCurrentItem(1);
						else
								if (*(parameters->find("valuetype"))=="ref")
										valueType->setCurrentItem(2);
								else
										if (*(parameters->find("valuetype"))=="object")
												valueType->setCurrentItem(3);
										else
												valueType->setCurrentItem(0);
						parameters->remove("valuetype");
				};
		AttributesTab::setParameters(parameters);
}

ObjectTab::ObjectTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 13, 5, 5);
	
		codeBase=new QLineEdit(this, "codebase");
		codeBase->setMinimumSize(codeBase->sizeHint());
		QLabel *label=new QLabel(codeBase, i18n("Base URI"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *codeBaseButton=new QPushButton(this);
		codeBaseButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		codeBaseButton->setMinimumSize(codeBaseButton->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(codeBase, 0, 0, 1, 3);
		grid->addWidget(codeBaseButton, 0, 4);

		classID=new QLineEdit(this, "classid");
		classID->setMinimumSize(classID->sizeHint());
		label=new QLabel(classID, i18n("Code Location"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *classIDButton=new QPushButton(this);
		classIDButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		classIDButton->setMinimumSize(classIDButton->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(classID, 1, 1, 1, 3);
		grid->addWidget(classIDButton, 1, 4);

		data=new QLineEdit(this, "data");
		data->setMinimumSize(data->sizeHint());
		label=new QLabel(data, i18n("Data Location"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *dataButton=new QPushButton(this);
		dataButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		dataButton->setMinimumSize(dataButton->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(data, 2, 2, 1, 3);
		grid->addWidget(dataButton, 2, 4);

		archive=new QLineEdit(this, "archive");
		archive->setMinimumSize(archive->sizeHint());
		label=new QLabel(archive, i18n("Archive Location"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *archiveButton=new QPushButton(this);
		archiveButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		archiveButton->setMinimumSize(archiveButton->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addMultiCellWidget(archive, 3, 3, 1, 3);
		grid->addWidget(archiveButton, 3, 4);

		QLineEdit *lineEdit=new QLineEdit(this, "standby");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Standby Message"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addMultiCellWidget(lineEdit, 4, 4, 1, 3);
	
		lineEdit=new QLineEdit(this, "codetype");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Code Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(lineEdit, 5, 1);
	
		lineEdit=new QLineEdit(this, "type");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Data Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 2);
		grid->addWidget(lineEdit, 5, 3);
	
		lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addWidget(lineEdit, 6, 1);
		
		lineEdit=new QLineEdit(this, "height");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Height"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 2);
		grid->addWidget(lineEdit, 6, 3);
	
		lineEdit=new QLineEdit(this, "hspace");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("HSpace"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 0);
		grid->addWidget(lineEdit, 7, 1);
		
		lineEdit=new QLineEdit(this, "vspace");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("VSpace"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 2);
		grid->addWidget(lineEdit, 7, 3);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("aligntop.xpm"));
		align->insertItem(loader->loadIcon("alignmiddle.xpm"));
		align->insertItem(loader->loadIcon("alignbottom.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 8, 0);
		grid->addWidget(align, 8, 1);
	
		lineEdit=new QLineEdit(this, "border");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Border"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 8, 2);
		grid->addWidget(lineEdit, 8, 3);
	
		usemap=new QLineEdit(this, "usemap");
		usemap->setMinimumSize(usemap->sizeHint());
		label=new QLabel(usemap, i18n("Image Map"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *mapButton=new QPushButton(this);
		mapButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		mapButton->setMinimumSize(mapButton->sizeHint());
		grid->addWidget(label, 9, 0);
		grid->addMultiCellWidget(usemap, 9, 9, 1, 3);
		grid->addWidget(mapButton, 9, 4);
	
		lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 10, 0);
		grid->addWidget(lineEdit, 10, 1);
	
		lineEdit=new QLineEdit(this, "tabindex");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Tab Index"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 10, 2);
		grid->addWidget(lineEdit, 10, 3);
	
		declare=new QCheckBox("Declare Only", this, "declare");
		declare->setMinimumSize(declare->sizeHint());
		grid->addMultiCellWidget(declare, 11, 11, 0, 3);
	
		grid->setRowStretch(12, 1);
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->activate();
		
		connect(codeBaseButton, SIGNAL(clicked()), this, SLOT(codeBaseClicked()));
		connect(classIDButton, SIGNAL(clicked()), this, SLOT(classIDClicked()));
		connect(dataButton, SIGNAL(clicked()), this, SLOT(dataClicked()));
		connect(archiveButton, SIGNAL(clicked()), this, SLOT(archiveClicked()));
		connect(mapButton, SIGNAL(clicked()), this, SLOT(usemapClicked()));
}

QString ObjectTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"right\"";break;
						case 3: s+=" align=\"top\"";break;
						case 4: s+=" align=\"middle\"";break;
						case 5: s+=" align=\"bottom\"";break;
						};
		if (declare->isChecked()) s+=" declare";
		return s;
}

void ObjectTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="right")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="top")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="middle")
														align->setCurrentItem(4);
												else
														if (*(parameters->find("align"))=="bottom")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		if (parameters->find("declare"))
				{
						declare->setChecked(TRUE);
						parameters->remove("declare");
				};
		AttributesTab::setParameters(parameters);
}

void ObjectTab::codeBaseClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		codeBase->setText(fileName);
}

void ObjectTab::classIDClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		classID->setText(fileName);
}

void ObjectTab::dataClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		data->setText(fileName);
}

void ObjectTab::archiveClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		archive->setText(fileName);
}

void ObjectTab::usemapClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		usemap->setText(fileName);
}

AppletTab::AppletTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 9, 5, 5);
	
		codeBase=new QLineEdit(this, "codebase");
		codeBase->setMinimumSize(codeBase->sizeHint());
		QLabel *label=new QLabel(codeBase, i18n("Base URI"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *codeBaseButton=new QPushButton(this);
		codeBaseButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		codeBaseButton->setMinimumSize(codeBaseButton->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(codeBase, 0, 0, 1, 3);
		grid->addWidget(codeBaseButton, 0, 4);

		archive=new QLineEdit(this, "archive");
		archive->setMinimumSize(archive->sizeHint());
		label=new QLabel(archive, i18n("Archive Location"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *archiveButton=new QPushButton(this);
		archiveButton->setPixmap(loader->loadIcon("fileopen.xpm"));
		archiveButton->setMinimumSize(archiveButton->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(archive, 1, 1, 1, 3);
		grid->addWidget(archiveButton, 1, 4);

		QLineEdit *lineEdit=new QLineEdit(this, "code");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Class File"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addMultiCellWidget(lineEdit, 2, 2, 1, 3);
	
		lineEdit=new QLineEdit(this, "object");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Object"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addMultiCellWidget(lineEdit, 3, 3, 1, 3);
	
	 lineEdit=new QLineEdit(this, "alt");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Alternate Text"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addMultiCellWidget(lineEdit, 4, 4, 1, 3);
	
		lineEdit=new QLineEdit(this, "width");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Width"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 0);
		grid->addWidget(lineEdit, 5, 1);
		
		lineEdit=new QLineEdit(this, "height");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Height"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 5, 2);
		grid->addWidget(lineEdit, 5, 3);
	
		lineEdit=new QLineEdit(this, "hspace");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("HSpace"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 0);
		grid->addWidget(lineEdit, 6, 1);
		
		lineEdit=new QLineEdit(this, "vspace");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("VSpace"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 6, 2);
		grid->addWidget(lineEdit, 6, 3);
	
		align=new QComboBox(FALSE, this, "align");
		align->insertItem(i18n("Implied"));
		align->insertItem(loader->loadIcon("alignleft.xpm"));
		align->insertItem(loader->loadIcon("alignright.xpm"));
		align->insertItem(loader->loadIcon("aligntop.xpm"));
		align->insertItem(loader->loadIcon("alignmiddle.xpm"));
		align->insertItem(loader->loadIcon("alignbottom.xpm"));
		align->setMinimumSize(align->sizeHint());
		label=new QLabel(align, i18n("Align"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 0);
		grid->addWidget(align, 7, 1);
	
		lineEdit=new QLineEdit(this, "name");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Name"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 7, 2);
		grid->addWidget(lineEdit, 7, 3);
	
		grid->setRowStretch(8, 1);
		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->activate();
		
		connect(codeBaseButton, SIGNAL(clicked()), this, SLOT(codeBaseClicked()));
		connect(archiveButton, SIGNAL(clicked()), this, SLOT(archiveClicked()));
}

QString AppletTab::getParameters()
{
		QString s=AttributesTab::getParameters();
		if (align->currentItem())
				switch (align->currentItem())
						{
						case 1: s+=" align=\"left\"";break;
						case 2: s+=" align=\"right\"";break;
						case 3: s+=" align=\"top\"";break;
						case 4: s+=" align=\"middle\"";break;
						case 5: s+=" align=\"bottom\"";break;
						};
		return s;
}

void AppletTab::setParameters(StrDict *parameters)
{
		if (parameters->find("align"))
				{
						if (*(parameters->find("align"))=="left")
								align->setCurrentItem(1);
						else
								if (*(parameters->find("align"))=="right")
										align->setCurrentItem(2);
								else
										if (*(parameters->find("align"))=="top")
												align->setCurrentItem(3);
										else
												if (*(parameters->find("align"))=="middle")
														align->setCurrentItem(4);
												else
														if (*(parameters->find("align"))=="bottom")
																align->setCurrentItem(5);
														else
																align->setCurrentItem(0);
						parameters->remove("align");
				};
		AttributesTab::setParameters(parameters);
}

void AppletTab::codeBaseClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		codeBase->setText(fileName);
}

void AppletTab::archiveClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		archive->setText(fileName);
}

HeadTab::HeadTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 2, 3, 5);
	
		profile=new QLineEdit(this, "profile");
		profile->setMinimumSize(profile->sizeHint());
		QLabel *label=new QLabel(profile, i18n("Profile"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addWidget(profile, 0, 1);
		grid->addWidget(button, 0, 2);
		connect(button, SIGNAL(clicked()), this, SLOT(profileClicked()));
	
		grid->setColStretch(1, 1);
		grid->setRowStretch(1, 1);
		grid->activate();
}

void HeadTab::profileClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		profile->setText(fileName);
}

LinkTab::LinkTab(const char *baseDir, QWidget *parent, const char *name):AttributesTab(parent, name)
{
		KIconLoader *loader=kapp->getIconLoader();
	
		basePath=baseDir;
	
		QGridLayout *grid=new QGridLayout(this, 6, 5, 5);
	
		href=new QLineEdit(this, "href");
		href->setMinimumSize(href->sizeHint());
		QLabel *label=new QLabel(href, i18n("HREF"), this);
		label->setMinimumSize(label->sizeHint());
		QPushButton *button=new QPushButton(this);
		button->setPixmap(loader->loadIcon("fileopen.xpm"));
		button->setMinimumSize(button->sizeHint());
		grid->addWidget(label, 0, 0);
		grid->addMultiCellWidget(href, 0, 0, 1, 3);
		grid->addWidget(button, 0, 4);
		connect(button, SIGNAL(clicked()), this, SLOT(hrefClicked()));
	
		QLineEdit *lineEdit=new QLineEdit(this, "target");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Target"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 1, 0);
		grid->addMultiCellWidget(lineEdit, 1, 1, 1, 3);
	
		lineEdit=new QLineEdit(this, "rel");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Rel"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 0);
		grid->addWidget(lineEdit, 2, 1);
	
		lineEdit=new QLineEdit(this, "rev");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Rev"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 2, 2);
		grid->addWidget(lineEdit, 2, 3);
	
		lineEdit=new QLineEdit(this, "type");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Type"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 0);
		grid->addWidget(lineEdit, 3, 1);
	
		lineEdit=new QLineEdit(this, "media");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Media"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 3, 2);
		grid->addWidget(lineEdit, 3, 3);
	
		lineEdit=new QLineEdit(this, "hreflang");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("HREF Language"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 0);
		grid->addWidget(lineEdit, 4, 1);
	
		lineEdit=new QLineEdit(this, "charset");
		lineEdit->setMinimumSize(lineEdit->sizeHint());
		label=new QLabel(lineEdit, i18n("Character Set"), this);
		label->setMinimumSize(label->sizeHint());
		grid->addWidget(label, 4, 2);
		grid->addWidget(lineEdit, 4, 3);

		grid->setColStretch(1, 1);
		grid->setColStretch(3, 1);
		grid->setRowStretch(5, 1);
		grid->activate();
}

void LinkTab::hrefClicked()
{
		QString fileName=KFileDialog::getOpenFileName(0, "*.*htm*|HTML Files (*.*htm*)\n*.css *.jss|Style Sheets (*.css *.jss)\n*|All Files", this);
		if (fileName.isEmpty()) return;
		QExtFileInfo file(fileName);
		file.convertToRelative(basePath);
		fileName=file.filePath();
		href->setText(fileName);
}

/*============================TagEditDialog===========================*/

TagEditDialog::TagEditDialog(const HTMLTag *element, HTMLEdit *parent, const char *name):QTabDialog(parent, name, TRUE)
{
		AttributesTab *mainAttrs;
		switch (element->dialogTab)
				{
				case ABSENT: mainAttrs = 0; break;
				case DOCTYPE: mainAttrs = new DocTypeTab(this, "MainTab"); break;
				case HEAD: mainAttrs = new HeadTab(parent->projectRoot(), this, "MainTab"); break;
				case META: mainAttrs = new MetaTab(this, "MainTab"); break;
				case BODY: mainAttrs = new BodyTab(parent->projectRoot(), this, "MainTab"); break;
				case ALIGN: mainAttrs = new AlignTab(this, "MainTab"); break;
				case QUOTE: mainAttrs = new QuoteTab(parent->projectRoot(), this, "MainTab"); break;
				case BR: mainAttrs = new BRTab(this, "MainTab"); break;
				case PRE: mainAttrs = new PreTab(this, "MainTab"); break;
				case EDIT: mainAttrs = new EditTab(parent->projectRoot(), this, "MainTab"); break;
				case UL: mainAttrs = new ULTab(this, "MainTab"); break;
				case OL: mainAttrs = new OLTab(this, "MainTab"); break;
				case LI: mainAttrs = new LITab(this, "MainTab"); break;
				case COMPACT: mainAttrs = new CompactTab(this, "MainTab"); break;
				case TABLE: mainAttrs = new TableTab(this, "MainTab"); break;
				case CAPTION: mainAttrs = new CaptionTab(this, "MainTab"); break;
				case THEAD: mainAttrs = new THeadTab(this, "MainTab"); break;
				case COLGROUP: mainAttrs = new ColGroupTab(this, "MainTab"); break;
				case TR: mainAttrs = new TRTab(this, "MainTab"); break;
				case TD: mainAttrs = new TDTab(this, "MainTab"); break;
				case A: mainAttrs = new ATab(parent->projectRoot(), this, "MainTab"); break;
				case LINK: mainAttrs = new LinkTab(parent->projectRoot(), this, "MainTab"); break;
				case BASE: mainAttrs = new BaseTab(parent->projectRoot(), this, "MainTab"); break;
				case IMG: mainAttrs = new ImageTab(parent->projectRoot(), this, "MainTab"); break;
				case OBJECT: mainAttrs = new ObjectTab(parent->projectRoot(), this, "MainTab"); break;
				case PARAM: mainAttrs = new ParamTab(this, "MainTab"); break;
				case APPLET: mainAttrs = new AppletTab(parent->projectRoot(), this, "MainTab"); break;
				case MAP: mainAttrs = new MapTab(this, "MainTab"); break;
				case AREA: mainAttrs = new AreaTab(parent->projectRoot(), this, "MainTab"); break;
				case STYLE: mainAttrs = new StyleTab(this, "MainTab"); break;
				case BASEFONT: mainAttrs = new BaseFontTab(this, "MainTab"); break;
				case FONT: mainAttrs = new FontTab(this, "MainTab"); break;
				case HR: mainAttrs = new HRTab(this, "MainTab"); break;
				case FRAMESET: mainAttrs = new FrameSetTab(this, "MainTab"); break;
				case FRAME: mainAttrs = new FrameTab(parent->projectRoot(), this, "MainTab"); break;
				case IFRAME: mainAttrs = new IFrameTab(parent->projectRoot(), this, "MainTab"); break;
				case FORM: mainAttrs = new FormTab(parent->projectRoot(), this, "MainTab"); break;
				case INPUT: mainAttrs = new InputTab(parent->projectRoot(), this, "MainTab"); break;
				case BUTTON: mainAttrs = new ButtonTab(this, "MainTab"); break;
				case SELECT: mainAttrs = new SelectTab(this, "MainTab"); break;
				case OPTGROUP: mainAttrs = new OptGroupTab(this, "MainTab"); break;
				case OPTION: mainAttrs = new OptionTab(this, "MainTab"); break;
				case TEXTAREA: mainAttrs = new TextAreaTab(this, "MainTab"); break;
				case ISINDEX: mainAttrs = new IsIndexTab(this, "MainTab"); break;
				case LABEL: mainAttrs = new LabelTab(this, "MainTab"); break;
				case LEGEND: mainAttrs = new LegendTab(this, "MainTab"); break;
				case SCRIPT: mainAttrs = new ScriptTab(parent->projectRoot(), this, "MainTab"); break;
				default:
						//						mainAttrs = new AttributesTab(this, "MainTab");
						mainAttrs = 0;
				};
		if (mainAttrs) addTab(mainAttrs, i18n("&Main"));
		if (element->coreAttrs)
				{
						CoreTab *coreAttrs = new CoreTab(this, "CoreTab");
						addTab(coreAttrs, i18n("C&ore"));
				};
		if (element->events)
				{
						EventsTab *events = new EventsTab(this, "EventsTab");
						addTab(events, i18n("&Events"));
				};
		if (element->i18n)
				{
						I18nTab *i18n = new I18nTab(this, "I18nTab");
						addTab(i18n, i18n("&i18n"));
				};
		setOkButton(i18n("&OK"));
		setCancelButton(i18n("&Cancel"));
}

QString TagEditDialog::getParameters()
{
		QObjectList *childs=queryList(0, ".*Tab");
		QObjectListIt childsIt(*childs);
		QString s;
		QObject *child;
		while ((child=childsIt.current())!=0)
				{
						s+=((AttributesTab*)child)->getParameters();
						++childsIt;
				};
		delete childs;
		return s;
}

void TagEditDialog::setParameters(StrDict *parameters)
{
		QObjectList *childs=queryList(0, ".*Tab");
		QObjectListIt childsIt(*childs);
		QObject *child;
		while ((child=childsIt.current())!=0)
				{
						((AttributesTab*)child)->setParameters(parameters);
						++childsIt;
				};
		delete childs;
}
