/* $Id: docking.cpp,v 1.1.1.1 1998/05/06 19:13:31 matt Exp $
   $Log: docking.cpp,v $

   Revision 1.2  1998/03/22 07:14:36  matt
   *** empty log message ***

   Revision 1.1  1998/03/02 14:28:43  matt
   Added animation


   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   */

#include <kwm.h>
#include <kmsgbox.h>
#include <kstring.h>
#include <kapp.h>

#include "docking.h"
#include "kget.h"

QList<QPixmap> *DockWidget::animDock;

DockWidget::DockWidget(const char *name)
  : QWidget(0, name, 0) {

  docked = false;

  QString path = KApplication::getKApplication()->kde_datadir() + "/kget/pics/";

  // Load animation
  if ( animDock->count() == 0 ) {
    animDock->setAutoDelete( true );
    for ( int i = 0; i < 7; i++ ) {
      QString n1;

      QPixmap *p1 = new QPixmap();
      n1.sprintf( "/kget-small%i.xpm", i );
      p1->load( path + n1 );
      if ( p1->isNull() ) {
	QString e;
	e << i18n( "Could not load icon\n" ) << n1.data();
	KMsgBox::message( this, i18n( "KGet Error" ), e.data(),
			  KMsgBox::EXCLAMATION );
      }
      animDock->append( p1 );
    }
  }

  // popup menu for right mouse button
  popup_m = new QPopupMenu();
  toggleID = popup_m->insertItem(i18n("Restore"),
				 this, SLOT(toggle_window_state()));

  // timer for showing statistics
  clocktimer = new QTimer(this);
  connect(clocktimer, SIGNAL(timeout()), SLOT(timeclick()));
  clocktimer->start( 200 );
  
  KDNDDropZone *myDropZone = new KDNDDropZone(this, DndURL);
  connect( myDropZone, SIGNAL( dropAction( KDNDDropZone *) ), 
	   this, SLOT( slotDropEvent( KDNDDropZone *) ) );

}


DockWidget::~DockWidget() {
  clocktimer->stop();
}


void DockWidget::slotDropEvent( KDNDDropZone *zone ){
  p_kmain->slotDropEvent ( zone );
}



void DockWidget::dock() {

  if (!docked) {

    // prepare panel to accept this widget
    KWM::setDockWindow (this->winId());

    // that's all the space there is
    this->setFixedSize(24, 24);

    // finally dock the widget
    this->show();

    docked = true;
  } 
}

void DockWidget::undock() {

  if (docked) {

    // the widget's window has to be destroyed in order 
    // to undock from the panel. Simply using hide() is
    // not enough (seems to be necessary though).

    this->hide();

    this->destroy(true, true);

    // recreate window for further dockings
    this->create(0, true, false);

    docked = false;
  }
}

const bool DockWidget::isDocked() {

  return docked;

}

void DockWidget::paintEvent (QPaintEvent *e) {

  (void) e;

  paintIcon();

}

void DockWidget::paintIcon () {

  if ( p_kmain->isTransfer() ){
    QPixmap *p1 = animDock->next();
    if (p1 == 0L)
      p1 = animDock->first();
    bitBlt(this, 0, 0, p1);
  }
  else
    bitBlt(this, 0, 0, animDock->first());

  return;
}


void DockWidget::timeclick() {

  if(this->isVisible()){
    paintIcon();
  }  
}


void DockWidget::mousePressEvent(QMouseEvent *e) {

  // open/close kget window on left mouse button 
  if ( e->button() == LeftButton ) {
    toggle_window_state();
  }

  // open popup menu on right mouse button
  if ( e->button() == RightButton ) {
    int x = e->x() + this->x();
    int y = e->y() + this->y();

    QString text;
    if(p_kmain->isVisible())
      text = i18n("Minimize");
    else
      text = i18n("Restore");
    
    popup_m->changeItem(text, toggleID);
    popup_m->popup(QPoint(x, y));
    popup_m->exec();
  }

}

void DockWidget::toggle_window_state() {

  // restore/hide connect-window
  if(p_kmain != 0L)  {
    if (p_kmain->isVisible())
      p_kmain->hide();
    else 
      p_kmain->show();
  }
}

